/* debug.h for Eterm.
 * 21 Feb 1998, vendu.
 */

#ifndef _DEBUG_H
# define _DEBUG_H

/* Assert macros stolen from my work on Ebar.  If these macros break with your cpp, let me know -- mej@eterm.org */
#  define NOP ((void)0)

#  ifdef DEBUG_MALLOC
#    if defined(__FILE__) && defined(__LINE__)
#      define ASSERT(x)  do {if (!(x)) {fatal_error("ASSERT failed at %s:%d:  %s", __FILE__, __LINE__, #x);}} while (0);
#    else
#      define ASSERT(x)  do {if (!(x)) {fatal_error("ASSERT failed:  %s", #x);}} while (0);
#    endif
#  else
#    if defined(__FILE__) && defined(__LINE__)
#      define ASSERT(x)  do {if (!(x)) {print_warning("ASSERT failed at %s:%d:  %s", __FILE__, __LINE__, #x);}} while (0);
#    else
#      define ASSERT(x)  do {if (!(x)) {print_warning("ASSERT failed:  %s", #x);}} while (0);
#    endif
#  endif

#ifdef __FILE__
#ifdef __LINE__
#define __DEBUG(s)		"%s, line %d: " s, __FILE__, __LINE__
#endif
#endif

#ifndef __DEBUG
#define __DEBUG(s)		s
#endif

#define __DEBUG_1(a)		fprintf(stderr, __DEBUG(a))
#define __DEBUG_2(a,b)		fprintf(stderr, __DEBUG(a), b)
#define __DEBUG_3(a,b,c)	fprintf(stderr, __DEBUG(a), b, c)
#define __DEBUG_4(a,b,c,d)	fprintf(stderr, __DEBUG(a), b, c, d)
#define __DEBUG_5(a,b,c,d,e)	fprintf(stderr, __DEBUG(a), b, c, d, e)

/* Macros for printing debugging messages */
# ifdef DEBUG_OPTIONS
#  define D_OPTIONS		__DEBUG_1
#  define D_OPTIONS2		__DEBUG_2
#  define D_OPTIONS3		__DEBUG_3
#  define D_OPTIONS4		__DEBUG_4
# else
#  define D_OPTIONS(a)		NOP
#  define D_OPTIONS2(a,b)	NOP
#  define D_OPTIONS3(a,b,c) 	NOP
#  define D_OPTIONS4(a,b,c,d)	NOP
# endif

# ifdef DEBUG_TTY
#  define D_TTY			__DEBUG_1
#  define D_TTY2		__DEBUG_2
#  define D_TTY3		__DEBUG_3
#  define D_TTY4		__DEBUG_4
# else
#  define D_TTY(a) NOP
#  define D_TTY2(a,b) NOP
#  define D_TTY3(a,b,c) 	NOP
#  define D_TTY4(a,b,c,d) 	NOP
# endif

# ifdef DEBUG_TTYMODE
#  define D_TTYMODE		__DEBUG_1
#  define D_TTYMODE2		__DEBUG_2
#  define D_TTYMODE3		__DEBUG_3
#  define D_TTYMODE4		__DEBUG_4
# else
#  define D_TTYMODE(a) 		NOP
#  define D_TTYMODE2(a,b) 	NOP
#  define D_TTYMODE3(a,b,c) 	NOP
#  define D_TTYMODE4(a,b,c,d) 	NOP
# endif

# ifdef DEBUG_CMD
#  define D_CMD			__DEBUG_1
#  define D_CMD2		__DEBUG_2
#  define D_CMD3		__DEBUG_3
#  define D_CMD4		__DEBUG_4
# else
#  define D_CMD(a) 		NOP
#  define D_CMD2(a,b)		NOP
#  define D_CMD3(a,b,c)		NOP
#  define D_CMD4(a,b,c,d) 	NOP
# endif

# ifdef DEBUG_IMLIB
#  define D_IMLIB		__DEBUG_1
#  define D_IMLIB2		__DEBUG_2
#  define D_IMLIB3		__DEBUG_3
#  define D_IMLIB4		__DEBUG_4
# else
#  define D_IMLIB(a)		NOP
#  define D_IMLIB2(a,b)		NOP
#  define D_IMLIB3(a,b,c)	NOP
#  define D_IMLIB4(a,b,c,d)	NOP
# endif

# ifdef DEBUG_PIXMAP
#  define D_PIXMAP		__DEBUG_1
#  define D_PIXMAP2		__DEBUG_2
#  define D_PIXMAP3		__DEBUG_3
#  define D_PIXMAP4		__DEBUG_4
#  define D_PIXMAP5		__DEBUG_5
# else
#  define D_PIXMAP(a) 		NOP
#  define D_PIXMAP2(a,b)	NOP
#  define D_PIXMAP3(a,b,c) 	NOP
#  define D_PIXMAP4(a,b,c,d) 	NOP
#  define D_PIXMAP5(a,b,c,d,e) 	NOP
# endif

# ifdef DEBUG_SCREEN
#  define D_SCREEN		__DEBUG_1
#  define D_SCREEN2		__DEBUG_2
#  define D_SCREEN3		__DEBUG_3
#  define D_SCREEN4		__DEBUG_4
#  define D_SCREEN5		__DEBUG_5
# else
#  define D_SCREEN(a) 		NOP
#  define D_SCREEN2(a,b)	NOP
#  define D_SCREEN3(a,b,c) 	NOP
#  define D_SCREEN4(a,b,c,d) 	NOP
#  define D_SCREEN5(a,b,c,d,e) 	NOP
# endif

# ifdef DEBUG_X11
#  define D_X11			__DEBUG_1
#  define D_X112		__DEBUG_2
#  define D_X113		__DEBUG_3
#  define D_X114		__DEBUG_4
#  define D_X115		__DEBUG_5
# else
#  define D_X11(a)		NOP
#  define D_X112(a,b)	 	NOP
#  define D_X113(a,b,c) 	NOP
#  define D_X114(a,b,c,d) 	NOP
#  define D_X115(a,b,c,d,e) 	NOP
# endif

# ifdef DEBUG_EVENTS
#  define D_EVENTS		__DEBUG_1
#  define D_EVENTS2		__DEBUG_2
#  define D_EVENTS3		__DEBUG_3
#  define D_EVENTS4		__DEBUG_4
# else
#  define D_EVENTS(a) 		NOP
#  define D_EVENTS2(a,b)	NOP
#  define D_EVENTS3(a,b,c) 	NOP
#  define D_EVENTS4(a,b,c,d) 	NOP
# endif

# ifdef DEBUG_SCROLLBAR
#  define D_SCROLLBAR		__DEBUG_1
#  define D_SCROLLBAR2		__DEBUG_2
#  define D_SCROLLBAR3		__DEBUG_3
#  define D_SCROLLBAR4		__DEBUG_4
# else
#  define D_SCROLLBAR(a) 	NOP
#  define D_SCROLLBAR2(a,b) 	NOP
#  define D_SCROLLBAR3(a,b,c) 	NOP
#  define D_SCROLLBAR4(a,b,c,d)	NOP
# endif

# ifdef DEBUG_THREADS
#  define D_THREADS		__DEBUG_1
#  define D_THREADS2		__DEBUG_2
#  define D_THREADS3		__DEBUG_3
#  define D_THREADS4		__DEBUG_4
# else
#  define D_THREADS(a)		NOP
#  define D_THREADS2(a,b)	NOP
#  define D_THREADS3(a,b,c) 	NOP
#  define D_THREADS4(a,b,c,d) 	NOP
# endif

#endif /* _DEBUG_H */
