/* SoundPaths.c - Sound/SoundSet paths
 * 
 *  WSoundPrefs - WMSound Server Preferences Program
 * 
 *  Copyright (c) 1998 Pascal G. Hofstee
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, 
 *  USA.
 */


#include "WSoundPrefs.h"
#include <unistd.h>

typedef struct _Panel {
    WMFrame *frame;
    char *sectionName;
    
    CallbackRec callbacks;

    WMWindow *win;

    WMFrame *sndF;
    WMList *sndL;
    WMButton *sndaB;
    WMButton *sndrB;
    WMTextField *sndT;

    WMFrame *sndsetF;
    WMList *sndsetL;
    WMButton *sndsetaB;
    WMButton *sndsetrB;
    WMTextField *sndsetT;

    WMColor *red;
    WMColor *black;
    WMColor *white;
    WMFont *font;
} _Panel;



#define ICON_FILE	"paths"


static void
addPathToList(WMList *list, int index, char *path)
{
    char *fpath = wexpandpath(path);
    WMListItem *item;

    item = WMInsertListItem(list, index, path);

    if (access(fpath, X_OK)!=0) {
	item->uflags = 1;
    }
    free(fpath);
}


static void
showData(_Panel *panel)
{
    proplist_t array, val;
    int i;
    
    array = GetObjectForKey("SoundPath");
    if (!array || !PLIsArray(array)) {
	if (array)
	    wwarning(_("bad value in option SoundPath. Using default path list"));
	addPathToList(panel->sndL, -1, "~/GNUstep/Library/WindowMaker/Sounds");
	addPathToList(panel->sndL, -1, "/usr/local/share/WindowMaker/Sounds");
    } else {
	for (i=0; i<PLGetNumberOfElements(array); i++) {
	    val = PLGetArrayElement(array, i);
	    addPathToList(panel->sndL, -1, PLGetString(val));
	}
    }

    array = GetObjectForKey("SoundSetPath");
    if (!array || !PLIsArray(array)) {
	if (array)
	    wwarning(_("bad value in option SoundSetPath. Using default path list"));
	addPathToList(panel->sndsetL, -1, "~/GNUstep/Library/WindowMaker/SoundSets");
	addPathToList(panel->sndsetL, -1, "/usr/local/share/WindowMaker/SoundSets");
    } else {
	for (i=0; i<PLGetNumberOfElements(array); i++) {
	    val = PLGetArrayElement(array, i);
	    addPathToList(panel->sndsetL, -1, PLGetString(val));
	}
    }
}


static void
pushButton(WMWidget *w, void *data)
{
    _Panel *panel = (_Panel*)data;
    int i;

    /* sound paths */
    if (w == panel->sndaB) {
	char *text = WMGetTextFieldText(panel->sndT);

	if (text && strlen(text) > 0) {
	    i = WMGetListSelectedItemRow(panel->sndL);
	    if (i >= 0) i++;
	    addPathToList(panel->sndL, i, text);
	    WMSetListBottomPosition(panel->sndL, 
				    WMGetListNumberOfRows(panel->sndL));
	}
	if (text)
	    free(text);

	WMSetTextFieldText(panel->sndT, NULL);
    } else if (w == panel->sndrB) {
	i = WMGetListSelectedItemRow(panel->sndL);

	if (i>=0)
	    WMRemoveListItem(panel->sndL, i);
    }

    /* soundset paths */
    if (w == panel->sndsetaB) {
	char *text = WMGetTextFieldText(panel->sndsetT);

	if (text && strlen(text) > 0) {
	    i = WMGetListSelectedItemRow(panel->sndsetL);
	    if (i >= 0) i++;
	    addPathToList(panel->sndsetL, i, text);
	    WMSetListBottomPosition(panel->sndsetL, 
				    WMGetListNumberOfRows(panel->sndsetL));
	}
	if (text)
	    free(text);
	WMSetTextFieldText(panel->sndsetT, NULL);
    } else if (w == panel->sndsetrB) {
	i = WMGetListSelectedItemRow(panel->sndsetL);
	
	if (i>=0)
	    WMRemoveListItem(panel->sndsetL, i);
    }
}


static void
textEditedObserver(void *observerData, WMNotification *notification)
{
    _Panel *panel = (_Panel*)observerData;

    switch ((int)WMGetNotificationClientData(notification)) {
     case WMReturnTextMovement:
	if (WMGetNotificationObject(notification) == panel->sndT)
	    WMPerformButtonClick(panel->sndaB);
	else
	    WMPerformButtonClick(panel->sndsetaB);
	break;

     case WMIllegalTextMovement:
	if (WMGetNotificationObject(notification) == panel->sndT) {
	    WMSetButtonImage(panel->sndaB, NULL);
	    WMSetButtonImage(panel->sndaB, NULL);
	} else {	
	    WMSetButtonImage(panel->sndsetaB, NULL);
	    WMSetButtonImage(panel->sndsetaB, NULL);
	}
	break;
    } 
}


static void
textBeginObserver(void *observerData, WMNotification *notification)
{
    _Panel *panel = (_Panel*)observerData;
    WMScreen *scr = WMWidgetScreen(panel->win);
    WMPixmap *arrow1 = WMGetSystemPixmap(scr, WSIReturnArrow);
    WMPixmap *arrow2 = WMGetSystemPixmap(scr, WSIHighlightedReturnArrow);

    if (WMGetNotificationObject(notification)==panel->sndT) {
	WMSetButtonImage(panel->sndaB, arrow1);
	WMSetButtonAltImage(panel->sndaB, arrow2);
    } else {	
	WMSetButtonImage(panel->sndsetaB, arrow1);
	WMSetButtonAltImage(panel->sndsetaB, arrow2);    
    }
}



static void
listClick(WMWidget *w, void *data)
{
    _Panel *panel = (_Panel*)data;
    char *t;
    
    if (w == panel->sndL) {
	t = WMGetListSelectedItem(panel->sndL)->text;
	WMSetTextFieldText(panel->sndT, t);
    } else {
	t = WMGetListSelectedItem(panel->sndsetL)->text;
	WMSetTextFieldText(panel->sndsetT, t);
    }
}



static void
paintItem(WMList *lPtr, int index, Drawable d, char *text, int state, WMRect *rect)
{
    int width, height, x, y;
    _Panel *panel = (_Panel*)WMGetHangedData(lPtr);
    WMScreen *scr = WMWidgetScreen(lPtr);
    Display *dpy = WMScreenDisplay(scr);

    width = rect->size.width;
    height = rect->size.height;
    x = rect->pos.x;
    y = rect->pos.y;

    if (state & WLDSSelected)
	XFillRectangle(dpy, d, WMColorGC(panel->white), x, y, width, 
		       height);
    else 
	XClearArea(dpy, d, x, y, width, height, False);

    if (state & 1) {
	WMDrawString(scr, d, WMColorGC(panel->red), panel->font, x+4, y, 
		     text, strlen(text));
    } else {
	WMDrawString(scr, d, WMColorGC(panel->black), panel->font, x+4, y, 
		     text, strlen(text));
    }
}



static void
storeData(_Panel *panel)
{
    proplist_t list;
    proplist_t tmp;
    int i;
    char *p;
    
    list = PLMakeArrayFromElements(NULL, NULL);
    for (i=0; i<WMGetListNumberOfRows(panel->sndL); i++) {
	p = WMGetListItem(panel->sndL, i)->text;
	tmp = PLMakeString(p);
	PLAppendArrayElement(list, tmp);
    }
    SetObjectForKey(list, "SoundPath");
    
    list = PLMakeArrayFromElements(NULL, NULL);
    for (i=0; i<WMGetListNumberOfRows(panel->sndsetL); i++) {
	p = WMGetListItem(panel->sndsetL, i)->text;
	tmp = PLMakeString(p);
	PLAppendArrayElement(list, tmp);
    }
    SetObjectForKey(list, "SoundSetPath");
}



static void
createPanel(Panel *p)
{
    _Panel *panel = (_Panel*)p;
    WMScreen *scr = WMWidgetScreen(panel->win);

    panel->white = WMWhiteColor(scr);
    panel->black = WMBlackColor(scr);
    panel->red = WMCreateRGBColor(scr, 0xffff, 0, 0, True);
    panel->font = WMSystemFontOfSize(scr, 12);

    panel->frame = WMCreateFrame(panel->win);
    WMResizeWidget(panel->frame, FRAME_WIDTH, FRAME_HEIGHT);
    WMMoveWidget(panel->frame, FRAME_LEFT, FRAME_TOP);

    /* sound path */
    panel->sndF = WMCreateFrame(panel->frame);
    WMResizeWidget(panel->sndF, 230, 210);
    WMMoveWidget(panel->sndF, 25, 10);
    WMSetFrameTitle(panel->sndF, _("Sound Search Paths"));
    
    panel->sndL = WMCreateList(panel->sndF);
    WMResizeWidget(panel->sndL, 200, 120);
    WMMoveWidget(panel->sndL, 15, 20);
    WMSetListAction(panel->sndL, listClick, panel);
    WMSetListUserDrawProc(panel->sndL, paintItem);
    WMHangData(panel->sndL, panel);
    
    panel->sndaB = WMCreateCommandButton(panel->sndF);
    WMResizeWidget(panel->sndaB, 90, 24);
    WMMoveWidget(panel->sndaB, 125, 145);
    WMSetButtonText(panel->sndaB, _("Add"));
    WMSetButtonAction(panel->sndaB, pushButton, panel);
    WMSetButtonImagePosition(panel->sndaB, WIPRight);

    panel->sndrB = WMCreateCommandButton(panel->sndF);
    WMResizeWidget(panel->sndrB, 90, 24);
    WMMoveWidget(panel->sndrB, 15, 145);
    WMSetButtonText(panel->sndrB, _("Remove"));
    WMSetButtonAction(panel->sndrB, pushButton, panel);

    panel->sndT = WMCreateTextField(panel->sndF);
    WMResizeWidget(panel->sndT, 200, 20);
    WMMoveWidget(panel->sndT, 15, 175);
    WMAddNotificationObserver(textEditedObserver, panel, 
			      WMTextDidEndEditingNotification, panel->sndT);
    WMAddNotificationObserver(textBeginObserver, panel,
			      WMTextDidBeginEditingNotification, panel->sndT);

    WMMapSubwidgets(panel->sndF);

    /* soundset path */
    panel->sndsetF = WMCreateFrame(panel->frame);
    WMResizeWidget(panel->sndsetF, 230, 210);
    WMMoveWidget(panel->sndsetF, 270, 10);
    WMSetFrameTitle(panel->sndsetF, _("SoundSet Search Paths"));
    
    panel->sndsetL = WMCreateList(panel->sndsetF);
    WMResizeWidget(panel->sndsetL, 200, 120);
    WMMoveWidget(panel->sndsetL, 15, 20);
    WMSetListAction(panel->sndsetL, listClick, panel);
    WMSetListUserDrawProc(panel->sndsetL, paintItem);
    WMHangData(panel->sndsetL, panel);
    
    panel->sndsetaB = WMCreateCommandButton(panel->sndsetF);
    WMResizeWidget(panel->sndsetaB, 90, 24);
    WMMoveWidget(panel->sndsetaB, 125, 145);
    WMSetButtonText(panel->sndsetaB, _("Add"));
    WMSetButtonAction(panel->sndsetaB, pushButton, panel);
    WMSetButtonImagePosition(panel->sndsetaB, WIPRight);
    
    panel->sndsetrB = WMCreateCommandButton(panel->sndsetF);
    WMResizeWidget(panel->sndsetrB, 90, 24);
    WMMoveWidget(panel->sndsetrB, 15, 145);
    WMSetButtonText(panel->sndsetrB, _("Remove"));
    WMSetButtonAction(panel->sndsetrB, pushButton, panel);
    
    panel->sndsetT= WMCreateTextField(panel->sndsetF);
    WMResizeWidget(panel->sndsetT, 200, 20);
    WMMoveWidget(panel->sndsetT, 15, 175);
    WMAddNotificationObserver(textEditedObserver, panel, 
			      WMTextDidEndEditingNotification, panel->sndsetT);
    WMAddNotificationObserver(textBeginObserver, panel,
			      WMTextDidBeginEditingNotification, panel->sndsetT);

    WMMapSubwidgets(panel->sndsetF);
    
    WMRealizeWidget(panel->frame);
    WMMapSubwidgets(panel->frame);
    
    showData(panel);
}



Panel*
InitPaths(WMScreen *scr, WMWindow *win)
{
    _Panel *panel;

    panel = wmalloc(sizeof(_Panel));
    memset(panel, 0, sizeof(_Panel));

    panel->sectionName = _("Search Path Configuration");

    panel->win = win;

    panel->callbacks.createWidgets = createPanel;
    panel->callbacks.updateDomain = storeData;
    
    AddSection(panel, ICON_FILE);

    return panel;
}
