/*  WSoundPrefs.c - main window and other basic stuff
 *
 *  WSoundPrefs - WMSound Server Preferences Program
 *
 *  Copyright (c) 1998 Pascal G. Hofstee
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#include "WSoundPrefs.h"
#include <assert.h>


extern Panel *InitSoundEvents(WMScreen *scr, WMWindow *win);
extern Panel *InitPaths(WMScreen *scr, WMWindow *win);


#define MAX_SECTIONS 2

typedef struct _WSoundPrefs {
    WMWindow *win;

    WMScrollView *scrollV;
    WMFrame *buttonF;
    WMButton *sectionB[MAX_SECTIONS];

    int sectionCount;

    WMButton *saveBtn;
    WMButton *closeBtn;
    WMButton *undoBtn;
    WMButton *undosBtn;

    WMFrame *banner;
    WMLabel *nameL;
    WMLabel *versionL;
    WMLabel *creditsL;
    WMLabel *statusL;

    Panel *currentPanel;
} _WSoundPrefs;

static _WSoundPrefs WSoundPrefs;

/* user defaults dictionary */
proplist_t WMSoundDomain = NULL;



static Bool TIFFOK = False;


#define INITIALIZED_PANEL	(1<<0)



static void loadConfigurations(WMScreen *scr, WMWindow *mainw);

static void savePanelData(Panel *panel);


void
quit(WMWidget *w, void *data)
{
    exit(0);
}


static void
save(WMWidget *w, void *data)
{
    int i;
    proplist_t keyList;

/*
    puts("gathering data");
*/
    for (i=0; i<WSoundPrefs.sectionCount; i++) {
        PanelRec *rec = WMGetHangedData(WSoundPrefs.sectionB[i]);
        if ((rec->callbacks.flags & INITIALIZED_PANEL))
            savePanelData((Panel*)rec);
    }
    /* compare the user dictionary with the global and remove redundant data */
    keyList = PLGetAllDictionaryKeys(WMSoundDomain);

    PLRelease(keyList);
/*
    puts("storing data");
*/

    PLSave(WMSoundDomain, YES);
}


static void
undo(WMWidget *w, void *data)
{
    PanelRec *rec = (PanelRec*)WSoundPrefs.currentPanel;

    if (!rec)
        return;

    if (rec->callbacks.undoChanges
        && (rec->callbacks.flags & INITIALIZED_PANEL)) {
        (*rec->callbacks.undoChanges)(WSoundPrefs.currentPanel);
    }
}


static void
undoAll(WMWidget *w, void *data)
{
    int i;

    for (i=0; i<WSoundPrefs.sectionCount; i++) {
        PanelRec *rec = WMGetHangedData(WSoundPrefs.sectionB[i]);

        if (rec->callbacks.undoChanges
            && (rec->callbacks.flags & INITIALIZED_PANEL))
            (*rec->callbacks.undoChanges)((Panel*)rec);
    }
}



static void
createMainWindow(WMScreen *scr)
{
    WMScroller *scroller;
    WMFont *font;
    char buffer[128];

    WSoundPrefs.win = WMCreateWindow(scr, "wsoundprefs");
    WMResizeWidget(WSoundPrefs.win, 520, 390);
    WMSetWindowTitle(WSoundPrefs.win, _("WMSound Server Preferences"));
    WMSetWindowCloseAction(WSoundPrefs.win, quit, NULL);
    WMSetWindowMaxSize(WSoundPrefs.win, 520, 390);
    WMSetWindowMinSize(WSoundPrefs.win, 520, 390);
    WMSetWindowMiniwindowTitle(WSoundPrefs.win, _("WSoundPrefs"));

    WSoundPrefs.scrollV = WMCreateScrollView(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.scrollV, 500, 87);
    WMMoveWidget(WSoundPrefs.scrollV, 10, 10);
    WMSetScrollViewRelief(WSoundPrefs.scrollV, WRSunken);
    WMSetScrollViewHasHorizontalScroller(WSoundPrefs.scrollV, True);
    WMSetScrollViewHasVerticalScroller(WSoundPrefs.scrollV, False);
    scroller = WMGetScrollViewHorizontalScroller(WSoundPrefs.scrollV);
    WMSetScrollerArrowsPosition(scroller, WSANone);

    WSoundPrefs.buttonF = WMCreateFrame(WSoundPrefs.win);
    WMSetFrameRelief(WSoundPrefs.buttonF, WRFlat);

    WMSetScrollViewContentView(WSoundPrefs.scrollV, WMWidgetView(WSoundPrefs.buttonF));

    WSoundPrefs.undosBtn = WMCreateCommandButton(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.undosBtn, 90, 28);
    WMMoveWidget(WSoundPrefs.undosBtn, 135, 350);
    WMSetButtonText(WSoundPrefs.undosBtn, _("Revert Page"));
    WMSetButtonAction(WSoundPrefs.undosBtn, undo, NULL);

    WSoundPrefs.undoBtn = WMCreateCommandButton(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.undoBtn, 90, 28);
    WMMoveWidget(WSoundPrefs.undoBtn, 235, 350);
    WMSetButtonText(WSoundPrefs.undoBtn, _("Revert All"));
    WMSetButtonAction(WSoundPrefs.undoBtn, undoAll, NULL);

    WSoundPrefs.saveBtn = WMCreateCommandButton(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.saveBtn, 80, 28);
    WMMoveWidget(WSoundPrefs.saveBtn, 335, 350);
    WMSetButtonText(WSoundPrefs.saveBtn, _("Save"));
    WMSetButtonAction(WSoundPrefs.saveBtn, save, NULL);

    WSoundPrefs.closeBtn = WMCreateCommandButton(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.closeBtn, 80, 28);
    WMMoveWidget(WSoundPrefs.closeBtn, 425, 350);
    WMSetButtonText(WSoundPrefs.closeBtn, _("Close"));
    WMSetButtonAction(WSoundPrefs.closeBtn, quit, NULL);

    /* banner */
    WSoundPrefs.banner = WMCreateFrame(WSoundPrefs.win);
    WMResizeWidget(WSoundPrefs.banner, FRAME_WIDTH, FRAME_HEIGHT);
    WMMoveWidget(WSoundPrefs.banner, FRAME_LEFT, FRAME_TOP);
    WMSetFrameRelief(WSoundPrefs.banner, WRFlat);

    font = WMCreateFont(scr, "-*-times-bold-r-*-*-24-*-*-*-*-*-*-*");
    if (!font)
        font = WMBoldSystemFontOfSize(scr, 24);
    WSoundPrefs.nameL = WMCreateLabel(WSoundPrefs.banner);
    WMSetLabelTextAlignment(WSoundPrefs.nameL, WACenter);
    WMResizeWidget(WSoundPrefs.nameL, FRAME_WIDTH-20, 30);
    WMMoveWidget(WSoundPrefs.nameL, 10, 25);
    WMSetLabelFont(WSoundPrefs.nameL, font);
    WMSetLabelText(WSoundPrefs.nameL, "WMSound Server Preferences Utility");
    WMReleaseFont(font);

    WSoundPrefs.versionL = WMCreateLabel(WSoundPrefs.banner);
    WMResizeWidget(WSoundPrefs.versionL, FRAME_WIDTH-20, 20);
    WMMoveWidget(WSoundPrefs.versionL, 10, 65);
    WMSetLabelTextAlignment(WSoundPrefs.versionL, WACenter);
    sprintf(buffer, _("Version %s for Window Maker %s"), WVERSION, WMVERSION);
    WMSetLabelText(WSoundPrefs.versionL, buffer);

    WSoundPrefs.statusL = WMCreateLabel(WSoundPrefs.banner);
    WMResizeWidget(WSoundPrefs.statusL, FRAME_WIDTH-20, 60);
    WMMoveWidget(WSoundPrefs.statusL, 10, 100);
    WMSetLabelTextAlignment(WSoundPrefs.statusL, WACenter);
    WMSetLabelText(WSoundPrefs.statusL, _("Starting..."));

    WSoundPrefs.creditsL = WMCreateLabel(WSoundPrefs.banner);
    WMResizeWidget(WSoundPrefs.creditsL, FRAME_WIDTH-20, 60);
    WMMoveWidget(WSoundPrefs.creditsL, 10, FRAME_HEIGHT-60);
    WMSetLabelTextAlignment(WSoundPrefs.creditsL, WACenter);
    WMSetLabelText(WSoundPrefs.creditsL, _("Programming: Pascal G. Hofstee\n"
				"Based on WPrefs by: Alfredo K. Kojima\n"
				"Artwork by: Pascal G. Hofstee and Marco van Hylckama Vlieg\n"));


    WMMapSubwidgets(WSoundPrefs.win);

    WMUnmapWidget(WSoundPrefs.undosBtn);
    WMUnmapWidget(WSoundPrefs.undoBtn);
    WMUnmapWidget(WSoundPrefs.saveBtn);
}


static void
showPanel(Panel *panel)
{
    PanelRec *rec = (PanelRec*)panel;

    if (!(rec->callbacks.flags & INITIALIZED_PANEL)) {
        (*rec->callbacks.createWidgets)(panel);
        rec->callbacks.flags |= INITIALIZED_PANEL;
    }

    WMSetWindowTitle(WSoundPrefs.win, rec->sectionName);

    WMMapWidget(rec->frame);
}


static void
hidePanel(Panel *panel)
{
    PanelRec *rec = (PanelRec*)panel;

    WMUnmapWidget(rec->frame);
}



void    
Initialize(WMScreen *scr)                             
{
    char **list;
    int i;
    char *path;
    WMPixmap *icon;


    list = RSupportedFileFormats();
    for (i=0; list[i]!=NULL; i++) {
        if (strcmp(list[i], "TIFF")==0) {
            TIFFOK = True;
            break;                    
        }                                   
    }
    RFreeStringList(list);         

    if (TIFFOK)               
        path = WMPathForResourceOfType("WSoundPrefs.tiff", NULL);
    else
        path = WMPathForResourceOfType("WSoundPrefs.xpm", NULL);
    if (path) {
        RImage *tmp;

        tmp = RLoadImage(WMScreenRContext(scr), path, 0);
        if (!tmp) {
            wwarning("could not load image file %s:%s", path, RMessageForError(RErrorCode));
        } else {
            icon = WMCreatePixmapFromRImage(scr, tmp, 0);
            RDestroyImage(tmp);
            if (icon) {
                WMSetApplicationIconImage(scr, icon);
                WMReleasePixmap(icon);
            }
        }
        free(path);
    }

    memset(&WSoundPrefs, 0, sizeof(_WSoundPrefs));
    createMainWindow(scr);

    WMRealizeWidget(WSoundPrefs.win);
    WMMapWidget(WSoundPrefs.win);
    XFlush(WMScreenDisplay(scr));
    WMSetLabelText(WSoundPrefs.statusL, _("Loading WMSound Server configuration file..."));
    XFlush(WMScreenDisplay(scr));
    loadConfigurations(scr, WSoundPrefs.win);

    WMSetLabelText(WSoundPrefs.statusL, _("Initializing configuration panels..."));

    InitSoundEvents(scr, WSoundPrefs.win);
    InitPaths(scr, WSoundPrefs.win);

    WMRealizeWidget(WSoundPrefs.scrollV);

    WMSetLabelText(WSoundPrefs.statusL, _("Checking for WMSound Server..."));
    if (!wmsndFindSoundServer ()) {
	wwarning ("The WMSound Server doesn't seem to be running ... You will not be able to use the Play button.");
        WMRunAlertPanel(scr, WSoundPrefs.win, _("Warning"), _("The WMSound Server doesn't seem to be running ...\nYou will not be able to use the Play button."), _("OK"), NULL, NULL);
    }

    WMSetLabelText(WSoundPrefs.statusL, "This program is still under development. Backup your ~/GNUstep/Defaults directory, before using it.");
}

static void
loadConfigurations(WMScreen *scr, WMWindow *mainw)
{
    proplist_t db;
    char *path;
    FILE *file;
    char buffer[1024];
    char mbuf[1024];
    int v1, v2, v3;

    path = wdefaultspathfordomain("WMSound");

    db = PLGetProplistWithPath(path);
    if (db) {                           
        if (!PLIsDictionary(db)) {                            
            PLRelease(db);      
            db = NULL;
            sprintf(mbuf, _("WMSound domain (%s) is corrupted!"), path);
            WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
        }                              
    } else {
        sprintf(mbuf, _("Could not load WMSound domain (%s) from defaults database."),
                path);
        WMRunAlertPanel(scr, mainw, _("Error"), mbuf, _("OK"), NULL, NULL);
    }
    free(path);

    if (!db) {
        db = PLMakeDictionaryFromEntries(NULL, NULL, NULL);
    }

    WMSoundDomain = db;
}


static void
savePanelData(Panel *panel)
{
    PanelRec *rec = (PanelRec*)panel;
    
    if (rec->callbacks.updateDomain) {
        (*rec->callbacks.updateDomain)(panel);
    }
}


static void                                                    
changeSection(WMWidget *self, void *data)                                
{
    if (WSoundPrefs.banner) {
        WMDestroyWidget(WSoundPrefs.banner);
        WSoundPrefs.banner = NULL;
/*      WMMapWidget(WSoundPrefs.undosBtn);
        WMMapWidget(WSoundPrefs.undoBtn);
 */
        WMMapWidget(WSoundPrefs.saveBtn);
    }                                                          

    showPanel(data);

    if (WSoundPrefs.currentPanel)                                  
        hidePanel(WSoundPrefs.currentPanel);
    WSoundPrefs.currentPanel = data;
}


char*
LocateImage(char *name)
{
    char *path;
    char *tmp = wmalloc(strlen(name)+8);

    if (TIFFOK) {
        sprintf(tmp, "%s.tiff", name);
        path = WMPathForResourceOfType(tmp, "tiff");
    } else {
        sprintf(tmp, "%s.xpm", name);
        path = WMPathForResourceOfType(tmp, "xpm");
    }
    free(tmp);
    if (!path) {
        wwarning(_("could not locate image file %s\n"), name);
    }

    return path;
}


void
AddSection(Panel *panel, char *iconFile)
{
    WMButton *bPtr;
    WMPixmap *icon;
    RColor color;
    char *iconPath;

    assert(WSoundPrefs.sectionCount < MAX_SECTIONS);

    iconPath = LocateImage(iconFile);

    bPtr = WMCreateCustomButton(WSoundPrefs.buttonF, WBBStateLightMask
                                |WBBStateChangeMask);
    WMResizeWidget(bPtr, 64, 64);
    WMMoveWidget(bPtr, WSoundPrefs.sectionCount*64, 0);
    WMSetButtonImagePosition(bPtr, WIPImageOnly);
    WMSetButtonAction(bPtr, changeSection, panel);
    WMHangData(bPtr, panel);

    color.red = 0xae;
    color.green = 0xaa;
    color.blue = 0xae;
    color.alpha = 0;
    if (iconPath) {
        icon = WMCreateBlendedPixmapFromFile(WMWidgetScreen(WSoundPrefs.win),
                                             iconPath, &color);
        if (!icon)
            wwarning(_("could not load icon file %s"), iconPath);
    } else {
        icon = NULL;
    }

    WMSetButtonImage(bPtr, icon);

    if (icon)
        WMReleasePixmap(icon);

    color.red = 0xff;
    color.green = 0xff;
    color.blue = 0xff;
    color.alpha = 0;
    if (iconPath) {
        icon = WMCreateBlendedPixmapFromFile(WMWidgetScreen(WSoundPrefs.win),
                                             iconPath, &color);
        if (!icon)
            wwarning(_("could not load icon file %s"), iconPath);
    } else {
        icon = NULL;
    }

    WMSetButtonAltImage(bPtr, icon);

    if (icon)
        WMReleasePixmap(icon);

    WMMapWidget(bPtr);

    WSoundPrefs.sectionB[WSoundPrefs.sectionCount] = bPtr;

    if (WSoundPrefs.sectionCount > 0) {
        WMGroupButtons(WSoundPrefs.sectionB[0], bPtr);
    }

    WSoundPrefs.sectionCount++;

    WMResizeWidget(WSoundPrefs.buttonF, WSoundPrefs.sectionCount*64, 64);

    free(iconPath);
}

