/* WSoundPrefs.h- general definitions
 *
 *  WSoundPrefs - WMSound Server Preferences Program
 *
 *  Copyright (c) 1998 Pascal G.Hofstee
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#ifndef WSOUNDPREFS_H_
#define WSOUNDPREFS_H_

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <X11/Xlib.h>

#include <proplist.h>

#include <wraster.h>

#include <WINGs.h>
#include <WUtil.h>

#include <wmsnd.h>

#include "PLFunctions.h"

#define WVERSION        "0.9.1"
#define WMVERSION       "0.50.x"


typedef struct _Panel Panel;           

typedef struct {                                                   
    unsigned flags;                    /* reserved for WSoundPrefs.c Don't access it */

    void (*createWidgets)(Panel*);     /* called when showing for first time */
    void (*updateDomain)(Panel*);      /* save the changes to the dictionary */
    Bool (*validate)(Panel*);          /* do validation checks if necessary */
    void (*undoChanges)(Panel*);       /* reset values to those in the dictionary */
} CallbackRec;                                                           


/* all Panels must start with the following layout */                   
typedef struct PanelRec {                                               
    WMFrame *frame;                                                  

    char *sectionName;                 /* section name to display in titlebar */

    CallbackRec callbacks;
} PanelRec;

void AddSection(Panel *panel, char *iconFile);

char *LocateImage(char *name);

/* manipulate the dictionary for the WMSound domain */
proplist_t GetObjectForKey(char *defaultName);
void SetObjectForKey(proplist_t object, char *defaultName);

#define FRAME_TOP       105
#define FRAME_LEFT      -2
#define FRAME_WIDTH     524
#define FRAME_HEIGHT    235


/* 
 * Needed for HAVE_LIBINTL_H
 */              
#if HAVE_LIBINTL_H && I18N
# include <libintl.h>
# define _(text) gettext(text)
#else
# define _(text) (text)
#endif


#endif
