	/*

	Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#include "execdlg.h"
#include <qlayout.h>
#include <qlabel.h>
#include <qframe.h>
#include <kbuttonbox.h>
#include <qbttngrp.h>
#include <qradiobt.h>
#include <qlined.h>
#include <kapp.h>
#include <kmsgbox.h>

static void min_size(QWidget *w) {
  w->setMinimumSize(w->sizeHint());
} 

ExecDlg::ExecDlg(QWidget *parent, Arts::Synthesizer *Synthesizer) :QDialog(parent,"X", TRUE)
{
	this->Synthesizer = Synthesizer;

	setCaption("Arts Module Execution");

	mainlayout = new QVBoxLayout(this);

// caption label: "Synthesis running..."

	mainlayout->addSpacing(5);
	QLabel *captionlabel = new QLabel(this);
	QFont labelfont(captionlabel->font());
	labelfont.setPointSize(labelfont.pointSize()*3/2);
	captionlabel->setFont(labelfont);
	captionlabel->setText(QString(" ")+i18n("Synthesis running...")+QString(" "));
	captionlabel->setAlignment(AlignCenter);
	min_size(captionlabel);
	mainlayout->addWidget(captionlabel);

	cpuusagelabel = new QLabel(this);
	cpuusagelabel->setText(i18n("cpu usage: unknown"));

    cpuusagetimer = new QTimer( this );
	connect( cpuusagetimer, SIGNAL(timeout()),
		this, SLOT(updateCpuUsage()) );
	cpuusagetimer->start( 2000, false );

	min_size(cpuusagelabel);
	mainlayout->addWidget(cpuusagelabel);

//
	sliderlayout = new QVBoxLayout;
	mainlayout->addLayout(sliderlayout);

// hruler

	mainlayout->addSpacing(5);
	QFrame *ruler = new QFrame(this);
	ruler->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	ruler->setMinimumSize(50,2);
	mainlayout->addWidget(ruler);

	mainlayout->addSpacing(5);

// buttons

	QHBoxLayout *buttonlayout = new QHBoxLayout;
	mainlayout->addSpacing(5);
	mainlayout->addLayout(buttonlayout);
	mainlayout->addSpacing(5);

	buttonlayout->addSpacing(5);
	KButtonBox *bbox = new KButtonBox(this);

	QButton *helpbutton = bbox->addButton(i18n("Help"));

	bbox->addStretch(1);

	QButton *okbutton = bbox->addButton(i18n("Okay"));
	connect( okbutton, SIGNAL( clicked() ), SLOT(stop() ) );
	connect( okbutton, SIGNAL( clicked() ), SLOT(accept() ) );

	bbox->layout();
	//min_size(bbox);

	buttonlayout->addWidget(bbox);
	buttonlayout->addSpacing(5);

//	mainlayout->freeze();

	this->Synthesizer = Synthesizer;
};

void ExecDlg::addVar(QString var)
{
	printf("-------------------------------\n");
	printf("  got var %s\n",var.data());
	printf("-------------------------------\n");
	
	QScrollBar *bar = new SynthScrollBar(this,var,Synthesizer);

    bar->setMinimumSize(bar->sizeHint());
	bar->setOrientation(QScrollBar::Horizontal);
	sliderlayout->addWidget(bar);
}

void ExecDlg::start()
{
	mainlayout->freeze();
	Synthesizer->Start(128);
}

void ExecDlg::updateCpuUsage()
{
	char cpuusage[100];

	Arts::Status s = Synthesizer->getStatus();
	if(s.halted)
	{
		accept();
		stop();
		cpuusagetimer->stop();
		KMsgBox::message(this,i18n("Error"),i18n("Your synthesis has been interrupted due to too high cpu load."),KMsgBox::STOP);
	}
	sprintf(cpuusage,"%s%3.2f%%",i18n("cpu usage: "),s.cpu_usage*100);
	cpuusagelabel->setText(cpuusage);
}

void ExecDlg::stop()
{
	Synthesizer->Stop();
}

//--- SynthScrollBar

SynthScrollBar::SynthScrollBar(QWidget *parent, QString varname, Arts::Synthesizer *Synthesizer) :QScrollBar(parent)
{
	this->varname = varname;
	this->Synthesizer = Synthesizer;
	connect(this,SIGNAL(valueChanged(int)),this,SLOT(setSynthVar(int)));
	setValue(50);
}

void SynthScrollBar::setSynthVar(int newvalue)
{
//	Synthesizer->setVar(CORBA::string_dup(varname.data()), (float)(newvalue)/50-1);
//	not that way any more
}
