	/*

	Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#include "module.h"
#include "utils.h"

ModulePort::ModulePort(Module *owner, char *description, int portnr,
						 Direction direction, Arts::PortDesc *PortDesc)
{
	selected = false;

	this->owner = owner;				// whats the syntax for that again?
	this->portnr = portnr;
	this->direction = direction;
	this->PortDesc = PortDesc;
	this->description = description;

	pdID = PortDesc->ID();

	isinitarg = (PortDesc->Type().ConnType == Arts::property);
	conntype = none;
	route_owner = 0;
}

QString ModulePort::getID()
{
	QString id,dstr;

	if(direction == in) dstr = "in"; else dstr = "out";

	printf("portnr = %d, dstr = %s  => id =",portnr,dstr.data());
	id.sprintf("%s_%sport%d",owner->getID().data(),dstr.data(),portnr);
	printf("%s\n",id.data());
	return(id);
}

void ModulePort::save(KSimpleConfig& out)
{
/*
	out.setGroup(getID().data());

	out.writeEntry("connvalue",connvalue);
	out.writeEntry("strvalue",strvalue);
	switch(conntype)
	{
		case none: out.writeEntry("conntype","none");
			break;
		case dest: out.writeEntry("conntype","dest");
				   out.writeEntry("connmodule",connection->getID());
			break;
		case source: out.writeEntry("conntype","source");
			break;
		case value: out.writeEntry("conntype","value");
			break;
		case conf: out.writeEntry("conntype","conf");
			break;
	}
*/
}

void ModulePort::load(KSimpleConfig& in, list<Module *> &ModuleList)
{
	printf(">>mpload\n");
	in.setGroup(getID().data());

	float	connvalue = in.readDoubleNumEntry("connvalue");
	QString strvalue = in.readEntry("strvalue");

	QString str_conntype(in.readEntry("conntype"));

	if(str_conntype == "none") conntype = none;
	if(str_conntype == "dest")
	{
		QString connmodule = in.readEntry("connmodule");

		list<Module *>::iterator mi;
		for(mi=ModuleList.begin();mi!=ModuleList.end();mi++)
		{
			ModulePort *otherport = (*mi)->findPort(connmodule);
			if(otherport)
			{
				PortDesc->connectTo(otherport->PortDesc);
			}
		}
		
/*
		connection = NULL;
		conntype = dest;
		list<Module *>::iterator mi;
		for(mi=ModuleList.begin();mi!=ModuleList.end();mi++)
		{
			ModulePort *otherport = (*mi)->findPort(connmodule);

			if(otherport) connection = otherport;
		}

		if(!connection)
		{
			fprintf(stderr,"PANIC: connecting to object which is not there\n");
			conntype = none;
		}
*/
	}
	if(str_conntype == "source") conntype = source;
	if(str_conntype == "value")
	{
		conntype = value;
		if(isinitarg)
			PortDesc->StringValue(strvalue);
		else
			PortDesc->FloatValue(connvalue);
	}
	if(str_conntype == "conf") conntype = conf;
	printf("<<mpload\n");
}

Module::Module(Arts::ModuleDesc *module, Arts::StructureDesc *structuredesc)
{
	StructureDesc = Arts::StructureDesc::_duplicate(structuredesc);
	ModuleDesc = Arts::ModuleDesc::_duplicate(module);

	x = ModuleDesc->X();
	y = ModuleDesc->Y();
	initModule();
}

Module::Module(const Arts::ModuleInfo *minfo, Arts::StructureDesc *structuredesc)
{
	StructureDesc = Arts::StructureDesc::_duplicate(structuredesc);
	ModuleDesc = StructureDesc->createModuleDesc(*minfo);
	
	initModule();
}

void Module::initModule()
{
	QString iconname;

	selected = false;
	height = 1;
	type = ModuleDesc->Name();
	pixmap = NULL;

// test if pixmap available

	iconname = type+".xpm";
	FILE *test=fopen(iconname.data(),"r");
	if(test)
	{
		pixmap = new QPixmap(iconname);
		fclose(test);
	}
	
// create lists with inports & outports for this module
// and bind them to it ...

	printf("Getting ports...\n");
	Arts::PortDescSeq_var ports = ModuleDesc->Ports();
	long portpos = 0, incount = 0, outcount = 0;

	for(portpos=0; portpos<ports->length(); portpos++)
	{
		Arts::PortDesc *pd = (*ports)[portpos];
		ModulePort *p;

		printf("CREATING %s\n",pd->Name());
		switch(pd->Type().Direction)
		{
			case Arts::input:
					p = new ModulePort(this,pd->Name(),incount++,
							ModulePort::in,Arts::PortDesc::_duplicate(pd));
					inports.push_back(p);
				break;
			case Arts::output:
					p = new ModulePort(this,pd->Name(),outcount++,
							ModulePort::out,Arts::PortDesc::_duplicate(pd));
					outports.push_back(p);
				break;
			default:
				assert(false);	// shouldn't happen!
		}
	}

	width = 1+imax(inports.size(),outports.size()+1);

	mdID = ModuleDesc->ID();
}

Module::~Module()
{
	StructureDesc->freeModuleDesc(ModuleDesc);
}

void Module::save(KSimpleConfig &out)
{
	out.setGroup(getID().data());

	out.writeEntry("x",x);
	out.writeEntry("y",y);
	out.writeEntry("selected",selected);
	out.writeEntry("type",type.data());

	int i;
	list<ModulePort *>::iterator port;

	for(i=0, port=inports.begin();port != inports.end(); port++,i++)
		(*port)->save(out);

	for(i=0, port=outports.begin();port != outports.end(); port++,i++)
		(*port)->save(out);
}

void Module::load(KSimpleConfig &in)
{
	in.setGroup(getID().data());

	x = in.readNumEntry("x");
	y = in.readNumEntry("y");

	assert(ModuleDesc->moveTo(x,y));

	selected = in.readBoolEntry("selected");
}

void Module::loadPorts(KSimpleConfig &in, list<Module *> &ModuleList)
{
	int i;
	list<ModulePort *>::iterator port;

	for(i=0, port=inports.begin();port != inports.end(); port++,i++)
		(*port)->load(in, ModuleList);

	for(i=0, port=outports.begin();port != outports.end(); port++,i++)
		(*port)->load(in, ModuleList);
}

QString Module::getID()
{
	QString id;

	id.sprintf("module%d",modnr);
	return(id);
}

ModulePort *Module::findPort(QString id)
{
	list<ModulePort *>::iterator i;

	for(i=inports.begin();i != inports.end();i++)
		if((*i)->getID() == id) return(*i);

	for(i=outports.begin();i != outports.end();i++)
		if((*i)->getID() == id) return(*i);

	return(NULL);
}

void Module::portConnect()
{
/*
	list<ModulePort *>::iterator i;

	printf("Connecting Ports, module %s\n",(const char *)type);
	for(i=inports.begin();i != inports.end();i++)
	{
		ModulePort *in = (*i);
		if(in->conntype == ModulePort::dest)
			in->PortDesc->connectTo(in->connection->PortDesc);
		printf("- port %s\n",(const char *)in->description);
		if(in->isinitarg)
		{
			in->PortDesc->StringValue(in->strvalue);
		}
		else
		{
			if(in->conntype != ModulePort::none)
				printf("not connected\n");
			assert(in->conntype != ModulePort::conf);	// fixme	
		}
		printf("ok\n");
	}	
*/
}

void Module::move(int newx, int newy)
{
	x = newx;
	y = newy;
	ModuleDesc->moveTo(x,y);
}

/*
ModulePort *Module::findPort(int xoffset, Direction direction)
{
	list<ModulePort *> *ports;
	list<ModulePort *>::iterator i;

	int n;

	if(direction == 0) ports = &m->inports; else ports = &m->outports;

	i = ports->begin();
	n = xoffset-1-direction;

	if(n<ports->size() && n>=0)
	{
		while(n>0) { n--; i++; }
		return(*i);
	}
	return(NULL);
}
*/
