#include "structure.h"

Structure::Structure(Arts::Synthesizer *Synthesizer)
{
	this->Synthesizer = Synthesizer;
	StructureDesc = Synthesizer->createStructureDesc();
	StructureDesc->incRef();
	execID = 0;
}

Structure::~Structure()
{
	printf("%ld\n",StructureDesc->obtainID());
	printf("releasing structuredesc from structure\n");
	StructureDesc->decRef();
}

void Structure::startExecute()
{
	assert(!execID);

	execID = Synthesizer->createStructure(StructureDesc);
	assert(execID);
}

void Structure::stopExecute()
{
	assert(execID);
	Synthesizer->freeStructure(execID);
	execID = 0;
}

void Structure::save(const char *filename)
{
	Arts::StringSeq_var list = StructureDesc->saveToList();
	FILE *file = fopen(filename,"w");
	unsigned long i;

	for(i=0;i<list->length();i++) fprintf(file,"%s\n",(char *)(*list)[i]);
	fclose(file);
/*
	KSimpleConfig out(filename);

	list<Module *>::iterator i;
	for(i=ModuleList.begin(); i != ModuleList.end(); i++)
		(*i)->save(out);
*/
}

void Structure::clear()
{
	list<Module *>::iterator mi;

	for(mi = ModuleList.begin(); mi != ModuleList.end(); mi++) delete (*mi);

	ModuleList.erase(ModuleList.begin(), ModuleList.end());
	StructureDesc->clear();
}

void Structure::load(const char *filename)
{
	FILE *infile = fopen(filename,"r");
	Arts::StringSeq_var strseq = new Arts::StringSeq;

	char line[1024];
	unsigned long i = 0;

	while(fgets(line,1024,infile))
	{
		// cut eventual CR and/or LFs at the end of the line
		while(strlen(line) && line[strlen(line)-1] < 14)
			line[strlen(line)-1] = 0;

		strseq->length(i+1);
		(*strseq)[i++] = CORBA::string_dup(line);
	}
	fclose(infile);

	clear();

	StructureDesc->loadFromList(strseq);

	Arts::ModuleDescSeq_var modules = StructureDesc->Modules();

	for(i=0;i<modules->length();i++)
	{
		Module *m = new Module((*modules)[i],StructureDesc);

		ModuleList.push_back(m);
	}
}

void Structure::loadOld(const char *filename)
{
	clear();
	KSimpleConfig in(filename);

	int mod;
	for(mod=0;;mod++)
	{
		QString groupname;

		groupname.sprintf("module%d",mod);
		in.setGroup(groupname.data());

		char *moduletype = in.readEntry("type").data();
		if(!moduletype)
		{
			printf("module %s not present -> eof?\n",groupname.data());
			printf("setting up ports (restoring connections)\n");
			list<Module *>::iterator mi;
			for(mi=ModuleList.begin();mi != ModuleList.end(); mi++)
			{
				printf("I -> loadPorts (%s)\n",(char *)(*mi)->getID());
				(*mi)->loadPorts(in,ModuleList);
				printf("II -> portConnect\n");
				(*mi)->portConnect();
			}
			return;
		}

		int i;
		for(i=0;i<(Synthesizer->getModuleCount());i++)
		{
			Arts::ModuleInfo *minfo;
			if(Synthesizer->getModuleInfo(i,minfo))
			{
				if(strcmp(minfo->name,moduletype) == 0)
				{
					printf("loading module %d which has type %s\n",
													mod,(char *)minfo->name);

					Module *m=createModule(minfo);

					m->modnr = mod;
					m->load(in);
				}
			}
		}
	}
}

Module *Structure::createModule(const Arts::ModuleInfo *minfo)
{
	Module *m = new Module(minfo,StructureDesc);

	ModuleList.push_back(m);
	return m;
}

list<Module *> *Structure::getModuleList()
{
	return(&ModuleList);
}

long Structure::countSelected()
{
	list<Module *>::iterator mi;
	long targets = 0;

	for(mi=ModuleList.begin();mi!=ModuleList.end();mi++)
		if((*mi)->selected) targets++;

	return targets;
}

void Structure::deleteSelected()
{
	list<Module *>::iterator mi;

	printf("deleteSelected...\n");
	for(mi=ModuleList.begin();mi!=ModuleList.end();)
	{
		if((*mi)->selected)
		{
			delete (*mi);
			ModuleList.erase(mi);
			mi = ModuleList.begin();
		}
		else mi++;
	}
	printf("deleteSelected ok.\n");
}
