	/*

	Copyright (C) 1998 Stefan Westerfeld
                       stefan@space.twc.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    */

#ifndef _SYNTH_IMPL_H
#define _SYNTH_IMPL_H

#include "synth.h"
#include "synthmodule.h"
#include "moduleserver.h"
#include "artsobject.h"

#ifdef HAVE_MINI_STL
#include <ministl/list>
#include <ministl/string>
#else
#include <list>
#include <string>
#endif

class ArtsIOEvent;

class Synthesizer_impl :public ArtsObject_impl,
						public Arts::Synthesizer_skel
						 
{
	#define ARG_WRITE	1
	#define ARG_READ	2
	#define ARG_CONST	4

	class Arg {
	public:
		Arg();			// only there to make it work with ministl

		Arg(string name, int flags, float *memory);

		float *memory;  // where the argument will be stored later
		string name;
		int flags;	   // combination of ARG_READ, ARG_WRITE, ARG_CONST
	};

	list<Arg> ArgList;

	SynthModule	**SynthModules;
	int MCount;

	list<ArtsIOEvent *> IOEvents;

/** status watching -> don't bring down the system by hogging the cpu **/

	long status_stalled;
	long status_clocks;
	long status_cycles;
	bool status_halted;
	float status_cpu_usage;

protected:
	ModuleServer<SynthModule> *MS_SynthModule;
	float *AllocArg(string name, int flags);

	// Script commands. Might become CORBA commands one nice day ;)
	void Execute_Create(char *buffer);
	void Execute_Data(char *buffer);
	void Execute_Clear();

	CORBA::ORB_var *my_orb;

	long nextID;		// execution IDs

public:
	void Run(long cycles);

	Synthesizer_impl(CORBA::ORB_var *my_orb);	// sure not nice, but don't know
											// a better way...
	~Synthesizer_impl();

// exported CORBA functions

    CORBA::Long getModuleCount();
    CORBA::Boolean getModuleInfo( CORBA::Long nr, Arts::ModuleInfo*& info );

    void Start( CORBA::Long cycles );
    Arts::Status getStatus();
	void Stop( );

    Arts::StructureDesc *createStructureDesc();

	CORBA::Long createStructure(Arts::StructureDesc *structuredesc);
    CORBA::Boolean freeStructure(long int id);
};

#endif
