// Aspell check functions
// Copyright 1998 by Kevin Atkinson under the terms of the LGPL


#ifndef as_check__
#define as_check__

template <class ForwardIterator>
struct CheckResults {
  ForwardIterator begin;
  ForwardIterator end;
  CheckResults() {}
  CheckResults(const ForwardIterator &b, const ForwardIterator &e)
    : begin(b), end(e) {}
  operator bool() {return !(begin == end);}
};

struct AS_DoNothing {
  void operator () () const {}
};

template <class ForwardIterator>
inline CheckResults<ForwardIterator> check (const aspell &sc,
				     ForwardIterator i, ForwardIterator end) {
  return check(sc, i, end, AS_DoNothing());
}


template <class ForwardIterator, class StatusFunc>
CheckResults<ForwardIterator> check (const aspell &sc,
				     ForwardIterator i, ForwardIterator end,
				     const StatusFunc &print_status) {

  const char *alpha = sc.alpha_map();

  string word;
  bool   cur_is_alpha  = false;
  bool   prev_is_alpha = false;
  ForwardIterator word_start;
  for (; i != end; ++i) {
    cur_is_alpha = alpha[*i];
    if (cur_is_alpha != prev_is_alpha) {
      if (prev_is_alpha) {
	if (sc.check(word)) {
	  print_status();
	} else {
	  return CheckResults<ForwardIterator>(word_start, i);
	} 
      } else { // cur_is_alpha
	word.resize(0);
	word_start = i;
      }
    }
    if (cur_is_alpha) {
      word += *i;
    } else {
      // do nothing
    }
    prev_is_alpha = cur_is_alpha;
  }
  if (cur_is_alpha) {
    if (sc.check(word)) {
      print_status();
    } else {
      return CheckResults<ForwardIterator>(word_start, i);
    }
  }
  return CheckResults<ForwardIterator>(end,end);
}

#endif
