#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <hash_map>
#include <hash_set>
#include "language.hh"
#include "data.hh"
#include "as_convert.hh"
#include "hash-t.hh"
#include "hash_string.hh"

// Needed for solaris assembler compatability
class string2 : public string {};
class vector_ui : public vector<unsigned int> {};
class vector2 : public vector<pair<string2, vector_ui> > {
public:
  vector2(unsigned int i) : vector<pair<string2, vector_ui> >(i) {}
};

typedef 
  vector_hash_map
  < string2, vector_ui, hash_string<string2>, 
    is_nonexistent_container<string2>,
    make_nonexistent_container<string2>,
    equal_to<string2>, vector2
  >      
  SCC_SoundHash;

typedef
  vector_hash_set 
  < string2, hash_string<string2>, 
    is_nonexistent_container<string2>, 
    make_nonexistent_container<string2>,
    equal_to<string2> 
   >
  Intr_WordList;

class SCC_WordList_Internal : public Intr_WordList {};

SCC_WordList::SCC_WordList() {
  internal = new SCC_WordList_Internal;
}

SCC_WordList::~SCC_WordList() {
  delete internal;
}
  
bool SCC_WordList::add(const string &str) {
  return (internal->insert(static_cast<const string2&>(str))).second;
}

void del (const SC_Language* lang) {
  delete lang;
}

const SC_Error create_master2 (const string &base, unsigned int size, 
			       SCC_WordList &wl, const SC_Language *lang) {
  string word;
  SCC_SoundHash sound_prehash;
  ofstream WORDS;
  SCC_WordList_Internal &word_list = *wl.internal;
  word_list.resize(word_list.size()/.8);
  sound_prehash.resize(word_list.bucket_count());
  WORDS.open((base + ".wrd").c_str(), ios::out | ios::bin);

  WORDS << "master_wl" << " " << lang->name() << " ";
  WORDS << word_list.size() << " " << word_list.bucket_count() 
	  << " " << size + 1 << '\n';

  unsigned int pos = 1;
  unsigned int sound_size = 0;
  string sound;

  {
    vector<unsigned int> final_hash(word_list.bucket_count(), 0);

    SCC_WordList_Internal::table_type::iterator begin = word_list.table().begin();
    SCC_WordList_Internal::table_type::iterator i     = begin;
    SCC_WordList_Internal::table_type::iterator end   = word_list.table().end();
  
    unsigned int  m;
    WORDS.put('\0');
    for (; i != end; ++i) {
      if (word_list.is_nonexistent_func()(*i)) continue;
      WORDS << *i << '\0';


      sound = lang->to_soundslike(*i);
      sound_prehash[static_cast<const string2&>(sound)].push_back(pos);
      sound_size += sound.size() + 1 + sizeof(unsigned short int);
      // the m stuff is needed to get alignment correct.
      m = sound_size % sizeof(const char *);
      if (m != 0) 
	sound_size += sizeof(const char *) - m;

      final_hash[i-begin] = pos;
      pos += (*i).size() + 1;
    }
    
    WORDS.write(reinterpret_cast<char *>(&final_hash.front()),
		final_hash.size() * sizeof(unsigned int));
  }

  unsigned int pointer_size = sizeof(void *);

  sound_size = sizeof(const char *) // the inital blank string entree
               + sound_size
               + sound_prehash.size() * pointer_size;
  // sound_size is the size of the soundslike (2b) object

  sound_prehash.resize(sound_prehash.size()/.8);

  ofstream SOUNDS ((base + ".sl").c_str(), ios::out | ios::bin);

  SOUNDS << "master_ml" << " " << lang->name() << " ";
  SOUNDS << sound_prehash.size() << " " << sound_prehash.bucket_count()
	 << " " << sound_size << '\n';
  {
    unsigned short int count;
    unsigned int count_size = sizeof(unsigned short int);
    unsigned int list_size;

    vector<unsigned int> final_hash(sound_prehash.bucket_count(), count_size);

    SCC_SoundHash::table_type::iterator begin = sound_prehash.table().begin();
    SCC_SoundHash::table_type::iterator i     = sound_prehash.table().begin();
    SCC_SoundHash::table_type::iterator end   = sound_prehash.table().end();

    unsigned int m;
    pos = pointer_size;

    for (; i != end; ++i) {
      if (sound_prehash.is_nonexistent_func()(*i)) continue;
     
      count = (*i).second.size();
      SOUNDS.write(reinterpret_cast<char *>(&count),count_size);

      list_size =  count * sizeof(unsigned int);
      SOUNDS.write(reinterpret_cast<char *>(&(*i).second.front()), list_size);

      SOUNDS << (*i).first << '\0';

      final_hash[i-begin] = pos + list_size + count_size;
      pos += list_size + count_size + (*i).first.size() + 1;
      m = pos % pointer_size;
      if (m != 0)
	pos += pointer_size - m;
    }
    count = 0;
    SOUNDS.write(reinterpret_cast<char *>(&count),sizeof(unsigned short int));
    
    SOUNDS.write(reinterpret_cast<char *>(&final_hash.front()),
		 final_hash.size() * sizeof(unsigned int));
  }

  return SC_Error();
}
    
const SC_Error create_personal2(const string &base, 
				SCC_WordList &word_list,
				const SC_Language *lang) {
  ofstream WORDS((base + ".per").c_str());
  
  if (WORDS.rdstate() & ios::badbit) 
    return SC_Error(base + ".per", SC_Error::cant_write);
  
  WORDS << "personal_wl" << " " << lang->name() << endl;
  WORDS << word_list.internal->size() << endl;
  SCC_WordList_Internal::iterator i   = word_list.internal->begin();
  SCC_WordList_Internal::iterator end = word_list.internal->end();
  for (; i != end; ++i)
    WORDS << *i << " " << lang->to_soundslike(*i) << endl;

  return SC_Error();
}
