
#include <vector>
#include <string>
#include "as_error.hh"
#include "as_mwordlist.hh"

#ifndef _as_data_hh_
#define _as_data_hh_

class aspell;

typedef const_string SC_Word;
typedef const_string SC_Sound;

class SC_ReadOnly_Soundslikes_WordList;

class ReadOnlyWL {
private:
  struct Internal;
  Internal *internal;
private:
  ReadOnlyWL(const ReadOnlyWL&);
  ReadOnlyWL& operator= (const ReadOnlyWL&);
public:
  class WordList {
    typedef SC_ReadOnly_Soundslikes_WordList Real;
    const Real *info;
  public:
    WordList();
    WordList(const Real *i);
    typedef unsigned int size_type;
    typedef const char * value_type;

    typedef const char * * const_iterator;
    typedef const_iterator interator;
      
    const_iterator begin() const;
    const_iterator end() const;
    size_type size() const;
    bool empty() const;
  };

  class const_iterator {
    void *itr;
  public:
    const_iterator();
    const_iterator(void *);
    ~const_iterator();
    const_iterator(const const_iterator&);
    const_iterator& operator=(const const_iterator&);
    typedef const char * value_type;
    typedef const char * return_type;
    return_type operator*();
    const_iterator& operator++();
    const_iterator  operator++(int);
    const_iterator  operator+(unsigned int);
    friend bool operator==(const_iterator, const_iterator);
    friend bool operator!=(const_iterator, const_iterator);
  };
  typedef const_iterator iterator;
  typedef unsigned int   size_type;
  
  const_iterator begin() const;
  const_iterator end() const;
  size_type size() const;
  bool empty() const;
  
  ReadOnlyWL(aspell *sc);
  ~ReadOnlyWL();
  const SC_Error load(const string &);
  bool operator [] (const char *word) const;
  bool operator [] (const string &word) const;
  const WordList words_w_soundslike(const char *word); 
};

typedef vector<SC_Word> SC_Writable_Soundslikes_WordList;

class WritableWL {
  struct Internal;
  Internal * internal;
public:
  class WordList {
    typedef SC_Writable_Soundslikes_WordList Real;
    const Real *info;
  public:
    WordList();
    WordList(const Real *i);
    typedef unsigned int size_type;
    typedef const char * value_type;

    class const_iterator {
      void *itr;
    public:
      const_iterator();
      const_iterator(void *);
      ~const_iterator();
      const_iterator(const const_iterator&);
      const_iterator& operator=(const const_iterator&);
      typedef const char * value_type;
      typedef const char * return_type;
      return_type operator*();
      const_iterator& operator++();
      const_iterator  operator++(int);
      const_iterator  operator+(unsigned int);
      friend bool operator==(const_iterator, const_iterator);
      friend bool operator!=(const_iterator, const_iterator);
    };
    typedef const_iterator iterator;
      
    const_iterator begin() const;
    const_iterator end() const;
    size_type size() const;
    bool empty() const;
  };

  class const_iterator {
    void *itr;
  public:
    const_iterator();
    const_iterator(void *);
    ~const_iterator();
    const_iterator(const const_iterator&);
    const_iterator& operator=(const const_iterator&);
    typedef const char * value_type;
    typedef const char * return_type;
    return_type operator*();
    const_iterator& operator++();
    const_iterator  operator++(int);
    friend bool operator==(const_iterator, const_iterator);
    friend bool operator!=(const_iterator, const_iterator);
  };
  typedef const_iterator iterator;
  typedef unsigned int   size_type;

  const_iterator begin() const;
  const_iterator end() const;
  size_type      size() const;
  bool           empty() const;

  WritableWL(aspell *sc);
  const SC_Error load(const string &);
  const SC_Error merge(const string &);
  const SC_Error update();
  const SC_Error save();
  const SC_Error save_as(const string &);
  const char * file_name();
  void add(const string &w, const string &s);
  void clear();
  bool operator [] (const char *word) const;
  bool operator [] (const string &word) const;
  const WordList words_w_soundslike(const char *word); 
};

class WritableReplList {
private:
  struct Internal;
  Internal * internal;
public:
  class Replacements {
    vector<string> info;
  public:
    typedef vector<string>::const_iterator const_iterator;
    typedef const_iterator                 iterator;
    typedef vector<string>::size_type      size_type;
    Replacements(const string &mis, size_type num)
      {info.reserve(num+1); info.push_back(mis);}
    Replacements(const string &mis, const string &cor) : info(2) 
      {info[0] = mis; info[1] = cor;}
    const string& misspelled_word() const {return info[0];}
    const_iterator begin() const {return info.begin()+1;}
    const_iterator end()   const {return info.end();}
    size_type      size()  const {return info.size()-1;}
    bool add(const string &);
    void add_nocheck(const string &r) {info.push_back(r);}
    bool erase(const string &);
    bool exists(const string &);
  };
  class WordList {
  friend class WritableReplList;
  friend class WritableReplList::Internal;
    typedef vector<Replacements> Real;
    const Real *info;
  public:
    WordList();
    WordList(const Real *i);
    typedef unsigned int size_type;
    typedef const char * value_type;

    class const_iterator {
      void *itr;
    public:
      const_iterator();
      const_iterator(void *);
      ~const_iterator();
      const_iterator(const const_iterator&);
      const_iterator& operator=(const const_iterator&);
      typedef Replacements value_type;
      typedef const Replacements & return_type;
      const value_type & operator*();
      const_iterator& operator++();
      const_iterator  operator++(int);
      const_iterator  operator+(unsigned int);
      friend bool operator==(const_iterator, const_iterator);
      friend bool operator!=(const_iterator, const_iterator);
    };
    typedef const_iterator iterator;
      
    const_iterator begin() const;
    const_iterator end() const;
    size_type size() const;
    bool empty() const;
  };

private:
  WritableReplList(const WritableReplList&);
  WritableReplList& operator=(const WritableReplList&);
public:
  WritableReplList(aspell *sc);
  ~WritableReplList();
  const SC_Error load(const string &);
  const SC_Error merge(const string &);
  const SC_Error update();
  const SC_Error save();
  const SC_Error save_as(const string &);
  const char * file_name();
  
  void add(const string &mis, const string &cor, const string &s);

  const WordList words_w_soundslike(const char *word) const;
};

typedef WritableReplList::Replacements SC_ReplList;
typedef WritableReplList::WordList     SC_MRWordList;

class AS_WordLists {
private:
  ReadOnlyWL * main_;
  bool         del_main;
  WritableWL * personal_;
  bool         del_personal;
  WritableWL * session_;
  bool         del_session;

  AS_WordLists(const AS_WordLists&);
  AS_WordLists& operator= (const AS_WordLists&);
  
public:
  typedef SC_MWordList        sword_list_type;
  typedef SC_MWordList        SWordList;

  AS_WordLists(aspell *);
  ~AS_WordLists();

  ReadOnlyWL & main() {return *main_;}
  WritableWL & personal() {return *personal_;}
  WritableWL & session() {return *session_;}

  const ReadOnlyWL & main() const {return *main_;}
  const WritableWL & personal() const {return *personal_;}
  const WritableWL & session() const {return *session_;}

  bool operator [] (const char *word) const 
    {return (*main_)[word] || (*personal_)[word] || (*session_)[word];}
  bool operator [] (const string &word) const
    {return operator[] (word.c_str());}

  const SWordList words_w_soundslike(const char *word) const;
};

class AS_ReplLists {
  typedef WritableReplList::WordList sword_list_type;
  typedef WritableReplList::WordList SWordList;
  
  class value_type {
  public:
    const char * misspelled() const;
    const char * replacement() const;
    const char * soundslike() const;
  };

  WritableReplList *personal_;
  bool              del_personal;

  AS_ReplLists(const AS_ReplLists&);
  AS_ReplLists& operator= (const AS_ReplLists&);
public:
  AS_ReplLists(aspell *);
  ~AS_ReplLists();

  WritableReplList & personal() {return *personal_;}

  const WritableReplList & personal() const {return *personal_;}

  const SWordList words_w_soundslike(const char *word) const;
};

#endif
