// Aspell main C++ include file
// Copyright 1998 by Kevin Atkinson under the terms of the LGPL.

#ifndef aspell__
#define aspell__

#include <string>
#include <vector>
#include <fstream.h>
#include "as_error.hh"
#include "as_mwordlist.hh"
#include "as_data.hh"
#include "const_string.hh"

class SC_WordList;
class SC_Soundslikes;
class SC_Language;

typedef const_string SC_Word;
typedef const_string SC_Sound;

class aspell {
public:
  typedef SC_Error            error_type;
  typedef SC_Error            Error;
  typedef SC_Word             word_type;
  typedef SC_Word             Word;
  typedef SC_Sound            soundslike_type;
  typedef SC_Sound            Soundslike;
  typedef vector<Word>        word_list_type;
  typedef vector<Word>        WordList;
  typedef SC_MWordList        sword_list_type;
  typedef SC_MWordList        SWordList;
  typedef vector<string>      Suggestions;
  typedef vector<string>      suggestions_type;
  typedef vector<Soundslike>  soundslike_list_type;
  typedef vector<Soundslike>  SoundslikeList;
  
private:
  void populate();

  AS_WordLists            word_lists_;
  AS_ReplLists            repl_lists_;

private:
  Error                   error_;
  const SC_Language      *lang;
  bool                    del_lang;
  bool                    ignore_repl;

  static const WordList empty_list;

  void operator= (const aspell &other);
  aspell(const aspell &other);

public:
  // these are public so that other classes and functions can use them, 
  // DO NOT USE

  const Error set_check_lang(const string& lang, const string &lang_dir="");
  const char * alpha_map() const;

  double distance (const char *, const char *, const char *, const char *) const;

public:
  aspell(const string &m, const string &p = ".aspell", 
	 const string &r = ".aspell",  bool ir = false);
  //aspell(const string &m, const string &p, const SC_Language *l);
  ~aspell();

  AS_WordLists & word_lists() {return word_lists_;}
  AS_ReplLists & repl_lists() {return repl_lists_;}

  const AS_WordLists & word_lists() const {return word_lists_;}
  const AS_ReplLists & repl_lists() const {return repl_lists_;}
  
  ReadOnlyWL & main_dict() {return word_lists_.main();}
  WritableWL & personal_dict() {return word_lists_.personal();}
  WritableWL & session_dict() {return word_lists_.session();}
  WritableReplList & personal_repl() {return repl_lists_.personal();}

  const ReadOnlyWL & main_dict() const {return word_lists_.main();}
  const WritableWL & personal_dict() const {return word_lists_.personal();}
  const WritableWL & session_dict() const {return word_lists_.session();}
  const WritableReplList & personal_repl() const {return repl_lists_.personal();}

  const Error& problem() const {return error_;}
  const Error& error() const {return error_;}
  operator bool() const {return error_;}

  const char * lang_name() const;
  string to_lower(const string &word) const;

  Suggestions suggest(const string &word) const;
  const Suggestions suggest_ultra(const string &word) const;

  bool check(const string &word) const;
  bool check(const char *word) const;
  bool check_notrim(const string &word) const;
  bool check_notrim(const char *word) const;
  bool check_raw(const char *word) const;
  bool check_raw(const string &word) const;

  const Word& get(const string &word) const;
  const Word& get(const char *word) const;

  void add_personal(const string &word);
  void add_session(const string &word);

  void save_dicts();

  void clear_session();

  const string to_soundslike(const string &word) const;
  const SWordList soundslike(const string &word) const;

  const string to_phoneme(const string &word) const;

  double score(const char *word1, const char *word2) const;

  bool ignore_replacements();
  bool ignore_replacements(bool);
  
  void store_replacement(const string& mis, const string &cor);
};

#define SpellChecker aspell

#endif
