
#include "gen_lang_loader.hh"
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>

void SC_GenericLangLoader::init(const string &name) {
  name_intr = name;
  name_     = name.c_str();
  to_lower_ = new          char[256];
  to_upper_ = new          char[256];
  is_alpha_ = new          char[256];
  soundslike_chars_ = new  char[257];
}  

SC_GenericLangLoader::SC_GenericLangLoader(const string &name) {
  init(name);
}

SC_GenericLangLoader::SC_GenericLangLoader(const string &name,
			       const string &path) {
  init(name);
  get_from_file(path+name+".lang");
}

SC_GenericLangLoader::~SC_GenericLangLoader() {
  delete[] to_lower_;
  delete[] to_upper_;
  delete[] is_alpha_;
  delete[] soundslike_chars_;
}

void SC_GenericLangLoader::get_from_file(const string &name) {
  ifstream data(name.c_str());
  string temp;
  bool soundslike_chars[256] = {0};
  
  for (int i = 0; i != 256; ++i) {
    to_lower_[i] = i;
    to_upper_[i] = i;
    is_alpha_[i] = 0;
  }

  while (data && temp != "{") data >> temp;

  string& upper_lower = temp; 
  while (data && upper_lower != "}") {
    data >> upper_lower;
    to_lower_[upper_lower[1]] = upper_lower[0];
    to_upper_[upper_lower[0]] = upper_lower[1];
    is_alpha_[upper_lower[0]] = 1;
    is_alpha_[upper_lower[1]] = 1;
    soundslike_chars[upper_lower[0]] = true;
  }
    
  while (data && temp != "{") data >> temp;

  char v;
  while (data) {
    data >> v;
    if (v == '}') break;
    is_alpha_[v] = 2;
    is_alpha_[to_upper_[v]] = 2;
    soundslike_chars[v] = false;
  }

  temp = v;
  while (data && temp != "{") data >> temp;

  while (data) {
    data >> v;
    if (v == '}') break;
    is_alpha_[v] = 3;
  }

  int j = 0;
  for (int i = 0; i != 256; ++i) {
    if (soundslike_chars[i]) 
      soundslike_chars_[j++] = i;
  }

  soundslike_chars_[j] = 0;
}

void SC_GenericLangLoader::print_code_to_file() {
  ofstream os( ("asl_" + name_intr + ".hh").c_str() );
  print_code_header(os);
  ofstream file( ("asl_" + name_intr + ".cc").c_str() );
  file << endl;
  file << "#include \"language.hh\"" << endl;
  file << "#include \"" << "asl_" << name_intr << ".hh" << "\"" << endl;
  file << endl;
  print_code_body(file);
}

void SC_GenericLangLoader::print_code_header(const string &class_name, ostream &out = cout) {
  out << endl;
  out << "#ifndef __" << class_name << "__" << endl;
  out << "#define __" << class_name << "__" << endl;
  out << endl;
  out << "const SC_Language * new_" << class_name << "();" << endl;
  out << endl;
  out << "#endif" << endl;
  out << endl;
}

void SC_GenericLangLoader::print_code_header(ostream &out) {
  string nname = name_intr;
  nname[0] = to_upper(nname[0]);
  print_code_header("SC_" + nname, out);
}
  

void SC_GenericLangLoader::print_code_body(const string &class_name, const string &var_name, 
				const char *var, int length, ostream &out) {
  out << "const char " << class_name << "::" << var_name << "[" << length << "] = {" << endl;
  int i = 0;
  for (;;) {
    out << "  ";
    for (;;) {
      out << setw(3) << 0 + static_cast<unsigned char>(var[i]);
      ++i;
      if (i == length || !(i % 16)) break;
      out << ",";
    }
    if (i == length) break;
    out << "," << endl;
  }
  out << endl << "};" << endl << endl;
}

void SC_GenericLangLoader::print_code_body(const string &class_name, 
					   ostream &out) {
  out << endl;
  out << "class " << class_name << " : public SC_Language {" << endl;
  out << "private:" << endl;
  out << "  static const char to_lower_intr[256];" << endl;
  out << "  static const char to_upper_intr[256];" << endl;
  out << "  static const char is_alpha_intr[256];" << endl;
  int l = 0;
  for (; soundslike_chars_[l]; ++l);
  out << "  static const char        soundslike_chars_intr[" << l+1 << "];" << endl;
  out << "public:" << endl;
  out << "  " << class_name << "();" << endl;
  out << "  ~" << class_name << "();" << endl;
  out << "};" << endl;
  out << endl;
  print_code_body(class_name, "to_lower_intr", to_lower_, 256, out);
  print_code_body(class_name, "to_upper_intr", to_upper_, 256, out);
  print_code_body(class_name, "is_alpha_intr", is_alpha_, 256, out);
  l = 0;
  for (; soundslike_chars_[l]; ++l);
  print_code_body(class_name, "soundslike_chars_intr", 
		  soundslike_chars_, l+1, out);
  out << endl;
  out << class_name << "::" << class_name << "() { " << endl;
  out << "  name_ = \"" << name_ << "\";" << endl;
  out << "  to_lower_ = to_lower_intr;" << endl;
  out << "  to_upper_ = to_upper_intr;" << endl;
  out << "  is_alpha_ = is_alpha_intr;" << endl;                          
  out << "  soundslike_chars_ = soundslike_chars_intr;" << endl;
  out << "} " << endl;
  out << endl;
  out << class_name << "::~" << class_name << "() {}" << endl;
  out << endl;
  out << "const SC_Language * new_" << class_name << "() {" << endl;
  out << "  return new " << class_name << ";" << endl;
  out << "}" << endl;
  out << endl;
}

void SC_GenericLangLoader::print_code_body(ostream &out) {
  string nname = name_intr;
  nname[0] = to_upper(nname[0]);
  print_code_body("SC_" + nname, out);
}
