
#include "as_config.h"
#include "language.hh"
#include "aspell.hh"
#include "const_string.hh"
#include <pair.h>

#include "gen_lang.hh"
#include "asl_english.hh"

const SC_Language * SC_Languages (const string &lang, const string &path) {
  typedef pair<const_string, const SC_Language * (*)()> lookup_pair;
  static const int size = 1;
  static const lookup_pair lookup[size] = {
    lookup_pair("english", new_SC_English)
  };
  for (int i = 0; i != size; ++i) {
    if (lang == lookup[i].first.c_str())
      return (*lookup[i].second)();
  }
  return new_SC_GenericLang(lang,path);
  return 0;
}  

string SC_Language::to_soundslike(const const_string &word) const {
  string new_word; 

  for (const_string::const_iterator i = word.begin(); *i; ++i) 
    if (is_alpha_[static_cast<unsigned char>(*i)] == consonant)
      new_word += to_lower(*i);
 
  return new_word;
}

bool SC_Language::have_phoneme() const {
  return false;
}

string SC_Language::to_phoneme(const const_string &word) const {
  return "";
}

int SC_Language::case_pattern (const const_string &word) const {
  const_string::iterator i = word.begin();
  bool first_upper = false;
  int  size = 0;
  int  upper_count = 0;
  if (*i) {
    ++size;
    if (is_upper(*i)) {
      first_upper = true;
      ++upper_count;
    }
  }
  for (;*i; ++i) {
    ++size;
    if (is_upper(*i)) ++upper_count;
  }

  if (upper_count == size)
    return all_upper;
  else if (first_upper)
    return first_upper;
  else
    return all_lower;
}

string SC_Language::fix_case (int pattern, const const_string &word) const {
  string new_word;
  new_word.reserve(word.size());
  const_string::iterator i = word.begin();
  switch (static_cast<CasePattern>(pattern)) {
  case all_lower:
    for (; *i; ++i) new_word += *i;
    break;
  case first_upper:
    if (*i) {new_word += to_upper(*i); ++i;}
    for (; *i; ++i) new_word += *i;
    break;
  case all_upper:
    for (; *i; ++i) new_word += to_upper(*i);
    break;
  }
  return new_word;
}

bool SC_Language::trim_n_try(const aspell &sc, const const_string &word) const {
  // assumes word is at least 1 char long
  string temp = word.c_str();
  if (is_special(temp[0])) {
    temp.erase(0,1);
    if (sc.check_notrim(temp))
      return true;
    temp = word; 
  }
  if (is_special(temp[temp.size()-1])) {
    temp.erase(temp.size()-1, 1);
    if (sc.check_notrim(temp))
      return true;
  }
  if (temp.size() && is_special(temp[0])) {
    temp.erase(0,1);
    if (sc.check_notrim(temp))
      return true;
  }
  return false;
}
    
  
