#ifndef __Language_
#define __Language_

#include <string>
#include "const_string.hh"

class aspell;
 
class SC_Language {
protected:
  enum CasePattern {all_lower, first_upper, all_upper};
  static const char consonant = 1, vowel = 2, special = 3;

  const char *name_;
  const char *to_lower_;
  const char *to_upper_;
  const char *is_alpha_;
  const char *soundslike_chars_;

  SC_Language() {}
public:
  virtual ~SC_Language() {}

  virtual string to_soundslike(const const_string &word) const;

  virtual bool have_phoneme() const;
  virtual string to_phoneme(const const_string &word) const;

  virtual int case_pattern (const const_string &word) const;
  virtual string fix_case (int pattern, const const_string &word) const;

  virtual bool trim_n_try (const aspell &sc, const const_string &word) const;

private:
  SC_Language(const SC_Language&);
  const SC_Language operator= (const SC_Language&);

public:
  const char * name() const {return name_;}
  bool is_alpha(const char c) const {return is_alpha_[static_cast<unsigned char>(c)];}
  const char * is_alpha() const {return is_alpha_;}
  const char * alpha_map() const {return is_alpha_;}
  char to_upper(const char c) const {return to_upper_[static_cast<unsigned char>(c)];}
  char to_lower(const char c) const {return to_lower_[static_cast<unsigned char>(c)];}
  bool is_upper(const char c) const {return to_upper_[static_cast<unsigned char>(c)] == c;}
  bool is_lower(const char c) const {return to_lower_[static_cast<unsigned char>(c)] == c;}
  bool is_special(const char c) const {return is_alpha_[static_cast<unsigned char>(c)] == 3;}
  string to_lower(const char *word) const
    {string new_word; for (; *word; ++word) new_word += to_lower(*word); return new_word;}
  string to_lower(const const_string &word) const {return to_lower(word.c_str());}
  string to_lower(const string &word) const {return to_lower(word.c_str());}
  string to_upper(const char *word) const
    {string new_word; for (; *word; ++word) new_word += to_upper(*word); return new_word;}
  string to_upper(const const_string &word) const {return to_upper(word.c_str());}
  string to_upper(const string &word) const {return to_upper(word.c_str());}
  const char * soundslike_chars() const {return soundslike_chars_;}
  string to_soundslike(const char *word) const {return to_soundslike(const_string(word));}
  string to_soundslike(const string &word) const {return to_soundslike(word.c_str());}
  string to_phoneme(const char *word) const {return to_phoneme(const_string(word));}
  string to_phoneme(const string &word) const {return to_phoneme(word.c_str());}
  bool trim_n_try (const aspell &sc, const string &word) const
    {return trim_n_try(sc, word.c_str());}
  bool trim_n_try (const aspell &sc, const char *word) const
    {return trim_n_try(sc, const_string(word));}
};

const SC_Language * SC_Languages (const string &lang, const string &path = "");

#endif
