#include "mwordlist.hh"

SC_MWordList_Iterator::SC_MWordList_Iterator(const SCMWL_Internal* intr)       // begin
  : internal(intr), which(main), offset(0) 
  {advance_past_nulls();}

SC_MWordList_Iterator::SC_MWordList_Iterator(const SCMWL_Internal* intr, bool)  // end
  : internal(intr), which(end), offset(0) {}

const SC_MWordList_Iterator::value_type SC_MWordList_Iterator::operator*() const {
  switch (which) {
  case main:
    return *(internal->main_list.begin()+offset);
  case personal:
    return *(internal->personal_list.begin()+offset);
  case session:
    return *(internal->session_list.begin()+offset);
  case end:
    break;
  }
  return SC_MWordList::value_type();
}

SC_MWordList_Iterator& SC_MWordList_Iterator::operator++() {
  ++offset;
  switch (which) {
  case main:
    if (offset==internal->main_list.size()) {
      which=personal;
      offset=0;
      advance_past_nulls();
    }
    break;
  case personal:
    if (offset==internal->personal_list.size()) {
      which=session;
      offset=0;
      advance_past_nulls();
    }
    break;
  case session:
    if (offset==internal->session_list.size()) {
      which=end;
      offset=0;
      advance_past_nulls();
    }
    break;
  case end:
    break;
  }
  return *this;
}
      
void SC_MWordList_Iterator::advance_past_nulls() {
  switch (which) {
  case main:
    if (internal->main_list.empty()) {
      which = personal;
      if (!internal->personal_list.empty()) 
	// ^^^ don't break if internal->personal is also empty
	//     continue on to the personal case...
	break;                    
    } else {
      break;
    }
  case personal:
    if (internal->personal_list.empty()) {
      which = session;
      if (!internal->session_list.empty())
	break;
    } else {
      break;
    }
  case session:
    if (internal->session_list.empty()) {
      which = end;
    }
  case end:
    break;
  }
}

const SC_MWordList_Iterator SC_MWordList_Iterator::operator++ (int) {
  SC_MWordList_Iterator temp = *this;
  operator++();
  return temp;
}

//
//
//

SC_MWordList::SC_MWordList(const SCMWL_Internal& intr) {
  internal = new SCMWL_Internal(intr);
}

SC_MWordList::SC_MWordList() {
  internal = 0;
}

SC_MWordList::SC_MWordList(const SC_MWordList& other) {
  internal = new SCMWL_Internal(*other.internal);
}

SC_MWordList& SC_MWordList::operator=(const SC_MWordList& other) {
  delete internal;
  internal = new SCMWL_Internal(*other.internal);
  return *this;
}

SC_MWordList::~SC_MWordList() {
  delete internal;
}

SC_MWordList::size_type SC_MWordList::size() const {
  size_type s = 0;
  s += internal->main_list.size();
  s += internal->personal_list.size();
  s += internal->session_list.size();
  return s;
}


  
  

