#include "aspell.hh"
#include "data.hh"
#include "language.hh"
#include "mwordlist.hh"

//
// data_access functions
//

const aspell::WordList aspell::empty_list;

aspell::aspell(const string &m, const string &p, const string &r, bool ir)
  : word_lists_(this), repl_lists_(this), lang(0), del_lang(true),
    ignore_repl(ir)
{
  error_ = main_dict().load(m);
  if (error_) return;
  error_ = personal_dict().load(".aspell");
  if (error_) return;
  error_ = personal_repl().load(".aspell");
}

aspell::~aspell() {
  if (del_lang)
    delete lang;
}

const char * aspell::lang_name() const {
  return lang->name();
}

string aspell::to_lower(const string &word) const {
  return lang->to_lower(word);
}

const char * aspell::alpha_map() const {
  return lang->alpha_map();
}

const SC_Error aspell::set_check_lang (const string &l, 
				       const string &lang_dir) {
  if (!lang) {
    lang = SC_Languages(l,lang_dir);
    del_lang = true;
    if (!lang) {
      error_ = Error("", l, Error::unknown_lang);
      return error_;
    }
  } else {
    if (l != lang->name()) {
      error_ = Error("",l, Error::mismatched_lang);
      return error_;
    }
  }
  return Error();
}

void aspell::save_dicts() {
  personal_dict().save();
  personal_repl().save();
}

void aspell::add_personal(const string &word) {
  personal_dict().add(word, lang->to_soundslike(word));
}

void aspell::add_session(const string &word) {
  session_dict().add(word, lang->to_soundslike(word));
}

void aspell::clear_session() {
  session_dict().clear();
}

bool aspell::ignore_replacements() {
  return ignore_repl;
}

bool aspell::ignore_replacements(bool r) {
  bool temp = ignore_repl;
  ignore_repl = r;
  return temp;
}

void aspell::store_replacement(const string& mis, const string &cor) {
  string::size_type pos;
  if (!ignore_repl && 
      (
       pos = cor.find(' '), 
       pos == string::npos 
         ? check(cor)
         : (check(cor.substr(0,pos)) && check(cor.substr(pos+1))) 
      )
      && cor != suggest(mis)[0])
  {
    personal_repl().add(mis, cor, lang->to_soundslike(mis));
  }
}

//
// simple functions
//

bool aspell::check(const string &word) const {
  if (word_lists()[word]) return true; 
  else if (word_lists()[lang->to_lower(word)]) return true;
  return lang->trim_n_try(*this, word);
}

bool aspell::check(const char *word) const {
  if (word_lists()[word]) return true; 
  else if (word_lists()[lang->to_lower(word)]) return true;
  return lang->trim_n_try(*this, word);
}

bool aspell::check_notrim(const string &word) const {
  if (word_lists()[word.c_str()]) return true;
  else return word_lists()[lang->to_lower(word)];
}

bool aspell::check_notrim(const char *word) const {
  if (word_lists()[word]) return true; 
  else return word_lists()[lang->to_lower(word)];
}

bool aspell::check_raw(const char *word) const {
  return word_lists()[word];
}

bool aspell::check_raw(const string &word) const {
  return word_lists()[word];
}

const string aspell::to_soundslike(const string &word) const {
  return lang->to_soundslike(word);
}

const aspell::SWordList aspell::soundslike(const string &word) const {
  return word_lists().words_w_soundslike(lang->to_soundslike(word).c_str());
}

const string aspell::to_phoneme(const string &word) const {
  return lang->to_phoneme(word);
}
