/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef CSCOLL_H
#define CSCOLL_H

#include "geometry/math3d.h"
#include "engine/basic/csobject.h"
#include "util/csvector.h"

class csSector;

/**
 * A collection object is for conveniance of the script language.
 * Objects are (currently) not really hierarchical in Crystal Space.
 * A collection simulates a hierarchy. The script can then perform
 * operations like 'move' and 'transform' on all the objects in
 * the collection together.
 */
class Collection : public csObjectTrigger
{
private:
  /// The list of objects contained in this Collection.
  csVector objects;

public:
  /**
   * Create a new Collection with the given name.
   */
  Collection (char* name);

  ///
  virtual ~Collection () {}

  /**
   * Set the location of all objects in the collection.
   */
  void set_move (csSector* home, const Vector3& v) 
  { set_move (home, v.x, v.y, v.z); }

  /**
   * Set the location of all objects in the collection.
   */
  void set_move (csSector* home, float x, float y, float z);

  /**
   * Set the transformation matrix for all objects in the collection.
   */
  void set_transform (Matrix3& matrix);

  /**
   * Relative move.
   */
  void move (float dx, float dy, float dz);

  /**
   * Relative move.
   */
  void move (const Vector3& v) 
  { move (v.x, v.y, v.z); }

  /**
   * Relative transform.
   */
  void transform (Matrix3& matrix);

  /**
   * Really do the transformation and moving of all objects in
   * the collection. This should be called after calling any of
   * the other set_move, set_transform, move or transform calls.
   */
  void transform ();

  /**
   * Find an object with the given name inside this collection.
   */
  csObject* find_object (char* name);

  /**
   * Get the number of objects in this collection.
   */
  int get_num_objects () const { return objects.Length(); }

  /// Add an object to the collection.
  void add_object(csObject* obj) { objects.Push((csSome)obj); }

  ///
  csObject* operator[] (int i) { return (csObject*) (objects[i]); }
};

#endif /*CSCOLL_H*/
