/*
    Copyright (C) 1998 by Ayal Zwi Pinkus
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "engine/bezier.h"
#include "engine/curve.h"
#include "engine/basic/polyset.h"

static csBezierCache theBezierCache;

#define TODO //@

csCurveTesselated::csCurveTesselated (int num_v, int num_t)
{
  num_vertices = num_v;
  CHK (vertices = new csCurveVertex[num_v]);
  num_triangles = num_t;
  CHK (triangles = new csCurveTriangle[num_t]);
}

csCurveTesselated::~csCurveTesselated ()
{
  CHK (delete[] vertices);
  CHK (delete[] triangles);
}

csCurveTemplate::csCurveTemplate(char* name, int type)
  : csObject (name, type) {}


csCurveTesselated* csBezier::tesselate (int res)
{
  if (res == previous_resolution && previous_tesselation)
    return previous_tesselation;

  previous_resolution = res;
  CHK(delete previous_tesselation);

  CHK( previous_tesselation = 
       new csCurveTesselated ((res+1)*(res+1), 2*res*res));

  TDtDouble cpt[9][5];
  TDtDouble *controls[9] = 
  {
    cpt[0], cpt[1], cpt[2], cpt[3], cpt[4],
    cpt[5], cpt[6], cpt[7], cpt[8],
  };

  int i,j;

  for (i=0;i<3;i++)
    for (j=0;j<3;j++)
      {
	controls[i*3+j][0] = points[i][j].x;
	controls[i*3+j][1] = points[i][j].y;
	controls[i*3+j][2] = points[i][j].z;

	controls[i*3+j][3] = texture_coords[i][j].x; 
	controls[i*3+j][4] = texture_coords[i][j].y; 
      }

  for (i=0;i<=res;i++)
  for (j=0;j<=res;j++)
  {
    TDtDouble point[5];
    BezierPoint(point, controls, i, j,res,BinomiumMap());
    csCurveVertex& vtx = previous_tesselation->getVertex(i+(res+1)*j);
    vtx.object_coord.x = point[0];
    vtx.object_coord.y = point[1];
    vtx.object_coord.z = point[2];
    //
    vtx.txt_coord.x    = point[3];
    vtx.txt_coord.y    = point[4];
  }

  for (i=0;i<res;i++)
  {
    for (j=0;j<res;j++)
    {
      csCurveTriangle& up = 
	previous_tesselation->getTriangle (2*(i+j*res));
      csCurveTriangle& down = 
	previous_tesselation->getTriangle (2*(i+j*res)+1);
      int tl = i+(res+1)*j;
      int tr = i+(res+1)*j+1;

      int bl = i+(res+1)*(j+1);
      int br = i+(res+1)*(j+1)+1;
      up.i1 = tl;
      up.i2 = br;
      up.i3 = tr;

      down.i1 = br;
      down.i2 = tl;
      down.i3 = bl;
    }
  }

  return previous_tesselation;
}



csBezier::csBezier(char* name, int type)
  : csCurve (name, type) 
{
  int i,j;
  for (i=0;i<3;i++)
    for (j=0;j<3;j++)
      {
	texture_coords[i][j].x = (0.5*i);
	texture_coords[i][j].y = (0.5*j);
      }
  previous_tesselation = NULL;
  previous_resolution = -1;
}

csBezier::~csBezier ()
{
  CHK(delete previous_tesselation);
}

void csBezier::set_control_point(int index, int control_id)
{
  getControlPoint(index) = parent->curve_vertex(control_id);
  getTextureCoord(index) = parent->curve_texel(control_id);
}




csBezierTemplate::csBezierTemplate(char* name, int type)
  : csCurveTemplate (name, type) 
{
  parent = NULL;
  int i;
  for (i=0;i<9;i++)
    {
      ver_id[i]=0;
    }
}

void csBezierTemplate::set_vertex(int index, int ver_ind)
{
  ver_id[index]=ver_ind;
}
int csBezierTemplate::get_vertex(int index) 
{
  return ver_id[index];
}

int csBezierTemplate::num_vertices()
{
  return 9;
}

csCurve* csBezierTemplate::makeCurve(char* name, int type)
{
  csBezier* p = new csBezier (name, type);
  p->setTexture(get_texture ());
  return p;
}


