
// Texture-related interfaces.

#include "engine/itexture.h"
#include "engine/texture.h"
#include "engine/world.h"

// IMipMapContainer interface.

IMPLEMENT_COMPOSITE_UNKNOWN( TextureMM, MipMapContainer )

IMPLEMENT_GET_PROPERTY( GetInVideoMemory, in_memory, bool, TextureMM, MipMapContainer ) 
IMPLEMENT_SET_PROPERTY( SetInVideoMemory, in_memory, bool, TextureMM, MipMapContainer )
  
IMPLEMENT_GET_PROPERTY_PTR( GetHighColorCache, hicolorcache, HighColorCache_Data*, TextureMM, MipMapContainer)
IMPLEMENT_SET_PROPERTY( SetHighColorCache, hicolorcache, HighColorCache_Data*, TextureMM, MipMapContainer)

IMPLEMENT_GET_PROPERTY_PTR( GetPrivateColorMap, priv_cmap->rgb_values, unsigned char*, TextureMM, MipMapContainer)
IMPLEMENT_GET_PROPERTY_PTR( GetPrivateToGlobalConversion, priv_cmap->priv_to_global, unsigned char*, TextureMM, MipMapContainer)
IMPLEMENT_GET_PROPERTY_PTR( GetName, get_name(), char*, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetTransparent, transparent, int, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetMeanColor, mean_color, int, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetMeanRed, mean_r, float, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetMeanGreen, mean_g, float, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetMeanBlue, mean_b, float, TextureMM, MipMapContainer )
IMPLEMENT_GET_PROPERTY( GetNumberOfColors, usage->get_num_colors(), int, TextureMM, MipMapContainer )

IMPLEMENT_METHOD( FreeLoadInformation, free_load_info, TextureMM, MipMapContainer )
IMPLEMENT_METHOD( ComputeColorUsage, compute_color_usage, TextureMM, MipMapContainer )
 
STDMETHODIMP IMipMapContainer::SetTransparent(int red, int green, int blue) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )
    
    pThis->set_transparent( red, green, blue ); 
    return S_OK; 
}

STDMETHODIMP IMipMapContainer::GetColorUsage( int idx, RGBPalEntry& retval ) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )
    
    retval = pThis->get_usage( idx ); 
    return S_OK;  
}

STDMETHODIMP IMipMapContainer::RemapPalette( ITextureContainer* new_palette ) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )
    
    pThis->remap_palette( GetTexturesFromITextureContainer(new_palette) ); 
    return S_OK; 
}

STDMETHODIMP IMipMapContainer::RemapPaletteGlobal( ITextureContainer* new_palette ) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )
    
    pThis->remap_palette_global( GetTexturesFromITextureContainer(new_palette) ); 
    return S_OK; 
}

STDMETHODIMP IMipMapContainer::RemapPalettePrivate( ITextureContainer* new_palette )
{
    METHOD_PROLOGUE( TextureMM, MipMapContainer )

    pThis->remap_palette_private(GetTexturesFromITextureContainer(new_palette) ); 
    return S_OK; 
}

STDMETHODIMP IMipMapContainer::BlendMipmapZero(ITextureContainer* tex) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )

    pThis->blend_mipmap0(GetTexturesFromITextureContainer(tex)); 
    return S_OK;  
}

STDMETHODIMP IMipMapContainer::CreateMipmaps(ITextureContainer* tex) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )

    pThis->create_mipmaps(GetTexturesFromITextureContainer(tex)); 
    return S_OK; 
}

STDMETHODIMP IMipMapContainer::LoadedCorrectly() 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )

    return pThis->loaded_correctly() ? S_OK : S_FALSE; 
}

STDMETHODIMP IMipMapContainer::GetTexture(int lev, ITextureMap** retval) 
{ 
    METHOD_PROLOGUE( TextureMM, MipMapContainer )

    *retval = GetITextureMapFromTexture(pThis->get_texture(lev)); 
	((IUnknown*)*retval)->AddRef();

    return S_OK; 
}

// ITextureMap for Texture8

STDMETHODIMP_(ULong) XTextureMap8::Release()
{
	return m_pThis->Release();
}

STDMETHODIMP_(ULong) XTextureMap8::AddRef()
{
	return m_pThis->AddRef();
}

STDMETHODIMP XTextureMap8::QueryInterface( REFIID riid, void** ppv )
{
	return m_pThis->QueryInterface( riid, ppv );
}

STDMETHODIMP XTextureMap8::GetWidth(int& retval)
{
    retval = m_pThis->width;
     return S_OK;
}

STDMETHODIMP XTextureMap8::GetHeight(int& retval)
{
    retval = m_pThis->height;
    return S_OK;
}

STDMETHODIMP XTextureMap8::GetWidthShift(int& retval)
{
    retval = m_pThis->shf_w;
    return S_OK;
}

STDMETHODIMP XTextureMap8::GetHeightShift(int& retval)
{
    retval = m_pThis->shf_h;
    return S_OK;
}

STDMETHODIMP XTextureMap8::GetWidthMask(int& retval)
{
    retval = m_pThis->and_w;
    return S_OK;
}

STDMETHODIMP XTextureMap8::GetHeightMask(int& retval)
{
    retval = m_pThis->and_h;
    return S_OK;
}


STDMETHODIMP XTextureMap8::GetBitmap(unsigned char** retval)
{
    *retval = m_pThis->bitmap;
    return S_OK;
}

STDMETHODIMP XTextureMap8::GetParent( IMipMapContainer** retval )
{
    *retval = GetIMipMapContainerFromTextureMM(m_pThis->parent);
	(*retval)->AddRef();

    return S_OK;
}


// ITextureMap for Texture32

STDMETHODIMP_(ULong) XTextureMap32::Release()
{
	return m_pThis->Release();
}

STDMETHODIMP_(ULong) XTextureMap32::AddRef()
{
	return m_pThis->AddRef();
}

STDMETHODIMP XTextureMap32::QueryInterface( REFIID riid, void** ppv )
{
	return m_pThis->QueryInterface( riid, ppv );
}


STDMETHODIMP XTextureMap32::GetWidth(int& retval)
{
    retval = m_pThis->width;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetHeight(int& retval)
{
    retval = m_pThis->height;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetWidthShift(int& retval)
{
    retval = m_pThis->shf_w;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetHeightShift(int& retval)
{
    retval = m_pThis->shf_h;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetWidthMask(int& retval)
{
    retval = m_pThis->and_w;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetHeightMask(int& retval)
{
    retval = m_pThis->and_h;
    return S_OK;
}


STDMETHODIMP XTextureMap32::GetBitmap(unsigned char** retval)
{
    *retval = (unsigned char*)m_pThis->bitmap;
    return S_OK;
}

STDMETHODIMP XTextureMap32::GetParent( IMipMapContainer** retval )
{
    *retval = GetIMipMapContainerFromTextureMM(m_pThis->parent);
	(*retval)->AddRef();

    return S_OK;
}


// ITextureContainer

IMPLEMENT_COMPOSITE_UNKNOWN( Textures, TextureContainer )

STDMETHODIMP ITextureContainer::Initialize(IWorld* world) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    return pThis->initialize( GetWorldFromIWorld(world) ) ? S_OK : E_UNEXPECTED; 
}

STDMETHODIMP ITextureContainer::FindRGBPrecise(int r, int g, int b, int& retval) 
{
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->find_rgb_slow( r, g, b ); 
    return S_OK;
}

STDMETHODIMP ITextureContainer::AllocateNewRGB (int r, int g, int b, int dist) 
{
    METHOD_PROLOGUE( Textures, TextureContainer )
    pThis->alloc_rgb( r, g, b, dist ); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetMipMapContainer(char* name, IMipMapContainer** retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    *retval = GetIMipMapContainerFromTextureMM( pThis->get_texture_mm(name) ); 
	(*retval)->AddRef();

    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetMipMapContainerByIndex(int idx, IMipMapContainer** retval)
{
    METHOD_PROLOGUE( Textures, TextureContainer )
    *retval = GetIMipMapContainerFromTextureMM( pThis->get_texture_mm(idx) ); 
	(*retval)->AddRef();

    return S_OK; 
}

STDMETHODIMP ITextureContainer::FindRGB(int r, int g, int b, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->find_rgb( r, g, b ); 
    return S_OK;
}

STDMETHODIMP ITextureContainer::FindColor(int r, int g, int b, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->find_color(r,g,b); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::FindRealRGB(int r, int g, int b, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->find_rgb_real(r,g,b); 
    return S_OK; 
}

IMPLEMENT_METHOD( InitializeLimited, initialize_limited, Textures, TextureContainer )
IMPLEMENT_METHOD( ComputePalette, compute_palette, Textures, TextureContainer )
IMPLEMENT_METHOD( AllocatePalette, alloc_palette, Textures, TextureContainer )

IMPLEMENT_GET_PROPERTY_PTR( GetPalette, get_palette(), RGBcolor*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( Get8BitWhiteLUT, lt_white8, TextureTablesWhite8*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( Get16BitWhiteLUT, lt_white16, TextureTablesWhite16*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetFastXXXLUT, lt_fastxxx, TextureTablesFastXxx*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetTrueRGBLUT, lt_truergb, TextureTablesTrueRgb*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetPrivateRGBLUT, lt_truergb_private, TextureTablesTrueRgbPriv*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetCurrentRGBLUT, lt_light, PalIdxLookup*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetAlphaLUT, lt_alpha, TextureTablesAlpha*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY_PTR( GetPaletteLUT, lt_pal, TextureTablesPalette*, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetRedLevel, get_level_red(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetGreenLevel, get_level_green(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetBlueLevel, get_level_blue(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetRed, red(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetBlue, blue(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetYellow, yellow(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetGreen, green(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetWhite, white(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetBlack, black(), int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetAlphaMask, alpha_mask, unsigned short, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetMixingType, mixing, int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetMode, txtMode, int, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetLightMapsOnly, do_lightmaponly, bool, Textures, TextureContainer )
IMPLEMENT_GET_PROPERTY(GetLightMapsGrid, do_lightmapgrid, bool, Textures, TextureContainer )


STDMETHODIMP ITextureContainer::ComputeLightTables(IWorld* world) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    pThis->compute_light_tables( GetWorldFromIWorld(world) ); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetPaletteAllocated( int idx, int& retval ) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->get_pal_alloced(idx); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetRedPalette( int idx, int& retval ) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->get_pal_red(idx); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetGreenPalette( int idx, int& retval ) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->get_pal_green(idx); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::GetBluePalette( int idx, int& retval ) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->get_pal_blue(idx); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::Load(char* buf, IWorld* world) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    pThis->load( buf, GetWorldFromIWorld(world) ); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::MixLights(int r, int g, int b, int p, unsigned char& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = (unsigned char)pThis->mix_lights( r, g, b, p ); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::AddRedLightPrivate(int r, int l, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->add_light_red_private(r, l); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::AddGreenLightPrivate(int g, int l, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->add_light_green_private(g, l); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::AddBlueLightPrivate(int b, int l, int& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->add_light_blue_private(b, l); 
    return S_OK; 
}

STDMETHODIMP ITextureContainer::MixLights16Bit(int r, int g, int b, int p, unsigned short& retval) 
{ 
    METHOD_PROLOGUE( Textures, TextureContainer )
    retval = pThis->mix_lights_16(r, g, b, p); 
    return S_OK; 
}

