/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "engine/msectobj.h"
#include "engine/sector.h"
#include "engine/world.h"

SectorMSO::SectorMSO ()
{
  next_sector = prev_sector = NULL;
  next_mso = prev_mso = NULL;
  sector = NULL;
  mso = NULL;
}

SectorMSO::~SectorMSO ()
{
  if (sector) sector->unlink_mso (this);
  if (mso) mso->unlink_mso (this);
}


MultiSectorObject::MultiSectorObject (csObject* obj)
{
  object = obj;
  first_sector = NULL;
  world = NULL;
}

MultiSectorObject::MultiSectorObject ()
{
  object = NULL;
  first_sector = NULL;
  world = NULL;
}

MultiSectorObject::~MultiSectorObject ()
{
  while (first_sector) { CHK (delete first_sector); }
  if (world) world->remove (this);
}


void MultiSectorObject::unlink_mso (SectorMSO* lp)
{
  if (lp->next_mso) lp->next_mso->prev_mso = lp->prev_mso;
  if (lp->prev_mso) lp->prev_mso->next_mso = lp->next_mso;
  else first_sector = lp->next_mso;
  lp->prev_mso = lp->next_mso = NULL;
  lp->mso = NULL;
}

void MultiSectorObject::add_mso (SectorMSO* lp)
{
  lp->next_mso = first_sector;
  lp->prev_mso = NULL;
  if (first_sector) first_sector->prev_mso = lp;
  first_sector = lp;
  lp->mso = this;
}

void MultiSectorObject::init_one_sector (csSector* s)
{
  while (first_sector) { CHK (delete first_sector); }
  CHK (SectorMSO* mso = new SectorMSO ());
  s->add_mso (mso);
  add_mso (mso);
}

