
#include "com/com.h"
#include "engine/polygon/ipolygon.h"
#include "engine/polygon/polygon.h"
#include "engine/polygon/polytext.h"
#include "engine/light/lghtmap.h"
#include "engine/basic/polyset.h"
#include "general/stdassrt.h"
#include "engine/texture.h"

IMPLEMENT_COMPOSITE_UNKNOWN( PolygonSet, PolygonSet )

STDMETHODIMP IPolygonSet::GetName(char** szName)
{
    METHOD_PROLOGUE( PolygonSet, PolygonSet );
    *szName = pThis->get_name();
    return S_OK;
}

IMPLEMENT_COMPOSITE_UNKNOWN( Polygon3D, Polygon3D )

STDMETHODIMP IPolygon3D::GetName(char** szName)
{
    METHOD_PROLOGUE( Polygon3D, Polygon3D );
    *szName = pThis->get_name();
    return S_OK;
}

STDMETHODIMP IPolygon3D::GetParent(IPolygonSet** retval)
{
    METHOD_PROLOGUE( Polygon3D, Polygon3D );
    PolygonSet* pPolySet = (PolygonSet*)pThis->get_parent();
    
	*retval = GetIPolygonSetFromPolygonSet(pPolySet);
	(*retval)->AddRef();

    return S_OK;
}

STDMETHODIMP IPolygon3D::GetLightMap(ILightMap** retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	if (pThis->lightmap)
	{
	  *retval = GetILightMapFromLightMap( pThis->lightmap );
	  (*retval)->AddRef();
	}
	else *retval = NULL;

	return S_OK;
}

STDMETHODIMP IPolygon3D::GetFog(int& retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	retval = pThis->get_fog();
	return S_OK;
}

STDMETHODIMP IPolygon3D::GetAlpha(int& retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	retval = pThis->get_alpha();
	return S_OK;
}

STDMETHODIMP IPolygon3D::UsesMipMaps(void)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	return pThis->no_mipmap ? S_FALSE : S_OK;
}

STDMETHODIMP IPolygon3D::GetTextureByZ(float z, IPolygonTexture** retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	*retval = GetIPolygonTextureFromPolyTexture(pThis->get_polytex(z));
	(*retval)->AddRef();

	return S_OK;
}

STDMETHODIMP IPolygon3D::GetTexture(int mipmap, IPolygonTexture** retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	*retval = GetIPolygonTextureFromPolyTexture(pThis->get_polytex(mipmap));
	(*retval)->AddRef();

	return S_OK;
}

STDMETHODIMP IPolygon3D::GetCameraVector(int v, ComVector3* retval)
{
	METHOD_PROLOGUE( Polygon3D, Polygon3D );
	Vector3 v3;
	v3 = pThis->vcam(v);

	//ASSERT( retval );
	retval->x = v3.x;
	retval->y = v3.y;
	retval->z = v3.z;

	return S_OK;
}


// IPolygonTexture implementation for PolyTexture

IMPLEMENT_COMPOSITE_UNKNOWN( PolyTexture, PolygonTexture )

IMPLEMENT_GET_PROPERTY( GetFDU, fdu, float, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetFDV, fdv, float, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetWidth, w, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetHeight, h, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetSize, size, long, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetIMinU, Imin_u, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetIMinV, Imin_v, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetMipMapSize, mipmap_size, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetMipMapShift, mipmap_shift, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetNumPixels, size, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetOriginalWidth, w_orig, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetShiftU, shf_u, int, PolyTexture, PolygonTexture )
IMPLEMENT_GET_PROPERTY( GetNumberDirtySubTex, count_dirty_subtextures(), int, PolyTexture, PolygonTexture ) 
IMPLEMENT_GET_PROPERTY( GetSubtexSize, subtex_size, int, PolyTexture, PolygonTexture ) 
IMPLEMENT_GET_PROPERTY( GetDynlightOpt, subtex_dynlight, bool, PolyTexture, PolygonTexture ) 

IMPLEMENT_METHOD( CreateDirtyMatrix, create_dirty_matrix, PolyTexture, PolygonTexture )
IMPLEMENT_METHOD( MakeAllDirty, make_all_dirty, PolyTexture, PolygonTexture )

STDMETHODIMP IPolygonTexture::CleanIfDirty (int lu, int lv, bool& retval)
{
  METHOD_PROLOGUE( PolyTexture, PolygonTexture );
  int idx = pThis->dirty_w*lv + lu;
  retval = (bool)(pThis->dirty_matrix[idx]);
  if (retval)
  {
    pThis->dirty_matrix[idx] = 0;
    pThis->dirty_cnt--;
  }
  return S_OK;
}

STDMETHODIMP IPolygonTexture::RecalculateDynamicLights(bool& recalc)
{
  METHOD_PROLOGUE( PolyTexture, PolygonTexture );
  recalc = pThis->recalc_dynamic_lights ();
  return S_OK;
}

STDMETHODIMP IPolygonTexture::GetLightMap(ILightMap** retval)
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	if (pThis->lm != NULL)
	{
		*retval = GetILightMapFromLightMap( pThis->lm );
		(*retval)->AddRef();
	}
	else *retval = NULL;

	return S_OK;
}

STDMETHODIMP IPolygonTexture::SetTCacheData(void* newval)
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	pThis->tcache_data = newval;
	return S_OK;
}

STDMETHODIMP IPolygonTexture::GetTCacheData(void** retval)
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	*retval = pThis->tcache_data;
	return S_OK;
}

STDMETHODIMP IPolygonTexture::GetPolygon(IPolygon3D** retval)
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	*retval = GetIPolygon3DFromPolygon3D( pThis->polygon );
	(*retval)->AddRef();

	return S_OK;
}

STDMETHODIMP IPolygonTexture::GetTexture(ITextureMap** retval)
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	*retval = GetITextureMapFromTexture( pThis->texture );
	(*retval)->AddRef();

	return S_OK;
}

STDMETHODIMP IPolygonTexture::GetTextureBox( float& fMinU, float& fMinV, float& fMaxU, float& fMaxV )
{
	METHOD_PROLOGUE( PolyTexture, PolygonTexture );
	
	fMinU = pThis->Fmin_u;
	fMaxU = pThis->Fmax_u;
	fMinV = pThis->Fmin_v;
	fMaxV = pThis->Fmax_v;

	return S_OK;
}
