/*
    CrystalSpace 3D renderer view
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "system/system.h"
#include "system/sysprint.h"
#include "general/csview.h"
#include "engine/world.h"

cs3Dview::cs3Dview (csWorld *iWorld)
{
  world = iWorld;
  CHK (view = new Polygon2D (16));
  CHK (camera = new csCamera ());
  clipper = NULL;

  if (csCamera::aspect == 0)
    csCamera::aspect = FRAME_HEIGHT;
  csCamera::inv_aspect = 1./csCamera::aspect;
}

cs3Dview::~cs3Dview ()
{
  CHK (delete world);
  CHK (delete camera);
  CHK (delete view);
  CHK (delete clipper);
}

void cs3Dview::SetRectangle (int x, int y, int w, int h)
{
  if (view) { CHK (delete view);  view = NULL; }
  bview = Box(x, y, x+w-1, y+h-1);
  CHK (delete clipper);
  clipper = NULL;
}

void cs3Dview::AddViewVertex (int x, int y)
{
  if (!view) CHKB (view = new Polygon2D (16));
  view->add_vertex(x,y);
  CHK (delete clipper);
  clipper = NULL;
}

void cs3Dview::Draw ()
{
  if (!clipper)
    if (view)
    {
      CHK (clipper = new csPolygonClipper (view->get_vertices (), view->get_num_vertices ()));
    }
    else
    {
      CHK (clipper = new csBoxClipper (bview));
    }

 world->draw (camera, clipper);
}

void cs3Dview::SetSector (csSector *sector)
{
  camera->set_sector (sector);
}
