/*
    CrystalSpace 3D renderer view
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSVIEW_H__
#define __CSVIEW_H__

#include "version.h"
#include "util/csbase.h"
#include "engine/polygon/polygon.h"
#include "geometry/math2d.h"

class Polygon2D;
class csCamera;
class csWorld;
class csSector;
class csClipper;

/**
 * The cs3Dview class encapsulates the top-level Crystal Space
 * renderer interface.
 */
class cs3Dview : public csBase
{
private:
  /// csClipper
  csClipper* clipper;

public:
  /// Clipping rectangle
  Box bview;
  /// Clipping region
  Polygon2D *view;

  /// csCamera
  csCamera *camera;
  /// World
  csWorld *world;

  /// Constructor
  cs3Dview (csWorld *iWorld);
  /// Destructor
  ~cs3Dview ();

  /// Clear clipping polygon
  void ClearView () { if (view) view->make_empty (); }
  /// Set clipping rectangle
  void SetRectangle (int x, int y, int w, int h);
  /// Add a vertex to clipping polygon (non-rectangular clipping)
  void AddViewVertex (int x, int y);
  /// Draw world view
  void Draw ();
  /// Set camera sector
  void SetSector (csSector *sector);
};

#endif //  __CSVIEW_H__
