/*
    Copyright (C) 1998 by Jorrit Tyberghein
    Written by Ivan Avramovic <iavramov@ibm.net>
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef _CSLOADER_H_
#define _CSLOADER_H_

#include "engine/basic/polyset.h"
#include "general/loader/loadinfo.h"

class PolygonTemplate;
class ThingTemplate;
class Textures;
class PolyPlane;
class Collection;
class csStatLight;
class Thing;
class Library;
class ImageFile;
class csWorld;
class Archive;
class csCurveTemplate;
class Wave;

enum { kTokenPSetVertex = 1, kTokenPSetPolygon, kTokenPSetBezier,
       kTokenPSetTexNr, kTokenPSetCircle,
       kTokenPSetTexlen, kTokenPSetTrigger, kTokenPSetActivate,
       kTokenPSetLightX, kTokenPSetBsp, kTokenPSetFog,
       kTokenPSetLast };

/**
 * The loader for CS worlds.
 */
class CSLoader
{
public:
  ///
  static Matrix3 load_matrix (char* buf);
  ///
  static Vector3 load_vector (char* buf);
  ///
  static PolyPlane* load_polyplane (char* buf, char* name = NULL);
  ///
  static Collection* load_collection (char* name, csWorld* world, char* buf);
  ///
  static csStatLight* load_statlight (char* buf);
  ///
  static PolygonSet& ps_process (PolygonSet& ps, PSLoadInfo& info, int cmd,
                                 char* name, char* params);
  ///
  static void skydome_process (csSector& sector, char* name, char* buf,
			       TextureMM* texture);
  ///
  static Thing* load_sixface (char* name, csWorld* w, char* buf,
                              Textures* textures, csSector* sec);
  ///
  static Thing* load_thing (char* name, csWorld* w, char* buf, 
                            Textures* textures, csSector* sec);
  ///
  static Polygon3D* load_poly3d (char* polyname, csWorld* w, char* buf, 
        Textures* textures, TextureMM* default_texture, float default_texlen,
        CLights* default_lightx, csSector* sec, PolygonSet* parent);

  static csCurve* load_bezier (char* polyname, csWorld* w, char* buf, 
        Textures* textures, TextureMM* default_texture, float default_texlen,
        CLights* default_lightx, csSector* sec, PolygonSet* parent);

  ///
  static ImageFile* load_image(char* name, csWorld* w, Archive* ar = NULL);
  ///
  static TextureMM& txt_process(TextureMM& t, char* buf);
  ///
  static PolygonTemplate* load_ptemplate (char* ptname, char* buf,
        Textures* textures, TextureMM* default_texture, float default_texlen,
        ThingTemplate* parent);
  ///
  static ThingTemplate* load_thingtpl(char* tname, char* buf,
                                      Textures* textures);
  ///
  static csCurveTemplate* load_beziertemplate (char* ptname, char* buf, 
					   Textures* textures, TextureMM* default_texture, float default_texlen,
					   ThingTemplate* parent);

  ///
  static ThingTemplate* load_sixtpl(char* tname,char* buf,Textures* textures);
  ///
  static csSector* load_room (char* secname, csWorld* w, char* buf, 
                            Textures* textures);
  ///
  static csSector* load_sector (char* secname, csWorld* w, char* buf, 
                              Textures* textures);
  /// Load a library definition.
  static bool load_library_def (Library* lib, csWorld* w, char* file, Textures* textures);
  /// Load a library definition.
  static bool load_library_def (Library* lib, csWorld* w, Archive* ar, char* buf, Textures* textures);

  ///
  static Wave* load_wave (char* name, csWorld* w, Archive* ar);
};

#endif
