/*
  Maze editor: menus, toolbars etc
  Copyright (C) 1998 by Jorrit Tyberghein
  Written by Andrew Zabolotny <bit@eltech.ru>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __ME_RES_H__

#define TOOLBAR_BITMAPS	"img/MazeD/ToolBar.png"
#define TOOLBAR_BUTTON_STYLE (CSBS_DISMISS | CSBS_SELECTABLE | CSBS_SHIFT | CSBS_MULTICHOOSE)

// Identification numbers for toolbars
#define MZID_TOOLBOX		0x80000000
#define MZID_WINDOWBAR		0x80000010

#define MZID_INFOBAR		0x80000020
#define MZID_INFO_PTR_X		0x80000021
#define MZID_INFO_PTR_Y		0x80000022
#define MZID_INFO_PTR_Z		0x80000023
#define MZID_INFO_VIEW_X	0x80000024
#define MZID_INFO_VIEW_Y	0x80000025
#define MZID_INFO_VIEW_Z	0x80000026
#define MZID_INFO_VIEW_PHI	0x80000027
#define MZID_INFO_VIEW_THETA	0x80000028
#define MZID_INFO_VIEW_SCALE	0x80000029
#define MZID_INFO_VERTICES	0x8000002a
#define MZID_INFO_POLYGONS	0x8000002b
#define MZID_INFO_LIGHTS	0x8000002c
#define MZID_INFO_TEXTURES	0x8000002d
#define MZID_INFO_FRAMES	0x8000002e
#define MZID_INFO_CURRENTOP	0x8000002f

#define MZID_ACCELERATOR	0x80000040
#define MZID_DRAFTVIEW		0x80000050
#define MZID_3DVIEW		0x80000060

static inline void ShowAbout (csComponent *app)
{
  char msg [400];
  sprintf (msg,
    "Crystal Space Maze Editor\n"
    "Version %s\n"
    "Released %s\n"
    "\n"
    "Design:\n"
    "        Andrew Zabolotny\n"
    "Implementation:\n"
    "        Andrew Zabolotny\n"
    "Moral support:\n"
    "        My wife Ulea\n"
    "        Jorrit Tyberghein\n",
    VERSION, LAST_MOD_DATE);
  MessageBox (app, "About Maze Editor", msg);
}

static inline void InitializeMainMenu (csComponent *app)
{
  csMenu *menu, *submenu, *subsubmenu, *subsubsubmenu;

  CHK (menu = new csMenu (app, csmfsBar, 0));
  menu->id = CSWID_MENUBAR;

  CHK (submenu = new csMenu (NULL));
  CHK ((void)new csMenuItem (menu, "~File", submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Open", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~World\tCtrl+O", cscmdMzOpenWorld));
      CHK ((void)new csMenuItem (subsubmenu, "~Model\tCtrl+Shift+O", cscmdMzOpenModel));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Save", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~World\tCtrl+S", cscmdMzSaveWorld));
      CHK ((void)new csMenuItem (subsubmenu, "~Model\tCtrl+Shift+S", cscmdMzSaveModel));
    CHK ((void)new csMenuItem (submenu, "~New world", cscmdNewWorld));
    CHK ((void)new csMenuItem (submenu));
    CHK ((void)new csMenuItem (submenu, "~Quit\tShift+Q", cscmdQuit));

  CHK (submenu = new csMenu (NULL));
  CHK ((void)new csMenuItem (menu, "~Edit", submenu));
    CHK ((void)new csMenuItem (submenu, "~Undo\tAlt+BackSpace", cscmdNothing));
    CHK ((void)new csMenuItem (submenu));
    CHK ((void)new csMenuItem (submenu, "~Copy\tCtrl+Ins", cscmdNothing));
    CHK ((void)new csMenuItem (submenu, "Cu~t\tShift+Del", cscmdNothing));
    CHK ((void)new csMenuItem (submenu, "~Paste\tShift+Ins", cscmdNothing));
    CHK ((void)new csMenuItem (submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Select", subsubmenu));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Vertex", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~All\tCtrl+'+'", cscmdMzVertexSelectAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Deselect all\tCtrl+'-'", cscmdMzVertexDeselectAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Invert all\tCtrl+'*'", cscmdMzVertexInvertAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "Select mode\t+", cscmdMzVertexSelect));
        CHK ((void)new csMenuItem (subsubsubmenu, "Deselect mode\t-", cscmdMzVertexDeselect));
        CHK ((void)new csMenuItem (subsubsubmenu, "Invert mode\t*", cscmdMzVertexInvert));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Polygon", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~All\tCtrl+Shift+'+'", cscmdMzPolySelectAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Deselect all\tCtrl+Shift+'-'", cscmdMzPolyDeselectAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Invert all\tCtrl+Shift+'*'", cscmdMzPolyInvertAll));
        CHK ((void)new csMenuItem (subsubsubmenu, "Select mode\tShift+'+'", cscmdMzPolySelect));
        CHK ((void)new csMenuItem (subsubsubmenu, "Deselect mode\tShift+'-'", cscmdMzPolyDeselect));
        CHK ((void)new csMenuItem (subsubsubmenu, "Invert mode\tShift+'*'", cscmdMzPolyInvert));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "C~reate", subsubmenu));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Vertex", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Insert\tIns", cscmdMzCreateVertex));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Delete\tDel", cscmdMzDeleteVertex));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~2D preset", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Triangle", cscmdMzCreateTriangle));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Square", cscmdMzCreateSquare));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Hexagon", cscmdMzCreateHexagon));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Octagon", cscmdMzCreateOctagon));
        CHK ((void)new csMenuItem (subsubsubmenu, "~N-gon", cscmdMzCreateNgon));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~3D preset", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Box", cscmdMzCreateBox));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Prism", cscmdMzCreatePrism));
        CHK ((void)new csMenuItem (subsubsubmenu, "P~yramid", cscmdMzCreatePyramid));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Torus", cscmdMzCreateTorus));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Sphere", cscmdMzCreateSphere));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Polygon", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Connect\tCtrl+Ins", cscmdMzCreatePolyConnect));
        CHK ((void)new csMenuItem (subsubsubmenu, "Triangle ~fan\tCtrl+Alt+Ins", cscmdMzCreatePolyTriFan));
        CHK ((void)new csMenuItem (subsubsubmenu, "Triangle ~strip\tCtrl+Shift+Ins", cscmdMzCreatePolyTriStrip));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Quad strip\tCtrl+Alt+Shift+Ins", cscmdMzCreatePolyQuadStrip));
      CHK ((void)new csMenuItem (subsubmenu, "~Light", cscmdMzCreateLight));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Modify", subsubmenu));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Selected", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "Delete ~vertices\tCtrl+Del", cscmdMzDeleteSelVertices));
        CHK ((void)new csMenuItem (subsubsubmenu, "Delete ~polygons\tCtrl+Shift+Del", cscmdMzDeleteSelPolygons));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Move\tm", cscmdMzMove));
        CHK ((void)new csMenuItem (subsubsubmenu, "~2D scale\ts", cscmdMzScale2D));
        CHK ((void)new csMenuItem (subsubsubmenu, "~3D scale\td", cscmdMzScale3D));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Rotate\tr", cscmdMzRotate));
      CHK (subsubsubmenu = new csMenu (NULL));
      CHK ((void)new csMenuItem (subsubmenu, "~Light", subsubsubmenu));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Move", cscmdNothing));
        CHK ((void)new csMenuItem (subsubsubmenu, "~Delete", cscmdNothing));
      CHK ((void)new csMenuItem (subsubmenu, "A~xis\tx", cscmdMzSetupAxis));

  CHK (submenu = new csMenu (NULL));
  CHK ((void)new csMenuItem (menu, "~Model", submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Create", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Room", cscmdMzCreateRoom));
      CHK ((void)new csMenuItem (subsubmenu, "~Thing", cscmdMzCreateThing));
      CHK ((void)new csMenuItem (subsubmenu, "~Sprite", cscmdMzCreateSprite));
    CHK ((void)new csMenuItem (submenu, "~Properties", cscmdMzModelProperties));
    CHK ((void)new csMenuItem (submenu));
    CHK ((void)new csMenuItem (submenu, "~Manager\tCtrl+M", cscmdMzModelManager));
    CHK ((void)new csMenuItem (submenu, "C~lear", cscmdMzClearModel));
    CHK ((void)new csMenuItem (submenu, "~Delete", cscmdMzDeleteModel));
    CHK ((void)new csMenuItem (submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Frame", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Insert", cscmdMzInsertModelFrame));
      CHK ((void)new csMenuItem (subsubmenu, "~Delete", cscmdMzDeleteModelFrame));
      CHK ((void)new csMenuItem (subsubmenu, "~Next\tCtrl+Up", cscmdMzNextModelFrame));
      CHK ((void)new csMenuItem (subsubmenu, "~Previous\tCtrl+Down", cscmdMzPrevModelFrame));

  CHK (submenu = new csMenu (NULL));
  CHK ((void)new csMenuItem (menu, "~Window", submenu));
    CHK ((void)new csMenuItem (submenu, "Close ~all", cscmdMzWindowCloseAll));
    CHK ((void)new csMenuItem (submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Draft view", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Create", cscmdMzDraftView));
      CHK ((void)new csMenuItem (subsubmenu, "~Setup\tCtrl+D", cscmdMzDraftViewSetup));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~3D view", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Create", cscmdMz3DView));
      CHK ((void)new csMenuItem (subsubmenu, "~Setup\tCtrl+V", cscmdMz3DViewSetup));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Toolbar", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Tool box", cscmdMzToolBox));
      CHK ((void)new csMenuItem (subsubmenu, "~Window manager", cscmdMzWindowManager));
      CHK ((void)new csMenuItem (subsubmenu, "~Information", cscmdMzInformation));
    CHK ((void)new csMenuItem (submenu));
    CHK (subsubmenu = new csMenu (NULL));
    CHK ((void)new csMenuItem (submenu, "~Layout", subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~1: One\tCtrl+1", cscmdMzWindowLayout1));
      CHK ((void)new csMenuItem (subsubmenu, "~2: Two\tCtrl+2", cscmdMzWindowLayout2));
      CHK ((void)new csMenuItem (subsubmenu, "~3: Three\tCtrl+3", cscmdMzWindowLayout3));
      CHK ((void)new csMenuItem (subsubmenu, "~4: Four\tCtrl+4", cscmdMzWindowLayout4));
      CHK ((void)new csMenuItem (subsubmenu));
      CHK ((void)new csMenuItem (subsubmenu, "~Reset", cscmdMzWindowLayoutReset));
    CHK ((void)new csMenuItem (submenu));
    CHK ((void)new csMenuItem (submenu, "L~ist", cscmdMzWindowList));

  CHK ((void)new csMenuItem (menu, CSMIS_NEWCOLUMN));

  CHK (submenu = new csMenu (NULL));
  CHK ((void)new csMenuItem (menu, "~Help", submenu));
    CHK ((void)new csMenuItem (submenu, "~Index", cscmdNothing));
    CHK ((void)new csMenuItem (submenu, "~General", cscmdNothing));
    CHK ((void)new csMenuItem (submenu, "~Context\tF1", cscmdNothing));
    CHK ((void)new csMenuItem (submenu));
    CHK ((void)new csMenuItem (submenu, "~About", cscmdMzAbout));

  menu->PlaceItems ();
  menu->SetRect (0, 0, app->bound.xmax, menu->TextHeight () + 8);
}

static inline void InitializeKeyboardAccelerator (csApp *app)
{
  CHK (csKeyboardAccelerator *ka = new csKeyboardAccelerator (app));
  ka->id = MZID_ACCELERATOR;
  ka->Command ('Q', CSMASK_SHIFT, cscmdQuit);
  ka->Command ('+', CSMASK_CTRL, cscmdMzVertexSelectAll);
  ka->Command ('-', CSMASK_CTRL, cscmdMzVertexDeselectAll);
  ka->Command ('*', CSMASK_CTRL, cscmdMzVertexInvertAll);
  ka->Command ('+', 0, cscmdMzVertexSelect);
  ka->Command ('-', 0, cscmdMzVertexDeselect);
  ka->Command ('*', 0, cscmdMzVertexInvert);
  ka->Command ('+', CSMASK_CTRL | CSMASK_SHIFT, cscmdMzPolySelectAll);
  ka->Command ('_', CSMASK_CTRL | CSMASK_SHIFT, cscmdMzPolyDeselectAll);
  ka->Command ('*', CSMASK_CTRL | CSMASK_SHIFT, cscmdMzPolyInvertAll);
  ka->Command ('+', CSMASK_SHIFT, cscmdMzPolySelect);
  ka->Command ('_', CSMASK_SHIFT, cscmdMzPolyDeselect);
  ka->Command ('*', CSMASK_SHIFT, cscmdMzPolyInvert);
  ka->Command (CSKEY_INS, 0, cscmdMzCreateVertex);
  ka->Command (CSKEY_DEL, 0, cscmdMzDeleteVertex);
  ka->Command (CSKEY_INS, CSMASK_CTRL, cscmdMzCreatePolyConnect);
  ka->Command (CSKEY_INS, CSMASK_CTRL | CSMASK_ALT, cscmdMzCreatePolyTriFan);
  ka->Command (CSKEY_INS, CSMASK_CTRL | CSMASK_SHIFT, cscmdMzCreatePolyTriStrip);
  ka->Command (CSKEY_INS, CSMASK_CTRL | CSMASK_SHIFT | CSMASK_ALT, cscmdMzCreatePolyQuadStrip);
  ka->Command ('1', CSMASK_CTRL, cscmdMzWindowLayout1);
  ka->Command ('2', CSMASK_CTRL, cscmdMzWindowLayout2);
  ka->Command ('3', CSMASK_CTRL, cscmdMzWindowLayout3);
  ka->Command ('4', CSMASK_CTRL, cscmdMzWindowLayout4);
  ka->Command (CSKEY_DEL, CSMASK_CTRL, cscmdMzDeleteSelVertices);
  ka->Command (CSKEY_DEL, CSMASK_CTRL | CSMASK_SHIFT, cscmdMzDeleteSelPolygons);
  ka->Command ('o', CSMASK_CTRL, cscmdMzOpenWorld);
  ka->Command ('O', CSMASK_CTRL | CSMASK_SHIFT, cscmdMzOpenModel);
  ka->Command ('s', CSMASK_CTRL, cscmdMzSaveWorld);
  ka->Command ('S', CSMASK_CTRL | CSMASK_SHIFT, cscmdMzSaveModel);
  ka->Command ('m', CSMASK_CTRL, cscmdMzModelManager);
  ka->Command ('d', CSMASK_CTRL, cscmdMzDraftViewSetup);
  ka->Command ('v', CSMASK_CTRL, cscmdMz3DViewSetup);

  ka->Command ('m', 0, cscmdMzMove);
  ka->Command ('s', 0, cscmdMzScale2D);
  ka->Command ('d', 0, cscmdMzScale3D);
  ka->Command ('r', 0, cscmdMzRotate);
  ka->Command ('x', 0, cscmdMzSetupAxis);

  ka->Command (CSKEY_UP, CSMASK_CTRL, cscmdMzNextModelFrame);
  ka->Command (CSKEY_DOWN, CSMASK_CTRL, cscmdMzPrevModelFrame);

  ka->Broadcast (CSKEY_HOME, 0, cscmdMzDraftViewOrigin);
  ka->Broadcast (CSKEY_HOME, CSMASK_CTRL, cscmdMzDraftViewFit);
}

static inline csSprite2D *GetBitmap (csApp *app, int iCode)
{
  switch (iCode)
  {
    case cscmdMzVertex:
      return NewBitmap (app, TOOLBAR_BITMAPS, 288, 0, 32, 32);
    case cscmdMzCreate2D:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 0, 32, 32);
    case cscmdMzCreate3D:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 0, 32, 32);
    case cscmdMzCreateLight:
      return NewBitmap (app, TOOLBAR_BITMAPS, 64, 0, 32, 32);
    case cscmdMzCreatePoly:
      return NewBitmap (app, TOOLBAR_BITMAPS, 96, 0, 32, 32);
    case cscmdMzSelect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 0, 32, 32);
    case cscmdMzMove:
      return NewBitmap (app, TOOLBAR_BITMAPS, 160, 0, 32, 32);
    case cscmdMzScale2D:
      return NewBitmap (app, TOOLBAR_BITMAPS, 160, 32, 32, 32);
    case cscmdMzScale3D:
      return NewBitmap (app, TOOLBAR_BITMAPS, 160, 64, 32, 32);
    case cscmdMzRotate:
      return NewBitmap (app, TOOLBAR_BITMAPS, 160, 96, 32, 32);
    case cscmdMzSetupAxis:
      return NewBitmap (app, TOOLBAR_BITMAPS, 160, 128, 32, 32);

    case cscmdMzCreateVertex:
      return NewBitmap (app, TOOLBAR_BITMAPS, 288, 32, 32, 32);
    case cscmdMzDeleteVertex:
      return NewBitmap (app, TOOLBAR_BITMAPS, 288, 64, 32, 32);
    case cscmdMzDeleteSelVertices:
      return NewBitmap (app, TOOLBAR_BITMAPS, 288, 96, 32, 32);
    case cscmdMzDeleteSelPolygons:
      return NewBitmap (app, TOOLBAR_BITMAPS, 288, 128, 32, 32);

    case cscmdMzCreateTriangle:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 32, 32, 32);
    case cscmdMzCreateSquare:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 64, 32, 32);
    case cscmdMzCreateHexagon:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 96, 32, 32);
    case cscmdMzCreateOctagon:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 128, 32, 32);
    case cscmdMzCreateNgon:
      return NewBitmap (app, TOOLBAR_BITMAPS, 0, 160, 32, 32);

    case cscmdMzCreateBox:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 32, 32, 32);
    case cscmdMzCreatePrism:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 64, 32, 32);
    case cscmdMzCreatePyramid:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 96, 32, 32);
    case cscmdMzCreateTorus:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 128, 32, 32);
    case cscmdMzCreateSphere:
      return NewBitmap (app, TOOLBAR_BITMAPS, 32, 160, 32, 32);

    case cscmdMzCreatePolyConnect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 96, 32, 32, 32);
    case cscmdMzCreatePolyTriFan:
      return NewBitmap (app, TOOLBAR_BITMAPS, 96, 64, 32, 32);
    case cscmdMzCreatePolyTriStrip:
      return NewBitmap (app, TOOLBAR_BITMAPS, 96, 96, 32, 32);
    case cscmdMzCreatePolyQuadStrip:
      return NewBitmap (app, TOOLBAR_BITMAPS, 96, 128, 32, 32);

    case cscmdMzVertexSelect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 32, 32, 32);
    case cscmdMzVertexDeselect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 64, 32, 32);
    case cscmdMzVertexInvert:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 96, 32, 32);
    case cscmdMzPolySelect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 128, 32, 32);
    case cscmdMzPolyDeselect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 160, 32, 32);
    case cscmdMzPolyInvert:
      return NewBitmap (app, TOOLBAR_BITMAPS, 128, 192, 32, 32);

    case cscmdMzWindowLayout1:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 0, 32, 32);
    case cscmdMzWindowLayout2:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 32, 32, 32);
    case cscmdMzWindowLayout3:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 64, 32, 32);
    case cscmdMzWindowLayout4:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 96, 32, 32);
    case cscmdMzWindowCloseAll:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 128, 32, 32);
    case cscmdMzWindowLayoutReset:
      return NewBitmap (app, TOOLBAR_BITMAPS, 256, 160, 32, 32);
    case cscmdMzCoordRotate:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 0, 32, 32);
    case cscmdMzZoomInOut:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 32, 32, 32);
    case cscmdMzZoomToRect:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 96, 32, 32);
    case cscmdMzSlideView:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 128, 32, 32);
    case cscmdMzToggleGrid:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 64, 32, 32);
    case cscmdMzToggleAxis:
      return NewBitmap (app, TOOLBAR_BITMAPS, 320, 160, 32, 32);

    default:
      return NULL;
  } /* endswitch */
}

static inline void InitializeToolbox (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzCreateLight,
    GetBitmap (app, cscmdMzCreateLight), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzVertex,
    GetBitmap (app, GLOBAL_VERTEXMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzCreate2D,
    GetBitmap (app, GLOBAL_CREATE2DMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzCreate3D,
    GetBitmap (app, GLOBAL_CREATE3DMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzCreatePoly,
    GetBitmap (app, GLOBAL_CREATEPOLYMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzSelect,
    GetBitmap (app, GLOBAL_SELECTMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
  csNewToolbarButton (toolbar, cscmdMzModify,
    GetBitmap (app, GLOBAL_MODIFYMODE), NULL, csbfsThinRect, CSBS_SHIFT | CSBS_MULTICHOOSE);
}

static inline void InitializeVertexToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzCreateVertex,
    GetBitmap (app, cscmdMzCreateVertex), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzDeleteVertex,
    GetBitmap (app, cscmdMzDeleteVertex), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzDeleteSelVertices,
    GetBitmap (app, cscmdMzDeleteSelVertices), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzDeleteSelPolygons,
    GetBitmap (app, cscmdMzDeleteSelPolygons), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
}

static inline void InitializeCreate2DToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzCreateTriangle,
    GetBitmap (app, cscmdMzCreateTriangle), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateSquare,
    GetBitmap (app, cscmdMzCreateSquare), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateHexagon,
    GetBitmap (app, cscmdMzCreateHexagon), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateOctagon,
    GetBitmap (app, cscmdMzCreateOctagon), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateNgon,
    GetBitmap (app, cscmdMzCreateNgon), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
}

static inline void InitializeCreate3DToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzCreateBox,
    GetBitmap (app, cscmdMzCreateBox), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreatePrism,
    GetBitmap (app, cscmdMzCreatePrism), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreatePyramid,
    GetBitmap (app, cscmdMzCreatePyramid), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateTorus,
    GetBitmap (app, cscmdMzCreateTorus), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreateSphere,
    GetBitmap (app, cscmdMzCreateSphere), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
}

static inline void InitializeCreatePolyToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzCreatePolyConnect,
    GetBitmap (app, cscmdMzCreatePolyConnect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreatePolyTriFan,
    GetBitmap (app, cscmdMzCreatePolyTriFan), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreatePolyTriStrip,
    GetBitmap (app, cscmdMzCreatePolyTriStrip), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzCreatePolyQuadStrip,
    GetBitmap (app, cscmdMzCreatePolyQuadStrip), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
}

static inline void InitializeSelectToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzVertexSelect,
    GetBitmap (app, cscmdMzVertexSelect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzVertexDeselect,
    GetBitmap (app, cscmdMzVertexDeselect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzVertexInvert,
    GetBitmap (app, cscmdMzVertexInvert), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzPolySelect,
    GetBitmap (app, cscmdMzPolySelect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzPolyDeselect,
    GetBitmap (app, cscmdMzPolyDeselect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzPolyInvert,
    GetBitmap (app, cscmdMzPolyInvert), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
}

static inline void InitializeModifyToolbar (csApp *app, csToolBar *toolbar)
{
  csNewToolbarButton (toolbar, cscmdMzMove,
    GetBitmap (app, cscmdMzMove), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzScale2D,
    GetBitmap (app, cscmdMzScale2D), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzScale3D,
    GetBitmap (app, cscmdMzScale3D), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzRotate,
    GetBitmap (app, cscmdMzRotate), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
/*
  csNewToolbarButton (toolbar, cscmdMzSetupAxis,
    GetBitmap (app, cscmdMzSetupAxis), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
*/
}

static inline void InitializeInfobar (csApp *app, csToolBar *toolbar)
{
  (void)app;
  int x;
  int th = toolbar->TextHeight ();
  int y1 = 3 + th * 1;
  int y2 = 3 + th * 2;
  int y3 = 3 + th * 3;
  int y4 = 3 + th * 4;

  CHK (csStatic *px = new csStatic (toolbar, NULL, "X", csscsText));
  CHK (csStatic *py = new csStatic (toolbar, NULL, "Y", csscsText));
  CHK (csStatic *pz = new csStatic (toolbar, NULL, "Z", csscsText));
  CHK (csStatic *pf = new csStatic (toolbar, NULL, "Pointer", csscsFrameLabel));
  x = 4;
  pf->SetRect (x, 2, x + 92, 2 + th * 5);
  px->SetRect (x + 4, y1, x + 16, y2);
  py->SetRect (x + 4, y2, x + 16, y3);
  pz->SetRect (x + 4, y3, x + 16, y4);

  CHK (px = new csStatic (toolbar, NULL, "", csscsText));
  CHK (py = new csStatic (toolbar, NULL, "", csscsText));
  CHK (pz = new csStatic (toolbar, NULL, "", csscsText));
  px->id = MZID_INFO_PTR_X; px->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  px->SetRect (x + 16, y1, x + 88, y2); px->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  py->id = MZID_INFO_PTR_Y; py->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  py->SetRect (x + 16, y2, x + 88, y3); py->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  pz->id = MZID_INFO_PTR_Z; pz->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  pz->SetRect (x + 16, y3, x + 88, y4); pz->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);

  CHK (csStatic *cx = new csStatic (toolbar, NULL, "Phi", csscsText));
  CHK (csStatic *cy = new csStatic (toolbar, NULL, "Theta", csscsText));
  CHK (csStatic *cz = new csStatic (toolbar, NULL, "Scale", csscsText));
  CHK (csStatic *cf = new csStatic (toolbar, NULL, "View", csscsFrameLabel));
  x = 100;
  cf->SetRect (x, 2, x + 184, 2 + th * 5);
  cx->SetRect (x + 4, y1, x + 44, y2);
  cy->SetRect (x + 4, y2, x + 44, y3);
  cz->SetRect (x + 4, y3, x + 44, y4);

  CHK (cx = new csStatic (toolbar, NULL, "", csscsText));
  CHK (cy = new csStatic (toolbar, NULL, "", csscsText));
  CHK (cz = new csStatic (toolbar, NULL, "", csscsText));
  cx->id = MZID_INFO_VIEW_PHI;   cx->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  cx->SetRect (x + 44, y1, x + 84, y2); cx->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  cy->id = MZID_INFO_VIEW_THETA; cy->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  cy->SetRect (x + 44, y2, x + 84, y3); cy->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  cz->id = MZID_INFO_VIEW_SCALE; cz->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  cz->SetRect (x + 44, y3, x + 84, y4); cz->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);

  CHK (px = new csStatic (toolbar, NULL, "X", csscsText));
  CHK (py = new csStatic (toolbar, NULL, "Y", csscsText));
  CHK (pz = new csStatic (toolbar, NULL, "Z", csscsText));
  px->SetRect (x + 88, y1, x + 108, y2);
  py->SetRect (x + 88, y2, x + 108, y3);
  pz->SetRect (x + 88, y3, x + 108, y4);

  CHK (px = new csStatic (toolbar, NULL, "", csscsText));
  CHK (py = new csStatic (toolbar, NULL, "", csscsText));
  CHK (pz = new csStatic (toolbar, NULL, "", csscsText));
  px->id = MZID_INFO_VIEW_X; px->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  px->SetRect (x + 108, y1, x + 180, y2); px->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  py->id = MZID_INFO_VIEW_Y; py->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  py->SetRect (x + 108, y2, x + 180, y3); py->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  pz->id = MZID_INFO_VIEW_Z; pz->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  pz->SetRect (x + 108, y3, x + 180, y4); pz->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);

  CHK (csStatic *sv = new csStatic (toolbar, NULL, "Vertices", csscsText));
  CHK (csStatic *sp = new csStatic (toolbar, NULL, "Polygons", csscsText));
  CHK (csStatic *sl = new csStatic (toolbar, NULL, "Lights", csscsText));
  CHK (csStatic *st = new csStatic (toolbar, NULL, "Textures", csscsText));
  CHK (csStatic *sr = new csStatic (toolbar, NULL, "Frames", csscsText));
  CHK (csStatic *sf = new csStatic (toolbar, NULL, "Statistics", csscsFrameLabel));
  x = 288;
  sf->SetRect (x, 2, x + 210, 2 + th * 5);
  sv->SetRect (x + 4, y1, x + 64, y2);
  sp->SetRect (x + 4, y2, x + 64, y3);
  sl->SetRect (x + 4, y3, x + 64, y4);
  st->SetRect (x + 106, y1, x + 166, y2);
  sr->SetRect (x + 106, y2, x + 166, y3);

  CHK (sv = new csStatic (toolbar, NULL, "", csscsText));
  CHK (sp = new csStatic (toolbar, NULL, "", csscsText));
  CHK (sl = new csStatic (toolbar, NULL, "", csscsText));
  CHK (st = new csStatic (toolbar, NULL, "", csscsText));
  CHK (sr = new csStatic (toolbar, NULL, "", csscsText));
  sv->id = MZID_INFO_VERTICES; sv->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  sv->SetRect (x + 64, y1, x + 104, y2); sv->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  sp->id = MZID_INFO_POLYGONS; sp->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  sp->SetRect (x + 64, y2, x + 104, y3); sp->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  sl->id = MZID_INFO_LIGHTS; sl->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  sl->SetRect (x + 64, y3, x + 104, y4); sl->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  st->id = MZID_INFO_TEXTURES; st->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  st->SetRect (x + 166, y1, x + 206, y2); st->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);
  sr->id = MZID_INFO_FRAMES; sr->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_D);
  sr->SetRect (x + 166, y2, x + 206, y3); sr->SetTextAlign (CSSTA_RIGHT | CSSTA_VCENTER);

  CHK (csStatic *op = new csStatic (toolbar, NULL, "", csscsText));
  op->id = MZID_INFO_CURRENTOP; op->SetColor (CSPAL_STATIC_ITEXT, cs_Color_Red_M);
  op->SetRect (4, 2 + th * 5, 498, 2 + th * 6); op->SetTextAlign (CSSTA_HCENTER | CSSTA_VCENTER);
  op->SetFont (csFontPolice);

  toolbar->SetSize (506, 2 + th * 6 + 2);
}

static inline void InitializeWindowManager (csApp *app, csToolBar *toolbar)
{
  csButton *b;
  b = csNewToolbarButton (toolbar, cscmdMzWindowLayout1,
    GetBitmap (app, cscmdMzWindowLayout1), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  b->SetState (CSS_GROUP, true);
  csNewToolbarButton (toolbar, cscmdMzWindowLayout2,
    GetBitmap (app, cscmdMzWindowLayout2), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzWindowLayout3,
    GetBitmap (app, cscmdMzWindowLayout3), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzWindowLayout4,
    GetBitmap (app, cscmdMzWindowLayout4), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  b = csNewToolbarButton (toolbar, cscmdMzWindowCloseAll,
    GetBitmap (app, cscmdMzWindowCloseAll), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE & ~CSBS_MULTICHOOSE);
  b->SetState (CSS_GROUP, true);
  csNewToolbarButton (toolbar, cscmdMzWindowLayoutReset,
    GetBitmap (app, cscmdMzWindowLayoutReset), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE & ~CSBS_MULTICHOOSE);
  b = csNewToolbarButton (toolbar, cscmdMzCoordRotate,
    GetBitmap (app, cscmdMzCoordRotate), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  b->SetState (CSS_GROUP, true);
  csNewToolbarButton (toolbar, cscmdMzSlideView,
    GetBitmap (app, cscmdMzSlideView), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzZoomInOut,
    GetBitmap (app, cscmdMzZoomInOut), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  csNewToolbarButton (toolbar, cscmdMzZoomToRect,
    GetBitmap (app, cscmdMzZoomToRect), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE);
  b = csNewToolbarButton (toolbar, cscmdMzToggleGrid,
    GetBitmap (app, cscmdMzToggleGrid), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE & ~CSBS_MULTICHOOSE);
  b->SetState (CSS_GROUP, true);
  b->SetPressed (F_DRAFT_GRID);
  b = csNewToolbarButton (toolbar, cscmdMzToggleAxis,
    GetBitmap (app, cscmdMzToggleAxis), NULL, csbfsThinRect, TOOLBAR_BUTTON_STYLE & ~CSBS_MULTICHOOSE);
  b->SetState (CSS_GROUP, true);
  b->SetPressed (F_DRAFT_AXIS);
}

#endif // __ME_RES_H__
