/*
  Maze editor: world loading
  Copyright (C) 1998 by Jorrit Tyberghein
  Written by Andrew Zabolotny <bit@eltech.ru>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <setjmp.h>
#include "me_app.h"
#include "me_text.h"
#include "engine/objects/thingtpl.h"
#include "engine/objects/cssprite.h"

static jmp_buf fatal_jmp;

void my_fatal_exit (int errorcode, bool canreturn)
{
  if (!canreturn)
    longjmp (fatal_jmp, errorcode + 1);
}

bool MazeEditor::LoadWorld (char *iFileName)
{
  csWorld world;
  LanguageLayer layer (&world, NULL);
  Invalidate (true);
  CsPrintf (MSG_INITIALIZATION, "-------------------\n");
  CsPrintf (MSG_INITIALIZATION, TEXT_LOADINGWORLDFILE, iFileName);

  void (*old_fatal_exit) (int, bool) = fatal_exit;
  fatal_exit = my_fatal_exit;
  if (setjmp (fatal_jmp))
  {
error:
    TextureMM::fIgnoreLoadingErrors = false;
    fatal_exit = old_fatal_exit;
    // Wait three seconds before continuing
    long t = System->Time () + 3000;
    while (System->Time () < t) ;
    return false;
  }
  TextureMM::fIgnoreLoadingErrors = true;
  bool retc = world.load_file (&layer, iFileName);
  if (!retc)
    goto error;
  fatal_exit = old_fatal_exit;
  TextureMM::fIgnoreLoadingErrors = false;

  // Get all textures
  if (world.get_textures ())
  {
    CsPrintf (MSG_INITIALIZATION, TEXT_IMPORTINGTEXTURES);
    for (int i = 0; i < world.get_textures ()->get_num_textures (); i++)
    {
      textures->add_texture (world.get_textures ()->get_texture_mm (0));
      world.get_textures ()->delete_texture (0, false);
    } /* endfor */
  } /* endif */


  // now load all rooms
  CsPrintf (MSG_INITIALIZATION, TEXT_IMPORTINGROOMS);
  int count = world.sectors.Length ();
  int no;
  for (no = 0 ; no < count ; no++)
  {
    bool done = false;
    for (int i = Models.Length () - 1; i >= 0; i--)
      if (((mz3DModel *)Models [i])->Join ((csSector *)world.sectors [no]))
      {
	done = true;
	break;
      }
    if (!done)
    {
      CHK (mz3DModel *m = new mz3DModel (csmtRoom, ModelsPath));
      CHK (Models.Push (m));
      m->Load ((csSector *)world.sectors [no]);
    }
  } /* endfor */

  // now transform world into mz3DModel's
  CsPrintf (MSG_INITIALIZATION, TEXT_IMPORTINGTHINGS);
  count = world.thing_templates.Length ();
  for (no = 0; no < count; no++)
  {
    CHK (mz3DModel *m = new mz3DModel (csmtThing, ModelsPath));
    CHK (Models.Push (m));
    m->Load ((ThingTemplate *)world.thing_templates [no]);
  } /* endfor */

  CsPrintf (MSG_INITIALIZATION, TEXT_IMPORTINGSPRITES);
  count = world.sprite_templates.Length ();
  for (no = 0; no < count; no++)
  {
    csSpriteTemplate *sprite = (csSpriteTemplate *)world.sprite_templates [no];
    CHK (mz3DModel *m = new mz3DModel (csmtSprite, ModelsPath));
    CHK (Models.Push (m));
    m->Load (sprite);
    for (int i = 0; i < sprite->get_num_actions (); i++)
    {
      CHK (mzSpriteAction *a = new mzSpriteAction (sprite->get_action (i)));
      m->InsertAction (a);
    } /* endfor */
  } /* endfor */

  SelectModel (0);
  GLOBAL_FITALL;
  return true;
}
