/*
    The simplest possible application using CrystalSpace
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
    This application is not intended to be an extensive example of CrystalSpace
    features. It is just a basical skeleton of a simple program that uses
    CrystalSpace library and implements just the minimal required functionality.
*/

#include "general/cs.h"

// 3D view object
cs3Dview *cs;

class SimpleApp : public SysSystemDriver
{
public:
  SimpleApp () : SysSystemDriver () {}

  virtual void NextFrame (long elapsed_time, long current_time);

  void DrawFrame ();
};

void SimpleApp::NextFrame (long elapsed_time, long current_time)
{
  SysSystemDriver::NextFrame (elapsed_time, current_time);

  float walkstep = .2;
  float turnstep = .1;

  if (Keyboard->Key.shift)
  {
    walkstep *= 2;
    turnstep *= 2;
  }
  if (Keyboard->Key.ctrl)
  {
    walkstep /= 2;
    turnstep /= 2;
  }

  if (Keyboard->Key.up)
    cs->camera->move (walkstep*VEC_FORWARD);
  if (Keyboard->Key.down)
    cs->camera->move (walkstep*VEC_BACKWARD);
  if (Keyboard->Key.left)
    if (Keyboard->Key.alt)
      cs->camera->move (walkstep*VEC_LEFT);
    else
      cs->camera->rotate (VEC_ROT_LEFT, turnstep);
  if (Keyboard->Key.right)
    if (Keyboard->Key.alt)
      cs->camera->move (walkstep*VEC_RIGHT);
    else
      cs->camera->rotate (VEC_ROT_RIGHT, turnstep);
  if (Keyboard->Key.pgup)
    if (Keyboard->Key.alt)
      cs->camera->rotate (VEC_TILT_LEFT, turnstep);
    else
      cs->camera->rotate (VEC_TILT_UP, turnstep);
  if (Keyboard->Key.pgdn)
    if (Keyboard->Key.alt)
      cs->camera->rotate (VEC_TILT_RIGHT, turnstep);
    else
      cs->camera->rotate (VEC_TILT_DOWN, turnstep);
  if (Keyboard->Key.esc)
    Shutdown = true;
  DrawFrame ();
}

void SimpleApp::DrawFrame ()
{
  if (!Gfx3D->BeginDraw (CSDRAW_3DGRAPHICS))
    return;

  cs->Draw ();

  Gfx3D->FinishDraw ();
  // Blit the image to screen
  Gfx3D->Print ();
}

int main (int argc, char *argv[])
{
  // Open configuration file
  config = new csIniFile ("cryst.cfg");

  // Application object
  SimpleApp app;

  // print some fancy info
  CsPrintf (MSG_INITIALIZATION, "Crystal Space version %s (%s).\n", VERSION, LAST_MOD_DATE);
  CsPrintf (MSG_INITIALIZATION, "Created by Jorrit Tyberghein and others...\n\n");

  // Initialize application
  app.Initialize (argc, argv);

  // Now create world object
  csWorld *world = new csWorld ();

  // Load world file
  char *worldfile = "../data/rgb.zip";
  if ((argc > 1) && (argv[1][0] != '-'))
    worldfile = argv[1];
  if (!world->load_file (NULL, worldfile))
    exit (-1);

  // Initialize 3D renderer wrapper object
  cs = new cs3Dview (world);

  // Initialize textures
  world->get_textures ()->initialize (world);
  // Initialize world
  world->initialize ();
  // Set starting sector
  cs->SetSector (world->get_sector ("room"));
  // Set viewing rectangle
  cs->SetRectangle (2, 2, FRAME_WIDTH - 4, FRAME_HEIGHT - 4);

  // Open the graphics context
  app.Open("Crystal Space simple example");

  world->get_textures ()->alloc_palette ();

  // Main loop
  app.Loop();
}

void debug_dump () { }
void cleanup () { }

