/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "util/def.h"
#include "general/console.h"
#include "general/walktest/main.h"
#include "general/walktest/infmaze.h"
#include "general/walktest/scon.h"
#include "geometry/csrect.h"
#include "engine/objects/thing.h"
#include "engine/wirefrm.h"
#include "general/loader/csloader.h"
#include "general/dumper.h"
#include "util/sparse3d.h"
#include "util/memheap.h"
#include "engine/library.h"
#include "render/igraph3d.h"
#include "general/csview.h"
#include "engine/stats.h"
#include "engine/light/light.h"
#include "engine/light/dynlight.h"
#include "scripts/csscript.h"
#include "scripts/intscri.h"
#include "engine/texture.h"
#include "util/inifile.h"
#include "general/command.h"
#include "engine/polygon/polytext.h"
#include "sound/cssnddrv.h"
#include "util/sound/waveload.h"
#include "util/sound/moduload.h"
#include "colldet/collider.h"

#if defined(OS_DOS) || defined(OS_WIN32) || defined (OS_OS2)
#  include <io.h>
#elif defined(OS_UNIX)
#  include <unistd.h>
#endif

WalkTest *Sys;
LanguageLayer* layer = NULL;
cs3Dview *cs;
InfiniteMaze* infinite_maze = NULL;

Wave* wMissile_boom = NULL;
Wave* wMissile_whoosh = NULL;

bool do_fps = true;
bool do_stats = false;
bool do_physics = false;
bool do_clear = false;
bool do_show_coord = false;
bool busy_perf_test = false;
bool do_show_z = false;
bool do_infinite = false;
bool do_cd = true;

float timeFPS=0.0;

//-----------------------------------------------------------------------------

void DrawZbuffer ()
{
  for (int y = 0; y < FRAME_HEIGHT; y++)
  {
    int gi_pixelbytes;
    GfxInfo->GetPixelBytes(gi_pixelbytes);
  
    if (gi_pixelbytes > 1)
    {
      UShort *dest;
      Gfx2D->GetPixelAt(0, y, (unsigned char**)&dest);
      
      ULong *zbuf;
      Gfx3D->GetZBufPoint (0, y, &zbuf);
      
      for (int x = 0; x < FRAME_WIDTH; x++)
        *dest++ = (unsigned short)(*zbuf++ >> 13);
    } 
    else
    {
      unsigned char *dest;
      Gfx2D->GetPixelAt(0, y, &dest);
      
      ULong *zbuf;
      Gfx3D->GetZBufPoint(0, y, &zbuf);
      
      for (int x = 0; x < FRAME_WIDTH; x++)
        *dest++ = (unsigned char)(*zbuf++ >> 16);
    }
  }
}

int collcount = 0;

void WalkTest::DrawFrame (long elapsed_time, long current_time)
{

  //not used since we need WHITE background not black
  int drawflags = 0; /* do_clear ? CSDRAW_CLEARSCREEN : 0; */
  if (do_clear || System->world->map_mode == MAP_ON)
  {
    if (Gfx3D->BeginDraw (CSDRAW_2DGRAPHICS) != S_OK)
      return;
    Gfx2D->Clear (System->world->map_mode == MAP_ON ? 0 : 255);
  }

  if (!System->Console->IsActive ()
   || ((SimpleConsole*)(System->Console))->IsTransparent ())
  {
    // Tell Gfx3D we're going to display 3D things
    if (Gfx3D->BeginDraw (drawflags | CSDRAW_3DGRAPHICS) != S_OK)
      return;
    if (System->world->map_mode != MAP_ON) cs->Draw ();
    // no need to clear screen anymore
    drawflags = 0;

    System->world->advance_sprite_frames (current_time);

    csDynLight* dyn = System->world->get_first_dyn_light ();
    while (dyn)
    {
      extern void HandleDynLight (csDynLight*);
      csDynLight* dn = dyn->get_next ();
      if (dyn->userdata) HandleDynLight (dyn);
      dyn = dn;
    }

#if 0 // why this ??? - A.Z.
    long g3d_interlacing;
    Gfx3D->GetRenderState(G3DRENDERSTATE_INTERLACINGENABLE, g3d_interlacing);

    if (g3d_interlacing != -1)
    {
      Gfx3D->SetRenderState(G3DRENDERSTATE_INTERLACINGENABLE, !g3d_interlacing);
      // DAN:
      //      csGraphics3D::ilace_fastmove = false;
    }
#endif
  } 

  // Start drawing 2D graphics
  if (Gfx3D->BeginDraw (drawflags | CSDRAW_2DGRAPHICS) != S_OK)
    return;
  if (do_show_z) DrawZbuffer ();
  if (System->world->map_mode)
  {
    System->world->wf->get_wireframe ()->draw (Gfx3D, System->world->wf->get_camera ());
  }
  SimpleConsole* scon = (SimpleConsole*)System->Console;
  scon->Print ();

  if (!System->Console->IsActive ())
  {
    if (do_fps)
    {
      GfxWrite(11, FRAME_HEIGHT-11, 0, -1, "FPS=%f", timeFPS);
      GfxWrite(10, FRAME_HEIGHT-10, scon->get_fg (), -1, "FPS=%f", timeFPS);
    } /* endif */
    if (do_stats)
    {
      char buffer[30];
      sprintf (buffer, "pc=%d pd=%d po=%d", Stats::polygons_considered,
        Stats::polygons_drawn, Stats::portals_drawn);
      GfxWrite(FRAME_WIDTH-24*8-1, FRAME_HEIGHT-11, 0, -1, "%s", buffer);
      GfxWrite(FRAME_WIDTH-24*8, FRAME_HEIGHT-10, scon->get_fg (), -1, "%s", buffer);
    }
    else if (do_show_coord)
    {
      char buffer[100];
      sprintf (buffer, "%2.2f,%2.2f,%2.2f: %s",
        cs->camera->v_world2cam().x, cs->camera->v_world2cam().y,
        cs->camera->v_world2cam().z, cs->camera->get_sector ()->get_name ());
      Gfx2D->Write(FRAME_WIDTH-24*8-1, FRAME_HEIGHT-11, 0, -1, buffer);
      Gfx2D->Write(FRAME_WIDTH-24*8, FRAME_HEIGHT-10, scon->get_fg (), -1, buffer);
    }
    else if (do_cd)
    {
      char buffer[200], names[30];
      sprintf (buffer,"CD %d cam pos %2.2f,%2.2f,%2.2f %c %c",
         collcount, 
         cs->camera->v_world2cam().x, cs->camera->v_world2cam().y,
         cs->camera->v_world2cam().z,
         Collider::falling?'F':' ', Collider::climbing?'C':' ');
      for (int i = 0; i < collcount; i++)
      {
        Collider *id1, *id2;
        if (Collider::report(&id1, &id2))
	  {
          sprintf(names,"[%s,%s] ",id1->get_name(),id2->get_name());
	  }
        strcat(buffer,names);
      }
      Gfx2D->Write(8-1, FRAME_HEIGHT-21, 0, -1, buffer);
      Gfx2D->Write(8, FRAME_HEIGHT-20, scon->get_fg (), -1, buffer);
    } /* endif */
  } /* endif */

  // Drawing code ends here
  Gfx3D->FinishDraw ();
  // Print the output.
  Gfx3D->Print (NULL);
}

int cnt = 1;
long time0 = -1;

void WalkTest::PrepareFrame (long elapsed_time, long current_time)
{
  CLights::LightIdle (); // SJI
  if (do_cd && Collider::init)
  {
    // TODO ALEX: In future this should depend on whether the whole world
    // or 'active' sectors need to be set up as monsters hunt for player
    // outside of current sector, but this will do for now.
    Collider::set_camera (cs->camera);	// TODO.
    // Test camera collision.
    collcount = Collider::collision_detect (cs->camera->get_sector (), cs->camera);    
  }
  if (do_physics)
   /* cs->camera->get_physics ()->spendTime () */ ;

  if (cnt <= 0)
  {
    long time1 = SystemGetTime ();
    if (time0 != -1)
    {
      if (time1 != time0)
        timeFPS=10000.0f/(float)(time1-time0);
    }
    cnt = 10;
    time0 = SystemGetTime ();
  }
  cnt--;

  layer->step_run ();
}

void perf_test ()
{
  busy_perf_test = true;
  long t1, t2, t;
  CsPrintf (MSG_CONSOLE, "Performance test busy...\n");
  t = t1 = SystemGetTime ();
  int i;
  for (i = 0 ; i < 100 ; i++)
  {
    layer->step_run ();
    Sys->DrawFrame (SystemGetTime ()-t, SystemGetTime ());
    t = SystemGetTime ();
  }
  t2 = SystemGetTime ();
  CsPrintf (MSG_CONSOLE, "%f secs to render 100 frames: %f fps\n",
        (float)(t2-t1)/1000., 100000./(float)(t2-t1));
  CsPrintf (MSG_DEBUG_0, "%f secs to render 100 frames: %f fps\n",
        (float)(t2-t1)/1000., 100000./(float)(t2-t1));
  cnt = 1;
  time0 = -1;
  busy_perf_test = false;
}

void CaptureScreen (void)
{
#if !defined(OS_MACOS)  // SJI - NON mac stuff - the mac has its own way of doing screen captures
  int i = 0;
  char name[255];
  UByte pall[768];
  UByte *pal = &pall[0];

  do
  {
    sprintf (name, "cryst%02d.pcx", i++);
  } while (i < 100 && (access (name, 0) == 0));

  if (i >= 100) return;

  RGBpaletteEntry* pPalette;
  GfxInfo->GetPalette( &pPalette );

  if (pPalette)
  {
	  for (i=0; i<256; i++)
	  {
		  *pal++ = pPalette[i].red;
		  *pal++ = pPalette[i].green;
		  *pal++ = pPalette[i].blue;
	  }
  }

  extern void WritePCX (char *name, unsigned char *data, UByte *pal,
			int width,int height);
  Gfx3D->BeginDraw(CSDRAW_2DGRAPHICS);
  
  unsigned char* pFirstPixel;
  Gfx2D->GetPixelAt(0,0, &pFirstPixel);
  
  WritePCX (name, pFirstPixel, pall, FRAME_WIDTH, FRAME_HEIGHT);
  Gfx3D->FinishDraw();
  CsPrintf (MSG_CONSOLE, "Screenshot: %s", name);

#endif // !OS_MACOS
}

/*---------------------------------------------
 * Our main event loop.
 *---------------------------------------------*/

/*
 * Do a large debug dump just before the program
 * exits. This function can be installed as a last signal
 * handler if the program crashes (for systems that support
 * this).
 */
void debug_dump ()
{
  cs->camera->save_file ("coord.bug");
  CsPrintf (MSG_DEBUG_0, "Camera saved in coord.bug\n");
  Dumper::dump (cs->camera);
  CsPrintf (MSG_DEBUG_0, "Camera dumped in debug.txt\n");
  Dumper::dump (System->world);
  CsPrintf (MSG_DEBUG_0, "World dumped in debug.txt\n");
}

/*
 * A sample script which just prints a message on screen.
 */
bool do_message_script (IntRunScript* sc, char* data)
{
  sc->get_layer ()->message (data);
  return true;
}

//---------------------------------------------------------------------------


void cleanup ()
{
  pprintf ("Cleaning up...\n");
  Collider::end_world();
  if (infinite_maze) CHKB (delete infinite_maze);
  if (cs) CHKB (delete cs);
  if (layer) CHKB (delete layer);
  if (Sys)
  {
    Sys->Close();
    CHK (delete Sys);
  }
  pprintf_close();              // DAN: this closes the pprintf routine
}

/*---------------------------------------------------------------------*
 * Demo stuff
 *---------------------------------------------------------------------*/

struct DemoInfo
{
  Textures* tex;
  csWorld* world;
};

DemoInfo* demo_info = NULL;

/*
 * The routine to update the screen in demo mode.
 */
void WalkTest::DemoWrite (char* buf)
{
  if (Console)
  {
    if (Gfx2D)
    {
      VERIFY_SUCCESS( Gfx2D->BeginDraw () );
      Gfx2D->Clear (0);
      Console->PutText (buf);
      Console->Print ();
      Gfx2D->FinishDraw ();
      
      csRect rect;
      Gfx2D->Print (NULL);
    }
  }
}

/*
 * Start the demo in the already open screen.
 */
void start_demo ()
{
  CHK (demo_info = new DemoInfo);
  demo_info->tex = NULL;
  demo_info->world = NULL;

  CHK (demo_info->world = new csWorld ());
  CHK (demo_info->tex = new Textures (4));

  demo_info->tex->initialize_limited ();

  ((SimpleConsole *)System->Console)->SetupColors (demo_info->tex);
  ((SimpleConsole *)System->Console)->SetMaxLines (1000);       // Some arbitrary high value.
  ((SimpleConsole *)System->Console)->SetTransparent (1);
  demo_info->tex->alloc_palette ();

  if (Gfx2D->BeginDraw() == S_OK)
  {
    Gfx2D->Print (NULL);
    Gfx2D->FinishDraw ();
  }
  System->DemoReady = true;
}

/*
 * Stop the demo.
 */
void stop_demo ()
{
  if (demo_info)
  {
    if (demo_info->world) CHKB (delete demo_info->world);
    if (demo_info->tex) CHKB (delete demo_info->tex);
    CHK (delete demo_info);
  }
}


/*---------------------------------------------------------------------*
 * Main function
 *---------------------------------------------------------------------*/
int main (int argc, char* argv[])
{
#if 0
  // This construction enables short-precision mode in the pentium
  // processor. I read somewhere that this should be faster. As
  // this doesn't seem to give a speed increase in Linux I don't enable it
  // yet. Maybe later when I test it in DOS?
  int OldFPUCW, FPUCW;
  asm volatile ("\
        fstcw (%0)              \n\
        movl (%0),%%eax         \n\
        andl $0xfcff,%%eax      \n\
        movl %%eax,(%1)         \n\
        fldcw (%1)              "
        :
        : "g" (&OldFPUCW), "g" (&FPUCW)
        : "eax");
#endif

  tables.initialize ();

  CHK (config = new csIniFile ("cryst.cfg"));
  CHK (Sys = new WalkTest ());

  int_script_reg.reg ("message", &do_message_script);

  csCamera::aspect = 0;
  //@@@ TODO: TextureCache::cache_size = config->GetInt ("TextureMapper", "CACHE", DEFAULT_CACHE_SIZE);
  do_fps = config->GetYesNo ("WalkTest", "FPS", true);
  do_stats = config->GetYesNo ("WalkTest", "STATS", false);
  do_physics = config->GetYesNo ("WalkTest", "PHYSICS", false);
  do_cd = config->GetYesNo ("WalkTest", "COLLDET", true);
  strcpy (WalkTest::world_file, config->GetStr ("World", "WORLDFILE", "world"));

  Polygon3D::zdist_mipmap1 = config->GetFloat ("TextureMapper", "DMIPMAP1", ZDIST_MIPMAP1);
  Polygon3D::zdist_mipmap2 = config->GetFloat ("TextureMapper", "DMIPMAP2", ZDIST_MIPMAP2);
  Polygon3D::zdist_mipmap3 = config->GetFloat ("TextureMapper", "DMIPMAP3", ZDIST_MIPMAP3);
  csSector::cfg_reflections = config->GetInt ("Lighting", "REFLECT", csSector::cfg_reflections);
  csSector::do_radiosity = config->GetYesNo ("Lighting", "RADIOSITY", csSector::do_radiosity);
  PolyTexture::do_accurate_things = config->GetYesNo ("Lighting", "ACCURATE_THINGS", PolyTexture::do_accurate_things);
  PolyTexture::cfg_cosinus_factor = config->GetFloat ("Lighting", "COSINUS_FACTOR", PolyTexture::cfg_cosinus_factor);

  Textures::Gamma = config->GetFloat ("TextureMapper", "GAMMA", 1.0);

  hilight = NULL;

  Sys->Initialize (argc, argv);

  if (!Sys->Open ("Crystal Space"))
  {
    CsPrintf (MSG_FATAL_ERROR, "Error opening system!\n");
    cleanup ();
    fatal_exit (0, false);
  }

  bool gfx3d_do_debug = config->GetYesNo ("WalkTest", "DEBUG", false);
  Gfx3D->SetRenderState( G3DRENDERSTATE_DEBUGENABLE, (long)gfx3d_do_debug );

  if (config->GetYesNo ("VideoDriver", "INTERLACING", false))
    Gfx3D->SetRenderState( G3DRENDERSTATE_INTERLACINGENABLE, 0L );
  else
    Gfx3D->SetRenderState( G3DRENDERSTATE_INTERLACINGENABLE, -1L ); 

  // Create console object
  CHK (System->Console = new SimpleConsole ());
  start_demo ();

  CsPrintf (MSG_INITIALIZATION, "Crystal Space version %s (%s).\n", VERSION, LAST_MOD_DATE);
  CsPrintf (MSG_INITIALIZATION, "Created by Jorrit Tyberghein and others...\n\n");

  CHK (csWorld* world = new csWorld ());
  CHK (cs = new cs3Dview (world));
  Command::Initialize (world, cs->camera);
  Gfx3D->SetCamera( GetICameraFromCamera(cs->camera) );

  CHK (layer = new LanguageLayer (world, cs->camera));
  infinite_maze = NULL;
  Textures* tex;
  csSector* room;

  if (do_infinite)
  {
    CsPrintf (MSG_INITIALIZATION, "Creating initial room!...\n");
    Polygon3D::do_force_recalc = true;
    world->start_world ();
    srand (time (NULL));

    CHK (tex = new Textures (10));
    world->set_textures (tex);
    CHK (Library* lib = new Library ("standard"));
    world->libraries.Push (lib);
    CSLoader::load_library_def (lib, world, "standard.zip", tex);

    ImageFile* image = CSLoader::load_image ("stone4.gif", world, lib->get_archive ());
    TextureMM* tm = tex->new_texture ("txt", image);
    tm->alloc_mipmaps ();
    image = CSLoader::load_image ("mystone2.gif", world, lib->get_archive ());
    tm = tex->new_texture ("txt2", image);
    tm->alloc_mipmaps ();

    tex->initialize (world);

    CHK (infinite_maze = new InfiniteMaze ());
    room = infinite_maze->create_six_room (world, 0, 0, 0)->sector;
    infinite_maze->create_six_room (world, 0, 0, 1);
    infinite_maze->create_six_room (world, 0, 0, 2);
    infinite_maze->create_six_room (world, 1, 0, 2);
    infinite_maze->create_six_room (world, 0, 1, 2);
    infinite_maze->create_six_room (world, 1, 1, 2);
    infinite_maze->create_six_room (world, 0, 0, 3);
    infinite_maze->create_six_room (world, 0, 0, 4);
    infinite_maze->create_six_room (world, -1, 0, 4);
    infinite_maze->create_six_room (world, -2, 0, 4);
    infinite_maze->create_six_room (world, 0, -1, 3);
    infinite_maze->create_six_room (world, 0, -2, 3);
    infinite_maze->create_six_room (world, 0, 1, 3);
    infinite_maze->create_six_room (world, 0, 2, 3);
    infinite_maze->connect_infinite (0, 0, 0, 0, 0, 1);
    infinite_maze->connect_infinite (0, 0, 1, 0, 0, 2);
    infinite_maze->connect_infinite (0, 0, 2, 0, 0, 3);
    infinite_maze->connect_infinite (0, 0, 2, 1, 0, 2);
    infinite_maze->connect_infinite (0, 0, 2, 0, 1, 2);
    infinite_maze->connect_infinite (1, 1, 2, 0, 1, 2);
    infinite_maze->connect_infinite (1, 1, 2, 1, 0, 2);
    infinite_maze->connect_infinite (0, 0, 3, 0, 0, 4);
    infinite_maze->connect_infinite (-1, 0, 4, 0, 0, 4);
    infinite_maze->connect_infinite (-2, 0, 4, -1, 0, 4);
    infinite_maze->connect_infinite (0, 0, 3, 0, -1, 3);
    infinite_maze->connect_infinite (0, -1, 3, 0, -2, 3);
    infinite_maze->connect_infinite (0, 0, 3, 0, 1, 3);
    infinite_maze->connect_infinite (0, 1, 3, 0, 2, 3);
    infinite_maze->create_loose_portal (-2, 0, 4, -2, 1, 4);

    world->initialize ();
  }
  else
  {
    CsPrintf (MSG_INITIALIZATION, "Loading world '%s'...\n", WalkTest::world_file);
    if (!world->load_file (layer, WalkTest::world_file))
    {
      CsPrintf (MSG_FATAL_ERROR, "Loading of world failed!\n");
      cleanup ();
      fatal_exit (0, false);
    }
    tex = world->get_textures ();
    tex->initialize (world);
    world->initialize ();
    char* strt = world->start_sector ? world->start_sector : "room";
    room = (csSector*)world->sectors.FindName (strt);
    if (!room)
    {
      CsPrintf (MSG_FATAL_ERROR,
          "World file does not contain a room called '%s' which is used\nas a starting point!\n",
	  strt);
      cleanup ();
      fatal_exit (0, false);
    }
  }

  if (do_cd)
  {
    Collider::init = true;
    Collider::init_world(System->world);
  }

  CsPrintf (MSG_INITIALIZATION, "--------------------------------------\n");

  // Wait one second before starting.
  long t = Sys->Time ()+1000;
  while (Sys->Time () < t) ;

  // Reinit console object
  ((SimpleConsole *)System->Console)->SetupColors (tex);
  ((SimpleConsole *)System->Console)->SetMaxLines ();
  ((SimpleConsole *)System->Console)->SetTransparent ();
  System->Console->Clear ();

  cs->SetSector (room);
  cs->camera->set_position (world->start_vec);
#if 1
  cs->SetRectangle (2, 2, FRAME_WIDTH - 4, FRAME_HEIGHT - 4);
#else
  cs->AddViewVertex (2, 60);
  cs->AddViewVertex (2, FRAME_HEIGHT - 60);
  cs->AddViewVertex (40, FRAME_HEIGHT - 2);
  cs->AddViewVertex (FRAME_WIDTH - 40, FRAME_HEIGHT - 2);
  cs->AddViewVertex (FRAME_WIDTH - 2, FRAME_HEIGHT - 60);
  cs->AddViewVertex (FRAME_WIDTH - 2, 60);
  cs->AddViewVertex (FRAME_WIDTH - 40, 2);
  cs->AddViewVertex (40, 2);
#endif

  stop_demo ();

  tex->alloc_palette ();
  CHK (world->wf = new WireFrameCam (tex));

//  Command::perform ("fclear", "on");
//  Command::perform ("coordload", NULL);
//  Command::perform ("portals", "off");
//  Command::perform ("lighting", "off");
//  Command::perform ("transp", "off");
//  Command::perform ("texture", "off");

#ifdef DO_SOUND
  Wave* w = world->get_wave ("tada.wav");
  if (w)
  {
    w->convertTo (Sys->Sound->getFrequency (), Sys->Sound->is16Bits (), false);
    Sys->Sound->playEphemeral (w);
  }
  else
    CsPrintf (MSG_INITIALIZATION, "Cannot Load tada.wav!\n");

  wMissile_boom = world->get_wave ("boom.wav");
  if (wMissile_boom)
    wMissile_boom->convertTo (Sys->Sound->getFrequency (), Sys->Sound->is16Bits (), false);
  else
    CsPrintf (MSG_INITIALIZATION, "Cannot Load boom.wav!\n");

  wMissile_whoosh = world->get_wave ("whoosh.wav");
  if (wMissile_whoosh)
    wMissile_whoosh->convertTo (Sys->Sound->getFrequency (), Sys->Sound->is16Bits (), false);
  else
    CsPrintf (MSG_INITIALIZATION, "Cannot Load whoosh.wav!\n");

  Module *m=ModuleLoader::load ("blues.mod");
  if( m)
  {
    Channel_ID id=Sys->Sound->add (m);
    //Sys->Sound->startChannel(id); // <- it's broken at this time, so wait a little...
  }
  else
    CsPrintf (MSG_INITIALIZATION, "Cannot load blues.mod!\n");
#endif

  Command::start_script ("autoexec.cfg");
  char cmd_buf[512];
  while (Command::get_script_line (cmd_buf, 511))
    Command::perform_line (cmd_buf);

  Sys->Loop ();

  cleanup ();

  return (1);
}
