/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef MAIN_H
#define MAIN_H

#include <stdarg.h>
#include "geometry/math2d.h"
#include "system/sysdriv.h"

class Polygon3D;
class cs3Dview;
class WalkTest;

///
struct csKeyMap
{
  csKeyMap* next, * prev;
  int key, shift, alt, ctrl;
  char* cmd;
};

///
class WalkTest : public SysSystemDriver
{
public:
  /// The world file to load
  static char world_file[100];

public:
  ///
  WalkTest ();

  ///
  virtual void NextFrame (long elapsed_time, long current_time);
  ///
  void PrepareFrame (long elapsed_time, long current_time);
  ///
  void DrawFrame (long elapsed_time, long current_time);

  /// Override ParseArg to handle additional arguments
  virtual void ParseArg (int argc, char* argv[], int& i);
  /// Override Help to show additional arguments help
  virtual void Help ();
  /// Override DemoWrite for nice demo messaging
  virtual void DemoWrite (char* msg);

private:
  ///
  static void handle_key_forward (float speed, bool shift, bool alt, bool ctrl);
  ///
  static void handle_key_backwards (float speed, bool shift, bool alt, bool ctrl);
  ///
  static void handle_key_left (float speed, bool shift, bool alt, bool ctrl);
  ///
  static void handle_key_right (float speed, bool shift, bool alt, bool ctrl);
  ///
  static void handle_key_pgup (float, bool shift, bool alt, bool ctrl);
  ///
  static void handle_key_pgdn (float, bool shift, bool alt, bool ctrl);
  ///
  static void eatkeypress (int key, bool shift, bool alt, bool ctrl);
};

extern bool do_physics;
extern bool do_clear;
extern bool do_fps;
extern bool do_stats;
extern bool do_show_coord;
extern bool do_coord_check;
extern bool do_cd;
extern bool busy_perf_test;
extern bool do_show_z;
extern Vector2 coord_check_vector;
extern Polygon3D* hilight;
extern cs3Dview *cs;
extern WalkTest* Sys;

extern void perf_test ();
extern void CaptureScreen ();

#endif // MAIN_H
