/*
    Crystal Space polygon clipping routines
    Copyright (C) 1998 by Jorrit Tyberghein
    Contributed by Ivan Avramovic <iavramov@ibm.net> and
                   Andrew Zabolotny <bit@eltech.ru>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "geometry/polyclip.h"
#include "util/util.h"
#include "util/def.h"
#include "util/memory.h"
#include "system/sysprint.h"

//---------------------------------------------------------------------------

Vector2* csBoxClipper::Clip (Vector2 *Polygon, int Count, int &OutCount)
{
  CHK (Vector2* v = new Vector2[Count+5]);
   memcpy (v, Polygon, Count * sizeof (Vector2));
  OutCount = Count;
  Box bbox = Box(-100000., -100000., 100000., 100000.);
  if (Clip(v, OutCount, Count+5, &bbox)) return v;
   else
   {
    CHK (delete[] v);
    OutCount = 0;
    return NULL;
   }
}

bool csBoxClipper::Clip (Vector2 *Polygon, int& Count, int MaxCount, 
                       Box *BoundingBox)
{
  Vector2 OutPoly[100];
  Vector2 *InPolygon = Polygon;
  int InVertices = Count;
  Vector2 *OutPolygon = OutPoly;
  int OutVertices = Count;
  int vert; 

  if (!region.overlap (*BoundingBox)) return false;

  
  if (BoundingBox->minx < region.minx) /* clip against left x line */
  {
    float px = InPolygon[0].x, py = InPolygon[0].y;
    bool pVertIn = (px > region.minx);
    OutVertices = 0;
    int isectCount = 0;
    for (vert = 1; vert <= InVertices; vert++)
    {
     float cx, cy;
     if (vert < InVertices) 
     { cx = InPolygon[vert].x;  cy = InPolygon[vert].y; }
     else { cx = InPolygon[0].x;  cy = InPolygon[0].y; }
     bool cVertIn = (cx > region.minx);
     
     if (pVertIn)
     {
      OutPolygon[OutVertices].x = px;  OutPolygon[OutVertices].y = py;
      OutVertices++;
      if (OutVertices == MaxCount) break;
     }   
     if (pVertIn != cVertIn)
     {
      OutPolygon[OutVertices].x = region.minx;
      OutPolygon[OutVertices].y = cy - (cx-region.minx)*(cy-py)/(cx-px);
      OutVertices++;  
      if (OutVertices == MaxCount) break;
      isectCount++;
      if (isectCount >= 2)
      {
       if (cVertIn)
       {
        memcpy (&OutPolygon[OutVertices], &InPolygon[vert],
                (InVertices - vert) * sizeof(OutPolygon[0]));
        OutVertices += InVertices - vert;
       }
       break;
      }
     }
     px = cx; py = cy;  pVertIn = cVertIn; 
    } /* for (vert = 1; vert <= InVertices; vert++) */

    if (OutVertices < 3) { Count = 0; return false; }
    InVertices = OutVertices;
    InPolygon = OutPolygon;
    if (OutPolygon == OutPoly) OutPolygon = Polygon;
     else OutPolygon = OutPoly;
  }
  if (BoundingBox->miny < region.miny) /* clip against upper y line */
  {
    float px = InPolygon[0].x, py = InPolygon[0].y;
    bool pVertIn = (py > region.miny);
    OutVertices = 0;
    int isectCount = 0;
    for (vert = 1; vert <= InVertices; vert++)
    {
     float cx, cy;
     if (vert < InVertices) 
     { cx = InPolygon[vert].x;  cy = InPolygon[vert].y; }
     else { cx = InPolygon[0].x;  cy = InPolygon[0].y; }
     bool cVertIn = (cy > region.miny);
     
     if (pVertIn)
     {
      OutPolygon[OutVertices].x = px;  OutPolygon[OutVertices].y = py;
      OutVertices++;
      if (OutVertices == MaxCount) break;
     }   
     if (pVertIn != cVertIn)
     {
      OutPolygon[OutVertices].x = cx - (cy-region.miny)*(cx-px)/(cy-py);
      OutPolygon[OutVertices].y = region.minx;
      OutVertices++;  
      if (OutVertices == MaxCount) break;
      isectCount++;
      if (isectCount >= 2)
      {
       if (cVertIn)
       {
        memcpy (&OutPolygon[OutVertices], &InPolygon[vert],
                (InVertices - vert) * sizeof(OutPolygon[0]));
        OutVertices += InVertices - vert;
       }
       break;
      }
     }
     px = cx; py = cy;  pVertIn = cVertIn; 
    } /* for (vert = 1; vert <= InVertices; vert++) */

    if (OutVertices < 3) { Count = 0; return false; }
    InVertices = OutVertices;
    InPolygon = OutPolygon;
    if (OutPolygon == OutPoly) OutPolygon = Polygon;
     else OutPolygon = OutPoly;
  }
  if (BoundingBox->maxx > region.maxx) /* clip against right x line */
  {
    float px = InPolygon[0].x, py = InPolygon[0].y;
    bool pVertIn = (px < region.maxx);
    OutVertices = 0;
    int isectCount = 0;
    for (vert = 1; vert <= InVertices; vert++)
    {
     float cx, cy;
     if (vert < InVertices) 
     { cx = InPolygon[vert].x;  cy = InPolygon[vert].y; }
     else { cx = InPolygon[0].x;  cy = InPolygon[0].y; }
     bool cVertIn = (cx < region.maxx);
     
     if (pVertIn)
     {
      OutPolygon[OutVertices].x = px;  OutPolygon[OutVertices].y = py;
      OutVertices++;
      if (OutVertices == MaxCount) break;
     }   
     if (pVertIn != cVertIn)
     {
      OutPolygon[OutVertices].x = region.maxx;
      OutPolygon[OutVertices].y = cy - (cx-region.maxx)*(cy-py)/(cx-px);
      OutVertices++;  
      if (OutVertices == MaxCount) break;
      isectCount++;
      if (isectCount >= 2)
      {
       if (cVertIn)
       {
        memcpy (&OutPolygon[OutVertices], &InPolygon[vert],
                (InVertices - vert) * sizeof(OutPolygon[0]));
        OutVertices += InVertices - vert;
       }
       break;
      }
     }
     px = cx; py = cy;  pVertIn = cVertIn; 
    } /* for (vert = 1; vert <= InVertices; vert++) */

    if (OutVertices < 3) { Count = 0; return false; }
    InVertices = OutVertices;
    InPolygon = OutPolygon;
    if (OutPolygon == OutPoly) OutPolygon = Polygon;
     else OutPolygon = OutPoly;
  }
  if (BoundingBox->maxy > region.maxy) /* clip against lower y line */
  {
    float px = InPolygon[0].x, py = InPolygon[0].y;
    bool pVertIn = (py < region.maxy);
    OutVertices = 0;
    int isectCount = 0;
    for (vert = 1; vert <= InVertices; vert++)
    {
     float cx, cy;
     if (vert < InVertices) 
     { cx = InPolygon[vert].x;  cy = InPolygon[vert].y; }
     else { cx = InPolygon[0].x;  cy = InPolygon[0].y; }
     bool cVertIn = (cy < region.maxy);
     
     if (pVertIn)
     {
      OutPolygon[OutVertices].x = px;  OutPolygon[OutVertices].y = py;
      OutVertices++;
      if (OutVertices == MaxCount) break;
     }   
     if (pVertIn != cVertIn)
     {
      OutPolygon[OutVertices].x = cx - (cy-region.maxy)*(cx-px)/(cy-py);
      OutPolygon[OutVertices].y = region.maxy;
      OutVertices++;  
      if (OutVertices == MaxCount) break;
      isectCount++;
      if (isectCount >= 2)
      {
       if (cVertIn)
       {
        memcpy (&OutPolygon[OutVertices], &InPolygon[vert],
                (InVertices - vert) * sizeof(OutPolygon[0]));
        OutVertices += InVertices - vert;
       }
       break;
      }
     }
     px = cx; py = cy;  pVertIn = cVertIn; 
    } /* for (vert = 1; vert <= InVertices; vert++) */

    if (OutVertices < 3) { Count = 0; return false; }
    InVertices = OutVertices;
    InPolygon = OutPolygon;
    if (OutPolygon == OutPoly) OutPolygon = Polygon;
     else OutPolygon = OutPoly;
  }

  Count = OutVertices;
  if (Count < 3) {  Count=0; return false; }
  if (InPolygon!=Polygon) 
   memcpy (Polygon, InPolygon, Count * sizeof (Vector2));
  BoundingBox->start_bounding_box (*InPolygon++);
  while (--OutVertices)
   BoundingBox->add_bounding_vertex_smart (*InPolygon++);
  return true;
}

//---------------------------------------------------------------------------

csPolygonClipper::csPolygonClipper (Vector2 *Clipper, int Count, bool mirror,
  bool copy)
{
  int vert;

  ClipPolyIsACopy = mirror | copy;
  if (mirror)
  {
    CHK (ClipPoly = new Vector2 [Count]);
    for (vert = 0; vert < Count; vert++)
      ClipPoly [Count - 1 - vert] = Clipper [vert];
  } else if (copy)
  {
    CHK (ClipPoly = new Vector2 [Count]);
    for (vert = 0; vert < Count; vert++)
      ClipPoly [vert] = Clipper [vert];
  } else
    ClipPoly = Clipper;

  CHK (ClipData = new SegData [ClipPolyVertices = Count]);
  // Precompute some data for each clipping edge
  for (vert = 0; vert < ClipPolyVertices; vert++)
  {
    int next = (vert == ClipPolyVertices - 1 ? 0 : vert + 1);
    ClipData [vert].dx = ClipPoly [next].x - ClipPoly [vert].x;
    ClipData [vert].dy = ClipPoly [next].y - ClipPoly [vert].y;
  } /* endfor */

  // Initialize bounding box
  ClipBox.start_bounding_box (ClipPoly [0]);
  for (vert = 1; vert < ClipPolyVertices; vert++)
    ClipBox.add_bounding_vertex (ClipPoly [vert]);
}

csPolygonClipper::~csPolygonClipper ()
{
  if (ClipData)
    CHKB (delete [] ClipData);
  if (ClipPolyIsACopy)
    CHKB (delete [] ClipPoly);
}

bool csPolygonClipper::IsInside (float x, float y)
{
  for (int vert = 0; vert < ClipPolyVertices; vert++)
    if (x * ClipData [vert].dy - y * ClipData [vert].dx < 0)
      return false;
  return true;
}

Vector2 *csPolygonClipper::Clip (Vector2 *Polygon, int Count, int &OutCount)
{
  #include "polyclip.inc"

  if (OutVertices < 3)
  {
    OutCount = 0;
    return NULL;
  } else
  {
    CHK (Vector2 *result = new Vector2 [OutVertices]);
    memcpy (result, InPolygon, OutVertices * sizeof (Vector2));
    OutCount = OutVertices;
    return result;
  } /* endif */
}

bool csPolygonClipper::Clip (Vector2 *Polygon, int &Count, int MaxCount,
  Box *BoundingBox)
{
  if (!ClipBox.overlap (*BoundingBox))
    return false;

  #include "polyclip.inc"

  if (OutVertices < 3)
  {
    Count = 0;
    return false;
  } else
  {
    if (OutVertices > MaxCount)
      OutVertices = MaxCount;
    Count = OutVertices;
    memcpy (Polygon, InPolygon, Count * sizeof (Vector2));
    BoundingBox->start_bounding_box (*InPolygon++);
    while (--OutVertices)
      BoundingBox->add_bounding_vertex_smart (*InPolygon++);
    return true;
  } /* endif */
}

void csPolygonClipper::dump (char* name)
{
  CsPrintf (MSG_DEBUG_0, "PolygonClipper '%s'\n", name);
  int i;
  for (i = 0 ; i < ClipPolyVertices ; i++)
    CsPrintf (MSG_DEBUG_0, "  %d: (%f,%f)\n", i, ClipPoly[i].x, ClipPoly[i].y);
}

