############################################
# These are the dependencies for Crystal Space.
# This is an include file for the makefiles.
# Don't use this directly as a makefile.
############################################

# The following include file should define all variables
# needed for this makefile to work.
include mk/user.mak

# MAKE_DLL should be defined by the calling makefile if it
# is going to be used for compiling a library (shared or not)

# The following include should re/define system-dependent variables
MAKESECTION=defines
include $(SYSMAKEFILE)

.SUFFIXES: .dep
.PHONY: all soft regsvr cs wt si mz pe doc depend \
	clean cleancs cleanlib cleandepend

############################################
# Depending on the type of optimization choosen we disable assembler support.
ifneq ($(OPT),optimize)
  DO_ASM=no
endif
# If we don't want DLLs, make some changes
ifeq ($(USE_DLL),no)
  DLL_FLAG=-DCS_STATIC_LINKED
  override MAKE_DLL=no
  SYS_SRC_DLL=
endif
############################################

############################################
# Directory for object files
ifeq ($(OPT),optimize)
  SHORTOPT=o
else
  ifeq ($(OPT),debug)
    SHORTOPT=d
  else
    ifeq ($(OPT),profile)
      SHORTOPT=p
    endif
  endif
endif
OUTBASE=out/
ifeq ($(DO_DLL),yes)
  OUTSUFFIX=$(DLL)
endif
OUTOS=$(OUTBASE)$(OS)/
OUT=$(OUTOS)$(DISP)_$(SHORTOPT)$(OUTSUFFIX)/
############################################

############################################
# Targets
ifndef WALKTEST
  WALKTEST=cryst$(EXE)
endif
ifndef CSREGSVR
  CSREGSVR=csregsvr$(EXE)
endif
ifndef SOFTRNDR
  ifeq ($(USE_DLL),no)
    SOFTRNDR=$(OUT)$(LIB_PREFIX)soft$(LIB)
  else
    SOFTRNDR=$(DLL_DIR)softrndr$(DLL)
  endif
endif
ifndef SIMPLE
  SIMPLE=simple$(EXE)
endif
ifndef CSWSTEST
  CSWSTEST=cswstest$(EXE)
endif
ifndef MAZED
  MAZED=mazed$(EXE)
endif
ifndef PRIMEDIT
  PRIMEDIT=primedit$(EXE)
endif
############################################

S_COM=com/*.cpp
S_ENGINE=engine/*.cpp engine/polygon/*.cpp engine/basic/*.cpp engine/light/*.cpp \
   engine/objects/*.cpp engine/2d/*.cpp
S_GENERAL=general/*.cpp memory.cpp
S_LOADER=general/loader/*.cpp
S_GEOMETRY=geometry/*.cpp
S_SCRIPTS=scripts/*.cpp
S_UTIL=util/*.cpp util/gfx/csimage.cpp util/gfx/pcx.cpp util/input/*.cpp util/sound/*.cpp
S_SYSTEM=system/*.cpp $(SYS_SRC_EXE)
S_CSWS=csws/*.cpp
S_CD=colldet/*.cpp
ifeq ($(DO_SOUND),yes)
  S_SOUND=sound/*.cpp
endif

GFX_LIBS=

ifeq ($(DO_PNG),yes)
  S_GFORM+=util/gfx/pngimage.cpp
  PNG=-DDO_PNG
  GFX_LIBS+=$(PNG_LIBS)
endif
ifeq ($(DO_GIF),yes)
  S_GFORM+=util/gfx/gifimage.cpp
  GIF=-DDO_GIF
endif
ifeq ($(DO_BMP),yes)
  S_GFORM+=util/bmp/bmpimage.cpp
  BMP=-DDO_BMP
endif
ifeq ($(DO_TGA),yes)
  S_GFORM+=util/gfx/tgaimage.cpp
  TGA=-DDO_TGA
endif
ifeq ($(DO_JPG),yes)
  S_GFORM+=util/gfx/jpgimage.cpp
  JPG=-DDO_JPG
  GFX_LIBS+=$(JPG_LIBS)
endif

SRC=$(wildcard $(S_ENGINE) $(S_GENERAL) $(S_GEOMETRY) \
  $(S_SCRIPTS) $(S_UTIL) $(S_SYSTEM) $(S_GFORM) $(S_LOADER) \
  $(S_CD) $(S_SOUND) $(S_COM))

SRC_WALKTEST = $(wildcard general/walktest/*.cpp)
SRC_SOFTRNDR = $(wildcard render/software/*.cpp render/graph3d.cpp \
  render/softw16/*.cpp geometry/math3d.cpp com/guids.cpp \
  com/cscom.cpp com/inttable.cpp util/memheap.cpp $(SYS_SRC_DLL))
SRC_MAZED    = $(wildcard $(S_CSWS) general/mazed/*.cpp)
SRC_CSWSTEST = $(wildcard $(S_CSWS) general/cswstest/*.cpp)
SRC_SIMPLE   = $(wildcard general/simple/*.cpp)
SRC_PRIMEDIT = $(wildcard general/primedit/*.cpp) util/util.cpp
SRC_CSREGSVR = $(wildcard general/csregsvr/csregsvr.cpp $(SYS_SRC_REG))

HEADER=$(wildcard *.h */*.h */*/*.h */*/*/*.h)

OBJ1=$(SRC:.cpp=.o)
OBJ2=$(OBJ1:.S=.o)
OBJ          = $(addprefix $(OUT),$(OBJ2:.s=.o))
WALKTEST_OBJ = $(addprefix $(OUT),$(SRC_WALKTEST:.cpp=.o))
SOFTRNDR_OBJ = $(addprefix $(OUT),$(SRC_SOFTRNDR:.cpp=.o))
MAZED_OBJ    = $(addprefix $(OUT),$(SRC_MAZED:.cpp=.o))
CSWSTEST_OBJ = $(addprefix $(OUT),$(SRC_CSWSTEST:.cpp=.o))
SIMPLE_OBJ   = $(addprefix $(OUT),$(SRC_SIMPLE:.cpp=.o))
PRIMEDIT_OBJ = $(addprefix $(OUT),$(SRC_PRIMEDIT:.cpp=.o))
CSREGSVR_OBJ = $(addprefix $(OUT),$(SRC_CSREGSVR:.cpp=.o))

ifneq ($(DO_ASM),yes)
  ASM=-DNO_ASSEMBLER
endif
ifeq ($(OPT),optimize)
  HOW=$(CFLAGS_GENERAL) $(CFLAGS_OPTIMIZE)
  HOWL=$(LFLAGS_GENERAL) $(LFLAGS_OPTIMIZE)
endif
ifeq ($(OPT),debug)
  HOW=$(CFLAGS_GENERAL) $(CFLAGS_DEBUG)
  HOWL=$(LFLAGS_GENERAL) $(LFLAGS_DEBUG)
endif
ifeq ($(OPT),profile)
  HOW=$(CFLAGS_GENERAL) $(CFLAGS_PROFILE)
  HOWL=$(LFLAGS_GENERAL) $(LFLAGS_PROFILE)
endif
ifeq ($(MAKE_DLL),yes)
  HOWDLL=$(CFLAGS_GENERAL) $(CFLAGS_DLL)
  HOWDLLL=$(LFLAGS_GENERAL) $(LFLAGS_DLL)
endif

# If no native COM support, notify the compiler
ifeq ($(NATIVE_COM),no)
  COM_FLAG=-DNO_COM_SUPPORT
endif

# SHM
ifeq ($(DO_SHM),yes)
  XSHARED=-DDO_SHM
endif

ifeq ($(DO_MMX),yes)
  MMX=-DDO_MMX
endif

SYSFLAGS=-DOS_$(OS) -DPROC_$(PROC) -DCOMP_$(COMP) -DDISP_$(DISP)
INC_PATH=$(INCLUDE_DIRS) -I .
LIBS=$(EXTRA_LIBS) $(GFX_LIBS) $(Z_LIBS)
CFLAGS=$(HOW) $(HOWDLL) $(SYSFLAGS) $(XSHARED) $(MMX) $(ASM) $(MEM) $(PNG) $(GIF) $(BMP) $(TGA) $(JPG) $(COM_FLAG) $(DLL_FLAG)
LFLAGS=$(HOWL) $(HOWDLLL) -L $(OUT)
SED_DEPEND=-e "s|^\([^ ].*\)\(\..*: *\)\(.*\1\)\(\.c.*\)|$$\(OUT\)\3\2\3\4|" $(SYS_SED_DEPEND)

# Directories for output files
OUTDIRS=$(OUTBASE) $(OUTOS) $(OUT)  $(DLL_DIR) \
 $(addprefix $(OUT),$(filter-out $(OUTBASE)%,$(sort $(dir $(wildcard */* */*/* */*/*/*)))))

  
# The following include should make additional defines using above variables
MAKESECTION=postdefines
include $(SYSMAKEFILE)

all: $(OUTDIRS) $(WALKTEST) $(SOFTRNDR) $(CSREGSVR) $(MAZED) $(CSWSTEST) $(SIMPLE) $(PRIMEDIT)

soft: $(OUTDIRS) $(SOFTRNDR)

regsvr: $(OUTDIRS) $(CSREGSVR)

cs: $(OUTDIRS) $(WALKTEST)

wt: $(OUTDIRS) $(CSWSTEST)

si: $(OUTDIRS) $(SIMPLE)

mz: $(OUTDIRS) $(MAZED)

pe: $(OUTDIRS) $(PRIMEDIT)

$(SOFTRNDR): $(SOFTRNDR_OBJ)
ifeq ($(MAKE_DLL),no)
	$(AR) cr $@ $^
else
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)
endif

$(WALKTEST): $(OBJ) $(WALKTEST_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS) $(LFLAGS_CS)

$(CSREGSVR): $(CSREGSVR_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)

$(MAZED): $(OBJ) $(MAZED_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)

$(CSWSTEST): $(OBJ) $(CSWSTEST_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)

$(SIMPLE): $(OBJ) $(SIMPLE_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)

$(PRIMEDIT): $(OBJ) $(PRIMEDIT_OBJ)
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)

clean:
	-$(RMDIR) out
	-$(RM) debug.txt cryst.reg $(WALKTEST) $(SOFTRNDR) $(CSREGSVR) $(MAZED) \
		$(CSWSTEST) $(SIMPLE) $(PRIMEDIT) $(STAT_SOFTRNDR)

cleancs:
	-$(RM) $(OBJ) $(WALKTEST_OBJ) $(WALKTEST)

cleanlib:
	-$(RM) $(SOFTRNDR_OBJ) $(STAT_SOFTRNDR) $(SOFTRNDR)

cleandepend:
	-$(RM) $(OUTOS)*.dep

doc:
	doc++ -H -d ../csdoc -f $(HEADER)

depend: $(OUTBASE) $(OUTOS) cleandepend
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC) | sed $(SED_DEPEND) >$(OUTOS)cs.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_WALKTEST) | sed $(SED_DEPEND) >$(OUTOS)walktest.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_SOFTRNDR) | sed $(SED_DEPEND) >$(OUTOS)softrndr.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_MAZED) | sed $(SED_DEPEND) >$(OUTOS)mazed.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_SIMPLE) | sed $(SED_DEPEND) >$(OUTOS)simple.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_PRIMEDIT) | sed $(SED_DEPEND) >$(OUTOS)primedit.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_CSREGSVR) | sed $(SED_DEPEND) >$(OUTOS)csregsvr.dep

$(OUT)%.o : %.cpp
	$(CXX) -o $@ $< -c $(CFLAGS) $(INC_PATH)

$(OUT)%.o : %.c
	$(CC) -o $@ $< -c $(CFLAGS) $(INC_PATH)

$(OUT)%.o : %.S
	$(CC) $< -o $@ -c $(CFLAGS)

$(OUT)%.o : %.s
	$(CC) $< -o $@ -c $(CFLAGS)

$(OUTDIRS):
	$(MKDIR)

# The following include should define system-dependent targets
MAKESECTION=targets
include $(SYSMAKEFILE)

-include $(OUTOS)cs.dep
-include $(OUTOS)walktest.dep
-include $(OUTOS)mazed.dep
-include $(OUTOS)simple.dep
-include $(OUTOS)primedit.dep
-include $(OUTOS)softrndr.dep
-include $(OUTOS)csregsvr.dep
