# This is an include file for all the makefiles which describes system specific
# settings. Also have a look at mk/user.mak.

include mk/system/belib2d.mak

#---------------------------------------------------- rootdefines & defines ---#
ifneq (,$(findstring defines,$(MAKESECTION)))

# Processor. Can be one of: INTEL, SPARC, POWERPC, M68K, UNKNOWN
PROC=INTEL

# Operating system. Can be one of: SOLARIS, LINUX, IRIX, BSD, UNIX, DOS, MACOS, AMIGAOS, WIN32, OS2, BE
OS=BE

# Compiler. Can be one of: GCC, WCC (Watcom C++), MPWERKS, VC (Visual C++), UNKNOWN
COMP=GCC

# Display type. Can be one of: X11, SVGA, GGI, OTHER (for a non-unix OS)
DISP=OTHER

# Typical extension for executables on this system (e.g. EXE=.exe)
EXE=

# Typical extension for dynamic libraries on this system.
DLL=.so

# Extra libraries needed on this system.
EXTRA_LIBS= -lbe -lgame

# Where can the Zlib library be found on this system?
Z_LIBS= -lz

# Where can the PNG library be found on this system?
PNG_LIBS= -lpng

# Where can the JPG library be found on this system?
JPG_LIBS= -ljpeg

# Where can the optional sound libraries be found on this system?
SOUND_LIBS=

# Indicate where special include files can be found.
INCLUDE_DIRS=

# General flags for the compiler which are used in any case.
CFLAGS_GENERAL=-Wall -Wno-multichar -Wno-ctor-dtor-privacy 

# Flags for the compiler which are used when optimizing.
CFLAGS_OPTIMIZE=-O3 

# Flags for the compiler which are used when debugging.
CFLAGS_DEBUG=-g -O0

# Flags for the compiler which are used when profiling.
CFLAGS_PROFILE=-pg -O -g

# Flags for the compiler which are used when building a shared library.
#CFLAGS_DLL=
CFLAGS_DLL=

# General flags for the linker which are used in any case.
LFLAGS_GENERAL=

# Flags for the linker which are used when debugging.
LFLAGS_DEBUG= -gdwarf-2

# Flags for the linker which are used when profiling.
LFLAGS_PROFILE=-pg

# Flags for the linker which are used when building a shared library.
LFLAGS_DLL= -nostart -Xlinker

# System dependent source files belonging to Crystal Space. These are
# files which are used by the executables.
SYS_SRC_EXE=system/be/csbe.cpp system/be/loadlib.cpp system/def_sys/printf.cpp system/def_sys/fopen.cpp

# System dependent source files belonging to csregsvr.
SYS_SRC_REG=system/be/loadlib.cpp

# System dependent source files belonging to Crystal Space. These are
# files which are used by the shared libraries.
SYS_SRC_DLL=system/be/loadlib.cpp

# System dependent source files to create the 2d driver.
SYS_LIB2D_SRC = system/be/driver2d/*.cpp

# Does this OS have native COM support?
NATIVE_COM=no

# Where to put dynamic libraries on this system?
DLL_DIR=add-ons/

# The C compiler.
CC=gcc

# The C++ compiler.
CXX=gcc

# The linker.
LINK=gcc

# The 'make' executable. This should be the GNU make!
MAKE=make

# Command sequence for creating a directory.
# Note that directories will have forward slashes. Please
# make sure that this command accepts that (or use 'subst' first).
MKDIR=mkdir '$(@:/=)'

# The command to remove all specified files.
RM=rm -f

# The command to remove a directory tree.
RMDIR=rm -rf

# Extra parameters for 'sed' which are used for doing 'make depend'.
SYS_SED_DEPEND=

#==================================================
# Extra operation system dependent options.
#==================================================

# Include support for the XSHM extension in Crystal Space.
# Note that you don't need to disable this if you don't have XSHM
# support in your X server because Crystal Space will autodetect
# the availability of XSHM.
DO_SHM=no

endif # ifeq ($(MAKESECTION),defines)
