################################################################################
#                       This is the makefile for DOS/DJGPP
################################################################################

# Enable for GCC later than 2.7.2 (2.8.0 and up)
#NO_EXCEPTIONS=-fno-exceptions

#---------------------------------------------------- rootdefines & defines ---#
ifneq (,$(findstring defines,$(MAKESECTION)))

.SUFFIXES: .exe .a

# Processor type
PROC=INTEL

# "Operating system", if it can be called so :-/
OS=DOS

# Compiler
COMP=GCC

# Always override "USE_DLL" to "no"
override USE_DLL=no

endif # ifneq (,$(findstring defines,$(MAKESECTION)))

#-------------------------------------------------------------- rootdefines ---#
ifeq ($(MAKESECTION),rootdefines)

# System-dependent help commands
define SYSHELP
	@echo "  make raw          Make the raw DOS SVGA driver"
endef

endif # ifeq ($(MAKESECTION),rootdefines)

#-------------------------------------------------------------- roottargets ---#
ifeq ($(MAKESECTION),roottargets)

.PHONY: raw

raw:
	@echo "+==================================================================="
	@echo "| Compiling raw DOS SVGA driver for compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=yes $@

endif # ifeq ($(MAKESECTION),roottargets)

#------------------------------------------------------------------ defines ---#
ifeq ($(MAKESECTION),defines)

# Display type
DISP=RAW

# Typical extension for executables on this system (e.g. EXE=.exe)
EXE=.exe

# Typical extension for dynamic libraries on this system.
DLL=

# Typical extension for static libraries
LIB=.a
AR=ar

# Typical prefix for library filenames
LIB_PREFIX=lib

# Extra libraries needed on this system.
EXTRA_LIBS=-lm -L . -lsoft -lraw

# Where can the Zlib library be found on this system?
Z_LIBS=-lz

# Where can the PNG library be found on this system?
PNG_LIBS=-lpng

# Where can the JPG library be found on this system?
JPG_LIBS=-ljpeg

# Where can the optional sound libraries be found on this system?
SOUND_LIBS=

# Indicate where special include files can be found.
INCLUDE_DIRS=

# General flags for the compiler which are used in any case.
CFLAGS_GENERAL=-Wall -m486 -fno-rtti $(NO_EXCEPTIONS)

# Flags for the compiler which are used when optimizing.
CFLAGS_OPTIMIZE=-O6 -malign-loops=2 -malign-jumps=2 -malign-functions=2

# Flags for the compiler which are used when debugging.
CFLAGS_DEBUG=-g3

# Flags for the compiler which are used when profiling.
CFLAGS_PROFILE=-pg -O -g

# Flags for the compiler which are used when building a shared library.
CFLAGS_DLL=

# General flags for the linker which are used in any case.
LFLAGS_GENERAL=

# Flags for the linker which are used when optimizing.
LFLAGS_OPTIMIZE=-s

# Flags for the linker which are used when debugging.
LFLAGS_DEBUG=-g

# Flags for the linker which are used when profiling.
LFLAGS_PROFILE=-pg

# Flags for the linker which are used when building a shared library.
LFLAGS_DLL=

# System dependent source files belonging to Crystal Space. These are
# files which are used by the executables.
SYS_SRC_EXE=system/djgpp/djgpp.cpp system/djgpp/printf.cpp \
	system/djgpp/djmousys.s system/djgpp/djkeysys.s \
	system/def_sys/fopen.cpp system/def_sys/timing.cpp

# System dependent source files belonging to csregsvr.
SYS_SRC_REG=

# System dependent source files belonging to Crystal Space. These are
# files which are used by the shared libraries.
SYS_SRC_DLL=

# Where to put the dynamic libraries on this system?
DLL_DIR=

# Does this OS have native COM support?
NATIVE_COM=no

# The C compiler.
CC=gcc

# The C++ compiler.
CXX=gcc

# The linker.
LINK=gcc

# Command sequence for creating a directory.
# Note that directories will have forward slashes. Please
# make sure that this command accepts that (or use 'subst' first).
MKDIR=-mkdir $(subst /,\,$(@:/=))

# The command to remove all specified files.
RM=rm -f

# The command to remove a directory tree.
RMDIR=rm -rf

# Extra parameters for 'sed' which are used for doing 'make depend'.
SYS_SED_DEPEND=

endif # ifeq ($(MAKESECTION),defines)

#-------------------------------------------------------------- postdefines ---#
ifeq ($(MAKESECTION),postdefines)

# The raw 2D DOS SVGA driver
RAW=$(OUT)libraw$(LIB)
SRC_RAW=$(wildcard system/djgpp/raw/*.cpp \
	system/fonts.cpp render/graph2d.cpp render/igraph2d.cpp \
	com/guids.cpp com/cscom.cpp com/inttable.cpp \
	geometry/csrect.cpp $(SYS_SRC_DLL))
RAW_OBJ = $(addprefix $(OUT),$(SRC_RAW:.cpp=.o))

# Tell MAKE that most executables depend on video and rendering drivers
cs mz wt si pe: soft raw

endif # ifeq ($(MAKESECTION),postdefines)

#------------------------------------------------------------------ targets ---#
ifeq ($(MAKESECTION),targets)

.PHONY: dosdepend dosclean doscleancs doscleanlib raw

# Chain rules
all: raw
depend: dosdepend
clean: dosclean
cleancs: doscleancs
cleanlib: doscleanlib
raw: $(OUTDIRS) $(RAW)

$(RAW): $(RAW_OBJ)
	$(AR) cr $@ $^

dosclean:
	$(RM) $(RAW)

doscleanlib:
	$(RM) $(RAW_OBJ) $(RAW)

dosdepend:
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_RAW) | sed $(SED_DEPEND) >$(OUTOS)raw.dep

-include $(OUTOS)raw.dep

endif # ifeq ($(MAKESECTION),targets)
