# This is a subinclude file used to define the rules needed
# to build the GLX 2D driver -- glrender

#-------------------------------------------------------------- rootdefines ---#
ifeq ($(MAKESECTION),rootdefines)

# System-dependent help commands
define SYSHELP
	@echo "  make glrender       Make the GL 3D driver"
	@echo "  make stat_glrender  Make the static GL 3D driver"
endef

endif # ifeq ($(MAKESECTION),rootdefines)

#-------------------------------------------------------------- roottargets ---#
ifeq ($(MAKESECTION),roottargets)

.PHONY: glrender

glrender:
	@echo "+==================================================================="
	@echo "| Compiling GL 3D driver for OS=$(OS), compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "| Don't forget to register this library with csregsvr!"
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=yes glrender

stat_glrender:
	@echo "+==================================================================="
	@echo "| Compiling static GL 3D driver for OS=$(OS), compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=no stat_glrender

endif # ifeq ($(MAKESECTION),roottargets)

#-------------------------------------------------------------- postdefines ---#
ifeq ($(MAKESECTION),postdefines)

# The 2D GLX driver
GL3D=glrender$(DLL)
SRC_GL3D = $(wildcard render/opengl/*.cpp \
	system/fonts.cpp \
	render/hicache.cpp render/graph2d.cpp render/igraph2d.cpp \
	com/guids.cpp com/cscom.cpp com/inttable.cpp \
	engine/basic/csrect.cpp util/memheap.cpp $(SYS_SRC_DLL))
GL3D_OBJ = $(addprefix $(OUT),$(SRC_GL3D:.cpp=.o))

STAT_GL3D=libglrender.a
SRC_STAT_GLXD = $(SRC_GL3D)
STAT_GL3D_OBJ = $(GL3D_OBJ)

endif # ifeq ($(MAKESECTION),postdefines)

#------------------------------------------------------------------ targets ---#
ifeq ($(MAKESECTION),targets)

.PHONY: glrender stat_gl3d glclean glcleanlib gldepend

# Chain rules
all: glrender
clean: glclean
cleanlib: glcleanlib
depend: gldepend

glrender: $(OUTDIRS) $(GL3D)

$(GL3D): $(GL3D_OBJ)
ifeq ($(MAKE_DLL),no)
	ar -r $@ $^
else
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)
endif

stat_glrender: $(OUTDIRS) $(STAT_GL3D)

$(STAT_GL3D): $(STAT_GL3D_OBJ)
	ar -r $@ $^

glclean:
	$(RM) $(GL3D) $(STAT_GL3D)

glcleanlib:
	$(RM) $(GL3D_OBJ) $(GLX2D) $(STAT_GL3D_OBJ) $(STAT_GL3D)

gldepend:
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_GL3D) | sed $(SED_DEPEND) >$(OUTOS)glrender.dep

-include $(OUT)glrender.dep

endif # ifeq ($(MAKESECTION),targets)
