# This is a subinclude file used to define the rules needed
# to build the GLX 2D driver -- glx2d

#-------------------------------------------------------------- rootdefines ---#
ifeq ($(MAKESECTION),rootdefines)

# System-dependent help commands
define SYSHELP
	@echo "  make glx2d       Make the GLX 2D driver"
	@echo "  make stat_glx2d  Make the static GLX 2D driver"
endef

endif # ifeq ($(MAKESECTION),rootdefines)

#-------------------------------------------------------------- roottargets ---#
ifeq ($(MAKESECTION),roottargets)

.PHONY: glx2d

glx2d:
	@echo "+==================================================================="
	@echo "| Compiling GLX 2D driver for OS=$(OS), compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "| Don't forget to register this library with csregsvr!"
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=yes glx2d

stat_glx2d:
	@echo "+==================================================================="
	@echo "| Compiling static GLX 2D driver for OS=$(OS), compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=no stat_glx2d

endif # ifeq ($(MAKESECTION),roottargets)

#-------------------------------------------------------------- postdefines ---#
ifeq ($(MAKESECTION),postdefines)

# The 2D GLX driver
GLX2D=glx2d$(DLL)
SRC_GLX2D = $(wildcard system/xlib/driver2d_glx/*.cpp \
	system/fonts.cpp render/graph2d.cpp render/igraph2d.cpp \
	com/guids.cpp com/cscom.cpp com/inttable.cpp \
	engine/basic/csrect.cpp $(SYS_SRC_DLL))
GLX2D_OBJ = $(addprefix $(OUT),$(SRC_GLX2D:.cpp=.o))

STAT_GLX2D=libglx2d.a
SRC_STAT_GLXD = $(SRC_GLX2D)
STAT_GLX2D_OBJ = $(GLX2D_OBJ)

endif # ifeq ($(MAKESECTION),postdefines)

#------------------------------------------------------------------ targets ---#
ifeq ($(MAKESECTION),targets)

.PHONY: glx2d stat_glx2d glxclean glxcleanlib glxdepend

# Chain rules
all: glx2d
clean: glxclean
cleanlib: glxcleanlib
depend: glxdepend

glx2d: $(OUTDIRS) $(GLX2D)

$(GLX2D): $(GLX2D_OBJ)
ifeq ($(MAKE_DLL),no)
	ar -r $@ $^
else
	$(LINK) -o $@ $^ $(LFLAGS) $(LIB_PATH) $(LIBS)
endif

stat_glx2d: $(OUTDIRS) $(STAT_GLX2D)

$(STAT_GLX2D): $(STAT_GLX2D_OBJ)
	ar -r $@ $^

glxclean:
	$(RM) $(GLX2D) $(STAT_GLX2D)

glxcleanlib:
	$(RM) $(GLX2D_OBJ) $(GLX2D) $(STAT_GLX2D_OBJ) $(STAT_GLX2D)

glxdepend:
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_GLX2D) | sed $(SED_DEPEND) >$(OUTOS)glx2d.dep

-include $(OUT)glx2d.dep

endif # ifeq ($(MAKESECTION),targets)
