################################################################################
#                        This is the makefile for OS2/EMX
#                       *** (tested only with pgcc-1.0) ***
################################################################################

#---------------------------------------------------
# NOTE: OS/2 makefiles for libjpeg, libpng and zlib
# can be found in system/os2 subdirectory
#---------------------------------------------------

#---------------------------------------------------- rootdefines & defines ---#
ifneq (,$(findstring defines,$(MAKESECTION)))

.SUFFIXES: .exe .dll .lib .res .rc

# CMD.EXE is faster than BASH, but sometimes crashes on very long
# command lines. You SHOULD replace this by a Unix-like shell (say bash)
# when generating dependencies! Note that you can do it from the command
# line as well, i.e.: make depend SHELL=bash
SHELL=cmd.exe
#SHELL=bash.exe

# Processor type. Most OS/2 systems runs on Intel's, I'm not sure IBM
# will ever ressurect the PowerPC version ... :-(
PROC=INTEL

# Operating system
OS=OS2

# Compiler
COMP=GCC

endif # ifneq (,$(findstring defines,$(MAKESECTION)))

#-------------------------------------------------------------- rootdefines ---#
ifeq ($(MAKESECTION),rootdefines)

# System-dependent help commands
define SYSHELP
	@echo "  make csdive       Make the OS/2 DIVE driver DLL"
endef

endif # ifeq ($(MAKESECTION),rootdefines)

#-------------------------------------------------------------- roottargets ---#
ifeq ($(MAKESECTION),roottargets)

.PHONY: csdive

csdive:
	@echo "+==================================================================="
	@echo "| Compiling OS/2 DIVE driver for compiler=$(COMP) and processor=$(PROC)."
	@echo "| Compiling in $(OPT) mode."
	@echo "+==================================================================="
	$(MAKE) -f mk/cs.mak MAKE_DLL=yes $@

endif # ifeq ($(MAKESECTION),roottargets)

#------------------------------------------------------------------ defines ---#
ifeq ($(MAKESECTION),defines)

# Display type
DISP=DIVE

# Typical extension for executables on this system (e.g. EXE=.exe)
EXE=.exe

# Typical extension for dynamic libraries on this system.
DLL=.dll

# Typical extension for static libraries
ifeq ($(OPT),optimize)
  LIB=.lib
  AR=emxomfar
else
  LIB=.a
  AR=ar
endif

# Typical prefix for library filenames
LIB_PREFIX=

# Extra libraries needed on this system.
ifeq ($(USE_DLL),no)
  EXTRA_LIBS=-L . -lsoft -lcsdive -L $(OUTOS) -lcsos2
endif

# Where can the Zlib library be found on this system?
Z_LIBS=-L ../zlib -lzdll

# Where can the PNG library be found on this system?
PNG_LIBS=-L ../libpng -lpngdll

# Where can the JPG library be found on this system?
JPG_LIBS=-L ../libjpeg -ljpegdll

# Where can the optional sound libraries be found on this system?
SOUND_LIBS=

# Indicate where special include files can be found.
INCLUDE_DIRS=-I ../zlib -I ../libpng -I ../libjpeg

# General flags for the compiler which are used in any case.
CFLAGS_GENERAL=-Wall -Zmt -mpentiumpro -march=i686 -fno-exceptions -fno-rtti

# Flags for the compiler which are used when optimizing.
CFLAGS_OPTIMIZE=-Zomf -O6 -malign-loops=2 -malign-jumps=2 -malign-functions=2

# Flags for the compiler which are used when debugging.
CFLAGS_DEBUG=-gstabs3

# Flags for the compiler which are used when profiling.
CFLAGS_PROFILE=-pg -O -g

# Flags for the compiler which are used when building a shared library.
CFLAGS_DLL=

# General flags for the linker which are used in any case.
LFLAGS_GENERAL=-Zmt -fno-exceptions

# Flags for the linker which are used when optimizing.
LFLAGS_OPTIMIZE=-Zomf -s -Zsys -Zsmall-conv

# Flags for the linker which are used when debugging.
LFLAGS_DEBUG=-g -Zcrtdll -Zstack 512

# Flags for the linker which are used when profiling.
LFLAGS_PROFILE=-pg -Zcrtdll -Zstack 512

# Flags for the linker which are used when building a shared library.
LFLAGS_DLL=-Zdll

# System dependent source files belonging to Crystal Space. These are
# files which are used by the executables.
SYS_SRC_EXE=system/def_sys/printf.cpp system/def_sys/timing.cpp system/def_sys/fopen.cpp\
	system/os2/csos2.cpp system/os2/libDIVE.cpp system/os2/libDIVEpm.cpp \
	system/os2/loadlib.cpp system/os2/scancode.cpp

# System dependent source files belonging to csregsvr.
SYS_SRC_REG=system/os2/loadlib.cpp

# System dependent source files belonging to Crystal Space. These are
# files which are used by the shared libraries.
SYS_SRC_DLL=system/os2/loadlib.cpp system/os2/dllentry.cpp

# Where to put the dynamic libraries on this system?
DLL_DIR=

# Does this OS have native COM support?
NATIVE_COM=no

# The C compiler.
CC=gcc

# The C++ compiler.
CXX=g++

# The linker.
LINK=g++

# The Resource Compiler. Older versions do not like the "-n" flag
RC=rc
RCFLAGS=-n -r -I .

# Command sequence for creating a directory.
# Note that directories will have forward slashes. Please
# make sure that this command accepts that (or use 'subst' first).
ifneq (,$(findstring cmd,$(SHELL)))
  MKDIR = mkdir $(subst /,\,$(@:/=))
else
  MKDIR = mkdir $@
endif

# The command to remove all specified files.
RM=rm -f

# The command to remove a directory tree.
RMDIR=rm -rf

# Extra parameters for 'sed' which are used for doing 'make depend'.
SYS_SED_DEPEND=-e "s|\.obj|.o|g"

endif # ifeq ($(MAKESECTION),defines)

#-------------------------------------------------------------- postdefines ---#
ifeq ($(MAKESECTION),postdefines)

# The 2D OS/2 DIVE driver
ifeq ($(USE_DLL),no)
  CSDIVE=$(OUT)csdive$(LIB)
else
  CSDIVE=csdive$(DLL)
endif
SRC_CSDIVE = $(wildcard system/os2/csdive/*.cpp \
	system/fonts.cpp render/graph2d.cpp render/igraph2d.cpp \
	com/guids.cpp com/cscom.cpp com/inttable.cpp \
	geometry/csrect.cpp $(SYS_SRC_DLL))
CSDIVE_OBJ = $(addprefix $(OUT),$(SRC_CSDIVE:.cpp=.o))

# Resource file for DIVE library
RES=$(OUTOS)libDIVE.res

# Additional library which contains functions undefined in os2.a
CSOS2LIB=$(OUTOS)csos2$(LIB)

# Tell MAKE that most executables depend on video and rendering drivers
cs mz wt si pe: soft csdive

endif # ifeq ($(MAKESECTION),postdefines)

#------------------------------------------------------------------ targets ---#
ifeq ($(MAKESECTION),targets)

.PHONY: csdive os2depend os2clean os2cleancs os2cleanlib

# Chain rules
all: csdive
depend: os2depend
clean: os2clean
cleancs: os2cleancs
cleanlib: os2cleanlib
csdive: $(OUTDIRS) $(CSOS2LIB) $(CSDIVE)

# Extra dependencies for targets
ifneq ($(USE_DLL),no)
$(SOFTRNDR):	system/os2/softrndr.def
else
DEP_RES = $(RES)
endif
$(WALKTEST):	$(DEP_RES) system/os2/cryst.def
$(MAZED):	$(DEP_RES) system/os2/mazed.def
$(CSWSTEST):	$(DEP_RES) system/os2/mazed.def
$(SIMPLE):	$(DEP_RES) system/os2/mazed.def
$(PRIMEDIT):	$(DEP_RES) system/os2/mazed.def
$(CSREGSVR):	system/os2/csregsvr.def

ifeq ($(MAKE_DLL),no)
$(CSDIVE): $(CSDIVE_OBJ)
	$(AR) cr $@ $^
else
$(CSDIVE): $(CSDIVE_OBJ) $(RES) system/os2/csdive/csdive.def
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS) -L $(OUTOS) -lcsos2
endif

$(RES): system/os2/csdive/libDIVE.rc
	$(RC) $(RCFLAGS) $^ $@

$(CSOS2LIB): system/os2/csdive/csdive.imp
	emximp -o $@ $<

os2clean:
	$(RM) $(CSDIVE)

os2cleanlib:
	$(RM) $(CSDIVE_OBJ) $(CSDIVE)

os2depend:
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_CSDIVE) | sed $(SED_DEPEND) >$(OUTOS)csdive.dep

-include $(OUTOS)csdive.dep

endif # ifeq ($(MAKESECTION),targets)
