############################################
# These are the dependencies for the Gnu-Win32 port of Crystal Space.
# This is an include file for the Gnu-Win32 makefiles.
# It assumes, for NT4 Machines, that the developer has applied
# SP3, available for free download from the MS Website.
# Don't use this directly as a makefile.
############################################

.SUFFIXES: .dep
.PHONY: all clean doc depend nothing

############################################
# The system-dependent part of makefile could/should override
# the following variables:
#
# Variable      | Default value | Example | Description
# --------------+---------------+---------+------------------------------
# CC            |               |         | C compiler
# CXX (*)       |               |         | C++ compiler
# LINK (*)      |               |         | linker
# LFLAGS        |     HOW       |         | Linker flags
# OSP           |     none      | Win32|Linux|Unix|Dos|Mac | Defines System Platform
# SYSFLAGS (*)  |               | -D...   | System definition flags
# E             |               | .exe    | Executable file extension
# DO_SHM        |     no        | yes|no  | Use libshm?
# IS_OPENGL     |   (depends)   | yes|no  | Use OpenGL API?
# DO_D3D        |   (depends)   | yes|no  | Use Direct3D? (windows)
# DO_ASM        |     yes       | yes|no  | Include ASM statements?
# DO_PNG        |     yes       | yes|no  | Support PNG?
# DO_GIF        |     yes       | yes|no  | Support GIF?
# DO_TGA        |     yes       | yes|no  | Support TGA?
# DO_JPG        |     yes       | yes|no  | Support JPEG?
# DO_CD 	|     yes	| yes|no  | Support Collision Detection?
# DO_SOUND	|     yes	| yes|no  | Sound support (Windows only currently)
# SYS_PNG_LIBS  |     -lpng     |         | PNG support library
# SYS_JPG_LIBS  |     -ljpeg    |         | JPEG support library
# SYS_Z_LIBS    |     -lz       |         | zlib library
# SYS_SRC       |               |         | System-dependent sources (without the 'system' part)
# SYS_INC_PATH  |               |         | System-dependent include path(s)
# SYS_LIB_PATH  |               |         | System-dependent library path(s)
# SYS_LIBS      |               |         | System-dependent libraries
# WALKTEST      |               |         | CrystalSpace test executable
# SIMPLE        |               |         | CrystalSpace simplest example executable
# CSWSTEST      |               |         | CrystalSpace Windowing System test executable
# MAZED         |               |         | Maze editor executable
############################################

############################################
# Choose optimization, debugging or profiling compile:
# OPT should be one of 'optimize', 'debug', or 'profile'.
OPT=optimize
#OPT=debug
#OPT=profile
############################################

############################################
# Targets
ifndef WALKTEST
WALKTEST=cryst$E
endif
ifndef SIMPLE
SIMPLE=simple$E
endif
ifndef CSWSTEST
CSWSTEST=cswstest$E
endif
ifndef MAZED
MAZED=mazed$E
endif
ifndef PRIMEDIT
PRIMEDIT=primedit$E
endif
############################################

############################################
# Comment the following line if you specifically
# want no assembler routines (if supported by your
# architecture). This will of course make Crystal
# Space run slower.
ifndef DO_ASM
ifeq ($(OPT),debug)
	DO_ASM=no
else
ifeq ($(OPT),profile)
	DO_ASM=no
else
	DO_ASM=yes
endif
endif
endif
############################################

############################################
# Comment the following line if your X server
# doesn't support the shared memory extension (only
# for X windows versions of Crystal Space).
# Note that you can still disable the shared memory
# extension with the -noshm commandline option even
# if you have enabled SHM support. So even if
# your server doesn't support SHM you should probably
# keep this enabled unless you miss the needed libraries.
ifndef DO_SHM
DO_SHM=no
endif
############################################

############################################
# With the following lines you can control which types
# of graphics formats that you want Crystal Space to
# support. This is mainly to reduce the size of the
# Crystal Space executable and possibly to avoid having
# to include 'libpng' (for PNG support).
# Comment a line if you want to remove support for
# that specific graphic format.
ifndef DO_PNG
DO_PNG=yes
endif
ifndef DO_GIF
DO_GIF=yes
endif
#ifndef DO_TGA
#DO_TGA=yes
#endif
#ifndef DO_JPG
#DO_JPG=no
#endif
############################################

############################################
# Comment out the following line if you do not want
# collision detection.
ifndef DO_CD
DO_CD=yes
endif
############################################

############################################
# Comment out the following line if you don't want
# sound support. Currently only Windows supports this.
ifndef DO_SOUND
DO_SOUND=yes
endif
############################################

############################################
# Uncomment the following line if you want to
# enable the memory debugger that is built in.
# This is only recommended for debugging purposes
# as it makes Crystal Space a lot slower.
# If you want more extensive memory checking
# then you should also enable MEM_CHECK_EXTENSIVE.
# In this mode CS also keeps all allocated memory
# even after it has been freed and fills it with
# some special value. That way you can detect
# if memory is still being used after it has been
# freed. Define MEM_CHECK_FILL symbol so that
# newly allocated memory will be filled with
# garbage to detect memory areas that are used
# uninitialized (since most OSes fills allocated
# memory with zeros).
#MEM=-DMEM_CHECK -DMEM_CHECK_FILL
#MEM=-DMEM_CHECK -DMEM_CHECK_EXTENSIVE
############################################

############################################
#  NT4 console builds can handle Posix based references, but
# must do so with the '\' instead of the '/'
#
# Object Directory setup for object files
ifndef OUTBASE
  OUTBASE=out/
endif
ifndef OUT
  _OUT=$(OUTBASE)$(patsubst -DOS_%,%,$(filter -DOS_%,$(SYSFLAGS)))_$(patsubst -DDISP_%,%,$(filter -DDISP_%,$(SYSFLAGS)))
  ifeq ($(OPT),debug)
    OUT=$(addsuffix _d,$(_OUT))/
  else
    ifeq ($(OPT),profile)
      OUT=$(addsuffix _p,$(_OUT))/
    else
      OUT=$(_OUT)/
    endif
  endif
endif
############################################

############################################
# Directory for object files -- Standard Posix
#	Use this for -DOS_UNIX and -DOS_DOS builds.
#ifndef OUTBASE
#  OUTBASE=out/
#endif
#ifndef OUT
#  _OUT=$(OUTBASE)$(patsubst -DOS_%,%,$(filter -DOS_%,$(SYSFLAGS)))
#  ifeq ($(OPT),debug)
#	OUT=$(addsuffix _d,$(_OUT))/
# else
#	OUT=$(_OUT)/
#  endif
#endif
############################################

############################################
# Canned command sequence for creating object directories
# Modified for an NT4 Gnu-Win32 development with the NT4
# Resource Kit(POSIX)installed -- comment if you are not
# using NT4 Command Prompt or do not have the NT4 Posix enhancements.
ifndef MKDIR
  MKDIR = c:/posix/mkdir $(@:/=)
endif
#
# The next command sequence is for creating the object directories 
# on a Unix based machine -- uncomment if your NT4 is an Alpha
# or other non-Win32 platform, also works for -DOS_DOS or a bash
# shell.
#ifndef MKDIR
#  MKDIR = mkdir $(@:/=)
#endif
############################################

S_ENGINE=engine/*.cpp engine/polygon/*.cpp engine/basic/*.cpp engine/light/*.cpp \
	 engine/objects/*.cpp engine/2d/*.cpp
S_GENERAL=general/*.cpp memory.cpp
S_LOADER=general/loader/*.cpp
S_GEOMETRY=geometry/*.cpp
S_RENDER=render/*.cpp render/software/*.cpp render/softw16/*.cpp render/direct3d/*.cpp
S_SCRIPTS=scripts/*.cpp
S_UTIL=util/*.cpp util/gfx/image.cpp util/gfx/pcx.cpp \
	util/input/*.cpp util/sound/*.cpp
S_SYSTEM=system/*.cpp $(addprefix system/,$(SYS_SRC))
S_CSWS=csws/*.cpp
ifeq ($(DO_CD),yes)
	S_CD=colldet/*.cpp
endif
ifeq ($(DO_SOUND),yes)
	S_SOUND=sound/*.cpp
endif

ifeq ($(DO_PNG),yes)
	S_GFORM+=util/gfx/pngimage.cpp
	ifdef SYS_PNG_LIBS
		PNG_LIBS=$(SYS_PNG_LIBS)
	else
		PNG_LIBS=-lpng
	endif
	PNG=-DDO_PNG
endif
ifeq ($(DO_GIF),yes)
	S_GFORM+=util/gfx/gifimage.cpp
	GIF=-DDO_GIF
endif
ifeq ($(DO_TGA),yes)
	S_GFORM+=util/gfx/tgaimage.cpp
	TGA=-DDO_TGA
endif
ifeq ($(DO_JPG),yes)
	S_GFORM+=util/gfx/jpgimage.cpp
	ifdef SYS_JPG_LIBS
		JPG_LIBS=$(SYS_JPG_LIBS)
	else
		JPG_LIBS=-ljpeg
	endif
	JPG=-DDO_JPG
endif
ifdef SYS_Z_LIBS
	Z_LIBS=$(SYS_Z_LIBS)
else
	Z_LIBS=-lz
endif

SRC=$(wildcard $(S_ENGINE) $(S_GENERAL) $(S_GEOMETRY) $(S_RENDER) \
	$(S_SCRIPTS) $(S_UTIL) $(S_SYSTEM) $(S_GFORM) $(S_LOADER) \
	$(S_CD) $(S_SOUND))

SRC_WALKTEST = $(wildcard general/walktest/*.cpp)
SRC_MAZED    = $(wildcard $(S_CSWS) general/mazed/*.cpp)
SRC_CSWSTEST = $(wildcard $(S_CSWS) general/cswstest/*.cpp)
SRC_SIMPLE   = $(wildcard general/simple/*.cpp)
SRC_PRIMEDIT = $(wildcard general/primedit/*.cpp) util/util.cpp

HEADER=$(wildcard *.h */*.h */*/*.h */*/*/*.h)

OBJ1=$(SRC:.cpp=.o)
OBJ2=$(OBJ1:.S=.o)
OBJ          = $(addprefix $(OUT),$(OBJ2:.s=.o))
WALKTEST_OBJ = $(addprefix $(OUT),$(SRC_WALKTEST:.cpp=.o))
MAZED_OBJ    = $(addprefix $(OUT),$(SRC_MAZED:.cpp=.o))
CSWSTEST_OBJ = $(addprefix $(OUT),$(SRC_CSWSTEST:.cpp=.o))
SIMPLE_OBJ   = $(addprefix $(OUT),$(SRC_SIMPLE:.cpp=.o))
PRIMEDIT_OBJ = $(addprefix $(OUT),$(SRC_PRIMEDIT:.cpp=.o))

ifeq ($(DO_ASM),yes)
else
	ASM=-DNO_ASSEMBLER
endif
ifeq ($(DO_SHM),yes)
	XSHARED=-DDO_SHM
endif

ifeq ($(OPT),optimize)
	HOW=$(HOW_OPTIMIZE)
endif

ifeq ($(OPT),debug)
	HOW=$(HOW_DEBUG)
endif

ifeq ($(OPT),profile)
	HOW=$(HOW_PROFILE)
endif

# Collision detection
ifeq ($(DO_CD),yes)
# Compile definitions.
CD_DEF= -DDO_CD -DVTYPE=float
endif                                

ifndef LFLAGS
LFLAGS=$(HOW)
endif

INC_PATH=$(SYS_INC_PATH) $(CD_DEF) -I . -I ../zlib -I ../libpng -I ../libjpeg
LIB_PATH=$(SYS_LIB_PATH) -L $(OUT) -L ../zlib -L ../libpng -L ../libjpeg
LIBS=$(SYS_LIBS) $(Z_LIBS) $(PNG_LIBS) $(JPG_LIBS)
CFLAGS=$(HOW) $(SYSFLAGS) $(ASM) $(MEM) $(XSHARED) $(D3D) $(PNG) $(GIF) $(TGA) $(JPG)
SED_DEPEND=-e "s|^\([^ ].*\)\(\..*: *\)\(.*\1\)\(\.c.*\)|$(OUT)\3\2\3\4|" $(SYS_SED_DEPEND)

# Directories for output files
OUTDIRS=$(OUTBASE) $(OUT)\
 $(addprefix $(OUT),$(filter-out $(OUTBASE)%,$(sort $(dir $(wildcard */* */*/*)))))

help:
	@echo "----------------------------------------------------------------"
	@echo "Choose any of the following targets:"
	@echo "  make cs       To make the Crystal Space demo"
	@echo "  make wt       To make the CSWS test program"
	@echo "  make si       To make the Simple demo"
	@echo "  make mz       To make MazeD, the CS editor (NOT FINISHED)"
	@echo "  make pe       To make PrimEdit, the primitive polygon editor (NOT FINISHED)"
	@echo "  make all      To make all"
	@echo "----------------------------------------------------------------"
	@echo "  make doc      Make documentation using Doc++"
	@echo "  make clean    Clean everything"
	@echo "  make depend   Make dependencies (recommended!)"
	@echo "----------------------------------------------------------------"

all: $(OUTDIRS) $(WALKTEST) $(MAZED) $(CSWSTEST) $(SIMPLE) $(PRIMEDIT)

cs: $(OUTDIRS) $(WALKTEST)

wt: $(OUTDIRS) $(CSWSTEST)

si: $(OUTDIRS) $(SIMPLE)

mz: $(OUTDIRS) $(MAZED)

pe: $(OUTDIRS) $(PRIMEDIT)

$(WALKTEST): $(OBJ) $(WALKTEST_OBJ)
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS)

$(MAZED): $(OBJ) $(MAZED_OBJ)
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS)

$(CSWSTEST): $(OBJ) $(CSWSTEST_OBJ)
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS)

$(SIMPLE): $(OBJ) $(SIMPLE_OBJ)
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS)

$(PRIMEDIT): $(OBJ) $(PRIMEDIT_OBJ)
	$(LINK) $(LFLAGS) -o $@ $^ $(LIB_PATH) $(LIBS)

clean:
	rm -rf out
	rm -f debug.txt \
	 $(WALKTEST) $(MAZED) $(CSWSTEST) $(SIMPLE) $(PRIMEDIT)

doc:
	doc++ -H -d ../csdoc -f $(HEADER)

depend: $(OUTBASE) $(OUT)
	-rm $(OUT)*.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC) | sed $(SED_DEPEND) >$(OUT)cs.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_WALKTEST) | sed $(SED_DEPEND) >$(OUT)walktest.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_MAZED) | sed $(SED_DEPEND) >$(OUT)mazed.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_SIMPLE) | sed $(SED_DEPEND) >$(OUT)simple.dep
	$(CC) -MM $(CFLAGS) $(INC_PATH) $(SRC_PRIMEDIT) | sed $(SED_DEPEND) >$(OUT)primedit.dep

$(OUT)%.o : %.cpp
	$(CXX) -c $(CFLAGS) $(INC_PATH) -o $@ $<

$(OUT)%.o : %.c
	$(CC) -c $(CFLAGS) $(INC_PATH) -o $@ $<

$(OUT)%.o : %.S
	$(CC) -c $(CFLAGS) $< -o $@

$(OUT)%.o : %.s
	$(CC) -c $(CFLAGS) $< -o $@

$(OUT)%.o : %.asm
	$(CC) -c $(CFLAGS) $< -o %@

$(OUTDIRS):
	$(MKDIR)

-include $(OUT)cs.dep
-include $(OUT)walktest.dep
-include $(OUT)mazed.dep
-include $(OUT)simple.dep
-include $(OUT)primedit.dep
