#####################################################################
## The compiler/os/processor flags:                                ##
SYSFLAGS=-DCOMP_WCC -DOS_DOS -DPROC_INTEL -DDISP_D
#####################################################################

#####################################################################
## Choose between optimizing                                       ##
##  486                                                            ##
#HOW_OPTIMIZE=/fpi87 /fp3 /4r /omantril /s /zp4
##  Pentium, k5, 5x86                                              ##
HOW_OPTIMIZE=/fpi87 /fp5 /5r /omantril /s /zp4
#/fpi87 /fp5 /5r /omantril /s /zp4
##  PentiumPro, Pentium2, K6, 6x86                                 ##
#HOW_OPTIMIZE=/fpi87 /fp6 /6r /obpmantrile /s /zp4
#####################################################################

#.SUFFIXES:
#.SUFFIXES: .cpp .exe .dep
.EXTENSIONS:
.EXTENSIONS: .exe .obj .c .cpp

# Tools
CC=wcc386
CXX=wpp386
LINK=wlink
#LINK=wcl386
LFLAGS=/bt=dos /l=dos4g /k512000

WALKTEST=cryst.exe
MAZED=MazeD.exe
SIMPLE=simple.exe
CSWSTEST=mainwst.exe

all: install $(WALKTEST)

cs: $(WALKTEST)
mz: $(MAZED)
wt: $(CSWSTEST)
si: $(SIMPLE)

SYS_HEADER=wcc/watcom.h
SYS_LIBS=system\wcc\video.lib
SYS_INC_PATH=-i=$(%WATCOM)\H
SYS_LIB_PATH=$(%WATCOM)\LIB386 $(%WATCOM)\LIB386\DOS

#####################################################################
# These are the dependencies for Crystal Space.
# This is an include file for the makefiles.
# Don't use this directly as a makefile.
#####################################################################

#####################################################################
# The system-dependent part of makefile could/should override
# the following variables:
#
# Variable      | Default value | Range  | Description
# --------------+---------------+--------+------------------------------
# CC            |               |        | C compiler
# CXX (*)       |               |        | C++ compiler
# LINK (*)      |               |        | linker
# LFLAGS        |       CFLAGS  |        | Linker flags
# SYSFLAGS (*)  |               | -D...  | System definition flags
# DO_SHM        |       no      | yes|no | Use libshm?
# DO_D3D        |       no      | yes|no | Use Direct3D? (windows)
# DO_ASM        |       yes     | yes|no | Include ASM statements?
# DO_PNG        |       yes     | yse|no | Support PNG?
# DO_GIF        |       yes     | yes|no | Support GIF?
# DO_TGA        |       yes     | yes|no | Support TGA?
# DO_JPG        |       yes     | yes|no | Support JPEG?
# SYS_PNG_LIBS  |       -lpng   |        | PNG support library
# SYS_JPG_LIBS  |       -ljpeg  |        | JPEG support library
# SYS_Z_LIBS    |       -lz     |        | zlib library
# SYS_SRC       |               |        | System-dependent sources (without the 'system' part)
# SYS_INC_PATH  |               |        | System-dependent include path(s)
# SYS_LIB_PATH  |               |        | System-dependent library path(s)
# SYS_LIBS      |               |        | System-dependent libraries
# WALKTEST(*)   |               |        | walktest executable
# MAZED         |               |        | maze editor executable
# O             |       .o      | .o .obj| Object file extension
#####################################################################

#####################################################################
# Choose optimization, debugging or profiling compile:
# OPT should be one of 'optimize', 'debug', or 'profile'.
OPT=optimize
HOW=
#####################################################################

#####################################################################
# Comment the following line if you specifically
# want no assembler routines (if supported by your
# architecture). This will of course make Crystal
# Space run slower.
!ifndef DO_ASM
DO_ASM=no
!endif
#####################################################################

#####################################################################
# Comment the following line if your X server
# doesn't support the shared memory extension (only
# for X windows versions of Crystal Space).
# Note that you can still disable the shared memory
# extension with the -noshm commandline option even
# if you have enabled SHM support. So even if
# your server doesn't support SHM you should probably
# keep this enabled unless you miss the needed libraries.
!ifndef DO_SHM
DO_SHM=no
XSHARED=
!endif
#####################################################################

#####################################################################
# Comment the following line if you don't
# want Direct3D support (only for Windows)
!ifndef DO_D3D
DO_D3D=no
!endif
#####################################################################

#####################################################################
# With the following lines you can control which types
# of graphics formats that you want Crystal Space to
# support. This is mainly to reduce the size of the
# Crystal Space executable and possibly to avoid having
# to include 'libpng' (for PNG support).
# Comment a line if you want to remove support for
# that specific graphic format.
!ifndef DO_PNG
DO_PNG=yes
PNG_LIBS=
PNG_OBJ=
PNG=
!endif
!ifndef DO_GIF
DO_GIF=yes
GIF_OBJ=
GIF=
!endif
!ifndef DO_TGA
DO_TGA=yes
TGA_OBJ=
TGA=
!endif
!ifndef DO_JPG
DO_JPG=yes
JPG_LIBS=
JPG_OBJ=
JPG=
!endif
#####################################################################

#####################################################################
# Uncomment the following line if you want to
# enable the memory debugger that is built in.
# This is only recommended for debugging purposes
# as it makes Crystal Space a lot slower.
# If you want more extensive memory checking
# then you should also enable MEM_CHECK_EXTENSIVE.
# In this mode CS also keeps all allocated memory
# even after it has been freed and fills it with
# some special value. That way you can detect
# if memory is still being used after it has been
# freed.
#MEM=-DMEM_CHECK
#MEM=-DMEM_CHECK -DMEM_CHECK_EXTENSIVE
MEM=
#####################################################################

!ifeq DO_ASM yes
ASM=
!else
ASM=-DNO_ASSEMBLER
!endif

!ifeq DO_SHM yes
XSHARED=-DDO_SHM
!endif

!ifdef DO_D3D yes
D3D=
!else
D3D=-DNO_DIRECT3D
!endif

!ifndef LFLAGS
LFLAGS=$(CFLAGS)
!endif

!ifeq OPT optimize
HOW=$(HOW_OPTIMIZE)
!endif

!ifeq OPT debug
HOW=$(HOW_DEBUG)
!endif

!ifeq OPT profile
HOW=$(HOW_PROFILE)
!endif


GENERAL =general\
main.obj: $(GENERAL)walktest\main.cpp
  $(CXX) $(GENERAL)walktest\$^& @walktest.flg
keys.obj: $(GENERAL)walktest\keys.cpp
  $(CXX) $(GENERAL)walktest\$^& @walktest.flg
scon.obj: $(GENERAL)walktest\scon.cpp
  $(CXX) $(GENERAL)walktest\$^& @walktest.flg

mazed.obj: $(GENERAL)mazed\mazed.cpp
  $(CXX) $(GENERAL)mazed\$^& @walktest.flg
me_app.obj: $(GENERAL)mazed\me_app.cpp
  $(CXX) $(GENERAL)mazed\$^& @walktest.flg
me_3d.obj: $(GENERAL)mazed\me_3d.cpp
  $(CXX) $(GENERAL)mazed\$^& @walktest.flg
me_draft.obj: $(GENERAL)mazed\me_draft.cpp
  $(CXX) $(GENERAL)mazed\$^& @walktest.flg
me_model.obj: $(GENERAL)mazed\me_model.cpp
  $(CXX) $(GENERAL)mazed\$^& @walktest.flg

WALKTEST_OBJ =main.obj keys.obj scon.obj

mains.obj: $(GENERAL)simple\mains.cpp
  $(CXX) $(GENERAL)simple\$^& @walktest.flg

SIMPLE_OBJ =mains.obj

#console.obj: $(GENERAL)console.cpp
#  $(CXX) $(GENERAL)$^& @walktest.flg
command.obj: $(GENERAL)command.cpp
  $(CXX) $(GENERAL)$^& @walktest.flg
csview.obj: $(GENERAL)csview.cpp
  $(CXX) $(GENERAL)$^& @walktest.flg

GENERAL_OBJ = command.obj csview.obj

UTIL =util\
util.obj: $(UTIL)util.cpp
  $(CXX) $(UTIL)$^& @walktest.flg
inifile.obj: $(UTIL)inifile.cpp
  $(CXX) $(UTIL)$^& @walktest.flg
token.obj: $(UTIL)token.cpp
  $(CXX) $(UTIL)$^& @walktest.flg
parser.obj: $(UTIL)parser.cpp
  $(CXX) $(UTIL)$^& @walktest.flg
memory.obj: $(UTIL)memory.cpp
  $(CXX) $(UTIL)$^& @walktest.flg
archive.obj: $(UTIL)archive.cpp
  $(CXX) $(UTIL)$^& @walktest.flg

GFX =gfx\
image.obj: $(UTIL)$(GFX)image.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg
pcx.obj: $(UTIL)$(GFX)pcx.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg

pngimage.obj: $(UTIL)$(GFX)pngimage.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg
gifimage.obj: $(UTIL)$(GFX)gifimage.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg
tgaimage.obj: $(UTIL)$(GFX)tgaimage.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg
jpgimage.obj: $(UTIL)$(GFX)jpgimage.cpp
  $(CXX) $(UTIL)$(GFX)$^& @walktest.flg

INPUT =input\
csevent.obj:  $(UTIL)$(INPUT)csevent.cpp
  $(CXX) $(UTIL)$(INPUT)$^& @walktest.flg
cseventq.obj: $(UTIL)$(INPUT)cseventq.cpp
  $(CXX) $(UTIL)$(INPUT)$^& @walktest.flg
csinput.obj:  $(UTIL)$(INPUT)csinput.cpp
  $(CXX) $(UTIL)$(INPUT)$^& @walktest.flg

UTIL_OBJ =util.obj inifile.obj token.obj parser.obj memory.obj archive.obj &
image.obj pcx.obj csevent.obj cseventq.obj csinput.obj

SYSTEM =system\
system.obj: $(SYSTEM)system.cpp
  $(CXX) $(SYSTEM)$^& @walktest.flg
csendian.obj: $(SYSTEM)csendian.cpp
  $(CXX) $(SYSTEM)$^& @walktest.flg
fonts.obj: $(SYSTEM)fonts.cpp
  $(CXX) $(SYSTEM)$^& @walktest.flg

DEF_SYS =def_sys\
timing.obj: $(SYSTEM)$(DEF_SYS)timing.cpp
  $(CXX) $(SYSTEM)$(DEF_SYS)$^& @walktest.flg

WCC =wcc\
csdoswat.obj: $(SYSTEM)$(WCC)csdoswat.cpp
  $(CXX) $(SYSTEM)$(WCC)$^& @walktest.flg
fopen.obj: $(SYSTEM)$(WCC)fopen.cpp
  $(CXX) $(SYSTEM)$(WCC)$^& @walktest.flg
keyboard.obj: $(SYSTEM)$(WCC)keyboard.cpp
  $(CXX) $(SYSTEM)$(WCC)$^& @walktest.flg
mouse.obj: $(SYSTEM)$(WCC)mouse.cpp
  $(CXX) $(SYSTEM)$(WCC)$^& @walktest.flg
printf.obj: $(SYSTEM)$(WCC)printf.cpp
  $(CXX) $(SYSTEM)$(WCC)$^& @walktest.flg

SYSTEM_OBJ = system.obj csendian.obj fonts.obj printf.obj timing.obj &
csdoswat.obj fopen.obj keyboard.obj mouse.obj

GEOMETRY =geometry\
math3d.obj: $(GEOMETRY)math3d.cpp
  $(CXX) $(GEOMETRY)$^& @walktest.flg
bsp.obj: $(GEOMETRY)bsp.cpp
  $(CXX) $(GEOMETRY)$^& @walktest.flg
polyclip.obj: $(GEOMETRY)polyclip.cpp
  $(CXX) $(GEOMETRY)$^& @walktest.flg

GEOMETRY_OBJ =polyclip.obj math3d.obj bsp.obj

ENGINE=engine\
sector.obj: $(ENGINE)sector.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
world.obj: $(ENGINE)world.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
camera.obj: $(ENGINE)camera.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
stats.obj: $(ENGINE)stats.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
texture.obj: $(ENGINE)texture.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
wirefrm.obj: $(ENGINE)wirefrm.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
textrans.obj: $(ENGINE)textrans.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
physics.obj: $(ENGINE)physics.cpp
  $(CXX) $(ENGINE)$^& @walktest.flg
csspr2d.obj: $(ENGINE)2d\csspr2d.cpp
  $(CXX) $(ENGINE)2d\$^& @walktest.flg

BASIC =basic\
csvector.obj: $(ENGINE)$(BASIC)csvector.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg
csstrvec.obj: $(ENGINE)$(BASIC)csstrvec.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg
csobjvec.obj: $(ENGINE)$(BASIC)csobjvec.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg
csobject.obj: $(ENGINE)$(BASIC)csobject.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg
csrect.obj: $(ENGINE)$(BASIC)csrect.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg
polyset.obj: $(ENGINE)$(BASIC)polyset.cpp
  $(CXX) $(ENGINE)$(BASIC)$^& @walktest.flg

OBJECTS =objects\
cssprite.obj: $(ENGINE)$(OBJECTS)cssprite.cpp
  $(CXX) $(ENGINE)$(OBJECTS)$^& @walktest.flg
thingtpl.obj: $(ENGINE)$(OBJECTS)thingtpl.cpp
  $(CXX) $(ENGINE)$(OBJECTS)$^& @walktest.flg
thing.obj: $(ENGINE)$(OBJECTS)thing.cpp
  $(CXX) $(ENGINE)$(OBJECTS)$^& @walktest.flg

POLYGON =polygon\
polygon.obj: $(ENGINE)$(POLYGON)polygon.cpp
  $(CXX) $(ENGINE)$(POLYGON)$^& @walktest.flg
polytext.obj: $(ENGINE)$(POLYGON)polytext.cpp
  $(CXX) $(ENGINE)$(POLYGON)$^& @walktest.flg
polyplan.obj: $(ENGINE)$(POLYGON)polyplan.cpp
  $(CXX) $(ENGINE)$(POLYGON)$^& @walktest.flg
portal.obj: $(ENGINE)$(POLYGON)portal.cpp
  $(CXX) $(ENGINE)$(POLYGON)$^& @walktest.flg

LIGHT =light\
light.obj: $(ENGINE)$(LIGHT)light.cpp
  $(CXX) $(ENGINE)$(LIGHT)$^& @walktest.flg
dynlight.obj: $(ENGINE)$(LIGHT)dynlight.cpp
  $(CXX) $(ENGINE)$(LIGHT)$^& @walktest.flg
fltlight.obj: $(ENGINE)$(LIGHT)fltlight.cpp
  $(CXX) $(ENGINE)$(LIGHT)$^& @walktest.flg
lightmap.obj: $(ENGINE)$(LIGHT)lightmap.cpp
  $(CXX) $(ENGINE)$(LIGHT)$^& @walktest.flg

ENGINE_OBJ =sector.obj world.obj camera.obj texture.obj wirefrm.obj textrans.obj &
           physics.obj csspr2d.obj csvector.obj csstrvec.obj csobjvec.obj &
           csobject.obj csrect.obj polyset.obj cssprite.obj thingtpl.obj thing.obj &
           polygon.obj polytext.obj polyplan.obj light.obj dynlight.obj &
           fltlight.obj lightmap.obj stats.obj portal.obj

RENDER =render\
graph2d.obj: $(RENDER)graph2d.cpp
  $(CXX) $(RENDER)$^& @walktest.flg
graph3d.obj: $(RENDER)graph3d.cpp
  $(CXX) $(RENDER)$^& @walktest.flg
hicache.obj: $(RENDER)hicache.cpp
  $(CXX) $(RENDER)$^& @walktest.flg

SOFTWARE =software\
scan.obj: $(RENDER)$(SOFTWARE)scan.cpp
  $(CXX) $(RENDER)$(SOFTWARE)$^& @walktest.flg
tcache.obj: $(RENDER)$(SOFTWARE)tcache.cpp
  $(CXX) $(RENDER)$(SOFTWARE)$^& @walktest.flg

SOFTW16 =softw16\
scan16.obj: $(RENDER)$(SOFTW16)scan16.cpp
  $(CXX) $(RENDER)$(SOFTW16)$^& @walktest.flg
tcache16.obj: $(RENDER)$(SOFTW16)tcache16.cpp
  $(CXX) $(RENDER)$(SOFTW16)$^& @walktest.flg

RENDER_OBJ =graph2d.obj graph3d.obj hicache.obj scan.obj tcache.obj &
           scan16.obj tcache16.obj

#!ifdef DO_D3D yes
#DIRECT3D =direct3d\
#d3dcache.obj: $(RENDER)$(DIRECT3D)d3dcache.cpp
#  $(CXX) $(RENDER)$(DIRECT3D)$^& @walktest.flg
#g3d_d3d.obj: $(RENDER)$(DIRECT3D)g3d_d3d.cpp
#  $(CXX) $(RENDER)$(DIRECT3D)$^& @walktest.flg
#RENDER_OBJ =$(RENDER_OBJ) d3dcache.obj g3d_d3d.obj
#!endif

SCRIPTS =scripts\
primscri.obj: $(SCRIPTS)primscri.cpp
  $(CXX) $(SCRIPTS)$^& @walktest.flg
csscript.obj: $(SCRIPTS)csscript.cpp
  $(CXX) $(SCRIPTS)$^& @walktest.flg
intscri.obj: $(SCRIPTS)intscri.cpp
  $(CXX) $(SCRIPTS)$^& @walktest.flg

SCRIPTS_OBJ =primscri.obj csscript.obj intscri.obj

CSWS =csws\
csapp.obj: $(CSWS)csapp.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csbutton.obj: $(CSWS)csbutton.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cscomp.obj: $(CSWS)cscomp.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csdialog.obj: $(CSWS)csdialog.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csgfxppl.obj: $(CSWS)csgfxppl.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csiline.obj: $(CSWS)csiline.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cslistbx.obj: $(CSWS)cslistbx.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csmenu.obj: $(CSWS)csmenu.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csmouse.obj: $(CSWS)csmouse.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csscrbar.obj: $(CSWS)csscrbar.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csstatic.obj: $(CSWS)csstatic.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cstimer.obj: $(CSWS)cstimer.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
csttlbar.obj: $(CSWS)csttlbar.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cswindow.obj: $(CSWS)cswindow.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cswsaux.obj: $(CSWS)cswsaux.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cswspal.obj: $(CSWS)cswspal.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cswsutil.obj: $(CSWS)cswsutil.cpp
  $(CXX) $(CSWS)$^& @walktest.flg
cstulbar.obj: $(CSWS)cstulbar.cpp
  $(CXX) $(CSWS)$^& @walktest.flg

MAZED_OBJ =mazed.obj me_app.obj me_3d.obj me_draft.obj me_model.obj &
           csapp.obj csbutton.obj cscomp.obj csdialog.obj csgfxppl.obj &
           csiline.obj cslistbx.obj csmenu.obj csmouse.obj csscrbar.obj &
           csstatic.obj cstimer.obj csttlbar.obj cswindow.obj cswsaux.obj &
           cswspal.obj cswsutil.obj cstulbar.obj

mainwst.obj: $(GENERAL)cswstest\mainwst.cpp
  $(CXX) $(GENERAL)cswstest\$^& @walktest.flg

CSWSTEST_OBJ =mainwst.obj &
           csapp.obj csbutton.obj cscomp.obj csdialog.obj csgfxppl.obj &
           csiline.obj cslistbx.obj csmenu.obj csmouse.obj csscrbar.obj &
           csstatic.obj cstimer.obj csttlbar.obj cswindow.obj cswsaux.obj &
           cswspal.obj cswsutil.obj

!ifeq DO_PNG yes
PNG_OBJ =pngimage.obj
PNG_LIBS=..\libpng\libpngf.lib
PNG=-DDO_PNG
!endif

!ifeq DO_GIF yes
GIF_OBJ =gifimage.obj
GIF=-DDO_GIF
!endif

!ifeq DO_TGA yes
TGA_OBJ =tgaimage.obj
TGA=-DDO_TGA
!endif

!ifeq DO_JPG yes
JPG_OBJ =jpgimage.obj
JPG_LIBS=..\libjpeg\libjpeg.lib
JPG=-DDO_JPG
!endif

INC_PATH=$(SYS_INC_PATH) -i=. -i=../zlib -i=../libpng -i=../libjpeg
CFLAGS=$(HOW) $(SYSFLAGS) $(INC_PATH) $(ASM) $(MEM) $(XSHARED) $(D3D) $(PNG) $(GIF) $(TGA) $(JPG)

#.BEFORE
install: .SYMBOLIC
        @%create walktest.flg
        @%append walktest.flg $(CFLAGS)

OBJ =$(GENERAL_OBJ) $(UTIL_OBJ) $(SYSTEM_OBJ) $(GEOMETRY_OBJ) &
     $(ENGINE_OBJ) $(RENDER_OBJ) $(SCRIPTS_OBJ) $(PNG_OBJ) &
     $(GIF_OBJ) $(TGA_OBJ) $(JPG_OBJ)

Z_LIBS=..\zlib\zlib.lib

# implicit rules
.cpp.obj :
  $(CXX) $^& $(CFLAGS)
.c.obj :
  $(CC) $^& $(CFLAGS)

LIB_PATH=$(SYS_LIB_PATH) -L ../zlib -L ../libpng -L ../libjpeg
LIBS=$(SYS_LIBS) $(Z_LIBS) $(PNG_LIBS) $(JPG_LIBS)

$(WALKTEST): $(WALKTEST_OBJ) $(OBJ) $(LIBS)
        @%create walktest.lnk
        @%append walktest.lnk name $(WALKTEST)
        @%append walktest.lnk system dos4g
        @%append walktest.lnk option stack=512000
        @%append walktest.lnk debug Watcom All
        @for %f in ($(LIBS)) do @%append walktest.lnk file %f
        @for %f in ($(WALKTEST_OBJ)) do @%append walktest.lnk file %f
        @for %f in ($(OBJ)) do @%append walktest.lnk file %f
        $(LINK) @walktest.lnk

$(MAZED): $(MAZED_OBJ) $(OBJ) $(LIBS)
        @%create walktest.lnk
        @%append walktest.lnk name $(MAZED)
        @%append walktest.lnk system dos4g
        @%append walktest.lnk option stack=512000
        @for %f in ($(LIBS)) do @%append walktest.lnk file %f
        @for %f in ($(MAZED_OBJ)) do @%append walktest.lnk file %f
        @for %f in ($(OBJ)) do @%append walktest.lnk file %f
        $(LINK) @walktest.lnk

$(CSWSTEST): $(CSWSTEST_OBJ) $(OBJ) $(LIBS)
        @%create walktest.lnk
        @%append walktest.lnk name $(CSWSTEST)
        @%append walktest.lnk system dos4g
        @%append walktest.lnk option stack=512000
        @for %f in ($(LIBS)) do @%append walktest.lnk file %f
        @for %f in ($(CSWSTEST_OBJ)) do @%append walktest.lnk file %f
        @for %f in ($(OBJ)) do @%append walktest.lnk file %f
        $(LINK) @walktest.lnk

$(SIMPLE): $(SIMPLE_OBJ) $(OBJ) $(LIBS)
        @%create walktest.lnk
        @%append walktest.lnk name $(SIMPLE)
        @%append walktest.lnk system dos4g
        @%append walktest.lnk option stack=512000
        @for %f in ($(LIBS)) do @%append walktest.lnk file %f
        @for %f in ($(SIMPLE_OBJ)) do @%append walktest.lnk file %f
        @for %f in ($(OBJ)) do @%append walktest.lnk file %f
        $(LINK) @walktest.lnk

system\wcc\video.lib:
        cd system\wcc\video
        wmake
        cd ..
        cd ..
        cd ..

..\zlib\zlib.lib:
        cd ..
        cd zlib\msdos
        copy makefile.wat ..\*.*
        cd ..
        wmake -f makefile.wat
        cd ..
        cd src

..\libpng\libpngf.lib:
        cd ..
        cd libpng
        wmake -f makefile.wat
        cd ..
        cd src

..\libjpeg\libjpeg.lib:
        cd ..
        cd libjpeg
        copy jconfig.wat jconfig.h
        copy makefile.wat makefile
        wmake
        cd ..
        cd src

clean:
        del *.obj
        del system\wcc\*.lib
        del *.exe
