
#include "com/com.h"
#include "render/direct3d/g3d_d3d.h"

#define DLL_NAME "Direct3DRender.dll"

static int gb_cRef = 0;
static DllRegisterData gb_regData;

extern const CLSID CLSID_Graphics3DDirect3D;

void _stdcall ModuleAddRef()
{
    gb_cRef++;
}

void _stdcall ModuleRelease()
{
    gb_cRef--;
}

HINSTANCE DllHandle;

// our main entry point...should be called when we're loaded.
BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	DllHandle = hinstDLL;

	gb_regData.clsid = CLSID_Graphics3DDirect3D;
		
	strcpy(gb_regData.szProgID, "crystalspace.graphics3d.direct3d");
	strcpy(gb_regData.szInProcServer, DLL_NAME );
	strcpy(gb_regData.szFriendlyName, "csGraphics3D Direct3D Driver");

	return TRUE;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
	return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)
{
        static csGraphics3DDirect3DFactory factory;

        if (rclsid == CLSID_Graphics3DDirect3D)
            return factory.QueryInterface(riid, ppv);

	// if we get here, rclsid is a class we don't implement
	*ppv = NULL;
	return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer()
{
	return csRegisterServer(&gb_regData);
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer()
{
	return csUnregisterServer(&gb_regData);
}

// Implementation of the csGraphics2DWin32 factory... ///////////

IMPLEMENT_UNKNOWN_NODELETE( csGraphics3DDirect3DFactory )

BEGIN_INTERFACE_TABLE( csGraphics3DDirect3DFactory )
	IMPLEMENTS_INTERFACE( IGraphicsContextFactory )
END_INTERFACE_TABLE()


STDMETHODIMP csGraphics3DDirect3DFactory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics3DDirect3D* pNew = new csGraphics3DDirect3D(piSystem);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}

STDMETHODIMP csGraphics3DDirect3DFactory::LockServer(BOOL bLock)
{
	if (bLock)
		gb_cRef++;
	else
		gb_cRef--;

	return S_OK;
}
