/*
    Copyright (C) 1998 by Jorrit Tyberghein and Dan Ogles
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// g2d_glide.h
// Graphics2DGlide Class Declaration
// Written by xtrochu and Nathaniel

#ifndef G2D_GLIDE_H
#define G2D_GLIDE_H

#include <glide.h>
 
#include "com/com.h"
#include "render/graph2d.h"
#include "render/glide/driver2d/xsysg2d.h"

extern const CLSID CLSID_Glide2DDriver;

extern const IID IID_IGraphics2DGlideFactory;

/// dummy interface
interface IGraphics2DGlideFactory : public IGraphics2DFactory
{
};

///
class csGraphics2DGlideFactory : public IGraphics2DGlideFactory 
{
public:
    DECLARE_IUNKNOWN()
    DECLARE_INTERFACE_TABLE(csGraphics2DGlideFactory)

    STDMETHOD(CreateInstance)(REFIID riid, ISystem* piSystem, void** ppv);
    STDMETHOD(LockServer)(BOOL bLock);
};

class csGraphics2DGlide : public csGraphics2D
{
  friend class csGraphics3DGlide;
  friend class csGraphics2DGlide;
  
public:
  csGraphics2DGlide(ISystem* piSystem);
  virtual ~csGraphics2DGlide(void);
  
  virtual bool Open (char *Title);
  virtual void Close ();
  
  virtual void Print (csRect *area = NULL);
  
  virtual void SetRGB(int i, int r, int g, int b);
 
  virtual bool BeginDraw(/*int Flag = (CSDRAW_2DGRAPHICS | CSDRAW_2DGRAPHICS_WRITE)*/);
  virtual void FinishDraw();
#if defined(OS_WIN32)
  virtual long GethWnd(unsigned long *hwnd);
#endif
  int GraphicsReady;
  static int Depth;
  
protected:
  // print to the system's device
  void SysPrintf(int mode, char* str, ...);

#if defined(OS_WIN32)
  HWND m_hWnd;
#endif
  bool bPalettized;
  bool bPaletteChanged;
  int glDrawMode;
  GrLfbInfo_t lfbInfo;
  bool locked;

  DECLARE_IUNKNOWN()
  DECLARE_INTERFACE_TABLE(csGraphics2DGlide)
  DECLARE_COMPOSITE_INTERFACE(XGlideGraphicsInfo)
};

#endif // G3D_GLIDE_H

