/*
    Copyright (C) 1998 by Jorrit Tyberghein and Dan Ogles

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
	
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// G2D_GLIDE.CPP
// Graphics2DGlide implementation file
// Written by xtrochu and Nathaniel

#if defined(DO_GLIDE)

#if (defined(OS_WIN32) || (defined(OS_DOS) && defined(COMP_WCC)) || defined(OS_LINUX))

#if defined(OS_WIN32)
#include <windows.h>
#define NAME "Crystal Glide"
#endif	// OS_WIN32
#include <stdio.h>
#include <stdlib.h>
#include <glide.h>

#include "system/system.h"
#include "util/inifile.h"
#include "render/graph3d.h"

#include "render/glide/g2dglide.h"
#include "render/glide/g3dglide.h"
#include "render/glide/glidelib.h"

int Graphics2DGlide::Depth=16;

/////The 2D Graphics Driver//////////////

#if defined(OS_WIN32)
extern long FAR PASCAL GlideWindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
extern HINSTANCE gb_hInstance;
extern int gb_nCmdShow;
static WORD wOldCW;
static BOOL MungeFPCW( WORD *pwOldCW )
{    
	BOOL ret = FALSE;    
	WORD wTemp, wSave; 
    __asm fstcw wSave    
	if (wSave & 0x300 ||            // Not single mode
		0x3f != (wSave & 0x3f) ||   // Exceptions enabled
		wSave & 0xC00)              // Not round to nearest mode    
	{
		__asm {            
			mov ax, wSave
	        and ax, not 300h    ;; single mode
		    or  ax, 3fh         ;; disable all exceptions
			and ax, not 0xC00   ;; round to nearest mode
			mov wTemp, ax            
			fldcw   wTemp        
		}        
		ret = TRUE;
	}    
	*pwOldCW = wSave;    
	return ret;
} 

static void RestoreFPCW(WORD wSave)
{
    __asm fldcw wSave
} 

inline void grWinKeyTrans(csSystemDriver* pSystemDriver, WPARAM wParam, bool shift, bool alt, bool ctrl, bool down)
{
	if(!pSystemDriver) return;
	
	switch(wParam)
	{
	case VK_END:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_END) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_END) ;
		break;
	case VK_ESCAPE:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_ESC) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_ESC) ;
		break;
	case VK_UP:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_UP) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_UP) ;
		break;
	case VK_DOWN:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_DOWN) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_DOWN) ;
		break;
	case VK_RETURN:
		if(down) pSystemDriver->Keyboard->do_keypress ('\n') ;
		else pSystemDriver->Keyboard->do_keyrelease ('\n') ;
		break;		
	case VK_LEFT:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_LEFT) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_LEFT) ;
		break;
		
	case VK_RIGHT:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_RIGHT) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_RIGHT) ;
		break;
		
	case VK_PRIOR:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_PGUP) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_PGUP) ;
		break;
		
	case VK_NEXT:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_PGDN) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_PGDN) ;
		break;
		
	case VK_MENU:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_ALT) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_ALT) ;
		break;
		
	case VK_CONTROL:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_CTRL) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_CTRL) ;
		break;
		
	case VK_SHIFT:
		if(down) pSystemDriver->Keyboard->do_keypress (CSKEY_SHIFT) ;
		else pSystemDriver->Keyboard->do_keyrelease (CSKEY_SHIFT);
		break;
	}
	
}

extern BOOL gb_bActive;

long FAR PASCAL GlideWindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	bool shift=false, ctrl=false;
	
	if(GetAsyncKeyState(VK_CONTROL)) ctrl=true;
	if(GetAsyncKeyState(VK_SHIFT)) shift=true;
	
	switch( message )
	{
	case WM_ACTIVATEAPP:
		gb_bActive = wParam;
		if(wParam)
		{
			GlideLib_grSstControlMode(GR_CONTROL_ACTIVATE);
#if defined(OS_WIN32)
			//((SysSystemDriver*)System)->SetFocus(true);
#endif
		}
		else
		{
			GlideLib_grSstControlMode(GR_CONTROL_DEACTIVATE);
#if defined(OS_WIN32)
			//((SysSystemDriver*)System)->SetFocus(false);
#endif
		}
		break;
	case WM_CREATE:
		break;
		
	case WM_DESTROY:
		PostQuitMessage( 0 );
		break;
	case WM_CHAR:
		if(System)
		{
			System->Keyboard->do_keypress (wParam) ;
			System->Keyboard->do_keyrelease (wParam) ;
		}
		break;
		
	case WM_KEYDOWN:
		grWinKeyTrans(System, wParam, shift, false, ctrl, true);
		if(wParam==VK_MENU)
			return 0;
		break;
		
	case WM_SYSKEYDOWN:
		grWinKeyTrans(System, wParam, shift, true, ctrl, true);
		if(wParam==VK_MENU)
			return 0;
		break;
		
	case WM_KEYUP:
		grWinKeyTrans(System, wParam, shift, false, ctrl, false);
		if(wParam==VK_MENU)
			return 0;
		break;
		
	case WM_SYSKEYUP:
		grWinKeyTrans(System, wParam, shift, true, ctrl, false);
		if(wParam==VK_MENU)
			return 0;
		break;
		
	case WM_LBUTTONDOWN:
		if(System)
			System->Mouse->do_buttonpress(1, LOWORD(lParam), HIWORD(lParam), wParam & MK_SHIFT , GetAsyncKeyState(VK_MENU), wParam & MK_CONTROL);
		break;
		
	case WM_LBUTTONUP:
		if(System)
			System->Mouse->do_buttonrelease(1, LOWORD(lParam), HIWORD(lParam));
		break;
		
	case WM_MBUTTONDOWN:
		if(System)
			System->Mouse->do_buttonpress(2, LOWORD(lParam), HIWORD(lParam), wParam & MK_SHIFT , GetAsyncKeyState(VK_MENU), wParam & MK_CONTROL);
		break;
		
	case WM_MBUTTONUP:
		if(System)
			System->Mouse->do_buttonrelease(2, LOWORD(lParam), HIWORD(lParam));
		break;
		
	case WM_RBUTTONDOWN:
		if(System)
			System->Mouse->do_buttonpress(3, LOWORD(lParam), HIWORD(lParam), wParam & MK_SHIFT , GetAsyncKeyState(VK_MENU), wParam & MK_CONTROL);
		break;
		
	case WM_RBUTTONUP:
		if(System)
			System->Mouse->do_buttonrelease(3, LOWORD(lParam), HIWORD(lParam));
		break;
		
	case WM_MOUSEMOVE:
		// Not yet ready for this!
		if(System)
			System->Mouse->do_mousemotion(LOWORD(lParam), HIWORD(lParam));
		break;
	}
	return DefWindowProc(hWnd, message, wParam, lParam);
}


#endif

extern void sys_fatalerror(char *s);

Graphics2DGlide::Graphics2DGlide(int argc, char* argv[]) : csGraphics2D ()
{
	hwnd=NULL;
	// make a window for rush or banshee ?
#if defined(OS_WIN32)
	WNDCLASS wc;
	MungeFPCW(&wOldCW);
	wc.style = 0;
	wc.lpfnWndProc = GlideWindowProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = gb_hInstance;
	wc.hIcon = LoadIcon( NULL, IDI_APPLICATION );
	wc.hCursor = LoadCursor( NULL, IDC_ARROW );
	wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
	wc.lpszClassName = NAME;
	if(!::RegisterClass( &wc ))
		sys_fatalerror("Cannot register CrystalSpace For Glide window class");
	
	CHK(glLib = new GlideLib);
#endif
	
	Depth=16;
	
	DrawPixel = DrawPixel16;   WriteChar = WriteChar16;
	GetPixelAt = GetPixelAt16; DrawSprite = DrawSprite16;
    
	// calculate CS's pixel format structure. 565
	pfmt.PixelBytes = 2;
	pfmt.PalEntries = 0;
	pfmt.RedMask = (1+2+4+8+16)<<11;
	pfmt.GreenMask = (1+2+4+8+16+32)<<5;
	pfmt.BlueMask = (1+2+4+8+16);
    
	complete_pixel_format();
	GraphicsReady=1;  
	CsPrintf (MSG_INITIALIZATION, "Using %d bits per pixel (%d color mode).\n", Depth, 1 << Depth);
}

Graphics2DGlide::~Graphics2DGlide(void)
{
	GraphicsReady=0;
	Close();
#if defined(OS_WIN32)
	RestoreFPCW(wOldCW);
	delete glLib;
#endif
}

bool Graphics2DGlide::Open(char *Title)
{
	if (!csGraphics2D::Open (Title))
		return false;
	
	
#if defined(OS_WIN32)
	// create the window.
	DWORD exStyle = 0;
	DWORD style = WS_OVERLAPPEDWINDOW;
	
	hwnd = CreateWindowEx(exStyle, NAME, Title, style,0,0,
		FRAME_WIDTH, FRAME_HEIGHT,
		NULL,NULL, gb_hInstance,NULL );
	if( !hwnd )
		sys_fatalerror("Cannot create CrystalSpace Glide window");
	if(config->GetYesNo("VideoDriver","FULL_SCREEN",TRUE))
		gb_nCmdShow=SW_MAXIMIZE;
	ShowWindow( hwnd, gb_nCmdShow );
	UpdateWindow( hwnd );
	::SetFocus( hwnd );
	// hwnd is window handler used by banshee/rush card
	GlideLib_grSstControlMode(GR_CONTROL_ACTIVATE);
#endif
	
	
	bPalettized = false;
	
	bPaletteChanged = false;
	
	return true;
}

void Graphics2DGlide::Close(void)
{
	csGraphics2D::Close ();
}

void Graphics2DGlide::Print (csRect *area)
{
}

#define GR_DRAWBUFFER GR_BUFFER_FRONTBUFFER

void Graphics2DGlide::FinishDraw ()
{
	Memory=NULL;
	for(int i = 0; i < FRAME_HEIGHT; i++)
		LineAddress [i] = 0;
	GlideLib_grLfbUnlock(glDrawMode,GR_DRAWBUFFER);
}

bool Graphics2DGlide::BeginDraw(/*int Flag*/)
{
	FxBool bret;
	lfbInfo.size=sizeof(GrLfbInfo_t);
/*	switch(Flag&(CSDRAW_2DGRAPHICS_READ|CSDRAW_2DGRAPHICS_WRITE))
	{
	case CSDRAW_2DGRAPHICS_READ:
		glDrawMode=GR_LFB_READ_ONLY;
		break;
	case CSDRAW_2DGRAPHICS_WRITE:
		glDrawMode=GR_LFB_WRITE_ONLY;
		break;
	default:
		return false;
	}
*/	glDrawMode=GR_LFB_WRITE_ONLY;
	bret=GlideLib_grLfbLock(glDrawMode|GR_LFB_IDLE,
		GR_DRAWBUFFER,
		GR_LFBWRITEMODE_565,
		GR_ORIGIN_ANY,
		FXFALSE,
		&lfbInfo);
	if(bret)
	{
		Memory=(unsigned char*)lfbInfo.lfbPtr;
		if(lfbInfo.origin==GR_ORIGIN_UPPER_LEFT)
		{
			for(int i = 0; i < FRAME_HEIGHT; i++)
				LineAddress [i] = i * lfbInfo.strideInBytes;
		}
		else
		{
			int omi =FRAME_HEIGHT-1;
			for(int i = 0; i < FRAME_HEIGHT; i++)
				LineAddress [i] = (omi--) * lfbInfo.strideInBytes;
		}
	}
	return bret;
}

void Graphics2DGlide::SetRGB(int i, int r, int g, int b)
{
	csGraphics2D::SetRGB (i, r, g, b);
	bPaletteChanged = true;
}

#else // OS/COMP 
#error "Cannot compile Glide on this Compiler/OS settings, remove DO_GLIDE from your makefile"
#endif // OS_..._LINUX


#endif // DO_GLIDE


