#include "system/system.h"
#include "render/hicache2.h"
#include "general/StdAssrt.h"

#include "engine/itexture.h"
#include "engine/polygon/ipolygon.h"
#include "engine/light/ilghtmap.h"

HighColorCacheAndManage::HighColorCacheAndManage(int max_size, HIGHCOLOR_TYPE type, int bpp,TextureMemoryManager * tm) : HighColorCache(max_size,type,bpp)
{
	// New For management.
	manager=tm;
}

HighColorCacheAndManage::~HighColorCacheAndManage()
{
	Clear();
}

void HighColorCacheAndManage::Add(IMipMapContainer *texture)
{
	HighColorCacheAndManage_Data *cached_texture;
  ITextureMap* piTM = NULL;
  int size = 0;
  
  if(type!=HIGHCOLOR_TEXCACHE) return;

  for (int c =0; c<4; c++)
  {
    int width, height;
    
    texture->GetTexture( c, &piTM );
    //ASSERT( piTM );
    
    piTM->GetWidth( width );
    piTM->GetHeight( height );
    
    size += width * height;
    
    piTM->Release();
  }

  size *= bpp/8;

  bool bIsInVideoMemory;
  texture->GetInVideoMemory( bIsInVideoMemory );
  
  if (bIsInVideoMemory)
	{
		// move unit to front (MRU)

    texture->GetHighColorCache((HighColorCache_Data**)&cached_texture);

    if(!cached_texture) return;

		if(cached_texture != head)
		{
			if(cached_texture->prev) cached_texture->prev->next = cached_texture->next;
			else head = cached_texture->next;
			if(cached_texture->next) cached_texture->next->prev = cached_texture->prev;
			else tail = cached_texture->prev;

			cached_texture->prev = NULL;
			cached_texture->next = head;
			if(head) head->prev = cached_texture;
			else tail = cached_texture;
			head = cached_texture;
		}
	}
	else
	{
	    	
		// unit is not in memory. load it into the cache
		while((total_size + size >= cache_size)&&(manager->hasFreeSpace(size)))
		{
			// out of memory. remove units from bottom of list.
      IMipMapContainer* piMMC;
			HighColorCacheAndManage_Data* l = (HighColorCacheAndManage_Data*)tail;

      l->pSource->QueryInterface( IID_IMipMapContainer, (void**)&piMMC );

			tail = tail->prev;
			if(tail) tail->next = NULL;
			else head = NULL;
			l->prev = NULL;

      piMMC->SetInVideoMemory(false);
      piMMC->SetHighColorCache(NULL);

			Unload(l);					// unload it.
//			manager->freeSpaceMem(l->mempos);
			delete l->mempos;
			l->mempos=0;
			num--;
			total_size -= l->lSize;
		}
		
		// now load the unit.
		num++;
		total_size += size;

		CHK (cached_texture = new HighColorCacheAndManage_Data);

		cached_texture->next = head;
		cached_texture->prev = NULL;
		if(head) head->prev = cached_texture;
		else tail = cached_texture;
		head = cached_texture;
		cached_texture->pSource = texture;
		cached_texture->lSize = size;

    texture->SetInVideoMemory( true );
    texture->SetHighColorCache(cached_texture);
//		cached_texture->mempos=manager->allocSpaceMem(size);
		Load(cached_texture);				// load it.
	}
}

void HighColorCacheAndManage::Add(IPolygonTexture *polytex)
{
	HighColorCacheAndManage_Data *cached_texture;
  ILightMap *piLM; 
	HighColorCacheAndManage_Data* l;

  polytex->GetLightMap( &piLM );
  if (!piLM) return;

	if(type!=HIGHCOLOR_LITCACHE) return;

  int size, width, height;
    
  piLM->GetWidth(width);
  piLM->GetHeight(height);
  size = width*height*(bpp/8);
    
  bool dl;
  polytex->RecalculateDynamicLights(dl);

  bool bInVideoMemory;
  piLM->GetInVideoMemory( bInVideoMemory );
  
  if (dl && bInVideoMemory)
	{
    piLM->GetHighColorCache((HighColorCache_Data**)&l);
    
    if(l->next)
      l->next->prev = l->prev;
    if(l->prev)
      l->prev->next = l->next;
    
    piLM->SetInVideoMemory(false);
    piLM->SetHighColorCache(NULL);
    
    Unload(l);					// unload it.
    //			manager->freeSpaceMem(l->mempos);
    delete l->mempos;
    l->mempos=0;						
    num--;
    total_size -= l->lSize;
    
    piLM->Release();
	}

  piLM->GetInVideoMemory( bInVideoMemory );
  if (bInVideoMemory)
	{
		// move unit to front (MRU)

    piLM->GetHighColorCache((HighColorCache_Data**)&cached_texture);

		if(!cached_texture) return;

		if(cached_texture != head)
		{
			if(cached_texture->prev) cached_texture->prev->next = cached_texture->next;
			else head = cached_texture->next;
			if(cached_texture->next) cached_texture->next->prev = cached_texture->prev;
			else tail = cached_texture->prev;

			cached_texture->prev = NULL;
			cached_texture->next = head;
			if(head) head->prev = cached_texture;
			else tail = cached_texture;
			head = cached_texture;
		}
	}
	else
	{
	    	
		// unit is not in memory. load it into the cache
		while(total_size + size >= cache_size)
		{
			// out of memory. remove units from bottom of list.
      ILightMap* ilm;
      
      l->pSource->QueryInterface( IID_ILightMap, (void**)&ilm );
      ASSERT( ilm );

			tail = tail->prev;
			if(tail) tail->next = NULL;
			else head = NULL;
			l->prev = NULL;

      ilm->SetInVideoMemory(false);
      ilm->SetHighColorCache(NULL);

			Unload(l);					// unload it.
//			manager->freeSpaceMem(l->mempos);
			delete l->mempos;
			l->mempos=0;						
						
			num--;
			total_size -= l->lSize;

      ilm->Release();
		}
		
		// now load the unit.
		num++;
		total_size += size;

		CHK (cached_texture = new HighColorCacheAndManage_Data);

		cached_texture->next = head;
		cached_texture->prev = NULL;
		if(head) head->prev = cached_texture;
		else tail = cached_texture;
    head = cached_texture;
    cached_texture->pSource = piLM;
    cached_texture->lSize = size;
    
    piLM->SetInVideoMemory(true);
    piLM->SetHighColorCache(cached_texture);
    
    cached_texture->pData = NULL;
//		cached_texture->mempos=manager->allocSpaceMem(size);
		Load(cached_texture);				// load it.
	}

  FINAL_RELEASE( piLM );
}

void HighColorCacheAndManage::Clear()
{
	while(head)
	{
		HighColorCacheAndManage_Data *n = (HighColorCacheAndManage_Data*)head->next;
		head->next = head->prev = NULL;
				
		Unload(head);
		manager->freeSpaceMem(((HighColorCacheAndManage_Data *)head)->mempos);
		delete ((HighColorCacheAndManage_Data *)head)->mempos;
		((HighColorCacheAndManage_Data *)head)->mempos=0;						

    if(type==HIGHCOLOR_TEXCACHE) 
    {
      IMipMapContainer* piMMC = NULL;
      
      head->pSource->QueryInterface( IID_IMipMapContainer, (void**)piMMC );
      assert( piMMC );
      
      piMMC->SetHighColorCache(NULL);
      piMMC->SetInVideoMemory(false);
        
      piMMC->Release();
    }
    else if(type==HIGHCOLOR_LITCACHE)
    {
      ILightMap* piLM;
    
      head->pSource->QueryInterface( IID_ILightMap, (void**)piLM );
      assert( piLM );
      
      piLM->SetHighColorCache(NULL);
      piLM->SetInVideoMemory(false);
        
      piLM->Release();
    }
 		head = n;
	}
	
	head = tail = NULL;
	total_size = 0;
	num = 0;
}
