/*
    Software renderer DLL
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "com/com.h"
#include "render/opengl/g3d_opengl.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#ifdef OS_WIN32
#define DLL_NAME "OpenGLRender.dll"
#elif defined (OS_OS2)
#define DLL_NAME "GLrender.dll"
#else
#define DLL_NAME "glrender.so"
#endif

static unsigned int gb_cRef = 0;
static DllRegisterData gb_regData;
extern const CLSID CLSID_Graphics3DOpenGL;

#ifndef CS_STATIC_LINKED
void STDAPICALLTYPE ModuleRelease ()
{
  gb_cRef--;
}

void STDAPICALLTYPE ModuleAddRef ()
{
  gb_cRef++;
}

// our main entry point...should be called when we're loaded.
STDAPI DllInitialize ()
{
  csCoInitialize(0);

  gb_regData.clsid = CLSID_Graphics3DOpenGL;
	
  strcpy(gb_regData.szProgID, "crystalspace.graphics3d.opengl");
  strcpy(gb_regData.szInProcServer, DLL_NAME );
  strcpy(gb_regData.szFriendlyName, "csGraphics3DOpenGL");
  return TRUE;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
  return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject (REFCLSID rclsid, REFIID riid, void** ppv)
{
  static csGraphics3DOpenGLFactory s_g3dFactory;

  if (rclsid == CLSID_Graphics3DOpenGL)
    return s_g3dFactory.QueryInterface (riid, ppv);

  // if we get here, rclsid is a class we don't implement
  *ppv = NULL;
  return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer ()
{
  return csRegisterServer (&gb_regData);
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer ()
{
  return csUnregisterServer (&gb_regData);
}
#endif


// Implementation of csGraphics3DOpenGLFactory /////////////////

IMPLEMENT_UNKNOWN_NODELETE (csGraphics3DOpenGLFactory)

BEGIN_INTERFACE_TABLE (csGraphics3DOpenGLFactory)
	IMPLEMENTS_INTERFACE (IGraphicsContextFactory)
END_INTERFACE_TABLE ()

STDMETHODIMP csGraphics3DOpenGLFactory::CreateInstance (REFIID riid,
	ISystem* piSystem, void** ppv)
{
  if (!piSystem)
  {
    *ppv = 0;
    return E_UNEXPECTED;
  }

  csGraphics3DOpenGL* pNew = new csGraphics3DOpenGL (piSystem);
  if (!pNew)
  {
    *ppv = 0;
    return E_OUTOFMEMORY;
  }
	
  return pNew->QueryInterface (riid, ppv);
}

STDMETHODIMP csGraphics3DOpenGLFactory::LockServer (BOOL bLock)
{
  if (bLock)
    gb_cRef++;
  else
    gb_cRef--;

  return S_OK;
}

