/*
    Copyright (C) 1998 by Jorrit Tyberghein and Dan Ogles.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
	
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#if defined(OS_WIN32)
#include <windows.h>
#endif

#include <GL/gl.h>
#include <GL/glu.h>

#include "system/isystem.h"
#include "render/igraph3d.h"
#include "engine/itexture.h"
#include "engine/light/ilghtmap.h"
#include "render/opengl/gl_texturecache.h"
#include "render/hicache.h"

// need definitions of R24(), G24(), and B24()
#include "engine/texture.h"
extern GLuint checkImage[64][64][4];


///
OpenGLTextureCache::OpenGLTextureCache(int size, int bitdepth)
  : HighColorCache(size,HIGHCOLOR_TEXCACHE,bitdepth)
{
}

void OpenGLTextureCache::Dump()
{
}

  
void OpenGLTextureCache::Load(HighColorCache_Data *d)
{
    IMipMapContainer *mipmap = (IMipMapContainer *)d->pSource;
    ITextureMap *texture;
    mipmap->GetTexture(0,&texture);

    int texture_width, texture_height;
    texture->GetWidth(texture_width);
    texture->GetHeight(texture_height);

    int active_alpha;
    mipmap->GetTransparent(active_alpha);

    GLuint *texturehandle = new GLuint;
    glGenTextures(1,texturehandle);

/*    CsPrintf(MSG_DEBUG_0,"caching texture in handle %d\n",*texturehandle);
    CsPrintf(MSG_DEBUG_0,"size (%d,%d)\n",texture_width,texture_height);
    CsPrintf(MSG_DEBUG_0,"texture data location %x\n",texture->get_bitmap32());*/

    glBindTexture(GL_TEXTURE_2D,*texturehandle);

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                   GL_NEAREST);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                   GL_NEAREST);

    unsigned char *tempdata = new unsigned char[texture_width*texture_height*4];
   ULong *source;
   texture->GetBitmap((unsigned char **)&source);
  unsigned char *dest = tempdata;
  for (int count = texture_width * texture_height;
        count > 0; count--)
  {
        dest[0] = R24(*source);
        dest[1] = G24(*source);
        dest[2] = B24(*source);
	// broken transparent textures
	if (active_alpha != -1 && (*source & 0xffffff) == active_alpha)
	   dest[3] = 0;
	else
	  dest[3] = 55;

        dest+=4; source++;
  }
/*  if (active_alpha != -1)
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,texture_width,
		texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		tempdata);
  else*/
    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,texture_width,
		texture_height,0,GL_RGBA,GL_UNSIGNED_BYTE,
		tempdata);

  delete []tempdata;

    d->pData = texturehandle;
    
}

///
void OpenGLTextureCache::Unload(HighColorCache_Data *d)
{
   if (d->pData)
   {
     glDeleteTextures(1,(GLuint *)d->pData);
     delete d->pData;
   }
}

///
OpenGLLightmapCache::OpenGLLightmapCache(int size, int bitdepth)
  : HighColorCache(size,HIGHCOLOR_LITCACHE,bitdepth)
{
}

///
void OpenGLLightmapCache::Dump()
{
}

///
void OpenGLLightmapCache::Load(HighColorCache_Data *d)
{
#if 0
  ILightMap *lightmap_info = (ILightMap *)d->pSource;

  // scale lightmap to be power-of-2 in size
  int lmheight, lmwidth;
  lightmap_info->GetWidth(lmwidth);
  lightmap_info->GetHeight(lmheight);

//   lmheight = floor(pow(2.0, ceil(log(lmheight)/log(2.0)) ) );
//   lmwidth = floor(pow(2.0, ceil(log(lmwidth)/log(2.0)) ) );

  unsigned char *red_data, *green_data, *blue_data;

  lightmap_info->GetMap(0,&red_data);
  lightmap_info->GetMap(1,&green_data);
  lightmap_info->GetMap(2,&blue_data);

  unsigned char  *lm_data = new unsigned char[lmwidth*lmheight*4];
  //unsigned char  *real_data = new unsigned char[rwidth*rheight*4+16];
  unsigned char *walkdata;
  for (int j=0; j<lmheight; j++)
  {
    walkdata = lm_data + (j) * lmwidth*4;

    for (int i=0; i<lmwidth; i++)
    {
      *walkdata++ = *red_data++;
      *walkdata++ = *blue_data++;
      *walkdata++ = *green_data++;
      *walkdata++ = 200;
    }
  }
  GLuint *lightmaphandle = new GLuint;
  glGenTextures(1,lightmaphandle);
  glBindTexture(GL_TEXTURE_2D,*lightmaphandle);

  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER,
                   GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                   GL_LINEAR);

  glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,lmwidth,
		lmheight,0,GL_RGBA,GL_UNSIGNED_BYTE,
		lm_data);
/*    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,64,
		64,0,GL_RGBA,GL_UNSIGNED_BYTE,
		checkImage);*/


  GLenum bob;
  bob = glGetError();
  if (bob != GL_NO_ERROR)
  { /*CsPrintf(MSG_DEBUG_0,"openGL error string: %s\n",gluErrorString(bob) );*/ }

  delete []lm_data; 

/*
    CsPrintf(MSG_DEBUG_0,"caching lightmap in handle %d\n",*lightmaphandle);
    CsPrintf(MSG_DEBUG_0,"size (%d,%d)\n",lmwidth,lmheight);
    CsPrintf(MSG_DEBUG_0,"lightmap data location %x\n",lightmap_info);
    */

  d->pData = lightmaphandle;    

#endif
  d->pData = NULL;
}

///
void OpenGLLightmapCache::Unload(HighColorCache_Data *d)
{
  if (d->pData)
  {
    glDeleteTextures(1,(GLuint *)d->pData);
    delete d->pData;
  }
}
