/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>

#include "render/software/scan.h"
#include "render/softw16/scan16.h"
#include "engine/polygon/ipolygon.h"
#include "engine/itexture.h"
#include "engine/light/ilghtmap.h"

#define STUPID_TEST 1

extern unsigned char* priv_to_global;
UShort* Scan16::pal_table;
ITextureContainer* Scan16::piTC;

#if !defined(NO_ASSEMBLER)

#  if defined(PROC_INTEL)
#    if defined(COMP_GCC)
#      include "system/i386/scanln16.h"
#    elif defined(COMP_VC)
#      include "system/i386/scanln16vc.h"
#    endif //COMP_???
#  endif //PROC_INTEL

#endif //!NO_ASSEMBLER

#ifndef NO_draw_scanline_map

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_map
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      *dd++ = srcTex[((vv>>16)<<shifter) + (uu>>16)];			\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#define SCANEND \
    do									\
    {									\
      *z_buffer++ = izz;						\
      izz += dzz;							\
    }									\
    while (z_buffer <= lastZbuf)
#include "scanln16.inc"

#endif // NO_draw_scanline_map

//------------------------------------------------------------------

#ifndef NO_draw_scanline_map_filter

extern int filter_bf;

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_map_filter
#define SCANMAP 1
#define SCANLOOP \
    int filter_du, filter_dv;						\
    do									\
    {									\
      if ((((long)dd) & filter_bf) != 0)				\
      {									\
        if ((uu&0xffff) < 64*256) filter_du = -1;			\
	else if ((uu&0xffff) > 192*256) filter_du = 1;			\
	else filter_du = 0;						\
        if ((vv&0xffff) < 64*256) filter_dv = -1;			\
	else if ((vv&0xffff) > 192*256) filter_dv = 1;			\
	else filter_dv = 0;						\
      }									\
      else filter_du = filter_dv = 0;					\
      *dd++ = srcTex[(((vv>>16)+filter_dv)<<shifter) + (uu>>16)+filter_du];	\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#define SCANEND \
    do									\
    {									\
      *z_buffer++ = izz;						\
      izz += dzz;							\
    }									\
    while (z_buffer <= lastZbuf)
#include "scanln16.inc"

#endif // NO_draw_scanline_map_filter

//------------------------------------------------------------------

#ifndef NO_draw_scanline_z_buf_map

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_z_buf_map
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      if (izz >= (int)(*z_buffer))					\
      {									\
	*dd++ = srcTex[((vv>>16)<<shifter) + (uu>>16)];			\
	*z_buffer++ = izz;						\
      }									\
      else								\
      {									\
        dd++;								\
        z_buffer++;							\
      }									\
      uu += duu;							\
      vv += dvv;							\
      izz += dzz;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_z_buf_map

//------------------------------------------------------------------

#ifndef NO_draw_scanline_map_alpha50

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_map_alpha50
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      UShort a = *dd;							\
      UShort b = srcTex[((vv>>16)<<shifter) + (uu>>16)];		\
      *dd++ = ((a & Scan::alpha_mask) >> 1) + ((b & Scan::alpha_mask) >> 1);		\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_map_alpha50

//------------------------------------------------------------------

#ifndef NO_draw_scanline_map_alpha_555

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_map_alpha_555
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      UShort a = *dd;							\
      UShort b = srcTex[((vv>>16)<<shifter) + (uu>>16)];		\
      int r1 = a >> 10;	\
      int g1 = (a >> 5) & 0x1f;	\
      int b1 = a & 0x1f;	\
      int r2 = b >> 10;	\
      int g2 = (b >> 5) & 0x1f;	\
      int b2 = b & 0x1f;	\
      r1 = ((256-Scan::alpha_fact)*r1 + r2*Scan::alpha_fact) >> 8;	\
      g1 = ((256-Scan::alpha_fact)*g1 + g2*Scan::alpha_fact) >> 8;	\
      b1 = ((256-Scan::alpha_fact)*b1 + b2*Scan::alpha_fact) >> 8;	\
      *dd++ = (r1<<10) | (g1<<5) | b1;	\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_map_alpha_555

//------------------------------------------------------------------

#ifndef NO_draw_scanline_map_alpha_565

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_map_alpha_565
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      UShort a = *dd;							\
      UShort b = srcTex[((vv>>16)<<shifter) + (uu>>16)];		\
      int r1 = a >> 11;	\
      int g1 = (a >> 5) & 0x3f;	\
      int b1 = a & 0x1f;	\
      int r2 = b >> 11;	\
      int g2 = (b >> 5) & 0x3f;	\
      int b2 = b & 0x1f;	\
      r1 = ((256-Scan::alpha_fact)*r1 + r2*Scan::alpha_fact) >> 8;	\
      g1 = ((256-Scan::alpha_fact)*g1 + g2*Scan::alpha_fact) >> 8;	\
      b1 = ((256-Scan::alpha_fact)*b1 + b2*Scan::alpha_fact) >> 8;	\
      *dd++ = (r1<<11) | (g1<<5) | b1;	\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_map_alpha_565

//------------------------------------------------------------------

#ifndef NO_draw_scanline_transp_map

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_transp_map
#define SCANMAP 1
#define SCANLOOP \
    do									\
    {									\
      UShort c = srcTex[((vv>>16)<<shifter) + (uu>>16)];		\
      if (c)								\
      {									\
        *dd++ = c;							\
	*z_buffer++ = izz;						\
      }									\
      else								\
      {									\
        dd++;								\
	z_buffer++;							\
      }									\
      uu += duu;							\
      vv += dvv;							\
      izz += dzz;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_transp_map

//------------------------------------------------------------------

#ifndef NO_draw_scanline

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline
#define SCANLOOP \
    do									\
    {									\
      *dd++ = pal_table[srcTex[((uu>>16)&ander_w) + ((vv>>shifter_h)&ander_h)]];\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#define SCANEND \
    do									\
    {									\
      *z_buffer++ = izz;						\
      izz += dzz;							\
    }									\
    while (z_buffer <= lastZbuf)
#include "scanln16.inc"

#endif // NO_draw_scanline

//------------------------------------------------------------------

#ifndef NO_draw_scanline_z_buf

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_z_buf
#define SCANLOOP \
    do									\
    {									\
      if (izz >= (int)(*z_buffer))					\
      {									\
        *dd++ = pal_table[srcTex[((uu>>16)&ander_w) + ((vv>>shifter_h)&ander_h)]];\
	*z_buffer++ = izz;						\
      }									\
      else								\
      {									\
        dd++;								\
        z_buffer++;							\
      }									\
      uu += duu;							\
      vv += dvv;							\
      izz += dzz;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_z_buf

//------------------------------------------------------------------

#ifndef NO_draw_scanline_z_buf_private

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_z_buf_private
#define SCANLOOP \
    do									\
    {									\
      if (izz >= (int)(*z_buffer))					\
      {									\
        *dd++ = pal_table[priv_to_global[srcTex[((uu>>16)&ander_w) + ((vv>>shifter_h)&ander_h)]]];\
	*z_buffer++ = izz;						\
      }									\
      else								\
      {									\
        dd++;								\
        z_buffer++;							\
      }									\
      uu += duu;							\
      vv += dvv;							\
      izz += dzz;							\
    }									\
    while (dd <= lastD)
#include "scanln16.inc"

#endif // NO_draw_scanline_z_buf_private

//------------------------------------------------------------------

#ifndef NO_draw_scanline_private

#undef SCANFUNC
#undef SCANEND
#undef SCANLOOP
#undef SCANMAP
#define SCANFUNC draw_scanline_private
#define SCANLOOP \
    do									\
    {									\
      *dd++ = pal_table[priv_to_global[srcTex[((uu>>16)&ander_w) + ((vv>>shifter_h)&ander_h)]]];\
      uu += duu;							\
      vv += dvv;							\
    }									\
    while (dd <= lastD)
#define SCANEND \
    do									\
    {									\
      *z_buffer++ = izz;						\
      izz += dzz;							\
    }									\
    while (z_buffer <= lastZbuf)
#include "scanln16.inc"

#endif // NO_draw_scanline_private

//------------------------------------------------------------------

#ifndef NO_draw_scanline_flat

void Scan16::draw_scanline_flat (int xx, unsigned char* d,
			       unsigned long* z_buf,
			       float inv_z, float u_div_z, float v_div_z)
{
  (void)u_div_z; (void)v_div_z;
  int color = Scan::flat_color;
  long izz = QInt24 (inv_z);
  long dzz = QInt24 (Scan::M);
  UShort* dd = (UShort*)d;
  while (xx > 0)
  {

    *dd++ = color;
    xx--;
    *z_buf++ = izz;
    izz += dzz;
  }
}

#endif // NO_draw_scanline_flat

//------------------------------------------------------------------

#ifndef NO_draw_scanline_z_buf_flat

void Scan16::draw_scanline_z_buf_flat (int xx, unsigned char* d,
			       unsigned long* z_buf,
			       float inv_z, float u_div_z, float v_div_z)
{
  (void)u_div_z; (void)v_div_z;
  int i;
  int izz1, dzz, izz;
  int color = Scan::flat_color;
  UShort* dd = (UShort*)d;

  izz = QInt24 (inv_z);

  while (xx > 0)
  {
    inv_z += Scan::dM;

    i = Scan::INTERPOL_STEP;
    if (xx < i) i = xx;
    xx -= i;

    izz1 = QInt24 (inv_z);
    dzz = (izz1-izz)/Scan::INTERPOL_STEP;

    while (i-- > 0)
    {
      if (izz >= (int)(*z_buf))
      {
	*dd++ = color;
	*z_buf++ = izz;
      }
      else
      {
        dd++;
        z_buf++;
      }
      izz += dzz;
    }
  }
}

#endif // NO_draw_scanline_z_buf_flat

//------------------------------------------------------------------

#ifndef NO_draw_scanline_fog_555

void Scan16::draw_scanline_fog_555 (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
  if (xx <= 0) return;
  (void)u_div_z; (void)v_div_z;
  UShort* dd = (UShort*)d;
  UShort* lastD = dd + xx-1;
  int dzz, izz;
  izz = QInt24 (inv_z);
  dzz = QInt24 (Scan::M);

  ULong fog_dens = Scan::fog_density;
  int fog_r = Scan::fog_red;
  int fog_g = Scan::fog_green;
  int fog_b = Scan::fog_blue;

  do
  {
    UByte dens_dist = tables.exp_table[fog_dens / *(z_buf) - fog_dens / izz];
    int r = (*dd) >> 10;
    int g = ((*dd) >> 5) & 0x1f;
    int b = (*dd) & 0x1f;
    r += (dens_dist * (fog_r - r)) >> 8;
    g += (dens_dist * (fog_g - g)) >> 8;
    b += (dens_dist * (fog_b - b)) >> 8;
    *dd++ = (r<<10) | (g<<5) | b;
    z_buf++;
    izz += dzz;
  }
  while (dd <= lastD);
}

#endif // NO_draw_scanline_fog_555

//------------------------------------------------------------------

#ifndef NO_draw_scanline_fog_565

void Scan16::draw_scanline_fog_565 (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
  if (xx <= 0) return;
  (void)u_div_z; (void)v_div_z;
  UShort* dd = (UShort*)d;
  UShort* lastD = dd + xx-1;
  int dzz, izz;
  izz = QInt24 (inv_z);
  dzz = QInt24 (Scan::M);

  ULong fog_dens = Scan::fog_density;
  int fog_r = Scan::fog_red;
  int fog_g = Scan::fog_green;
  int fog_b = Scan::fog_blue;

  do
  {
    UByte dens_dist = tables.exp_table[fog_dens / *(z_buf) - fog_dens / izz];
    int r = (*dd) >> 11;
    int g = ((*dd) >> 5) & 0x3f;
    int b = (*dd) & 0x1f;
    r += (dens_dist * (fog_r - r)) >> 8;
    g += (dens_dist * (fog_g - g)) >> 8;
    b += (dens_dist * (fog_b - b)) >> 8;
    *dd++ = (r<<11) | (g<<5) | b;
    z_buf++;
    izz += dzz;
  }
  while (dd <= lastD);
}

#endif // NO_draw_scanline_fog_565

//------------------------------------------------------------------

#ifndef NO_draw_scanline_fog_view_555

void Scan16::draw_scanline_fog_view_555 (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
  if (xx <= 0) return;
  (void)u_div_z; (void)v_div_z; (void)inv_z;
  UShort* dd = (UShort*)d;
  UShort* lastD = dd + xx-1;

  ULong fog_dens = Scan::fog_density;
  int fog_r = Scan::fog_red;
  int fog_g = Scan::fog_green;
  int fog_b = Scan::fog_blue;

  do
  {
    UByte dens_dist = tables.exp_table[fog_dens / *(z_buf)];
    int r = (*dd) >> 10;
    int g = ((*dd) >> 5) & 0x1f;
    int b = (*dd) & 0x1f;
    r += (dens_dist * (fog_r - r)) >> 8;
    g += (dens_dist * (fog_g - g)) >> 8;
    b += (dens_dist * (fog_b - b)) >> 8;
    *dd++ = (r<<10) | (g<<5) | b;
    z_buf++;
  }
  while (dd <= lastD);
}

#endif // NO_draw_scanline_fog_view_555

//------------------------------------------------------------------

#ifndef NO_draw_scanline_fog_view_565

void Scan16::draw_scanline_fog_view_565 (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
  if (xx <= 0) return;
  (void)u_div_z; (void)v_div_z; (void)inv_z;
  UShort* dd = (UShort*)d;
  UShort* lastD = dd + xx-1;

  ULong fog_dens = Scan::fog_density;
  int fog_r = Scan::fog_red;
  int fog_g = Scan::fog_green;
  int fog_b = Scan::fog_blue;

  do
  {
    UByte dens_dist = tables.exp_table[fog_dens / *(z_buf)];
    int r = (*dd) >> 11;
    int g = ((*dd) >> 5) & 0x3f;
    int b = (*dd) & 0x1f;
    r += (dens_dist * (fog_r - r)) >> 8;
    g += (dens_dist * (fog_g - g)) >> 8;
    b += (dens_dist * (fog_b - b)) >> 8;
    *dd++ = (r<<11) | (g<<5) | b;
    z_buf++;
  }
  while (dd <= lastD);
}

#endif // NO_draw_scanline_fog_view_565

//------------------------------------------------------------------


#ifndef NO_draw_scanline_zfill_only

void Scan16::draw_scanline_zfill_only (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
  (void)u_div_z; (void)v_div_z; (void)inv_z; (void)d;
  ULong* lastZbuf = z_buf + xx-1;
  int dzz, izz;
  izz = QInt24 (inv_z);
  dzz = QInt24 (Scan::M);

  do
  {
    *z_buf++ = izz;
    izz += dzz;
  }
  while (z_buf <= lastZbuf);
}

#endif // NO_draw_scanline_zfill_only

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline

void Scan16::draw_pi_scanline (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  while (_dest < _destend)
  {
    if (z >= *zbuff)
    {
      *_dest = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
      *zbuff = z;
    }
    _dest++;
    zbuff++;
    u += du;
    v += dv;
    z += dz;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline_zfill

void Scan16::draw_pi_scanline_zfill (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  while (_dest < _destend)
  {
    *_dest++ = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
    *zbuff++ = z;
    u += du;
    v += dv;
    z += dz;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline_zfill

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline_gouroud_zfill_555

void Scan16::draw_pi_scanline_gouroud_zfill_555 (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w,
  long r, long g, long b, long dr, long dg, long db)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  UShort tex;
  int r1, g1, b1;
  while (_dest < _destend)
  {
    tex = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
    r1 = tex >> 10;
    g1 = (tex >> 5) & 0x1f;
    b1 = tex & 0x1f;
    r1 = (r1*r) >> (16+5);
    g1 = (g1*g) >> (16+5);
    b1 = (b1*b) >> (16+5);
    *_dest++ = (r1<<10) | (g1<<5) | b1;
    *zbuff++ = z;
    u += du; v += dv; z += dz;
    r += dr; g += dg; b += db;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline_gouroud_zfill_555

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline_gouroud_zfill_565

void Scan16::draw_pi_scanline_gouroud_zfill_565 (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w,
  long r, long g, long b, long dr, long dg, long db)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  UShort tex;
  int r1, g1, b1;
  while (_dest < _destend)
  {
    tex = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
    r1 = tex >> 11;
    g1 = (tex >> 5) & 0x3f;
    b1 = tex & 0x1f;
    r1 = (r1*r) >> (16+5);
    g1 = (g1*g) >> (16+6);
    b1 = (b1*b) >> (16+5);
    *_dest++ = (r1<<11) | (g1<<5) | b1;
    *zbuff++ = z;
    u += du; v += dv; z += dz;
    r += dr; g += dg; b += db;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline_gouroud_zfill_565

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline_gouroud_555

void Scan16::draw_pi_scanline_gouroud_555 (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w,
  long r, long g, long b, long dr, long dg, long db)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  UShort tex;
  int r1, g1, b1;
  while (_dest < _destend)
  {
    if (z >= *zbuff)
    {
      tex = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
      r1 = tex >> 10;
      g1 = (tex >> 5) & 0x1f;
      b1 = tex & 0x1f;
      r1 = (r1*r) >> (16+5);
      g1 = (g1*g) >> (16+5);
      b1 = (b1*b) >> (16+5);
      *_dest = (r1<<10) | (g1<<5) | b1;
      *zbuff = z;
    }
    _dest++;
    zbuff++;
    u += du; v += dv; z += dz;
    r += dr; g += dg; b += db;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline_gouroud_555

//------------------------------------------------------------------

#ifndef NO_draw_pi_scanline_gouroud_565

void Scan16::draw_pi_scanline_gouroud_565 (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w,
  long r, long g, long b, long dr, long dg, long db)
{
  UShort *_dest = (UShort *)dest;
  UShort *_destend = _dest + len;
  UShort tex;
  int r1, g1, b1;
  while (_dest < _destend)
  {
    if (z >= *zbuff)
    {
      tex = pal_table[*(bitmap + ((v >> 16) << bitmap_log2w) + (u >> 16))];
      r1 = tex >> 11;
      g1 = (tex >> 5) & 0x3f;
      b1 = tex & 0x1f;
      r1 = (r1*r) >> (16+5);
      g1 = (g1*g) >> (16+6);
      b1 = (b1*b) >> (16+5);
      *_dest = (r1<<11) | (g1<<5) | b1;
      *zbuff = z;
    }
    _dest++;
    zbuff++;
    u += du; v += dv; z += dz;
    r += dr; g += dg; b += db;
  } /* endwhile */
}

#endif // NO_draw_pi_scanline_gouroud_565

//---------------------------------------------------------------------------

#ifndef NO_mmx_draw_scanline

void Scan16::mmx_draw_scanline (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
}

#endif // NO_mmx_draw_scanline

//------------------------------------------------------------------

#ifndef NO_mmx_draw_scanline_map

void Scan16::mmx_draw_scanline_map (int xx, unsigned char* d,
                              unsigned long* z_buf,
                              float inv_z, float u_div_z, float v_div_z)
{
}

#endif // NO_mmx_draw_scanline_map

//------------------------------------------------------------------

#ifndef NO_mmx_draw_pi_scanline

void Scan16::mmx_draw_pi_scanline (void *dest, int len, long *zbuff, long u, long du,
  long v, long dv, long z, long dz, unsigned char *bitmap, int bitmap_log2w)
{
}

#endif // NO_mmx_draw_pi_scanline

//------------------------------------------------------------------
