/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef TCACHE16_H
#define TCACHE16_H

#include "util/types.h"
#include "render/software/tcache.h"

interface ITextureContainer;

/**
 * This class implements the software texture cache for 16-bit mode.
 */
class TextureCache16 : public TextureCache
{
private:
  /**
   * Create a texture in the texture cache.
   */
  virtual void create_lighted_texture (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);

  /// Create a texture in the texture cache (true_rgb version).
  void create_lighted_texture_true_rgb (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);
  /// Create a texture in the texture cache (24bit version).
  void create_lighted_texture_24bit (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);
  /// Create a texture in the texture cache (true_rgb and private colormaps).
  void create_lighted_texture_true_rgb_private (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);

protected:
  /**
   * For debugging: overlay the lightmap grid on the lighted texture. This function should
   * be called after calling create_lighted_texture ().
   */
  virtual void show_lightmap_grid (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);

  /**
   * For debugging: don't add the texture but only the lightmap data.
   */
  virtual void create_lighted_texture_lightmaps (TCacheData& tcd, TCacheLightedTexture* tclt, ITextureContainer* i_tc);

public:
  ///
  TextureCache16 ();
  ///
  virtual ~TextureCache16 ();
};

#endif /*TCACHE16_H*/
