/*
    Software renderer DLL
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "com/com.h"
#include "render/graph3d.h"
#include "geometry/math3d.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#ifdef OS_WIN32
#define DLL_NAME "SoftwareRender.dll"
#elif defined (OS_OS2)
#define DLL_NAME "softrndr.dll"
#else
#define DLL_NAME "softrndr.so"
#endif

static unsigned int RefCount = 0;
extern const CLSID CLSID_Graphics3DSoftware;

static DllRegisterData RegData;

// our main entry point...should be called when we're loaded.
#ifdef CS_STATIC_LINKED
STDAPI Software_DllInitialize ()
#else
STDAPI DllInitialize ()
#endif
{
  csCoInitialize(0);

  RegData.clsid = CLSID_Graphics3DSoftware;
	
  strcpy(RegData.szProgID, "crystalspace.graphics3d.software");
  strcpy(RegData.szInProcServer, DLL_NAME );
  strcpy(RegData.szFriendlyName, "csGraphics3DSoftware");

  tables.initialize ();
  return TRUE;
}

#ifndef CS_STATIC_LINKED

void STDAPICALLTYPE ModuleRelease ()
{
  RefCount--;
}

void STDAPICALLTYPE ModuleAddRef ()
{
  RefCount++;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
  return RefCount ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject (REFCLSID rclsid, REFIID riid, void** ppv)
{
  static csGraphics3DSoftwareFactory s_g3dFactory;

  if (rclsid == CLSID_Graphics3DSoftware)
    return s_g3dFactory.QueryInterface (riid, ppv);

  // if we get here, rclsid is a class we don't implement
  *ppv = NULL;
  return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer ()
{
  return csRegisterServer (&RegData);
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer ()
{
  return csUnregisterServer (&RegData);
}
#endif


// Implementation of csGraphics3DSoftwareFactory /////////////////

IMPLEMENT_UNKNOWN_NODELETE (csGraphics3DSoftwareFactory)

BEGIN_INTERFACE_TABLE (csGraphics3DSoftwareFactory)
	IMPLEMENTS_INTERFACE (IGraphicsContextFactory)
END_INTERFACE_TABLE ()

STDMETHODIMP csGraphics3DSoftwareFactory::CreateInstance (REFIID riid,
	ISystem* piSystem, void** ppv)
{
  if (!piSystem)
  {
    *ppv = 0;
    return E_UNEXPECTED;
  }

  csGraphics3DSoftware* pNew = new csGraphics3DSoftware (piSystem);
  if (!pNew)
  {
    *ppv = 0;
    return E_OUTOFMEMORY;
  }
	
  return pNew->QueryInterface (riid, ppv);
}

STDMETHODIMP csGraphics3DSoftwareFactory::LockServer (BOOL bLock)
{
  if (bLock)
    RefCount++;
  else
    RefCount--;

  return S_OK;
}

