/*
    Copyright (C) 1998 by Jorrit Tyberghein
    Written by Nathaniel Saint Martin <noote@bigfoot.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSSOUND_H
#define __CSSOUND_H

#include "sound/channel.h"
#include "sound/module.h"

// some defines for priority function
#define Low_Priority -50
#define Normal_Priority 0
#define High_Priority 50

/// list of priority algorythm used
enum PRIORITY_FUNC
{
  Priority_None=1,
  Priority_Up,
  Priority_Down
};

///
class csSound  
{
public:
  ///
  csSound(int nchannels=16);
  ///
  virtual ~csSound();

  /// open sound device at 'frequency' Hz, enable or disable 16 bit and stereo
  virtual bool Open(int frequency, bool bit16, bool stereo);
  /// close device
  virtual bool Close();

  /// is device run ?
  virtual bool isActive();
  /// is 16 bit mode device
  virtual bool is16Bits();
  /// is a stereo device
  virtual bool isStereo();
  /// return frequency
  virtual int getFrequency();
  /**
   * this return if you sound device work alone (timer, thread...)
   * or if it need to be filled
   */
  virtual bool getBackGround();

  /// return sound buffer memory
  virtual void *getMemory();
  /// return sound buffer memory size
  virtual long getMemorySize();
  /// access to next buffer (in double buffer case)
  virtual bool nextMemory();

  /// set volume general (can depend of hardware used) [0...1] [low...full]
	virtual void setVolume(float v);
  /// get volume general
  virtual float getVolume();
  
  /// add a Wave and return his Channel_ID
  Channel_ID add(Wave *wave, bool inLoop=false, int priority=0);
  /// add a Module and return his Channel_ID
  Channel_ID add(Module *m, bool inLoop = false, int priority=0);
  /// play a Wave, kill it when finish to play
  Channel_ID playEphemeral(Wave *wave, int priority=0);
  /// delete a channel of the list and destroy it
  bool delChannel(Channel_ID id);

  /// return a channel by his id
  Channel *getChannel(Channel_ID id);
  /// add a channel
  void addChannel(Channel *c);

  /// set the priority function for the choice of channel which be mixed
  void setPriorityFunc(PRIORITY_FUNC mpf);

  /// mixing function
  void mixingFunction();
  /// mixing a module
	void mixingModuleFunction(Channel *c);
  /// update() is for no-background sound driver, it's call in principal loop for filling sound memory
  void update();

  /// return the number of channel wich be mixed
  int getNumberOfChannels();

	/// start playing a channel
  void startChannel(Channel_ID id);

protected:
	float volume;
  bool Active;
  int Frequency;
  bool Bits16;
  bool Stereo;
  bool BackGround;

  void *Memory;
  long MemorySize;

  /// list of all stored channels
  Channel *AllChannels;

  /// this function choice in the 'list' of channels which are the 'n' channels will be mixed 
  void setChannels();
  /// update the channels in the 'list' whose aren't be choiced by setChannels()
  void updateChannels();
  /// kill the channels in 'list' which are finish to be played
  void killChannels();

  /// current priority function choice
  int PriorityFunc;

  /// the channels
  Channel ** Channels;
  int numberChannels;
};

#endif // __CSSOUND_H
