/*
    Copyright (C) 1998 by Jorrit Tyberghein
    Written by Nathaniel Saint Martin <noote@bigfoot.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __MODULE_H
#define __MODULE_H

#include "util/def.h"


enum ModuleEffectType
{
  NoEffect=0,
  Arpeggio,
  PortamentoUp,
  PortamentoDown,
  TonePortamento,
  Vibrato,
  TonePortamentoVolumeSlide,
  VibratoVolumeSlide,
  Tremolo,
  SetPan,
  SampleOffset,
  PositionJump,
  VolumeSlide,
  SetVolume,
  PatternBreak,
  SetSpeed,
  SetFilter,
  FineTuneSlideUp,
  FineTuneSlideDown,
  GlissandoControl,
  VibratoControl,
  SetFineTune,
  PatternLoop,
  TremoloControl,
  PanControl,
  NoteRetrig,
  FineTuneVolumeUp,
  FineTuneVolumeDown,
  NoteCut,
  NoteDelay,
  PatternDelay,
  InvertLoop
};

typedef struct
{
 int sample;
 int period_index;
 ModuleEffectType effect;
 int effect_param;
 int effect_paramx;
 int effect_paramy;
} ModuleNote;

///
class ModuleRow
{
public:
  ///
  ModuleRow()
  {
    notes = NULL;
  }
  ///
  ~ModuleRow()
  {
    if(notes) CHKB (delete [] notes);
    notes = NULL;
  }

  ModuleNote *notes;
};

///
class ModulePattern
{
public:
  ///
  ModulePattern()
  {
    rows = NULL;
  }
  ///
  ~ModulePattern()
  {
    if(rows) CHKB (delete [] rows);
    rows = NULL;
  }
  ModuleRow *rows;
};

///
class ModuleSample
{
public:
  ///
  ModuleSample()
  {
    Name = NULL;
    Data = NULL;
  }
  ///
  ~ModuleSample()
  {
    if(Name) CHKB (delete [] Name);
    Name = NULL;
    if(Data) CHKB (delete Data);
    Data = NULL;
  }
  char *Name;
  int nFineTune, nVolume, nLength, nLoopStart, nLoopLength, nLoopEnd;
  Wave *Data;
};

typedef struct
{
  int sample;
  int pos;
  int period_index;
  int period;
  bool left;
  float freq;
  int volume;
  int mixvol;
  int porta;
  int portasp;
  int vibspe;
  int vibdep;
  int tremspe;
  int tremdep;
  int panval;
  int sinepos;
  int sineneg;
} ModuleTrack;

///
class Module  
{
public:
  ///
  Module();
  ///
  virtual ~Module();

  ///
  void Clean();

  ///
  char * getName();
  ///
  void setName(char *name);
  ///
  int getSpeed();
  ///
  void setSpeed(int speed);
  ///
  int getBPM();
  ///
  void setBPM(int bpm);
  ///
  ModuleSample * getSample(int index);
  ///
  void setSamples(int nsamples, ModuleSample *samples);
  ///
  void setOrders(int norder, int *orders, int max=-1);
  ///
  void setPatterns(int ntrack, int nrow, int npattern, ModulePattern *patterns);
  ///
  void setLeftRightChannelOnTrack(int track, bool left);

  ///
  Channel * makeChannel(int Frequency, bool bit16, bool stereo, int size);
  ///
  Channel * getChannel();

  ///
  void mixingFunction();
  ///
  bool isStarted();
  ///
  bool Stop();
  ///
  void Pause();
  ///
  bool Start();

private:
  void MixingSubFunction(long position, long size);
  void UpdateRow();
  void UpdateEffects();
  void DoTremolo(int track);
  void DoVibrato(int track);
  void DoPortamento(int track);
  void SlideVolume(int track, int amount);

  bool stereo;
	bool bit16;
  int frequency;

  bool started;
  bool pause;

  char *Name;
  int Speed;
  int BPM;

  int *Orders;
  int nOrders, maxOrders;
  ModulePattern *Patterns;
  int nPatterns;
  ModuleSample *Samples;
  int nSamples;
  ModuleTrack *Tracks;
  int nTracks;
  int nRows;

  int currentRowIndex;
  ModuleRow *currentRow;
  int currentOrder;
  int ticks;
  int samples_left;

  Channel *channel;
};

#endif // __MODULE_H
