#ifdef DO_AMIGAOS

#include "util/inifile.h"
#include "util/def.h"
#include "system/system.h"

#ifdef COMP_GCC
#define __NOLIBBASE__
#endif

extern "C" {
#include <exec/types.h>
#include <exec/memory.h>
#include <exec/execbase.h>

#include <intuition/intuition.h>
#include <intuition/screens.h>
#include <intuition/intuitionbase.h>

#include <libraries/asl.h>

#include <graphics/gfxbase.h>
#include <graphics/gfx.h>
#include <graphics/modeid.h>

#include <devices/inputevent.h>

#include <proto/exec.h>
#include <proto/intuition.h>
#include <proto/graphics.h>
#include <proto/asl.h>
#include <proto/keymap.h>

#include "system/amiga/Cryst_rev.h"

#include <stdarg.h>
}

#include <time.h>

extern struct ExecBase *SysBase;

//static char *version = VERSTAG;
static struct Screen *scr;
static struct Window *wnd;
static unsigned short *empty_pointer;
static struct RastPort temprp;

struct GfxBase *GfxBase;
struct IntuitionBase *IntuitionBase;
struct Library *AslBase;
struct Library *KeymapBase;

static BOOL aga;
static void __regargs (*c2p8_ptr)(      char *start,
                                        struct BitMap *bitmap);

extern APTR __regargs c2p8_reloc(       char *chunkyend,
                                        struct BitMap *bitmap)
                                        asm("_c2p8_reloc");

long csSystemDriver::Time()
{
  return time (NULL)*1000;
}

SysGraphics2D::SysGraphics2D(int argc, char* argv[]) : csGraphics2D ()
{
  signal(SIGINT, SIG_IGN);                  // FIXME: make a proper handler
  Memory = (char *)AllocVec(FRAME_WIDTH*FRAME_HEIGHT, MEMF_CLEAR|MEMF_ANY);
}

SysGraphics2D::~SysGraphics2D(void)
{
  if(Memory) { FreeVec(Memory); Memory = NULL; }
}

bool SysGraphics2D::Open(char *Title)
{
  if (!csGraphics2D::Open (Title))
    return false;

  struct ScreenModeRequester *smr;
  int display_id, width, height;

  GfxBase = (struct GfxBase *)OpenLibrary((unsigned char *)"graphics.library", (unsigned long)39);
  IntuitionBase = (struct IntuitionBase *)OpenLibrary((unsigned char *)"intuition.library", (unsigned long)39);
  AslBase = OpenLibrary((unsigned char *)"asl.library", (unsigned long)37);
  KeymapBase=OpenLibrary((unsigned char *)"keymap.library", (unsigned long)0);

  Title=NULL;               // do something with Title-> no warning
  smr = (struct ScreenModeRequester *)AllocAslRequestTags(
          ASL_ScreenModeRequest,
          ASLSM_DoDepth, FALSE,
          ASLSM_DoWidth, FALSE,
          ASLSM_DoHeight, FALSE,
          ASLSM_DoOverscanType, FALSE,
          ASLSM_MinWidth, FRAME_WIDTH,
          ASLSM_MinHeight, FRAME_HEIGHT,
          ASLSM_MinDepth, 8,
          ASLSM_MaxDepth, 8,
          TAG_DONE, 0);

  width = FRAME_WIDTH;
  height = FRAME_HEIGHT;

  if(AslRequest(smr, 0))
          display_id = smr->sm_DisplayID;
  else
          display_id = 0x0000;

  FreeAslRequest(smr);

  scr = OpenScreenTags(NULL,
          SA_Width, width,
          SA_Height, height,
          SA_Left, width/2 - FRAME_WIDTH/2,
          SA_Depth, 8,
          SA_ShowTitle, FALSE,
          SA_Quiet, TRUE,
          SA_DisplayID, display_id,
          SA_Type, CUSTOMSCREEN,
          TAG_END, 0);

  wnd = OpenWindowTags(NULL,
          WA_Left, 0,
          WA_Top, 0,
          WA_Width, FRAME_WIDTH,
          WA_Height, FRAME_HEIGHT,
          WA_RMBTrap, TRUE,
          WA_IDCMP, IDCMP_RAWKEY,
          WA_Borderless, TRUE,
          WA_Backdrop, TRUE,
          WA_SimpleRefresh, TRUE,
          WA_CustomScreen, (ULONG)scr,
          TAG_END, 0);

  empty_pointer = (unsigned short *)AllocVec(8, MEMF_CHIP|MEMF_CLEAR);
  SetPointer(wnd, empty_pointer, 0, 0, 0, 0);

  if(GetBitMapAttr(scr->RastPort.BitMap, BMA_FLAGS) & BMF_STANDARD)
  {
    CsPrintf (MSG_INITIALIZATION, "Using internal c2p routines...\n");
    aga = TRUE;
    c2p8_ptr = c2p8_reloc(Memory+FRAME_WIDTH*FRAME_HEIGHT, scr->RastPort.BitMap);
  }
  else
  {
    c2p8_ptr = NULL;
    aga = FALSE;
  }

  CopyMem(&scr->RastPort, &temprp, sizeof(struct RastPort));

  temprp.Layer = NULL;
  temprp.BitMap = AllocBitMap(FRAME_WIDTH*2, 1, 8, BMF_CLEAR, scr->RastPort.BitMap);

  return 1;
}

void SysGraphics2D::Close(void)
{
  if(wnd) { CloseWindow(wnd); wnd = NULL; }
  if(scr) { CloseScreen(scr); scr = NULL; }
  if(empty_pointer) { FreeVec(empty_pointer); empty_pointer = NULL; }
  if(c2p8_ptr) { FreeVec(c2p8_ptr); c2p8_ptr = NULL; }
  if(Memory) { FreeVec(Memory); Memory = NULL; }
  if(temprp.BitMap) { FreeBitMap(temprp.BitMap); temprp.BitMap = NULL; }
  if(KeymapBase) {CloseLibrary(KeymapBase); KeymapBase = NULL; }
  if(AslBase) { CloseLibrary(AslBase); AslBase = NULL; }
  if(IntuitionBase) { CloseLibrary((struct Library *)IntuitionBase); IntuitionBase = NULL; }
  if(GfxBase) { CloseLibrary((struct Library *)GfxBase); GfxBase = NULL; }
  csGraphics2D::Close ();
}

void SysGraphics2D::Print (csRect *area)
{
  if(aga)
  {
    c2p8_ptr(Memory, scr->RastPort.BitMap);
  } else
    WritePixelArray8(&scr->RastPort, (unsigned long)0, (unsigned long)0,
      (unsigned long)FRAME_WIDTH-1, (unsigned long)FRAME_HEIGHT-1, (unsigned char *)Memory, &temprp);
}

void SysGraphics2D::SetRGB(int i, int r, int g, int b)
{
  // set a rgb color in the palette of your graphic interface
  SetRGB32(&scr->ViewPort, i, r*0x01010101, g*0x01010101, b*0x01010101);
  csGraphics2D::SetRGB (i, r, g, b);
}

// Keyboard fonctions
SysKeyboardDriver::SysKeyboardDriver() : csKeyboardDriver()
{
  // Create your keyboard interface
}

SysKeyboardDriver::~SysKeyboardDriver(void)
{
  // Destroy your keyboard interface
  Close();
}

bool SysKeyboardDriver::Open(void)
{
  // Open your keyboard interface
  return 1;
}

void SysKeyboardDriver::Close(void)
{
  // Close your keyboard interface
}

// Mouse fonctions
SysMouseDriver::SysMouseDriver() : csMouseDriver()
{
// Initialize mouse system
}

SysMouseDriver::~SysMouseDriver(void)
{
  Close();
}

bool SysMouseDriver::Open()
{
// Open mouse system
  return 1;
}

void SysMouseDriver::Close()
{
// Close mouse system
}

// System loop !
void SysSystemDriver::Loop(void)
{
  char Key1;

  struct Message *msg;
  struct IntuiMessage copy;
  struct InputEvent ie;

  int key = 0, shift = 0, alt = 0, ctrl = 0;
  int key_up = 0, key_down = 0, key_left = 0, key_right = 0;

  // don't touch, please !

  while(!Shutdown && !ExitLoop)
  {
    if (key) Keyboard->do_keypress(key);
    if (key_up) Keyboard->do_keypress(key_up);
    if (key_down) Keyboard->do_keypress(key_down);
    if (key_left) Keyboard->do_keypress(key_left);
    if (key_right) Keyboard->do_keypress(key_right);

    NextFrame() ; // don't remove, please !

    while((msg = GetMsg(wnd->UserPort)))
    {
      CopyMem(msg, &copy, sizeof(struct IntuiMessage));
      ReplyMsg(msg);

      if(copy.Class ==  IDCMP_RAWKEY)
      {
        if(copy.Code & IECODE_UP_PREFIX)
        {
          copy.Code &= ~IECODE_UP_PREFIX;
          switch(copy.Code)
          {
            case 0x4c: key_up = 0; break;
            case 0x4d: key_down = 0; break;
            case 0x4f: key_left = 0; break;
            case 0x4e: key_right = 0; break;
          }
          key = 0;
        }
        else
        {
          ie.ie_Class=IECLASS_RAWKEY;
          ie.ie_SubClass=0;
          ie.ie_Code=copy.Code;
          ie.ie_Qualifier=copy.Qualifier;
          ie.ie_EventAddress=(APTR *) *((ULONG *)copy.IAddress);

          switch(copy.Code)
          {
            case 0x4c: key_up = CSKEY_UP; key = 0; break;
            case 0x4d: key_down = CSKEY_DOWN; key = 0; break;
            case 0x4f: key_left = CSKEY_LEFT; key = 0; break;
            case 0x4e: key_right = CSKEY_RIGHT; key = 0; break;
            case 0x3f: key = CSKEY_PGUP; break;
            case 0x1f: key = CSKEY_PGDN; break;
            case 0x1d: key = CSKEY_END; break;
            case 0x45: key = CSKEY_ESC; break;
            case 0x44: key = '\n'; break;                 // Return Key
            default: {MapRawKey(&ie,&Key1,1,NULL); key=(int) Key1; break;}
          }
        }
        shift = !!(copy.Qualifier & (IEQUALIFIER_LSHIFT|IEQUALIFIER_RSHIFT));
        alt = !!(copy.Qualifier & (IEQUALIFIER_LALT|IEQUALIFIER_RALT));
        ctrl = !!(copy.Qualifier & IEQUALIFIER_CONTROL);
      }
    }
  } // while (!Shutdown)
}

#endif // DO_AMIGAOS

