/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef OS_BE

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#include "util/def.h"
#include "system/system.h"
#include "util/inifile.h"
#include "system/be/csbe.h"
#include "system/be/driver2d/isysg2d.h"
#include "system/be/driver2d/CrystWindow.h"
//#include "render/graph3d.h"
#include "general/console.h"

static bool LoopStarted = false;// For Print() to know whenever to flush output

// The System driver ////////////////

BEGIN_INTERFACE_TABLE(SysSystemDriver)
  IMPLEMENTS_COMPOSITE_INTERFACE(BeLibSystemDriver)
  IMPLEMENTS_COMPOSITE_INTERFACE(System)
END_INTERFACE_TABLE()

IMPLEMENT_UNKNOWN_NODELETE(SysSystemDriver)

/* not too much to be said... */
class CrystApp : public BApplication {
public:
				CrystApp(SysSystemDriver *from) : BApplication("application/x-vnd.xsware-crystal") {
					driver = from;
					mspf = 100 * 1000;
				};
	void Pulse(void);
	bool QuitRequested();
	void CrystApp::MessageReceived(BMessage *message);

	void doMouseAction(BPoint point, int16 button, bool shift, bool alt, bool ctrl);
	void doKeyAction(int key, bool down, bool shift, bool alt, bool ctrl);
	bigtime_t mspf;
private:
	SysSystemDriver	*driver;
	BWindow			*aWindow;
	bigtime_t		milisecpf;
};

static CrystApp *app = NULL;
static bool firstRun;

bool CrystApp::QuitRequested()
{
	driver->Shutdown = true;
	return(TRUE);
}

void CrystApp::Pulse() {
//	bigtime_t		time;

//	time = system_time()+milisecpf;
//PostMessage('next');
//    driver->NextFrame();
//	time -= system_time();
#if 0
	if( time < 0) {
		SetPulseRate(1000000/reqfps);
	}
    if(g2d->win->Lock()){
    	if(g2d->view->IsInMotion())
    		Mouse->do_mousemotion (SystemGetTime (), g2d->view->lastloc.x, g2d->view->lastloc.y);
    	g2d->win->Unlock();
    }
#endif
}

void CrystApp::MessageReceived(BMessage *msg)
{
	bigtime_t	time = system_time();

	switch(msg->what) {
	// Switch between full-screen mode and windowed mode.
	case 'next' :
		driver->NextFrame(system_time () - time, system_time ());
		time = system_time() - time;
		mspf = time;
		break;
	case 'keys' :
		doKeyAction(msg->FindInt16("key"), msg->FindBool("down"),
			msg->FindBool("shift"),msg->FindBool("alt"),msg->FindBool("ctrl"));
		break;
	case 'mous' :
		doMouseAction(msg->FindPoint("loc"),msg->FindInt16("butn"),
			msg->FindBool("shift"),msg->FindBool("alt"),msg->FindBool("ctrl"));
		break;
	default :
		BApplication::MessageReceived(msg);
		break;
	}
}

void CrystApp::doMouseAction(BPoint point, int16 button, bool shift, bool alt, bool ctrl) 
{
//  		Mouse->do_mousemotion (g2d->view->lastloc.x, g2d->view->lastloc.y);
	if(button)
		driver->Mouse->do_buttonpress (SystemGetTime (), button, point.x, point.y, shift, alt, ctrl);
	else
		driver->Mouse->do_buttonrelease (SystemGetTime (), button, point.x, point.y);
}

void CrystApp::doKeyAction(int key, bool down, bool shift, bool alt, bool ctrl)
{
	switch (key) {
//		case B_Alt_L:
//		case B_Alt_R:      key = CSKEY_ALT; break;
//		case B_Control_L:
//		case B_Control_R:  key = CSKEY_CTRL; break;
//		case B_Shift_L:
//		case B_Shift_R:    key = CSKEY_SHIFT; break;
		case B_UP_ARROW:         	key = CSKEY_UP; break;
		case B_DOWN_ARROW:       	key = CSKEY_DOWN; break;
		case B_LEFT_ARROW:       	key = CSKEY_LEFT; break;
	    case B_RIGHT_ARROW:      	key = CSKEY_RIGHT; break;
	    case B_BACKSPACE:  			key = CSKEY_BACKSPACE; break;
	    case B_INSERT:     			key = CSKEY_INS; break;
	    case B_DELETE:     			key = CSKEY_DEL; break;
	    case B_PAGE_UP:    			key = CSKEY_PGUP; break;
	    case B_PAGE_DOWN:  			key = CSKEY_PGDN; break;
	    case B_HOME:       			key = CSKEY_HOME; break;
	    case B_END:        			key = CSKEY_END; break;
	    case B_ESCAPE:     			key = CSKEY_ESC; break;
	    case B_TAB:        			key = CSKEY_TAB; break;
	    case B_RETURN:     			key = CSKEY_ENTER; break;
//	    case B_F1_KEY:				key = CSKEY_F1; break;
//	    case B_F2_KEY:				key = CSKEY_F2; break;
//	    case B_F3_KEY:				key = CSKEY_F3; break;
//	    case B_F4_KEY:				key = CSKEY_F4; break;
//	    case B_F5_KEY:				key = CSKEY_F5; break;
//	    case B_F6_KEY:				key = CSKEY_F6; break;
//	    case B_F7_KEY:				key = CSKEY_F7; break;
//	    case B_F8_KEY:				key = CSKEY_F8; break;
//	    case B_F9_KEY:				key = CSKEY_F9; break;
//	    case B_F10_KEY:				key = CSKEY_F10; break;
//	    case B_F11_KEY:				key = CSKEY_F11; break;
//	    case B_F12_KEY:				key = CSKEY_F12; break;
        default:            		break;
	}
	if (down)
		driver->Keyboard->do_keypress (SystemGetTime (), key); //, shift, alt, ctrl);
	else
		driver->Keyboard->do_keyrelease (SystemGetTime (), key);
}


SysSystemDriver::SysSystemDriver () : csSystemDriver()
{
  app = new CrystApp(this);
  firstRun = true;
};

long csSystemDriver::Time()
{
  return system_time()/1000;
}

void SysSystemDriver::SetSystemDefaults ()
{
  csSystemDriver::SetSystemDefaults ();

  //FULL_SCREEN = false;

  void init_sig ();
//  init_sig ();
}

// Exit
void exit_clean (int rc)
{
  extern void cleanup ();
  cleanup ();
  exit (rc);
}

// System loop !
void SysSystemDriver::Loop(void)
{
	LoopStarted = true;
	bigtime_t		time;
	BView *dpy;
	int	reqfps = 8;
	thread_id	my_thread;

	IBeLibGraphicsInfo* piG2D = NULL;
	HRESULT hRes = Gfx2D->QueryInterface ((REFIID)IID_IBeLibGraphicsInfo, (void**)&piG2D);
	if (SUCCEEDED(hRes))
	{
		hRes = piG2D->GetDisplay (&dpy);
		piG2D->Release ();
		piG2D = NULL;
	}

	if(firstRun) {
		firstRun = false;
		my_thread = spawn_thread(SysSystemDriver::LoopThread, "LoopThread",
							 B_DISPLAY_PRIORITY, (void*)this);
		resume_thread(my_thread);
		app->SetPulseRate(1000000/reqfps);
		app->Run();
	} else while (!Shutdown && !ExitLoop) {
		// we want a frame to take at least 16 ms.
		time = system_time()+1000000/reqfps;
//		NextFrame();
#if 0
	    if(g2d->win->Lock()){
	    	if(g2d->view->IsInMotion())
	    		Mouse->do_mousemotion (SystemGetTime (), g2d->view->lastloc.x, g2d->view->lastloc.y);
	    	g2d->win->Unlock();
	    }
#endif
		time -= system_time();
		if (time > 0)
			snooze(time);
	}
}

/*
 * X specific extensions.
 */
bool SysSystemDriver::perform_extension (char* cmd, char* arg)
{
  (void)arg;

// @@@ WE NEED TO RETHINK THIS!
#if 0
  if (!strcasecmp ("sim_pal", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->recompute_simulated_palette ();
    return true;
  }
  else if (!strcasecmp ("sim_grey", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->recompute_grey_palette ();
    return true;
  }
  else if (!strcasecmp ("sim_332", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->restore_332_palette ();
    return true;
  }
#endif
  return false;
}

// COM Implementation

IMPLEMENT_COMPOSITE_UNKNOWN_AS_EMBEDDED( SysSystemDriver, BeLibSystemDriver )


/*
 * Signal handler to clean up and give some
 * final debugging information.
 */
extern void debug_dump ();
void handler (int sig)
{
  static bool in_exit = false;
  if (in_exit) exit (1);
  in_exit = true;

  if (sig == SIGINT)
    fprintf (stderr, "\n^C\n");
  else
    fprintf (stderr, "SIGNAL %d CAUGHT!!!\n", sig);

  int err;
  err = errno;
  printf ("error %d: %s\n", err, strerror (err));

  if (sig != SIGINT) debug_dump ();

  exit_clean (1);
} 

void init_sig ()
{
  void handler (int sig);

  signal (SIGHUP, handler);
  signal (SIGINT, handler);
//  signal (SIGTRAP, handler);
  signal (SIGABRT, handler);
  signal (SIGALRM, handler);
  signal (SIGTERM, handler);
  signal (SIGPIPE, handler);
  signal (SIGSEGV, handler);
  signal (SIGBUS, handler);
  signal (SIGFPE, handler);
  signal (SIGILL, handler);
}


// Keyboard fonctions
SysKeyboardDriver::SysKeyboardDriver() : csKeyboardDriver ()
{
  // Create your keyboard interface
}

bool SysKeyboardDriver::Open(csEventQueue *EvQueue)
{
  csKeyboardDriver::Open (EvQueue);
  // Open your keyboard interface
  return true;
}

void SysKeyboardDriver::Close ()
{
  csKeyboardDriver::Close();
  // Close your keyboard interface
}

// Mouse fonctions
SysMouseDriver::SysMouseDriver() : csMouseDriver ()
{
// Initialize mouse system
}

bool SysMouseDriver::Open(csEventQueue *EvQueue)
{
  csMouseDriver::Open (EvQueue);
// Open mouse system
  return 1;
}

void SysMouseDriver::Close()
{
// Close mouse system
}


//@@@ JORRIT: this is a bad place but I don't know where to define it otherwise
BOOL ModuleIsStopping ()
{
  return false;
}


// This is the thread doing the loop itself.
long SysSystemDriver::LoopThread(void *data)
{
	bigtime_t			delay;
	SysSystemDriver		*sys;
	bool	shutdown = false;
	
	// receive a pointer to the CrystWindow object.
	sys = (SysSystemDriver*)data;
	snooze(100000);
	
	// loop, frame after frame, until asked to quit.
	while (!shutdown) {
		
		// get the right to do direct screen access.
		//acquire_sem(w->drawing_lock);
		app->PostMessage('next');
//		if(be_app->Lock()) {
//			shutdown = sys->Shutdown;
//			delay = sys->mspf;
//			be_app->Unlock();
//		}
//		if(delay>0)
//			snooze(delay);
	snooze(app->mspf);

		// release the direct screen access
		//release_sem(w->drawing_lock);
	}	
	return 0;
}


#endif //OS_BE
