/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#define INITGUID

#include "com/com.h"
#include "system/isystem.h"
#include "system/be/driver2d/belibg2d.h"
#include "system/be/beitf.h"
#include "system/be/driver2d/CrystWindow.h"
#include "engine/basic/csrect.h"

BEGIN_INTERFACE_TABLE(csGraphics2DBeLib)
  IMPLEMENTS_COMPOSITE_INTERFACE_EX( IGraphics2D, XGraphics2D )
  IMPLEMENTS_COMPOSITE_INTERFACE_EX( IGraphicsInfo, XGraphicsInfo )
  IMPLEMENTS_COMPOSITE_INTERFACE_EX( IBeLibGraphicsInfo, XBeLibGraphicsInfo )
END_INTERFACE_TABLE()

IMPLEMENT_UNKNOWN_NODELETE(csGraphics2DBeLib)

// csGraphics2DXLib functions
csGraphics2DBeLib::csGraphics2DBeLib(ISystem* piSystem, bool bUses3D) : csGraphics2D (piSystem)
{
//  Screen* screen_ptr;
  HRESULT rc;
  IBeLibSystemDriver* piBeLib = NULL;

  rc = piSystem->QueryInterface ((REFIID)IID_IBeLibSystemDriver, (void**)&piBeLib);
  if (FAILED (rc))
  {
    //CsPrintf (MSG_FATAL_ERROR, "csGraphics2DBeLib::Open() -- ISystem does not support IBeLibSystemDriver.");
    printf ("csGraphics2DBeLib::Open() -- ISystem does not support IBeLibSystemDriver.");
    exit (1);
  }

  //CsPrintf (MSG_INITIALIZATION, "Crystal Space X windows version");
  printf ("Crystal Space BeOS version.\n");

depth=16;

	switch(depth) {
		case 15:
		case 16:
			cryst_bitmap = new BBitmap(BRect(0,0,Width-1,Height-1),(depth == 16) ? B_RGB16 : B_RGB15);
//								false ? ((depth == 16) ? B_RGB16_BIG : B_RGB15_BIG) :
//									  ((depth == 16) ? B_RGB16 : B_RGB15));
		    // calculate CS's pixel format structure.
		    pfmt.PixelBytes = 2;
		    pfmt.PalEntries = 0;
		    pfmt.RedMask   = (depth == 16) ? 0x1f <<11 : 0x1f <<10;
		    pfmt.GreenMask = (depth == 16) ? 0x3f << 5 : 0x1f << 5;
		    pfmt.BlueMask = 0x1f;
		    complete_pixel_format();
			break;
		case 8 :
		default:
			cryst_bitmap = new BBitmap(BRect(0,0,Width-1,Height-1),B_CMAP8);
			break;
	}

//  xim = NULL;
  Memory = NULL;

  // If in 16-bit mode, redirect drawing routines
  if (pfmt.PixelBytes == 2)
  {
    DrawPixel = DrawPixel16;
    WriteChar = WriteChar16;
    GetPixelAt = GetPixelAt16;
    DrawSprite = DrawSprite16;
  } /* endif */
}

csGraphics2DBeLib::~csGraphics2DBeLib(void)
{
  // Destroy your graphic interface
  Close();
  //if (Memory && sim_depth != 0) CHKB (delete [] Memory);
}

bool csGraphics2DBeLib::Open(char *Title)
{
	// Open your graphic interface
	if (!csGraphics2D::Open (Title)) return false;

	dpy = new	CrystView(BRect(0,0,Width-1,Height-1));
	window = new 	CrystWindow(BRect(32,32,Width+32,Height+32), "Crystal",dpy);
	
	window->Show();
	if(window->Lock()) {
		dpy->MakeFocus();
		window->Unlock();
	}

	Memory = (unsigned char *)cryst_bitmap->Bits();
//	for(int i = 0; i < FRAME_HEIGHT; i++)
//		LineAddress [i] = i * cryst_bitmap->BytesPerRow();

//  return 1;
  // Open window
//   = new BWindow (BRect(64, 16, Width, Height), "Yes man", TITLED_WINDOW);

//  attr.event_mask = ExposureMask | KeyPressMask | KeyReleaseMask |
//  	FocusChangeMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask;
//  XChangeWindowAttributes (dpy, window, CWEventMask, &attr);

  // Create backing store
//  if (!xim)
//  {
//    xim = XGetImage (dpy, window, 0, 0, Width, Height, AllPlanes, ZPixmap);
//    real_Memory = (unsigned char*)(xim->data);

    // If not simulating depth then Memory is equal to real_Memory.
    // If simulating then we allocate a new Memory array in the faked format.
//    if (!sim_depth) Memory = real_Memory;
//    else
//    {
//      CHK (Memory = new unsigned char [Width*Height*pfmt.PixelBytes]);
//    }
//CHK (Memory = new unsigned char [Width*Height*pfmt.PixelBytes]);
//  }
  Clear (0);
  return true;
}

void csGraphics2DBeLib::Close(void)
{
  // Close your graphic interface
  csGraphics2D::Close ();
}

void csGraphics2DBeLib::Print (csRect *area)
{
	if( window->Lock()) {
		dpy->DrawBitmap(cryst_bitmap);
//		dpy->DrawBitmapAsync(cryst_bitmap);
		window->Unlock();
	}
//      XPutImage (dpy, window, gc, xim, 0, 0, 0, 0, Width, Height);
}

void csGraphics2DBeLib::SetRGB(int i, int r, int g, int b)
{
  csGraphics2D::SetRGB (i, r, g, b);
}

