#include <stdlib.h>
#include "com/com.h"
#include "system/be/driver2d/belibg2d.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#define DLL_NAME "lib2d.so"

static unsigned int gb_cRef = 0;

extern const CLSID CLSID_Graphics2DBeLib;

static DllRegisterData gb_regData;

void STDAPICALLTYPE ModuleRelease(void)
{
    gb_cRef--;
}

void STDAPICALLTYPE ModuleAddRef(void)
{
    gb_cRef++;
}   

STDAPI DllInitialize ()
{
  csCoInitialize(0);
  gb_regData.clsid = CLSID_Graphics2DBeLib;

  strcpy(gb_regData.szProgID, "crystalspace.graphics2d.default");
  strcpy(gb_regData.szInProcServer, DLL_NAME );
  strcpy(gb_regData.szFriendlyName, "csGraphics2D BeLib Implementation");

  return TRUE;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
    return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)
{
    static csGraphics2DBeLibFactory s_g2dFactory;
    
    if (rclsid == CLSID_Graphics2DBeLib)
        return s_g2dFactory.QueryInterface(riid, ppv);
    
    // if we get here, rclsid is a class we don't implement
    *ppv = NULL;
    return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer()
{
    HRESULT hRes;
    hRes = csRegisterServer(&gb_regData);
    return hRes;
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer()
{
    HRESULT hRes;
    
    hRes = csRegisterServer(&gb_regData);
    return hRes;
}

// Implementation of the csGraphics2DBeLib factory... ///////////

IMPLEMENT_UNKNOWN_NODELETE( csGraphics2DBeLibFactory )

BEGIN_INTERFACE_TABLE( csGraphics2DBeLibFactory )
	IMPLEMENTS_INTERFACE( IGraphics2DFactory )
END_INTERFACE_TABLE()

STDMETHODIMP csGraphics2DBeLibFactory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics2DBeLib* pNew = new csGraphics2DBeLib(piSystem, false);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}

STDMETHODIMP csGraphics2DBeLibFactory::LockServer(BOOL bLock)
{
	if (bLock)
		gb_cRef++;
	else
		gb_cRef--;

	return S_OK;
}

