/*
    Copyright (C) 1998 by Jorrit Tyberghein
    Written by Dan Ogles

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __ISYSTEM_H__
#define __ISYSTEM_H__

#include "com/com.h"
#include "system/sysprint.h"

extern const IID IID_ISystem;

/**
 * This interface serves as a way for plug-ins to query Crystal Space about settings,
 * It also serves as a way for plug-ins to print through Crystal Space's printing
 * interfaces.
 */
interface ISystem : public IUnknown
{
  /// returns the width requested by the configuration.
  STDMETHOD (GetWidthSetting) (int& retval) = 0;
  /// returns the height requested by the configuration.
  STDMETHOD (GetHeightSetting) (int& retval) = 0;
  /// get the full screen setting
  STDMETHOD (GetFullScreenSetting) (bool& retval) = 0;
  /// get the depth setting
  STDMETHOD (GetDepthSetting) (int& retval) = 0;
  /// print a string to the specified device.
  STDMETHOD (Print) (int mode, char* string) = 0;
  /// check if system is shutting down
  STDMETHOD (GetShutdown) (bool &Shutdown) = 0;
};

#endif
