/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include "com/com.h"
#include "system/libggi/driver2d/ggig2d.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#define DLL_NAME "csggi2d.so"

static unsigned int gb_cRef = 0;

extern const CLSID CLSID_Graphics2DGGI;

static DllRegisterData gb_regData;

#ifdef CS_STATIC_LINKED
STDAPI GGI2D_DllInitialize ()
#else
STDAPI DllInitialize ()
#endif
{
    csCoInitialize(0);
    gb_regData.clsid = CLSID_Graphics2DGGI;
    
    strcpy(gb_regData.szProgID, "crystalspace.graphics2d.default");
    strcpy(gb_regData.szInProcServer, DLL_NAME );
    strcpy(gb_regData.szFriendlyName, "csGraphics2D GGI Implementation");
    return TRUE;
}

#ifndef CS_STATIC_LINKED

void STDAPICALLTYPE ModuleRelease(void)
{
    gb_cRef--;
}

void STDAPICALLTYPE ModuleAddRef(void)
{
    gb_cRef++;
}   

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
    return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)
{
    static csGraphics2DGGIFactory s_g2dFactory;
    
    if (rclsid == CLSID_Graphics2DGGI)
        return s_g2dFactory.QueryInterface(riid, ppv);
    
    // if we get here, rclsid is a class we don't implement
    *ppv = NULL;
    return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer()
{
    HRESULT hRes;
    hRes = csRegisterServer(&gb_regData);
    return hRes;
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer()
{
    HRESULT hRes;
    
    hRes = csRegisterServer(&gb_regData);
    return hRes;
}
#endif

// Implementation of the csGraphics2DGGI factory... ///////////

IMPLEMENT_UNKNOWN_NODELETE( csGraphics2DGGIFactory )

BEGIN_INTERFACE_TABLE( csGraphics2DGGIFactory )
	IMPLEMENTS_INTERFACE( IGraphics2DFactory )
END_INTERFACE_TABLE()

STDMETHODIMP csGraphics2DGGIFactory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics2DGGI* pNew = new csGraphics2DGGI(piSystem, false);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}

STDMETHODIMP csGraphics2DGGIFactory::LockServer(BOOL bLock)
{
	if (bLock)
		gb_cRef++;
	else
		gb_cRef--;

	return S_OK;
}

