/*
    Copyright (C) 1998 by Jorrit Tyberghein
    Written by Nathaniel Saint Martin <noote@bigfoot.com>
    Linux sound module by Gert Steenssens <gsteenss@eps.agfa.be>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __SYSSOUND_H
#define __SYSSOUND_H


#include "sound/cssnddrv.h"

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/msg.h>


// sound message struct
typedef struct soundentry
{
  long mtype;
  int shmid ;
  size_t shmsize;  
} soundmsg;


// block of sound data
class SoundBlock
{
public:

  // creates a new soundblock
  SoundBlock(size_t block_size);
  
  ~SoundBlock();

  // returns pointer to block of memory
  unsigned char* Memory(); 

  // returns size of block
  size_t MemorySize();
  
  // returns sound message for audio device
  soundmsg Message();

private:

  soundmsg sndentry;
  unsigned char* ptr;
 
};

// audio device 
class AudioDevice
{
public:

  AudioDevice();

  // opens device with specified params
  size_t Open(int frequency, bool bit16, bool stereo);

  // close device
  void Close();

  // play a soundblock on the device
  void Play(SoundBlock& sound);

private:  

  // starts the audio loop which writes sound data to the device
  void startAudioLoop();

private:

  int qid;
  pid_t pid;
  int audio;
  size_t block_size;

};



// system sound module
class SysSoundDriver : public csSoundDriver
{
public:
  //
  SysSoundDriver(int nchannels=16);

  // open sound device at 'frequency' Hz, enable or disable 16 bit and stereo
  virtual bool Open(int frequency, bool bit16, bool stereo);

  // close device
  virtual bool Close();

  // will be called by main system loop
  virtual void update();

private: 

  // used to check if any sound is playing
  bool ChannelsInUse();

  // used to setup timer when background=true (not currently used)
  void SetupTimer();
  
private:

  AudioDevice device; 
  size_t block_size;

}; 

#endif // __SYSSOUND_H
