/*
    Copyright (C) 1998 by Jorrit Tyberghein and Steve Israelson.
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*----------------------------------------------------------------
	Written by Steve Israelson 1997.
	
	Initial pass at this class.  Should eventually be modified to use
	DrawSprockets and InputSprockets.
----------------------------------------------------------------*/
#include "system/system.h"
#include "MacGraphics.h"

#include <SegLoad.h>
#include <Memory.h>
#include <Palettes.h>
#include <TextUtils.h>
#include <Types.h>
#include <Memory.h>
#include <Quickdraw.h>
#include <Fonts.h>
#include <Events.h>
#include <Menus.h>
#include <Windows.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <OSUtils.h>
#include <ToolUtils.h>

#include "util/inifile.h"
#include <stdarg.h>

// a class variable
static WindowPtr	mainPtr;

/*----------------------------------------------------------------
	Commandline parsing.
----------------------------------------------------------------*/

/*----------------------------------------------------------------
	Construct a graphics object.  This object provides a place on
	screen to draw, and a place offscreen to render into.
----------------------------------------------------------------*/
SysGraphics2D::SysGraphics2D(int argc, char* argv[]) : csGraphics2D ()
	{
  /* Initialize all the needed managers. */
	::InitGraf(&qd.thePort);
	::InitFonts();
	::InitWindows();
	::InitMenus();
	::TEInit();
	::InitDialogs(nil);
	::InitCursor();

	::GetDateTime((unsigned long*) &qd.randSeed);

	// some stuff for estimating the fps we are getting
	frame = 0;
	timer = ::TickCount();
	fpsString[0] = 0;

	offscreen = nil;	// we can not construct the offscreen until the color table is known
	// make an empty color table to be filled in later
	theCTab = (CTabHandle)::NewHandle(sizeof(ColorSpec) * 256 + 8);

	}

/*----------------------------------------------------------------
	When done rendering, destroy the graphics object.
----------------------------------------------------------------*/
SysGraphics2D::~SysGraphics2D()
	{
	::DisposeHandle((Handle)theCTab);
	if (offscreen)
		::DisposeGWorld(offscreen);
	if (mainPtr)
		::DisposeWindow(mainPtr);
	::RestoreDeviceClut(::GetMainDevice());
	}

// This won't work; however initialization code should be here not in main loop
bool SysGraphics2D::Open(char* Title)
{
  if (!csGraphics2D::Open (Title))
    return false;

  Graph->Init ();
  Graph->Clear(0);// is this neccessary???
}


/*----------------------------------------------------------------
	Toasts the window and restores the clut to the main device.
----------------------------------------------------------------*/
void SysGraphics2D::Close(void)
	{
	::DisposeWindow(mainPtr);
	::RestoreDeviceClut(::GetMainDevice());
	mainPtr = nil;
	}

/*----------------------------------------------------------------
	Initialize a color in the color palette.
----------------------------------------------------------------*/
void SysGraphics2D::SetRGB(int i, int r, int g, int b)
	{
	if (i < 0 || i > 255)
		return;
	(*theCTab)->ctTable[i].value = i;
	(*theCTab)->ctTable[i].rgb.red = r | (r << 8);
	(*theCTab)->ctTable[i].rgb.green = g | (g << 8);
	(*theCTab)->ctTable[i].rgb.blue = b | (b << 8);
	}

/*----------------------------------------------------------------
	Clear the offscreen to the color indicated.
----------------------------------------------------------------*/
void SysGraphics2D::Clear(int color)
	{
	if (!offscreen)
		return;
	::SetGWorld(offscreen, nil);
	::RGBForeColor(&(*theCTab)->ctTable[color].rgb);
	::PaintRect(&offscreen->portRect);
	}

/*----------------------------------------------------------------
	Store a single pixel into the offscreen buffer.
----------------------------------------------------------------*/
void SysGraphics2D::DrawPixel(int x, int y, int color)
	{
	*(pixels + x + y * rowBytes) = color;
	}

/*----------------------------------------------------------------
	Draw a line into the buffer.
----------------------------------------------------------------*/
void SysGraphics2D::DrawLine(int x1, int y1, int x2, int y2, int color)
	{
	if (!offscreen)
		return;
	::SetGWorld(offscreen, nil);
	::RGBForeColor(&(*theCTab)->ctTable[color].rgb);
	::MoveTo(x1, y1);
	::LineTo(x2, y2);
	}

/*----------------------------------------------------------------
	Draw a horizontal line into the buffer.
----------------------------------------------------------------*/
void SysGraphics2D::DrawHorLine(int x1, int x2, int y, int color)
	{
	SetLine(x1, y, x2, y, color);
	}

/*----------------------------------------------------------------
	Put the offscreen buffer into the window so the user can see it.
----------------------------------------------------------------*/
void SysGraphics2D::Print (csRect *area)
	{
	if (!offscreen)
		return;
	::SetGWorld((GWorldPtr)mainPtr, nil);
	::CopyBits((BitMap*)*theMap, &mainPtr->portBits,
			&offscreen->portRect, &offscreen->portRect,
			srcCopy, nil);
	
	// lets show the fps too
	if (++frame % 10 == 0)
		{
		fps = 6000 / (::TickCount() - timer); // frames per 10 secs
		timer = ::TickCount();
		::NumToString(fps , fpsString);
		}
	::MoveTo(10, 40);
	::DrawString(fpsString);
	}

// Mouse fonctions
SysMouseDriver::SysMouseDriver()
{
// Initialize mouse system
}

SysMouseDriver::~SysMouseDriver(void)
{
  Close();
}

bool SysMouseDriver::Open()
{
// Open mouse system
  return true;
}

void SysMouseDriver::Close()
{
// Close mouse system
}

// System loop !
#define TestKey(x, y) if(keyboard_keydown(x)) do_keypress (y, shift, alt, ctrl) 

void do_keypress (int key, int shift, int alt, int ctrl);

void SysSystemDriver::Loop(void)
{
  while(!Shutdown && !ExitLoop)
    {
      NextFrame (); // don't remove, please !
 
	int shift, ctrl, alt;

	shift = !!keyboard_keydown(SCAN_LSHIFT);
	alt =  !!keyboard_keydown(SCAN_ALT);
	ctrl =  !!keyboard_keydown(SCAN_CTRL);

	TestKey(SCAN_UP, CSKEY_UP);
	TestKey(SCAN_DOWN, CSKEY_DOWN);
	TestKey(SCAN_LEFT, CSKEY_LEFT);
	TestKey(SCAN_RIGHT, CSKEY_RIGHT);
	TestKey(SCAN_PGUP, CSKEY_PGUP);
	TestKey(SCAN_PGDN, CSKEY_PGDN);
	TestKey(SCAN_END, CSKEY_END);
	TestKey(SCAN_ESC, CSKEY_ESC);
	TestKey(SCAN_LANGLE, '[');
	TestKey(SCAN_RANGLE, ']');
	TestKey(SCAN_DOT, '.');
	TestKey(SCAN_COMA, ',');
	TestKey(SCAN_BACKSP, '\b');
	TestKey(SCAN_TAB, '\t');
	TestKey(SCAN_ENTER, '\n');
	TestKey(SCAN_A, 'a');
	TestKey(SCAN_B, 'b');
	TestKey(SCAN_C, 'c');
	TestKey(SCAN_D, 'd');
	TestKey(SCAN_E, 'e');
	TestKey(SCAN_F, 'f');
	TestKey(SCAN_G, 'g');
	TestKey(SCAN_H, 'h');
	TestKey(SCAN_I, 'i');
	TestKey(SCAN_J, 'j');
	TestKey(SCAN_K, 'k');
	TestKey(SCAN_L, 'l');
	TestKey(SCAN_M, 'm');
	TestKey(SCAN_N, 'n');
	TestKey(SCAN_O, 'o');
	TestKey(SCAN_P, 'p');
	TestKey(SCAN_Q, 'q');
	TestKey(SCAN_R, 'r');
	TestKey(SCAN_S, 's');
	TestKey(SCAN_T, 't');
	TestKey(SCAN_U, 'u');
	TestKey(SCAN_V, 'v');
	TestKey(SCAN_W, 'w');
	TestKey(SCAN_X, 'x');
	TestKey(SCAN_Y, 'y');
	TestKey(SCAN_Z, 'z');
   } // while (!Shutdown)
}

/*----------------------------------------------------------------
        Create a window to draw in and an offscreen buffer.
----------------------------------------------------------------*/
void SysGraphics2D::Init(void)
        {
        // finish up creating the color table
        (*theCTab)->ctSeed = ::GetCTSeed();
        (*theCTab)->ctFlags = 0;
        (*theCTab)->ctSize = 255;
        ::CTabChanged(theCTab);

        // make the offscreen
        Rect    bounds;
        bounds.left = bounds.top = 0;
        bounds.right = FRAME_WIDTH;
        bounds.bottom = FRAME_HEIGHT;
        if (::NewGWorld(&offscreen, 8, &bounds, theCTab, nil, 0) != noErr || offscreen == nil)
                ::ExitToShell();
        
        // locak it and get its address
        theMap = ::GetGWorldPixMap(offscreen);
        ::LockPixels(theMap);
        pixels = (unsigned char*)::GetPixBaseAddr(theMap);
        rowBytes = (*theMap)->rowBytes & 0x7fff;
        
        // make the main window
        Rect    windRect = {64, 64, FRAME_HEIGHT + 64, FRAME_WIDTH + 64}; // 640x480 offset to 64,64
        mainPtr =:: NewCWindow(nil, &windRect, "\pCrystal", true, documentProc, 
                                                (WindowPtr) -1, false, 0);
                
        // set the color table into the video card
        ::SetGWorld((CGrafPtr)mainPtr, nil);
        PaletteHandle thePalette = ::NewPalette(256, theCTab, pmExplicit + pmTolerant, 0);
        ::SetPalette(mainPtr, thePalette, false);
        ::ActivatePalette(mainPtr);
        ::SelectWindow(mainPtr);
        }


