/*
  OS/2 support for Crystal Space 3D library
  Copyright (C) 1998 by Jorrit Tyberghein
  Written by Andrew Zabolotny <bit@eltech.ru>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "com/com.h"
#include "geometry/csrect.h"
#include "util/input/csevent.h"

// shit ...
#undef SEVERITY_ERROR

#include "csdive.h"
#include "libDIVE.h"
#include "libDIVEprv.h"

#include <limits.h>

//------------------------------------------------------ Loading/Maintenance ---

static unsigned int RefCount = 0;

#ifndef CS_STATIC_LINKED
static DllRegisterData RegData;

void STDAPICALLTYPE ModuleRelease ()
{
  RefCount--;
}

void STDAPICALLTYPE ModuleAddRef ()
{
  RefCount++;
}   

// Initialization entry point
STDAPI DllInitialize ()
{
  csCoInitialize (0);
  RegData.clsid = CLSID_csGraphics2DOS2DIVE;

  strcpy (RegData.szProgID, "crystalspace.graphics2d.default");
  strcpy (RegData.szInProcServer, "csdive.dll");
  strcpy (RegData.szFriendlyName, "Crystal Space 2D graphics driver for OS/2 using DIVE");
  return TRUE;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow ()
{
  return RefCount ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject (REFCLSID rclsid, REFIID riid, void** ppv)
{
  static csGraphics2DFactoryOS2DIVE g2dFactory;

  if (rclsid == CLSID_csGraphics2DOS2DIVE)
    return g2dFactory.QueryInterface (riid, ppv);

  // if we get here, rclsid is a class we don't implement
  *ppv = NULL;
  return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by regsvr
STDAPI DllRegisterServer ()
{
  return csRegisterServer (&RegData);
}

// Called by regsvr
STDAPI DllUnregisterServer ()
{
  return csUnregisterServer (&RegData);
}

#endif // CS_STATIC_LINKED

//----------------------------------------------- csGraphics2DOS2DIVEFactory ---

IMPLEMENT_UNKNOWN_NODELETE (csGraphics2DFactoryOS2DIVE)

BEGIN_INTERFACE_TABLE (csGraphics2DFactoryOS2DIVE)
  IMPLEMENTS_INTERFACE (IGraphics2DFactory)
END_INTERFACE_TABLE ()

STDMETHODIMP csGraphics2DFactoryOS2DIVE::CreateInstance (REFIID riid,
  ISystem* piSystem, void**ppv)
{
  if (!piSystem)
  {
    *ppv = 0;
    return E_INVALIDARG;
  }

  csGraphics2DOS2DIVE *pNew = new csGraphics2DOS2DIVE (piSystem);
  if (!pNew)
  {
    *ppv = 0;
    return E_OUTOFMEMORY;
  }
		
  return pNew->QueryInterface (riid, ppv);
}

STDMETHODIMP csGraphics2DFactoryOS2DIVE::LockServer (BOOL bLock)
{
  if (bLock)
    RefCount++;
  else
    RefCount--;

  return S_OK;
}

//------------------------------------------------------ csGraphics2DOS2DIVE ---

BEGIN_INTERFACE_TABLE (csGraphics2DOS2DIVE)
  IMPLEMENTS_COMPOSITE_INTERFACE_EX (IGraphics2D, XGraphics2D)
  IMPLEMENTS_COMPOSITE_INTERFACE_EX (IGraphicsInfo, XGraphicsInfo)
END_INTERFACE_TABLE ()

IMPLEMENT_UNKNOWN_NODELETE (csGraphics2DOS2DIVE)

csGraphics2DOS2DIVE::csGraphics2DOS2DIVE (ISystem* piSystem) : csGraphics2D (piSystem)
{
  WindowX = INT_MIN;
  WindowY = INT_MIN;
  WindowWidth = -1;
  WindowHeight = -1;
  Memory = NULL;
  dblbuff = true;
  HardwareCursor = true;

  // Initialize module handle
#ifdef CS_STATIC_LINKED
  gdMH = NULLHANDLE;
#else
  extern unsigned long dll_handle;
  gdMH = dll_handle;
#endif

  // Initialize DIVE
  if (!gdDiveInitialize ())
  {
    printf ("Unable to initialize DIVE\n");
    exit (-1);
  }

  System = piSystem;
  ISystemDriverOS2* Sys;
  if (SUCCEEDED (piSystem->QueryInterface (IID_ISystemDriverOS2, (void**)&Sys)))
  {
    Sys->GetSettings (WindowX, WindowY, WindowWidth, WindowHeight, HardwareCursor);
    FINAL_RELEASE (Sys);
  }
  else
  {
    printf ("The system driver does not support the ISystemDriverOS2 interface\n");
    exit (-1);
  }

  // Set up FGVideoMode
  // Check first all available 16-bit modes for 'native' flag
  // if so, use it. Otherwise choose R5G6B5 format.
  PixelFormat = FOURCC_LUT8;
  switch (Depth)
  {
    case 8:
      break;
    case 15:
      PixelFormat = FOURCC_R555;
      break;
    case 16:
      for (int i = 0; i < (int)vmCount; i++)
      {
        if (vmList[i].PixelFormat == FOURCC_R565
         || vmList[i].PixelFormat == FOURCC_R664
         || vmList[i].PixelFormat == FOURCC_R555)
        {
          if (vmList[i].Flags & vmfNative)
          {
            PixelFormat = vmList[i].PixelFormat;
            break;
          }

          if ((vmList[i].PixelFormat == FOURCC_R565)
           || ((vmList[i].PixelFormat == FOURCC_R664)
            && (PixelFormat != FOURCC_R565))
           || (PixelFormat == FOURCC_LUT8))
            PixelFormat = vmList[i].PixelFormat;
        } /* endif */
      } /* endfor */
      break;
    default:
      printf ("ERROR: %d bits per pixel modes not supported\n", Depth);
      break;
  } /* endswitch */

  switch (PixelFormat)
  {
    case FOURCC_LUT8:
      pfmt.PalEntries = 256;
      pfmt.PixelBytes = 1;
      pfmt.RedMask    = 0xff;
      pfmt.GreenMask  = 0xff;
      pfmt.BlueMask   = 0xff;
      break;
    case FOURCC_R565:
      pfmt.PalEntries = 0;
      pfmt.PixelBytes = 2;
      pfmt.RedMask    = 0xf800;
      pfmt.GreenMask  = 0x07e0;
      pfmt.BlueMask   = 0x001f;
      break;
    case FOURCC_R664:
      pfmt.PalEntries = 0;
      pfmt.PixelBytes = 2;
      pfmt.RedMask    = 0xfc00;
      pfmt.GreenMask  = 0x03f0;
      pfmt.BlueMask   = 0x000f;
      break;
    case FOURCC_R555:
      pfmt.PalEntries = 0;
      pfmt.PixelBytes = 2;
      pfmt.RedMask    = 0x7c00;
      pfmt.GreenMask  = 0x03e0;
      pfmt.BlueMask   = 0x001f;
      break;
  } /* endswitch */
  complete_pixel_format ();
}

csGraphics2DOS2DIVE::~csGraphics2DOS2DIVE (void)
{
  Close ();
  // Deallocate DIVE resources
  gdDiveDeinitialize ();
}

bool csGraphics2DOS2DIVE::Open (char *Title)
{
  if (!csGraphics2D::Open (Title))
    return false;

  PMrq rq;
  u_int rc;
  FGVideoMode Mode = // selected mode with double buffering
  { Width, Height, PixelFormat, 2, vmfWindowed };

  // Create PM window
  rq.Parm.CreateWindow.Title = Title;
  if ((rc = PMcall (pmcmdCreateWindow, &rq)) != pmrcOK)
    return false;
  WinHandle = rq.Parm.CreateWindow.Handle;

  // Create DIVE contect
  rq.Parm.CreateCtx.Mode = &Mode;
  if ((rc = PMcall (pmcmdCreateDIVEctx, &rq)) != pmrcOK)
    return false;

  dW = rq.Parm.CreateCtx.dW;

  // Setup event handlers
  ISystemDriverOS2* Sys;
  if (SUCCEEDED (System->QueryInterface (IID_ISystemDriverOS2, (void**)&Sys)))
  {
    Sys->GetKeyboardHandler (KeyboardHandler, KeyboardHandlerParm);
    Sys->GetMouseHandler (MouseHandler, MouseHandlerParm);
    Sys->GetTerminateHandler (TerminateHandler, TerminateHandlerParm);
    Sys->GetFocusHandler (FocusHandler, FocusHandlerParm);
    Sys->GetSettings (WindowX, WindowY, WindowWidth, WindowHeight, HardwareCursor);
    FINAL_RELEASE (Sys);
    dW->SetKeyboardHandler (KeyboardHandlerStub, this);
    dW->SetMouseHandler (MouseHandlerStub, this);
    dW->SetTerminateHandler (TerminateHandler, TerminateHandlerParm);
    dW->SetFocusHandler (FocusHandler, FocusHandlerParm);
  }

  // Bind DIVE context to window
  rq.Parm.BindCtx.dW = dW;
  rq.Parm.BindCtx.Handle = WinHandle;
  rq.Parm.BindCtx.DesktopW = DesktopW;
  rq.Parm.BindCtx.DesktopH = DesktopH;
  if ((rc = PMcall (pmcmdBindDIVEctx, &rq)) != pmrcOK)
    return false;

  if ((WindowWidth != -1) && (WindowHeight != -1))
  {
    rq.Parm.Resize.dW = dW;
    rq.Parm.Resize.Width = WindowWidth;
    rq.Parm.Resize.Height = WindowHeight;
    rq.Parm.Resize.Center = true;
    PMcall (pmcmdResizeWindow, &rq);
  }
  if ((WindowX != INT_MIN) || (WindowY != INT_MIN))
  {
    rq.Parm.Locate.dW = dW;
    if (WindowX != INT_MIN)
      rq.Parm.Locate.x = ((DesktopW * WindowX) / 100) & ~1;
    else
      rq.Parm.Locate.x = INT_MIN;
    if (WindowY != INT_MIN)
      rq.Parm.Locate.y = (DesktopH * WindowY) / 100;
    else
      rq.Parm.Locate.y = INT_MIN;
    PMcall (pmcmdLocateWindow, &rq);
  }

  if (FullScreen)
    dW->Command (cmdFullScreen);

  // Update drawing routines on 16 bits per pixel
  if (pfmt.PixelBytes == 2)
  {
    DrawPixel = DrawPixel16;
    WriteChar = WriteChar16;
    GetPixelAt = GetPixelAt16;
    DrawSprite = DrawSprite16;
  } /* endif */

  // Show window
  dW->Show (TRUE);
  rq.Parm.ShowWin.dW = dW;
  rq.Parm.ShowWin.State = 1;
  if ((rc = PMcall (pmcmdShowWindow, &rq)) != pmrcOK)
    return false;

  UpdatePalette = FALSE;
  return true;
}

void csGraphics2DOS2DIVE::Close (void)
{
  PMrq rq;

  if (!dW)
    return;

  // Destroy DIVE context
  rq.Parm.DestroyCtx.dW = dW;
  PMcall (pmcmdDestroyDIVEctx, &rq);

  // Destroy PM window
  rq.Parm.DestroyWindow.Handle = WinHandle;
  PMcall (pmcmdDestroyWindow, &rq);

  dW = NULL;
}

void csGraphics2DOS2DIVE::Print (csRect *area)
{
  long switchmode = dblbuff ? DIVE_NEXTBUFFER : 0;
  // If we're in double-buffering mode, wait previous frame to complete
  if (dblbuff)
    dW->WaitSwitch ();
  // Now switch to next buffer
  if (area)
  {
    // EXPERIMENTAL FEATURE: although DIVE docs states that SetupBlitter
    // is a longplay operation, on some (all?) videocards it is still
    // faster to do a SetupBlitter() with a small area (that should be redrawn)
    // followed by a blit rather than blitting constantly entire image.

    RECTL rect;
    rect.xLeft = area->xmin;
    rect.yBottom = dW->BufferHeight () - 1 - area->ymax;
    rect.xRight = area->xmax + 1;
    rect.yTop = dW->BufferHeight () - 1 - area->ymin + 1;

    // re-scale
    float AspectX = (float)dW->WindowWidth () / (float)dW->BufferWidth ();
    float AspectY = (float)dW->WindowHeight () / (float)dW->BufferHeight ();
    rect.xLeft = (int) (rect.xLeft * AspectX);
    rect.xRight = (int) (rect.xRight * AspectX);
    rect.yBottom = (int) (rect.yBottom * AspectY);
    rect.yTop = (int) (rect.yTop * AspectY);
    // Increment top & right margin by one pixel to cover calculation errors
    rect.xRight++; rect.yTop++;
    dW->Switch (switchmode, &rect);
  } else
    dW->Switch (switchmode);
  // If we're not in double-buffering mode, wait previous frame to complete
  if (!dblbuff)
    dW->WaitSwitch ();
}

int csGraphics2DOS2DIVE::GetPage ()
{
  return dW->ActiveBuff ();
}

bool csGraphics2DOS2DIVE::DoubleBuffer (bool Enable)
{
  dblbuff = Enable;
  return true;
}

bool csGraphics2DOS2DIVE::DoubleBuffer ()
{
  return dblbuff;
}

void csGraphics2DOS2DIVE::Clear (int color)
{
  memset (Memory, color, Width * Height * pfmt.PixelBytes);
}

void csGraphics2DOS2DIVE::SetRGB (int i, int r, int g, int b)
{
  // set a rgb color in the palette of your graphic interface
  if (i < 0 && i > 255)
    return;
  DivePalette[i] = b | g << 8 | r << 16;
  UpdatePalette = TRUE;
  csGraphics2D::SetRGB (i, r, g, b);
}

bool csGraphics2DOS2DIVE::BeginDraw ()
{
  ULONG bpl;
  long switchmode = dblbuff ? DIVE_NEXTBUFFER : 0;

  Memory = dW->BeginPaint (&bpl, switchmode);
  // if paused, return false
  if (Memory == NULL)
  {
    bool Shutdown;
    System->GetShutdown (Shutdown);
    if (Shutdown)
      dW->Pause (FALSE);
    return false;
  } /* endif */

  if (bpl != LineAddressFrameW)
  {
    int i,addr,height;
    System->GetHeightSetting (height);

    for (i = 0, addr = 0; i < height; i++, addr += bpl)
      LineAddress [i] = addr;
    LineAddressFrameW = bpl;
  } /* endif */

  if (UpdatePalette && pfmt.PalEntries)
  {
    dW->SetCLUT(DivePalette, pfmt.PalEntries);
    UpdatePalette = FALSE;
  }
  return true;
}

void csGraphics2DOS2DIVE::FinishDraw ()
{
  dW->EndPaint ();
  Memory = NULL;
}

bool csGraphics2DOS2DIVE::SetMouseCursor (int iShape, IMipMapContainer *iBitmap)
{
  if (!HardwareCursor)
  {
    dW->MouseVisible (FALSE);
    return false;
  } /* endif */

  switch (iShape)
  {
    case csmcNone:
      dW->MouseVisible (FALSE);
      return true;
    case csmcArrow:
      dW->MouseCursor (SPTR_ARROW);
      dW->MouseVisible (TRUE);
      return true;
    case csmcLens:
    case csmcCross:
    case csmcPen:
      dW->MouseVisible (FALSE);
      return false;
    case csmcMove:
      dW->MouseCursor (SPTR_MOVE);
      dW->MouseVisible (TRUE);
      return true;
    case csmcSizeNWSE:
      dW->MouseCursor (SPTR_SIZENWSE);
      dW->MouseVisible (TRUE);
      return true;
    case csmcSizeNESW:
      dW->MouseCursor (SPTR_SIZENESW);
      dW->MouseVisible (TRUE);
      return true;
    case csmcSizeNS:
      dW->MouseCursor (SPTR_SIZENS);
      dW->MouseVisible (TRUE);
      return true;
    case csmcSizeEW:
      dW->MouseCursor (SPTR_SIZEWE);
      dW->MouseVisible (TRUE);
      return true;
    case csmcStop:
      dW->MouseCursor (SPTR_ILLEGAL);
      dW->MouseVisible (TRUE);
      return true;
    case csmcWait:
      dW->MouseCursor (SPTR_WAIT);
      dW->MouseVisible (TRUE);
      return true;
    default:
      dW->MouseVisible (FALSE);
      return false;
  } /* endswitch */
}

void csGraphics2DOS2DIVE::MouseHandlerStub (void *Self, int Button, int Down,
  int x, int y, int ShiftFlags)
{
  csGraphics2DOS2DIVE *This = (csGraphics2DOS2DIVE *)Self;
  if (!This)
    return;
  int ww = This->dW->WindowWidth (), wh = This->dW->WindowHeight ();
  if ((ww <= 0) || (wh <= 0))
    return;

  int fwidth, fheight;
  This->System->GetWidthSetting (fwidth);
  This->System->GetHeightSetting (fheight);

  float AspectX = (float)fwidth / (float)ww;
  float AspectY = (float)fheight / (float)wh;
  x = (int) (x * AspectX); y = (int) (((wh - 1 - y) * AspectY));
  This->MouseHandler (This->MouseHandlerParm, Button, Down, x, y,
    (ShiftFlags & dkf_SHIFT ? CSMASK_SHIFT : 0) |
    (ShiftFlags & dkf_CTRL ? CSMASK_CTRL : 0) |
    (ShiftFlags & dkf_ALT ? CSMASK_ALT : 0));
}

void csGraphics2DOS2DIVE::KeyboardHandlerStub (void *Self, unsigned char ScanCode,
  int Down, unsigned char RepeatCount, int ShiftFlags)
{
  csGraphics2DOS2DIVE *This = (csGraphics2DOS2DIVE *)Self;
  if (!This)
    return;
  This->KeyboardHandler (This->KeyboardHandlerParm, ScanCode, Down, ShiftFlags);
}
