/*
  OS/2 support for Crystal Space 3D library
  Copyright (C) 1998 by Jorrit Tyberghein
  Written by Andrew Zabolotny <bit@eltech.ru>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CSOS2_H__
#define __CSOS2_H__

#include <sys/types.h>
#include <dirent.h>
#include "render/graph2d.h"
#include "util/input/csinput.h"
#include "system/system.h"
#include "icsos2.h"

#define strcasecmp stricmp
#define strncasecmp strnicmp

extern "C" int getpid (void);
#define TEMP_FILE "zip%d.%d", getpid(), _gettid()
#define TEMP_DIR os2_tempdir()
// This is the function called by TEMP_DIR macro
static inline char *os2_tempdir()
{
  char *tmp;
  if ((tmp = getenv ("TMP")))
    return tmp;
  if ((tmp = getenv ("TEMP")))
    return tmp;
  return "";
}

#define getcwd os2_getcwd
static inline char *os2_getcwd (char *buffer, size_t size)
{
  char *ret = _getcwd2 (buffer, size);
  char *cur = ret;
  while (*cur)
  {
    if (*cur == '/')
      *cur = '\\';
    cur++;
  } /* endif */
  return ret;
}

#define __NO_GENERIC_ISDIR
static inline bool isdir (char *path, dirent *de)
{
  (void)path;
  return !!(de->d_attr & A_DIR);
}

/**
 * This is the System driver for OS/2. It implements all required
 * functionality for standard csSystemDriver class.
 */
class SysSystemDriver : public csSystemDriver
{
  /// Window position in percents
  int WindowX, WindowY;
  /// Window width and height
  int WindowWidth, WindowHeight;
  /// Use system cursor if true; otherwise use builtin CSWS software cursors
  bool HardwareCursor;

public:
  /// Initialize system-dependent data
  SysSystemDriver ();

  /// Check if configuration files requests 16 bits per pixel
  void SetSystemDefaults ();

  /**
   * This is a function that prints the commandline help text.
   * If system has system-dependent switches, it should override
   * this method and type its own text (possibly invoking
   * csSystemDriver::Help() first).
   */
  virtual void Help ();

  /**
   * System loop. This should be called last since it returns
   * only on program exit
   */
  virtual void Loop ();

  /// The system is idle: we can sleep for a while
  virtual void Sleep (int SleepTime);

  /// Implementation of ISystemDriverOS2
  class XSystemDriverOS2 : public ISystemDriverOS2
  {
    DECLARE_IUNKNOWN ()
    /// Get user settings
    STDMETHOD (GetSettings) (int &WindowX, int &WindowY,
      int &WindowWidth, int &WindowHeight, bool &HardwareCursor);
    /// Get OS/2-specific keyboard event handler routine
    STDMETHOD (GetKeyboardHandler) (KeyboardHandlerOS2 &Handler, void *&Parm);
    /// Get OS/2-specific mouse event handler routine
    STDMETHOD (GetMouseHandler) (MouseHandlerOS2 &Handler, void *&Parm);
    /// Get OS/2-specific mouse event handler routine
    STDMETHOD (GetTerminateHandler) (TerminateHandlerOS2 &Handler, void *&Parm);
    /// Get OS/2-specific mouse event handler routine
    STDMETHOD (GetFocusHandler) (FocusHandlerOS2 &Handler, void *&Parm);
  };

  // COM stuff
  DECLARE_IUNKNOWN ()
  DECLARE_INTERFACE_TABLE (SysSystemDriver)
  DECLARE_COMPOSITE_INTERFACE_EMBEDDED (SystemDriverOS2)

protected:
  /**
   * This is a system-dependent function which eats a single
   * command-line option (like -help, ...). If system-dependent
   * part does not recognize the option, it should pass it to
   * its parent class method.
   */
  virtual void ParseArg (int argc, char* argv[], int& i);
  /// Called when CrystalSpace window changes its "focused" state
  static void SysFocusChange (void *Self, int Enable);
  /// Called when window is closed by user
  static void Terminate (void *Self);
};

/**
 * This is the Keyboard class for OS/2. All of its functionality
 * is included in the 2D graphics driver; this class is just a
 * container for the handler function.
 */
class SysKeyboardDriver : public csKeyboardDriver
{
public:
  SysKeyboardDriver ();

  static void Handler (void *param, unsigned char ScanCode, bool Down,
    int ShiftFlags);
};

/**
 * This is the Mouse class for OS/2. All of its functionality
 * is included in the 2D graphics driver; this class is just a
 * container for the handler function.
 */
class SysMouseDriver : public csMouseDriver
{
public:
  SysMouseDriver ();

  static void Handler (void *param, int Button, int Down, int x, int y,
    int ShiftFlags);
};

#endif // __CSOS2_H__
