/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef DISP_SVGA

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <sys/timeb.h>

#include "system/system.h"
#include "util/inifile.h"
#include "render/graph3d.h"

#include <vga.h>
#include <vgagl.h>
#if defined (COMP_GCC) && defined (OS_DOS)
# include <vgakeybo.h>
#else
# include <vgakeyboard.h>
#endif
#include <vgamouse.h>

static int FRAME_DEPTH=-1;

//extern void draw_mouse ();
void init_sig ();
void initfont(void);

GraphicsContext physicalscreen;

void initfont(void)
{
  void *font;
  font = malloc(256 * 8 * 8 * BYTESPERPIXEL);
  gl_expandfont(8, 8, 1, gl_font8x8, font);
  gl_setfont(8, 8, font);
  gl_colorfont (8, 8, 2, font);
}

/*
 * Signal handler to clean up
 */
extern void debug_dump ();
void handler (int sig)
{
  static bool in_exit = false;
  if (in_exit) exit (1);
  in_exit = true;

  if (sig == SIGINT)
    fprintf (stderr, "\n^C\n");
  else
    fprintf (stderr, "SIGNAL %d CAUGHT!!!\n", sig);

  int err;
  keyboard_close ();
  mouse_close ();
  vga_setmode (TEXT);
  err = errno;
  //printf ("error %d: %s\n", err, strerror (err));

  //if (sig != SIGINT) debug_dump ();

  exit (1);
} 

void init_sig ()
{
  void handler (int sig);

  signal (SIGHUP, handler);
  signal (SIGINT, handler);
  signal (SIGTRAP, handler);
  signal (SIGABRT, handler);
  signal (SIGALRM, handler);
  signal (SIGTERM, handler);
#if !defined (OS_DOS)
  signal (SIGSTKFLT, handler);
#endif
  signal (SIGPIPE, handler);
  signal (SIGSEGV, handler);
#if !defined (OS_DOS)
  signal (SIGBUS, handler);
#endif
  signal (SIGFPE, handler);
  signal (SIGILL, handler);
}

// Keyboard fonctions
SysGraphics2D::SysGraphics2D(int argc, char* argv[]) : csGraphics2D ()
{
  CsPrintf (MSG_INITIALIZATION, "Crystal Space Linux/SVGALIB version.\n");
  CsPrintf (MSG_INITIALIZATION, "Using %dx%dx%d resolution.\n\n", FRAME_WIDTH, FRAME_HEIGHT,
         FRAME_DEPTH);

  init_sig ();

  gl_copyscreen (&physicalscreen);

  Memory = NULL;

  switch (FRAME_DEPTH)
  {
    case 8:
      pfmt.RedMask = pfmt.GreenMask = pfmt.BlueMask = 0;
      pfmt.PalEntries = 256; pfmt.PixelBytes = 1;
      break;

    case 15:
      pfmt.RedMask   = 0x1f << 10;
      pfmt.GreenMask = 0x1f << 5;
      pfmt.BlueMask  = 0x1f;
      pfmt.PalEntries = 0; 
      pfmt.PixelBytes = 2;

      DrawPixel = DrawPixel16;
      WriteChar = WriteChar16;
      GetPixelAt = GetPixelAt16;
      DrawSprite = DrawSprite16;
      break;

    case 16:
      pfmt.RedMask   = 0x1f << 11;
      pfmt.GreenMask = 0x3f << 5;
      pfmt.BlueMask  = 0x1f;
      pfmt.PalEntries = 0; 
      pfmt.PixelBytes = 2;

      DrawPixel = DrawPixel16;
      WriteChar = WriteChar16;
      GetPixelAt = GetPixelAt16;
      DrawSprite = DrawSprite16;
      break;
  }

  complete_pixel_format();
}

SysGraphics2D::~SysGraphics2D(void)
{
  // Destroy your graphic interface
}

bool SysGraphics2D::Open(char* Title)
{
  // Open your graphic interface

  if (!csGraphics2D::Open (Title))
    return false;

  //  vga_setmousesupport (1);
  int vgamode;

  switch (FRAME_DEPTH)
  {
    case 8:
      if (FRAME_WIDTH == 640 && FRAME_HEIGHT == 480) vgamode = G640x480x256;
      else if (FRAME_WIDTH == 800 && FRAME_HEIGHT == 600) vgamode = G800x600x256;
      else vgamode = G320x200x256;
      break;
    case 15:
      if (FRAME_WIDTH == 640 && FRAME_HEIGHT == 480) vgamode = G640x480x32K;
      else if (FRAME_WIDTH == 800 && FRAME_HEIGHT == 600) vgamode = G800x600x32K;
      else vgamode = G320x200x32K;
      break;
    case 16:
      if (FRAME_WIDTH == 640 && FRAME_HEIGHT == 480) vgamode = G640x480x64K;
      else if (FRAME_WIDTH == 800 && FRAME_HEIGHT == 600) vgamode = G800x600x64K;
      else vgamode = G320x200x64K;
      break;
    default:
      CsPrintf (MSG_FATAL_ERROR, "Unsupported depth %d\n", FRAME_DEPTH);
      return false;
    }

  if (vga_setmode (vgamode) == -1)
  {
    CsPrintf (MSG_FATAL_ERROR, "Specified screenmode is not available!\n");
    return false;
  }
  gl_setcontextvga (vgamode);
  gl_getcontext (&physicalscreen);
  gl_setcontextvgavirtual (vgamode);
  Memory = (unsigned char*)VBUF;
  gl_enablepageflipping (&physicalscreen);

  initfont ();
#if !defined (OS_DOS)
  mouse_init ("/dev/mouse", MOUSE_MICROSOFT, MOUSE_DEFAULTSAMPLERATE);
  mouse_setxrange (0, FRAME_WIDTH-1);
  mouse_setyrange (0, FRAME_HEIGHT-1);
  mouse_setwrap (MOUSE_NOWRAP);
#endif // OS_DOS

  Clear(0);

  return true;
}

void SysGraphics2D::Close(void)
{
  vga_setmode (TEXT);
  csGraphics2D::Close ();
}

void SysGraphics2D::Print (csRect *area)
{
    gl_copyscreen (&physicalscreen);
}

void SysGraphics2D::SetRGB(int i, int r, int g, int b)
{
  if (pfmt.PalEntries) vga_setpalette (i, r/4, g/4, b/4);

  csGraphics2D::SetRGB (i, r, g, b);
}

// Keyboard fonctions
SysKeyboardDriver::SysKeyboardDriver() : csKeyboardDriver ()
{
  int i;
  for (i = 0 ; i < 256 ; i++)
    remember_key[i] = false;
}

SysKeyboardDriver::~SysKeyboardDriver(void)
{
  // Destroy your keyboard interface
  Close();
}

bool SysKeyboardDriver::Open(csEventQueue* EvQueue)
{
  // Open your keyboard interface
  csKeyboardDriver::Open (EvQueue);
  keyboard_init ();
  return true;
}

void SysKeyboardDriver::Close(void)
{
  keyboard_close ();
}

#define SC_LEFTSHIFT 42
#define SC_RIGHTSHIFT 54
#define SC_COMMA 51
#define SC_DOT 52
#define SC_LEFT_BRA 26
#define SC_RIGHT_BRA 27

int SysKeyboardDriver::translate_scancode (int scancode)
{
  switch (scancode)
  {
    case SC_LEFTSHIFT: case SC_RIGHTSHIFT: return CSKEY_SHIFT;
    case SCANCODE_LEFTALT: case SCANCODE_RIGHTALT: return CSKEY_ALT;
    case SCANCODE_LEFTCONTROL: return CSKEY_CTRL;
    case SCANCODE_CURSORBLOCKUP: return CSKEY_UP;
    case SCANCODE_CURSORBLOCKDOWN: return CSKEY_DOWN;
    case SCANCODE_CURSORBLOCKLEFT: return CSKEY_LEFT;
    case SCANCODE_CURSORBLOCKRIGHT: return CSKEY_RIGHT;
    case SCANCODE_PAGEUP: return CSKEY_PGUP;
    case SCANCODE_PAGEDOWN: return CSKEY_PGDN;
    case SCANCODE_END: return CSKEY_END;
    case SCANCODE_ESCAPE: return CSKEY_ESC;
    case SC_LEFT_BRA: return '[';
    case SC_RIGHT_BRA: return ']';
    case SC_DOT: return '.';
    case SC_COMMA: return ',';
    case SCANCODE_BACKSPACE: return CSKEY_BACKSPACE;
    case SCANCODE_TAB: return CSKEY_TAB;
    case SCANCODE_ENTER: return CSKEY_ENTER;
    case SCANCODE_SPACE: return ' ';
    case SCANCODE_A: return 'a';
    case SCANCODE_B: return 'b';
    case SCANCODE_C: return 'c';
    case SCANCODE_D: return 'd';
    case SCANCODE_E: return 'e';
    case SCANCODE_F: return 'f';
    case SCANCODE_G: return 'g';
    case SCANCODE_H: return 'h';
    case SCANCODE_I: return 'i';
    case SCANCODE_J: return 'j';
    case SCANCODE_K: return 'k';
    case SCANCODE_L: return 'l';
    case SCANCODE_M: return 'm';
    case SCANCODE_N: return 'n';
    case SCANCODE_O: return 'o';
    case SCANCODE_P: return 'p';
    case SCANCODE_Q: return 'q';
    case SCANCODE_R: return 'r';
    case SCANCODE_S: return 's';
    case SCANCODE_T: return 't';
    case SCANCODE_U: return 'u';
    case SCANCODE_V: return 'v';
    case SCANCODE_W: return 'w';
    case SCANCODE_X: return 'x';
    case SCANCODE_Y: return 'y';
    case SCANCODE_Z: return 'z';
    case SCANCODE_0: return '0';
    case SCANCODE_1: return '1';
    case SCANCODE_2: return '2';
    case SCANCODE_3: return '3';
    case SCANCODE_4: return '4';
    case SCANCODE_5: return '5';
    case SCANCODE_6: return '6';
    case SCANCODE_7: return '7';
    case SCANCODE_8: return '8';
    case SCANCODE_9: return '9';
    default: return 0;
  }
}

void SysKeyboardDriver::checkkey (int scancode, bool shift, bool alt, bool ctrl)
{
  if (keyboard_keypressed (scancode))
  {
    remember_key[scancode] = true;
    int key = translate_scancode (scancode);
    do_keypress (key);
  }
  else if (remember_key[scancode])
  {
    remember_key[scancode] = false;
    int key = translate_scancode (scancode);
    do_keyrelease (key);
  }
}

void SysKeyboardDriver::handle_keys (bool shift, bool alt, bool ctrl)
{
  checkkey (SC_LEFTSHIFT, shift, alt, ctrl);
  checkkey (SC_RIGHTSHIFT, shift, alt, ctrl);
  checkkey (SCANCODE_LEFTALT, shift, alt, ctrl);
  checkkey (SCANCODE_RIGHTALT, shift, alt, ctrl);
  checkkey (SCANCODE_LEFTCONTROL, shift, alt, ctrl);
  checkkey (SCANCODE_CURSORBLOCKUP, shift, alt, ctrl);
  checkkey (SCANCODE_CURSORBLOCKDOWN, shift, alt, ctrl);
  checkkey (SCANCODE_CURSORBLOCKLEFT, shift, alt, ctrl);
  checkkey (SCANCODE_CURSORBLOCKRIGHT, shift, alt, ctrl);
  checkkey (SCANCODE_PAGEUP, shift, alt, ctrl);
  checkkey (SCANCODE_PAGEDOWN, shift, alt, ctrl);
  checkkey (SCANCODE_END, shift, alt, ctrl);
  checkkey (SCANCODE_ESCAPE, shift, alt, ctrl);
  checkkey (SC_LEFT_BRA, shift, alt, ctrl);
  checkkey (SC_RIGHT_BRA, shift, alt, ctrl);
  checkkey (SC_DOT, shift, alt, ctrl);
  checkkey (SC_COMMA, shift, alt, ctrl);
  checkkey (SCANCODE_BACKSPACE, shift, alt, ctrl);
  checkkey (SCANCODE_TAB, shift, alt, ctrl);
  checkkey (SCANCODE_ENTER, shift, alt, ctrl);
  checkkey (SCANCODE_SPACE, shift, alt, ctrl);
  checkkey (SCANCODE_A, shift, alt, ctrl);
  checkkey (SCANCODE_B, shift, alt, ctrl);
  checkkey (SCANCODE_C, shift, alt, ctrl);
  checkkey (SCANCODE_D, shift, alt, ctrl);
  checkkey (SCANCODE_E, shift, alt, ctrl);
  checkkey (SCANCODE_F, shift, alt, ctrl);
  checkkey (SCANCODE_G, shift, alt, ctrl);
  checkkey (SCANCODE_H, shift, alt, ctrl);
  checkkey (SCANCODE_I, shift, alt, ctrl);
  checkkey (SCANCODE_J, shift, alt, ctrl);
  checkkey (SCANCODE_K, shift, alt, ctrl);
  checkkey (SCANCODE_L, shift, alt, ctrl);
  checkkey (SCANCODE_M, shift, alt, ctrl);
  checkkey (SCANCODE_N, shift, alt, ctrl);
  checkkey (SCANCODE_O, shift, alt, ctrl);
  checkkey (SCANCODE_P, shift, alt, ctrl);
  checkkey (SCANCODE_Q, shift, alt, ctrl);
  checkkey (SCANCODE_R, shift, alt, ctrl);
  checkkey (SCANCODE_S, shift, alt, ctrl);
  checkkey (SCANCODE_T, shift, alt, ctrl);
  checkkey (SCANCODE_U, shift, alt, ctrl);
  checkkey (SCANCODE_V, shift, alt, ctrl);
  checkkey (SCANCODE_W, shift, alt, ctrl);
  checkkey (SCANCODE_X, shift, alt, ctrl);
  checkkey (SCANCODE_Y, shift, alt, ctrl);
  checkkey (SCANCODE_Z, shift, alt, ctrl);
  checkkey (SCANCODE_0, shift, alt, ctrl);
  checkkey (SCANCODE_1, shift, alt, ctrl);
  checkkey (SCANCODE_2, shift, alt, ctrl);
  checkkey (SCANCODE_3, shift, alt, ctrl);
  checkkey (SCANCODE_4, shift, alt, ctrl);
  checkkey (SCANCODE_5, shift, alt, ctrl);
  checkkey (SCANCODE_6, shift, alt, ctrl);
  checkkey (SCANCODE_7, shift, alt, ctrl);
  checkkey (SCANCODE_8, shift, alt, ctrl);
  checkkey (SCANCODE_9, shift, alt, ctrl);
}


// Mouse fonctions
SysMouseDriver::SysMouseDriver() : csMouseDriver ()
{
  // Initialize mouse system
  button1_pressed = false;
  button2_pressed = false;
  button3_pressed = false;
}

SysMouseDriver::~SysMouseDriver(void)
{
  Close();
}

bool SysMouseDriver::Open(csEventQueue* EvQueue)
{
  csMouseDriver::Open (EvQueue);
  // Open mouse system
  return true;
}

void SysMouseDriver::Close()
{
// Close mouse system
}

void SysMouseDriver::handle_mouse (bool shift, bool alt, bool ctrl)
{
  int button, x, y;;
  int rc = mouse_update ();
  x = mouse_getx ();
  y = mouse_gety ();

  button = mouse_getbutton ();
  if (button & MOUSE_LEFTBUTTON)
  {
    button1_pressed = true;
    do_buttonpress (1, x, y, shift, alt, ctrl);
  }
  else if (button1_pressed)
  {
    button1_pressed = false;
    do_buttonrelease (1, x, y);
  }
  if (button & MOUSE_MIDDLEBUTTON)
  {
    button2_pressed = true;
    do_buttonpress (2, x, y, shift, alt, ctrl);
  }
  else if (button2_pressed)
  {
    button2_pressed = false;
    do_buttonrelease (2, x, y);
  }
  if (button & MOUSE_RIGHTBUTTON)
  {
    button3_pressed = true;
    do_buttonpress (3, x, y, shift, alt, ctrl);
  }
  else if (button3_pressed)
  {
    button3_pressed = false;
    do_buttonrelease (3, x, y);
  }
  if (rc) do_mousemotion (x, y);
}

// System loop !
void SysSystemDriver::Loop(void)
{
  while(!Shutdown && !ExitLoop)
  {
    bool shift, ctrl, alt;
    keyboard_update ();
    shift = keyboard_keypressed (SC_LEFTSHIFT) || keyboard_keypressed (SC_RIGHTSHIFT);
    alt = keyboard_keypressed (SCANCODE_LEFTALT) || keyboard_keypressed (SCANCODE_RIGHTALT);
    ctrl = !!keyboard_keypressed (SCANCODE_LEFTCONTROL);

    ((SysMouseDriver*)Mouse)->handle_mouse (shift, alt, ctrl);
    NextFrame (); // don't remove, please !
    ((SysKeyboardDriver*)Keyboard)->handle_keys (shift, alt, ctrl);
  } // while (!Shutdown)
}

#endif //DISP_SVGA
