/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SVGA_H
#define SVGA_H

#include "system/system.h"
#include "render/graph2d.h"
#include "util/input/csinput.h"

/// SVGALIB version.
class SysSystemDriver : public csSystemDriver
{
public:
  SysSystemDriver () : csSystemDriver () { }

  virtual void Loop ();
};

/// SVGALIB version.
class SysGraphics2D : public csGraphics2D
{
public:
  SysGraphics2D(int argc, char* argv[]);
  virtual ~SysGraphics2D(void);

  virtual bool Open(char* Title);
  virtual void Close(void);

  virtual void Print (csRect *area = NULL);
  virtual void SetRGB (int i, int r, int g, int b);
};

/// SVGALIB version.
class SysKeyboardDriver : public csKeyboardDriver
{
private:
  /**
   * This array remembers whether a key is pressed. If the key is no longer
   * pressed later on and the flag in this array is true then we need to
   * send a do_keyrelease event. These entries are indexed by SVGALIB
   * scancode.
   */
  bool remember_key[256];

  /**
   * Translate an SVGALIB scancode to a CS key.
   */
  int translate_scancode (int scancode);

  /**
   * Check the given scancode. If the key is pressed send a do_keypress (),
   * if the key is not pressed and there was a keypress the previous time
   * then send a do_keyrelease ().
   */
  void checkkey (int scancode, bool shift, bool alt, bool ctrl);

public:
  SysKeyboardDriver();
  ~SysKeyboardDriver(void);

  bool Open(csEventQueue*);
  void Close(void);

  /**
   * This function should be called by SysSystemDriver::Loop. It will check
   * for all keys and possible send do_keypress/do_keyrelease events.
   */
  void handle_keys (bool shift, bool alt, bool ctrl);
};

/// SVGALIB version.
class SysMouseDriver : public csMouseDriver
{
private:
  /// Remember if button 1 was pressed before
  bool button1_pressed;
  bool button2_pressed;
  bool button3_pressed;

public:
  SysMouseDriver();
  ~SysMouseDriver(void);

  bool Open(csEventQueue*);
  void Close(void);

  /**
   * This function should be called by SysSystemDriver::Loop. It will
   * check for the mouse buttons and motions and call do_buttonpress,
   * do_buttonrelease or do_mousemotion if needed.
   */
  void handle_mouse (bool shift, bool alt, bool ctrl);
};

#endif // SVGA_H
