/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef SYSDRIV_H
#define SYSDRIV_H

// Include the system dependent driver classes.

#include "util/def.h"

#ifdef OS_WIN32
#  include "system/win32/win32.h"
#  include "system/win32/syssound.h"
#endif

#if defined(COMP_WCC) && defined(OS_DOS)
#  include "wcc/csdoswat.h"
#endif

#if defined(COMP_GCC) && defined(OS_DOS) && !defined(DISP_SVGA)
#  include "djgpp/djgpp.h"
#endif

#if defined(OS_MACOS)   // SJI - mac stuff
#  include "maclib/MacGraphics.h"
#  include "maclib/MacKeyboard.h"
#endif

#ifdef DISP_X11
#  include "xlib/xwin.h"
#endif

#ifdef DISP_SVGA
#  include "svgalib/svga.h"
#endif

#ifdef DISP_GGI
#  include "libggi/libggi.h"
#endif

#ifdef DO_AMIGAOS
#  include "amiga/amiga.h"
#endif

#ifdef OS_OS2
#  include "os2/csos2.h"
#endif

#ifdef OS_BE
#  include "be/csbe.h"
#endif

//--//--//--//--//--/ Allow system-dependent header files to override these --//

// Directory for temporary files
#ifndef TEMP_DIR
#  if defined(OS_UNIX)
#    define TEMP_DIR "/tmp/"
#  else
#    define TEMP_DIR ""
#  endif
#endif

// Name for temporary file
#ifndef TEMP_FILE
#  if defined(OS_UNIX)
#    define TEMP_FILE "cs%d.tmp", getpid()
#  else
#    define TEMP_FILE "$cs$.tmp"
#  endif
#endif

// How to make a directory (not entire path, only the last on the path)
#ifndef MKDIR
#  if defined(OS_WIN32) || (defined(OS_DOS) && !defined(COMP_GCC))
#    define MKDIR(path) _mkdir (path);
#  else
#    define MKDIR(path) mkdir (path, 0644);
#  endif
#endif

// Path separator character
#ifndef PATH_SEPARATOR
#  if defined(OS_MACOS)
#    define PATH_SEPARATOR ':'
#  elif defined(OS_OS2) || defined(OS_DOS) || defined(OS_WIN32)
#    define PATH_SEPARATOR '\\'
#  else
#    define PATH_SEPARATOR '/'
#  endif
#endif

// Maximal path length
#ifndef MAXPATHLEN
#  ifdef _MAX_FNAME
#    define MAXPATHLEN _MAX_FNAME
#  else
#    define MAXPATHLEN 256
#  endif
#endif

// Convert a character to upper case
#ifndef UPPERCASE
#  define UPPERCASE(c) ((c >= 'a' && c <= 'z') ? c - ('a' - 'A') : c)
#endif

// Convert a character to lower case
#ifndef LOWERCASE
#  define LOWERCASE(c) ((c >= 'A' && c <= 'Z') ? c + ('a' - 'A') : c)
#endif

#define APPEND_SLASH(str,len)		\
  if ((len)                             \
   && (str[len - 1] != '/')             \
   && (str[len - 1] != PATH_SEPARATOR)) \
  {                                     \
    str[len++] = PATH_SEPARATOR;        \
    str[len] = 0;                       \
  } /* endif */

// For systems without opendir()
#ifdef __NEED_OPENDIR_PROTOTYPE
struct DIR;
struct dirent;
extern "C" DIR *opendir (const char *name);
extern "C" dirent *readdir (DIR *dirp);
extern "C" int closedir (DIR *dirp);
//extern "C" void seekdir (DIR *dirp, long off);
//extern "C" long telldir (DIR *dirp);
//extern "C" void rewinddir (DIR *dirp);
#endif

#ifndef __NO_GENERIC_ISDIR
#if !defined(OS_UNIX)
#  include <io.h>
#endif
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
static inline bool isdir (char *path, dirent *de)
{
  char fullname [MAXPATHLEN];
  int pathlen = strlen (path);
  memcpy (fullname, path, pathlen + 1);
  APPEND_SLASH (fullname, pathlen);
  strcat (&fullname [pathlen], de->d_name);
  struct stat st;
  stat (fullname, &st);
  return ((st.st_mode & S_IFMT) == S_IFDIR);
}
#endif

#endif //SYSDRIV_H
