#include <stdlib.h>
#include "com/com.h"
#include "system/win32/opengl2d/oglg2d.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#define DLL_NAME "OpenGL2D.dll"

static unsigned int gb_cRef = 0;

extern const CLSID CLSID_csGraphics2DOpenGL;

static DllRegisterData gb_regData;
static DllRegisterData gb_reg3dData;

void STDAPICALLTYPE ModuleRelease(void)
{
    gb_cRef--;
}

void STDAPICALLTYPE ModuleAddRef(void)
{
    gb_cRef++;
}   

// our main entry point...should be called when we're loaded.
STDAPI DllInitialize()
{
    csCoInitialize(0);
    gb_regData.clsid = CLSID_csGraphics2DOpenGL;
    
    strcpy(gb_regData.szProgID, "crystalspace.graphics2d.defaultgl");
    strcpy(gb_regData.szInProcServer, DLL_NAME );
    strcpy(gb_regData.szFriendlyName, "csGraphics2D OpenGL Implementation");
    
	return S_OK;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
    return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)
{
    static csGraphics2DOpenGLFactory s_g2dFactory;
    
    if (rclsid == CLSID_csGraphics2DOpenGL)
        return s_g2dFactory.QueryInterface(riid, ppv);
    
    // if we get here, rclsid is a class we don't implement
    *ppv = NULL;
    return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer()
{
    HRESULT hRes;
    hRes = csRegisterServer(&gb_regData);
    if (FAILED(hRes))
        return hRes;
    
    return csRegisterServer(&gb_reg3dData);
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer()
{
    HRESULT hRes;
    
    hRes = csRegisterServer(&gb_regData);
    if (FAILED(hRes))
        return hRes;
    
    return csUnregisterServer(&gb_reg3dData);
}

// Implementation of the csGraphics2DOpenGL factory... ///////////

IMPLEMENT_UNKNOWN_NODELETE( csGraphics2DOpenGLFactory )

BEGIN_INTERFACE_TABLE( csGraphics2DOpenGLFactory )
	IMPLEMENTS_INTERFACE( IGraphics2DFactory )
        IMPLEMENTS_INTERFACE( IGraphics2DOpenGLFactory )
END_INTERFACE_TABLE()

STDMETHODIMP csGraphics2DOpenGLFactory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics2DOpenGL* pNew = new csGraphics2DOpenGL(piSystem, false);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}

STDMETHODIMP csGraphics2DOpenGLFactory::LockServer(BOOL bLock)
{
	if (bLock)
		gb_cRef++;
	else
		gb_cRef--;

	return S_OK;
}
