#include <stdlib.h>
#include "com/com.h"
#include "system/win32/driver2d/sysg2d.h"

// This is the name of the DLL. Make sure to change this if you change the DLL name!
// DAN: this might have to be changed for each OS, cuz each OS has a different extension for DLLs.
#define DLL_NAME "Driver2D.dll"

static unsigned int gb_cRef = 0;

extern const CLSID CLSID_Graphics2DDirectDraw;
extern const CLSID CLSID_Graphics2DDirectDrawWith3D;

static DllRegisterData gb_regData;
static DllRegisterData gb_reg3dData;

void STDAPICALLTYPE ModuleRelease(void)
{
    gb_cRef--;
}

void STDAPICALLTYPE ModuleAddRef(void)
{
    gb_cRef++;
}   

// our main entry point...should be called when we're loaded.
STDAPI DllInitialize()
{
    csCoInitialize(0);
    gb_regData.clsid = CLSID_Graphics2DDirectDraw;
    
    strcpy(gb_regData.szProgID, "crystalspace.graphics2d.default");
    strcpy(gb_regData.szInProcServer, DLL_NAME );
    strcpy(gb_regData.szFriendlyName, "csGraphics2D DirectDraw Implementation");
    
    // Direct3D-specifc coclass.
    gb_reg3dData.clsid = CLSID_Graphics2DDirectDrawWith3D;
    
    strcpy(gb_reg3dData.szProgID, "crystalspace.graphics2d.direct3d");
    strcpy(gb_reg3dData.szInProcServer, DLL_NAME );
    strcpy(gb_reg3dData.szFriendlyName, "csGraphics2D DirectDraw Implementation for Direct3D");

	return S_OK;
}

// return S_OK if it's ok to unload us now.
STDAPI DllCanUnloadNow()
{
    return gb_cRef ? S_FALSE : S_OK;
}

// used to get a COM class object from us.
STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, void** ppv)
{
    static csGraphics2DWin32Factory s_g2dFactory;
    static csGraphics2DWithDirect3DFactory s_g2dg3dFactory;
    
    if (rclsid == CLSID_Graphics2DDirectDraw)
        return s_g2dFactory.QueryInterface(riid, ppv);
    else if (rclsid == CLSID_Graphics2DDirectDrawWith3D)
        return s_g2dg3dFactory.QueryInterface(riid, ppv);
    
    // if we get here, rclsid is a class we don't implement
    *ppv = NULL;
    return CLASS_E_CLASSNOTAVAILABLE;
}

// Called by RegSvr32.exe
STDAPI DllRegisterServer()
{
    HRESULT hRes;
    hRes = csRegisterServer(&gb_regData);
    if (FAILED(hRes))
        return hRes;
    
    return csRegisterServer(&gb_reg3dData);
}

// Called by RegSvr32.exe
STDAPI DllUnregisterServer()
{
    HRESULT hRes;
    
    hRes = csRegisterServer(&gb_regData);
    if (FAILED(hRes))
        return hRes;
    
    return csUnregisterServer(&gb_reg3dData);
}

// Implementation of the csGraphics2DWin32 factory... ///////////

IMPLEMENT_UNKNOWN_NODELETE( csGraphics2DWin32Factory )

BEGIN_INTERFACE_TABLE( csGraphics2DWin32Factory )
	IMPLEMENTS_INTERFACE( IGraphics2DFactory )
        IMPLEMENTS_INTERFACE( IGraphics2DDirect3DFactory )
END_INTERFACE_TABLE()

BEGIN_INTERFACE_TABLE( csGraphics2DWithDirect3DFactory )
	IMPLEMENTS_INTERFACE( IGraphics2DFactory )
        IMPLEMENTS_INTERFACE( IGraphics2DDirect3DFactory )
END_INTERFACE_TABLE()


STDMETHODIMP csGraphics2DWin32Factory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics2DWin32* pNew = new csGraphics2DWin32(piSystem, false);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}

STDMETHODIMP csGraphics2DWin32Factory::LockServer(BOOL bLock)
{
	if (bLock)
		gb_cRef++;
	else
		gb_cRef--;

	return S_OK;
}

STDMETHODIMP csGraphics2DWithDirect3DFactory::CreateInstance(REFIID riid, ISystem* piSystem, void**ppv)
{
	if (!piSystem)
	{
		*ppv = 0;
		return E_INVALIDARG;
	}

	csGraphics2DWin32* pNew = new csGraphics2DWin32(piSystem, true);
	if (!pNew)
	{
		*ppv = 0;
		return E_OUTOFMEMORY;
	}
		
	return pNew->QueryInterface(riid, ppv);
}
