/*
    Copyright (C) 1998 by Jorrit Tyberghein

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef XLIBG2D_H
#define XLIBG2D_H

#include "com/com.h"
#include "render/graph2d.h"
#include "system/xlib/driver2d/xsysg2d.h"

// The CLSID to create csGraphics2DXLib instances
extern const CLSID CLSID_XLib2DDriver;

extern const IID IID_IGraphics2DXLibFactory;

/// dummy interface
interface IGraphics2DXLibFactory : public IGraphics2DFactory
{
};

///
class csGraphics2DXLibFactory : public IGraphics2DXLibFactory
{
public:
  DECLARE_IUNKNOWN()
  DECLARE_INTERFACE_TABLE(csGraphics2DXLibFactory)

  STDMETHOD(CreateInstance)(REFIID riid, ISystem* piSystem, void** ppv);
  STDMETHOD(LockServer)(BOOL bLock);
};

///

/// XLIB version.
class csGraphics2DXLib : public csGraphics2D
{
  friend class csGraphics3DSoftware;
private:
  Display* dpy;
  int screen_num;
  int display_width, display_height;
  Window window;
  XImage* xim;
  GC gc;
  Visual *visual;
  XVisualInfo vinfo;
  unsigned int vclass;

// Everything for simulated depth
  int sim_depth;
  csPixelFormat real_pfmt;	// Contains the real pfmt is simulating stuff
  unsigned char* real_Memory;	// Real memory to the display
  unsigned char* sim_lt8;	// 8-bit lookup table (with 16-bit index) for simulated depth
  UShort* sim_lt16;		// 16-bit lookup table (with 8-bit index) for simulated depth

  Colormap cmap;

public:
  csGraphics2DXLib (ISystem* piSystem, bool bUses3D);
  virtual ~csGraphics2DXLib ();

  virtual bool Open (char *Title);
  virtual void Close ();

  virtual bool BeginDraw () { return (Memory != NULL); }

  virtual void Print (csRect *area = NULL);
  virtual void SetRGB (int i, int r, int g, int b);

  /**
   * Special function that is available only on the X
   * version. This recomputes the colormap and the lookup table
   * for simulation of 15/16-bit on an 8-bit display to get an optimal
   * display. The default colormap simulates a 3:3:2 truecolor
   * display.
   */
  void recompute_simulated_palette ();

  /**
   * This function restores the standard 3:3:2 truecolor palette while
   * simulating 15/16-bit on an 8-bit display.
   */
  void restore_332_palette ();

  /**
   * Use greyscale palette for simulation of 15/16-bit on an 8-bit display.
   */
  void recompute_grey_palette ();

  /**
   * Extensions for X11 port.
   */
  virtual bool PerformExtension (char* args);

protected:
  DECLARE_IUNKNOWN()
  DECLARE_INTERFACE_TABLE(csGraphics2DXLib)
  DECLARE_COMPOSITE_INTERFACE(XXLibGraphicsInfo)
};

#endif //XLIBG2D_H

