/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef DO_X11

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

//changed Mar 9, 1998 by Jon Heiner for SGI compile
#ifndef OS_IRIX
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif

#include "system/system.h"
#include "util/inifile.h"
#include "system/xlib/xwin.h"
#include "render/graph3d.h"
#include "general/console.h"

#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/keysymdef.h>

#ifdef DO_SHM
#include <X11/extensions/XShm.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#endif /* DO_SHM */

#ifdef DO_OPENGL
#include <GL/glx.h>
#endif

Display* dpy = NULL;

int screen_num;
int display_width, display_height;
Window window;
XImage* xim = NULL;
GC gc;
#ifdef DO_SHM
  static XShmSegmentInfo shmi;
  XImage shm_image;
  bool do_shm;
#else
  bool do_shm = false;
#endif
#ifdef DO_OPENGL
GLXContext active_GLContext;
XVisualInfo *active_GLVisual = NULL;
#endif
static bool LoopStarted = false;// For Print() to know whenever to flush output

// Everything for simulated depth
int sim_depth = 0;
csPixelFormat real_pfmt;	// Contains the real pfmt is simulating stuff
unsigned char* real_Memory;	// Real memory to the display
unsigned char* sim_lt8 = NULL;	// 8-bit lookup table (with 16-bit index) for simulated depth
UShort* sim_lt16 = NULL;	// 16-bit lookup table (with 8-bit index) for simulated depth

Colormap cmap = 0;

void SysSystemDriver::SetSystemDefaults ()
{
  csSystemDriver::SetSystemDefaults ();

  FULL_SCREEN = false;

#ifdef DO_SHM
  do_shm = config->GetYesNo ("VideoDriver", "XSHM", true);
#endif
}

// Exit
void exit_clean (int rc)
{
  extern void cleanup ();
  cleanup ();
  exit (rc);
}

void SysSystemDriver::ParseArg (int argc, char* argv[], int &i)
{
  if (strcasecmp ("-mode", argv[i]) == 0)
  {
    i++;
    if (i < argc) SetMode (argv[i]);
  }
  else if (strcasecmp ("-shm", argv[i]) == 0)
  {
#   ifdef DO_SHM
      do_shm = true;
#   else
      CsPrintf (MSG_FATAL_ERROR, "SHM support not compiled into this version of Crystal Space!\n");
      exit_clean (0);
#   endif
  }
  else if (strcasecmp ("-noshm", argv[i]) == 0)
  {
#   ifdef DO_SHM
      do_shm = false;
#   else
      CsPrintf (MSG_FATAL_ERROR, "SHM support not compiled into this version of Crystal Space!\n");
      exit_clean (0);
#   endif
  }
  else if (strcasecmp ("-sdepth", argv[i]) == 0)
  {
    i++;
    sscanf (argv[i], "%d", &sim_depth);
    if (sim_depth != 8 && sim_depth != 15 && sim_depth != 16)
    {
      CsPrintf (MSG_FATAL_ERROR, "Crystal Space can't run in this simulated depth! (use 8, 15, or 16)!\n");
      exit (0);
    }
  }
  else csSystemDriver::ParseArg (argc, argv, i);
}

void SysSystemDriver::Help ()
{
  csSystemDriver::Help ();
  CsPrintf (MSG_STDOUT, "  -mode <w>x<y>      set resolution (default=%dx%d)\n",
	  FRAME_WIDTH, FRAME_HEIGHT);
  CsPrintf (MSG_STDOUT, "  -sdepth <depth>    set simulated depth (8, 15, or 16) (default=none)\n");
#ifdef DO_SHM
  CsPrintf (MSG_STDOUT, "  -shm/noshm         SHM extension (default '%sshm')\n",
	  do_shm ? "" : "no");
#endif
}

// System loop !
void SysSystemDriver::Loop(void)
{
  XEvent event;
  int state, key;
  bool down;
  LoopStarted = true;
  //int CompletionType = XShmGetEventBase (dpy) + ShmCompletion;
  while (!Shutdown && !ExitLoop)
  {
    NextFrame ();
    while (XCheckMaskEvent (dpy, ~0, &event))
      switch (event.type)
      {
        case ButtonPress:
          state = ((XButtonEvent*)&event)->state;
          Mouse->do_buttonpress (((XButtonEvent*)&event)->button,
		((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y,
            state & ShiftMask, state & Mod1Mask, state & ControlMask);
          break;

        case ButtonRelease:
          Mouse->do_buttonrelease (((XButtonEvent*)&event)->button,
		((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y);
          break;

        case MotionNotify:
          Mouse->do_mousemotion (((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y);
          break;

        case KeyPress:
          down = true;
          goto eatkey;

        case KeyRelease:
          down = false;
eatkey:
          key = XLookupKeysym ((XKeyEvent*)&event, 0);
          state = ((XKeyEvent*)&event)->state;
          switch (key)
          {
	    case XK_Meta_L:
	    case XK_Meta_R:
	    case XK_Alt_L:
	    case XK_Alt_R:      key = CSKEY_ALT; break;
	    case XK_Control_L:
	    case XK_Control_R:  key = CSKEY_CTRL; break;
	    case XK_Shift_L:
	    case XK_Shift_R:    key = CSKEY_SHIFT; break;
	    case XK_Up:         key = CSKEY_UP; break;
	    case XK_Down:       key = CSKEY_DOWN; break;
	    case XK_Left:       key = CSKEY_LEFT; break;
	    case XK_Right:      key = CSKEY_RIGHT; break;
	    case XK_BackSpace:  key = CSKEY_BACKSPACE; break;
	    case XK_Insert:     key = CSKEY_INS; break;
	    case XK_Delete:     key = CSKEY_DEL; break;
	    case XK_Page_Up:    key = CSKEY_PGUP; break;
	    case XK_Page_Down:  key = CSKEY_PGDN; break;
	    case XK_Home:       key = CSKEY_HOME; break;
	    case XK_End:        key = CSKEY_END; break;
	    case XK_Escape:     key = CSKEY_ESC; break;
	    case XK_Tab:        key = CSKEY_TAB; break;
	    case XK_Return:     key = CSKEY_ENTER; break;
	    case XK_F1:		key = CSKEY_F1; break;
	    case XK_F2:		key = CSKEY_F2; break;
	    case XK_F3:		key = CSKEY_F3; break;
	    case XK_F4:		key = CSKEY_F4; break;
	    case XK_F5:		key = CSKEY_F5; break;
	    case XK_F6:		key = CSKEY_F6; break;
	    case XK_F7:		key = CSKEY_F7; break;
	    case XK_F8:		key = CSKEY_F8; break;
	    case XK_F9:		key = CSKEY_F9; break;
	    case XK_F10:	key = CSKEY_F10; break;
	    case XK_F11:	key = CSKEY_F11; break;
	    case XK_F12:	key = CSKEY_F12; break;
            default:            break;
          }
          if (down)
            Keyboard->do_keypress (key);
          else
            Keyboard->do_keyrelease (key);
          break;
        default:
	  //if (event.type == CompletionType) shm_busy = 0;
	  break;
      } 
  }
}

/*
 * X specific extensions.
 */
bool SysSystemDriver::perform_extension (char* cmd, char* arg)
{
  (void)arg;

  if (!strcasecmp ("sim_pal", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->recompute_simulated_palette ();
    return true;
  }
  else if (!strcasecmp ("sim_grey", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->recompute_grey_palette ();
    return true;
  }
  else if (!strcasecmp ("sim_332", cmd))
  {
    ((SysGraphics2D*)(Graph3D->Graph2D))->restore_332_palette ();
    return true;
  }
  return false;
}

/*
 * Signal handler to clean up and give some
 * final debugging information.
 */
extern void debug_dump ();
void handler (int sig)
{
  static bool in_exit = false;
  if (in_exit) exit (1);
  in_exit = true;

  if (sig == SIGINT)
    fprintf (stderr, "\n^C\n");
  else
    fprintf (stderr, "SIGNAL %d CAUGHT!!!\n", sig);

  int err;
  err = errno;
  printf ("error %d: %s\n", err, strerror (err));

  if (sig != SIGINT) debug_dump ();

  exit_clean (1);
} 

void init_sig ()
{
  void handler (int sig);

  signal (SIGHUP, handler);
  signal (SIGINT, handler);
  signal (SIGTRAP, handler);
  signal (SIGABRT, handler);
  signal (SIGALRM, handler);
  signal (SIGTERM, handler);
  signal (SIGPIPE, handler);
  signal (SIGSEGV, handler);
  signal (SIGBUS, handler);
  signal (SIGFPE, handler);
  signal (SIGILL, handler);
}

// SysGraphics2D fonctions
SysGraphics2D::SysGraphics2D(int argc, char* argv[]) : csGraphics2D ()
{
  Screen* screen_ptr;
  (void)argc;  (void)argv;

  dpy = XOpenDisplay (NULL);

  screen_num = DefaultScreen (dpy);
  screen_ptr = DefaultScreenOfDisplay (dpy);
  display_width = DisplayWidth (dpy, screen_num);
  display_height = DisplayHeight (dpy, screen_num);

  init_sig ();

  // Determine visual information.
  Visual* visual = DefaultVisual (dpy, screen_num);

#   ifdef DO_OPENGL
  int desired_attributes[] = { GLX_RGBA, GLX_DOUBLEBUFFER, GLX_DEPTH_SIZE, 1, None };
 
  active_GLVisual = 
	glXChooseVisual(dpy,screen_num,desired_attributes);
  if (active_GLVisual)
  {
	sim_depth = 0;
	active_GLContext = glXCreateContext(dpy,active_GLVisual,0,GL_TRUE);
	cmap = XCreateColormap(dpy,RootWindow(dpy,active_GLVisual->screen),
		active_GLVisual->visual,AllocNone);
	visual = active_GLVisual->visual;
  }

#   endif
	
  pfmt.RedMask = 0xf00;//visual->red_mask;
  pfmt.GreenMask = 0xf0;//visual->green_mask;
  pfmt.BlueMask = 0xf;//visual->blue_mask;
  complete_pixel_format ();
  pfmt.PalEntries = visual->map_entries;
  if (visual->c_class == TrueColor) pfmt.PalEntries = 0;
  if (pfmt.PalEntries) pfmt.PixelBytes = 1;	// Palette mode
  else pfmt.PixelBytes = 2;			// Truecolor mode

  // First check if we can disable simulation because the format is the same.
  if (sim_depth == 8 && pfmt.PixelBytes == 1) sim_depth = 0;
  if (sim_depth == 15 && pfmt.PixelBytes == 2 && pfmt.RedMask == (0x1f<<10)) sim_depth = 0;
  if (sim_depth == 16 && pfmt.PixelBytes == 2 && pfmt.RedMask == (0x1f<<11)) sim_depth = 0;

  // A few simulations are not supported yet.
  if (sim_depth == 15 && pfmt.PixelBytes == 2 && pfmt.RedMask == (0x1f<<11))
  {
    CsPrintf (MSG_WARNING, "Simulating 15-bits on a 16-bit display is not implemented yet!\n");
    sim_depth = 0;
  }
  if (sim_depth == 16 && pfmt.PixelBytes == 2 && pfmt.RedMask == (0x1f<<10))
  {
    CsPrintf (MSG_WARNING, "Simulating 16-bits on a 15-bit display is not implemented yet!\n");
    sim_depth = 0;
  }

  // If we are going to simulate another depth then the real information
  // is put aside and we will fill pfmt with faked values.
  if (sim_depth != 0)
  {
    real_pfmt = pfmt;
    switch (sim_depth)
    {
      case 8:
        pfmt.RedMask = pfmt.GreenMask = pfmt.BlueMask = 0;
	pfmt.PalEntries = 256;
	pfmt.PixelBytes = 1;
	break;
      case 15:
      	pfmt.RedMask   = 0x1f<<10;
      	pfmt.GreenMask = 0x1f<<5;
      	pfmt.BlueMask  = 0x1f;
	pfmt.PalEntries = 0;
	pfmt.PixelBytes = 2;
	break;
      case 16:
      	pfmt.RedMask   = 0x1f<<11;
      	pfmt.GreenMask = 0x3f<<5;
      	pfmt.BlueMask  = 0x1f;
	pfmt.PalEntries = 0;
	pfmt.PixelBytes = 2;
	break;
    }
    complete_pixel_format ();
  }

#ifndef DO_OPENGL
  // Allocate the palette if not truecolor or if simulating truecolor
  // on an 8-bit display.
  if ((sim_depth == 0 && pfmt.PalEntries) || (sim_depth != 0 && real_pfmt.PalEntries))
    cmap = XCreateColormap (dpy, RootWindow (dpy, screen_num), visual, AllocAll);
  else
    cmap = 0;

  // If we are simulating truecolor on an 8-bit display then we will create
  // a truecolor 3:3:2 colormap. This is ugly but simulated depth is for testing
  // only. So who cares?
  if ((sim_depth == 15 || sim_depth == 16) && cmap)
  {
    int i;
    for (i = 0 ; i < 256 ; i++)
    {
      XColor color;
      color.pixel = i;
      color.red   = (((i&0xe0)>>5)<<5)*256;
      color.green = (((i&0x1c)>>2)<<5)*256;
      color.blue  = ((i&0x3)<<6)*256;
      color.flags = DoRed | DoGreen | DoBlue;
      XStoreColor (dpy, cmap, &color);
    }
  }

#endif

  // Create the simulated lookup tables if any.
  if (sim_depth == 15 && real_pfmt.PixelBytes == 1)
  {
    // Simulate 15-bit on 8-bit.
    CHK (sim_lt8 = new unsigned char [65536]);
    int i, r, g, b;
    for (i = 0 ; i < 32768 ; i++)
    {
      r = (i&0x7c00)>>10; r >>= 2;
      g = (i&0x03e0)>>5;  g >>= 2;
      b = (i&0x001f);     b >>= 3;
      sim_lt8[i] = (r<<5) | (g<<2) | b;
    }
  }
  else if (sim_depth == 16 && real_pfmt.PixelBytes == 1)
  {
    // Simulate 16-bit on 8-bit.
    CHK (sim_lt8 = new unsigned char [65536]);
    int i, r, g, b;
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0xf800)>>11; r >>= 2;
      g = (i&0x07e0)>>5;  g >>= 3;
      b = (i&0x001f);     b >>= 3;
      sim_lt8[i] = (r<<5) | (g<<2) | b;
    }
  }

  CsPrintf (MSG_INITIALIZATION, "Crystal Space X windows version");
  if (do_shm) CsPrintf (MSG_INITIALIZATION, " (Using SHM extension).\n");
  else CsPrintf (MSG_INITIALIZATION, ".\n");

  xim = NULL;
  Memory = NULL;

  // If in 16-bit mode, redirect drawing routines
  if (pfmt.PixelBytes == 2)
  {
    DrawPixel = DrawPixel16;
    WriteChar = WriteChar16;
    GetPixelAt = GetPixelAt16;
    DrawSprite = DrawSprite16;
  } /* endif */
}

SysGraphics2D::~SysGraphics2D(void)
{
  // Destroy your graphic interface
  Close();
  if (sim_lt8) CHKB (delete [] sim_lt8);
  if (sim_lt16) CHKB (delete [] sim_lt16);
  if (Memory && sim_depth != 0) CHKB (delete [] Memory);
}

bool SysGraphics2D::Open(char *Title)
{
  // Open your graphic interface
  if (!csGraphics2D::Open (Title)) return false;

  // Open window
#  ifdef DO_OPENGL
  XSetWindowAttributes winattr;
  winattr.border_pixel = 0;
  winattr.colormap = cmap;
  winattr.event_mask = ExposureMask | KeyPressMask | KeyReleaseMask |
     FocusChangeMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask;  
  window = XCreateWindow(dpy,RootWindow(dpy,active_GLVisual->screen), 0, 0,
	FRAME_WIDTH,FRAME_HEIGHT, 0 /*border width */,
	active_GLVisual->depth, InputOutput, active_GLVisual->visual, 
	CWBorderPixel | CWColormap | CWEventMask, &winattr);
  XMapWindow (dpy, window);
#  else
  window = XCreateSimpleWindow (dpy, DefaultRootWindow (dpy), 64, 16,
  	FRAME_WIDTH, FRAME_HEIGHT, 4, 0, 0);
  XMapWindow (dpy, window);
  XGCValues values;
  gc = XCreateGC (dpy, window, 0, &values);
  XSetForeground (dpy, gc, BlackPixel (dpy, screen_num));
  XSetLineAttributes (dpy, gc, 0, LineSolid, CapButt, JoinMiter);
  XSetGraphicsExposures (dpy, gc, False);

  XSetWindowAttributes attr;
  attr.event_mask = ExposureMask | KeyPressMask | KeyReleaseMask |
  	FocusChangeMask | PointerMotionMask | ButtonPressMask | ButtonReleaseMask;
  XChangeWindowAttributes (dpy, window, CWEventMask, &attr);

  if (cmap) XSetWindowColormap (dpy, window, cmap);
#  endif

  // Wait for expose event
  XEvent event;
  for (;;)
  {
    XNextEvent (dpy, &event);
    if (event.type == Expose) break;
  }

  // Create backing store
  if (!xim)
  {
    xim = XGetImage (dpy, window, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, AllPlanes, ZPixmap);
#   ifdef DO_SHM
    if (do_shm && !XShmQueryExtension (dpy))
    {
      do_shm = false;
      CsPrintf (MSG_INITIALIZATION, "Shm extension not available on display!\n");
    }
    if (do_shm)
    {
      shm_image = *xim;
      shmi.shmid = shmget (IPC_PRIVATE, xim->bytes_per_line*xim->height,
			       IPC_CREAT|0777);
      shmi.shmaddr = (char*)shmat (shmi.shmid, 0, 0);
      shmi.readOnly = FALSE;
      XShmAttach (dpy, &shmi);

      // Delete memory segment. The memory stays available until the last client
      // detaches from it.
      XSync (dpy, False);
      shmctl (shmi.shmid, IPC_RMID, 0);

      shm_image.data = shmi.shmaddr;
      real_Memory = (unsigned char*)shmi.shmaddr;
      shm_image.obdata = (char*)&shmi;
    }
    else
#   endif /* DO_SHM */
    {
      real_Memory = (unsigned char*)(xim->data);
    }

    // If not simulating depth then Memory is equal to real_Memory.
    // If simulating then we allocate a new Memory array in the faked format.
    if (!sim_depth) Memory = real_Memory;
    else
    {
      CHK (Memory = new unsigned char [FRAME_WIDTH*FRAME_HEIGHT*pfmt.PixelBytes]);
    }

//CHK (Memory = new unsigned char [FRAME_WIDTH*FRAME_HEIGHT*pfmt.PixelBytes]);
  }

#ifdef DO_OPENGL
  glXMakeCurrent(dpy,window,active_GLContext);
#endif

  Clear (0);
  return true;
}

void SysGraphics2D::Close(void)
{
  // Close your graphic interface
  csGraphics2D::Close ();
}

struct palent
{
  UShort idx;
  unsigned char r, g, b;
  int cnt;
};

int cmp_palent (const void* p1, const void* p2)
{
  const palent* pe1 = (palent*)p1;
  const palent* pe2 = (palent*)p2;
  if (pe1->cnt < pe2->cnt) return 1;
  else if (pe1->cnt > pe2->cnt) return -1;
  else return 0;
}

int find_rgb_palent (palent* pe, int r, int g, int b)
{
  int i, min, mindist;
  mindist = 1000L*256*256;
  min = -1;
  register int red, green, blue, dist;
  for (i = 0 ; i < 256 ; i++)
    if (pe[i].cnt)
    {
      red = r - pe[i].r;
      green = g - pe[i].g;
      blue = b - pe[i].b;
      dist = (299*red*red) + (587*green*green) + (114*blue*blue);
      if (dist == 0) return i;
      if (dist < mindist) { mindist = dist; min = i; }
    }
    else return min;
  return min;
}

void SysGraphics2D::restore_332_palette ()
{
  if (sim_depth != 15 && sim_depth != 16) return;

  CsPrintf (MSG_CONSOLE, "Restore 3:3:2 palette\n");
  System->Console->Print ();
  Print ();
  XFlush (dpy);

  int i, r, g, b;

  if (sim_depth == 15)
    for (i = 0 ; i < 32768 ; i++)
    {
      r = (i&0x7c00)>>10; r >>= 2;
      g = (i&0x03e0)>>5;  g >>= 2;
      b = (i&0x001f);     b >>= 3;
      sim_lt8[i] = (r<<5) | (g<<2) | b;
    }
  else
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0xf800)>>11; r >>= 2;
      g = (i&0x07e0)>>5;  g >>= 3;
      b = (i&0x001f);     b >>= 3;
      sim_lt8[i] = (r<<5) | (g<<2) | b;
    }

  for (i = 0 ; i < 256 ; i++)
  {
    XColor color;
    color.pixel = i;
    color.red   = (((i&0xe0)>>5)<<5)*256;
    color.green = (((i&0x1c)>>2)<<5)*256;
    color.blue  = ((i&0x3)<<6)*256;
    color.flags = DoRed | DoGreen | DoBlue;
    XStoreColor (dpy, cmap, &color);
  }

}

void SysGraphics2D::recompute_simulated_palette ()
{
  if (sim_depth != 15 && sim_depth != 16) return;

  CsPrintf (MSG_CONSOLE, "Recompute simulated palette\n");
  System->Console->Print ();
  Print ();
  XFlush (dpy);

  palent* pe;
  int i;

  // Create a palette of 16K entries
  CHK (pe = new palent [65536]);
  for (i = 0 ; i < 65536 ; i++)
  {
    pe[i].idx = i;
    pe[i].r = ((i&pfmt.RedMask) >> pfmt.RedShift) << (8-pfmt.RedBits);
    pe[i].g = ((i&pfmt.GreenMask) >> pfmt.GreenShift) << (8-pfmt.GreenBits);
    pe[i].b = ((i&pfmt.BlueMask) >> pfmt.BlueShift) << (8-pfmt.BlueBits);
    pe[i].cnt = 0;
  }

  // Count how many times each color appears.
  UShort* m = (UShort*)Memory;
  i = FRAME_WIDTH*FRAME_HEIGHT;
  while (i > 0)
  {
    pe[*m++].cnt++;
    i--;
  }

  // Fake that color 0 (black) is used very much so that
  // it will appear in front of all colors after sorting.
  pe[0].cnt = FRAME_WIDTH*FRAME_HEIGHT+1;

  // Sort based on count.
  qsort ((void*)pe, 65536, sizeof (palent), cmp_palent);

  // Fake that color 255 (white) is set to white.
  pe[255].r = 255;
  pe[255].g = 255;
  pe[255].b = 255;

  CsPrintf (MSG_CONSOLE, "Recomputing lookup table...\n");
  System->Console->Print ();
  Print ();
  XFlush (dpy);

  // Recompute the lookup table from 15/16-bit to 8-bit.
  int r, g, b;
  if (sim_depth == 15)
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0x7c00)>>10; r <<= 3;
      g = (i&0x03e0)>>5;  g <<= 3;
      b = (i&0x001f);     b <<= 3;
      sim_lt8[i] = find_rgb_palent (pe, r, g, b);
    }
  else
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0xf800)>>11; r <<= 3;
      g = (i&0x07e0)>>5;  g <<= 2;
      b = (i&0x001f);     b <<= 3;
      sim_lt8[i] = find_rgb_palent (pe, r, g, b);
    }

  // Allocate colors for the 256 most often occuring
  // colors on the display. This also makes the first 256
  // elements of pe the palette.
  for (i = 0 ; i < 256 ; i++)
  {
    XColor color;
    color.pixel = i;
    color.red   = pe[i].r*256;
    color.green = pe[i].g*256;
    color.blue  = pe[i].b*256;
    color.flags = DoRed | DoGreen | DoBlue;
    XStoreColor (dpy, cmap, &color);
  }

  CHK (delete [] pe);
  CsPrintf (MSG_CONSOLE, "Done!\n");
}

void SysGraphics2D::recompute_grey_palette ()
{
  if (sim_depth != 15 && sim_depth != 16) return;

  CsPrintf (MSG_CONSOLE, "Compute grey palette\n");
  System->Console->Print ();
  Print ();
  XFlush (dpy);

  palent* pe;
  int i;

  // Create a grey palette
  CHK (pe = new palent [256]);
  for (i = 0 ; i < 256 ; i++)
  {
    pe[i].idx = i;
    pe[i].r = i;
    pe[i].g = i;
    pe[i].b = i;
    pe[i].cnt = 1;
  }

  CsPrintf (MSG_CONSOLE, "Recomputing lookup table...\n");
  System->Console->Print ();
  Print ();
  XFlush (dpy);

  // Recompute the lookup table from 15/16-bit to 8-bit.
  int r, g, b;
  if (sim_depth == 15)
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0x7c00)>>10; r <<= 3;
      g = (i&0x03e0)>>5;  g <<= 3;
      b = (i&0x001f);     b <<= 3;
      sim_lt8[i] = find_rgb_palent (pe, r, g, b);
    }
  else
    for (i = 0 ; i < 65536 ; i++)
    {
      r = (i&0xf800)>>11; r <<= 3;
      g = (i&0x07e0)>>5;  g <<= 2;
      b = (i&0x001f);     b <<= 3;
      sim_lt8[i] = find_rgb_palent (pe, r, g, b);
    }

  // Allocate colors for the 256 most often occuring
  // colors on the display. This also makes the first 256
  // elements of pe the palette.
  for (i = 0 ; i < 256 ; i++)
  {
    XColor color;
    color.pixel = i;
    color.red   = pe[i].r*256;
    color.green = pe[i].g*256;
    color.blue  = pe[i].b*256;
    color.flags = DoRed | DoGreen | DoBlue;
    XStoreColor (dpy, cmap, &color);
  }

  CHK (delete [] pe);
  CsPrintf (MSG_CONSOLE, "Done!\n");
}

void SysGraphics2D::Print (csRect *area)
{
  if (sim_depth)
  {
    if (sim_depth == 15 || sim_depth == 16)
    {
      UShort* src = (UShort*)Memory;
      unsigned char* dst = real_Memory;
      int i = FRAME_WIDTH*FRAME_HEIGHT;
      while (i > 0)
      {
        *dst++ = sim_lt8[*src++];
        i--;
      }
    }
    else if (sim_depth == 8)
    {
      // If it is the first time we come here we will create
      // the lookup-table from 15/16-bit to 8-bit. We can only do
      // it here because the palette can change (gamma correction).
      // If the palette changes this lookup table is automatically made
      // invalid (by SetRGB).
      if (!sim_lt16)
      {
        CHK (sim_lt16 = new UShort [256]);
	int i, r, g, b;
	for (i = 0 ; i < 256 ; i++)
	{
	  r = Palette[i].red;
	  g = Palette[i].green;
	  b = Palette[i].blue;
	  sim_lt16[i] =
		((r >> (8-real_pfmt.RedBits))   << real_pfmt.RedShift) |
		((g >> (8-real_pfmt.GreenBits)) << real_pfmt.GreenShift) |
		((b >> (8-real_pfmt.BlueBits))  << real_pfmt.BlueShift);
        }
      }
      unsigned char* src = Memory;
      UShort* dst = (UShort*)real_Memory;
      int i = FRAME_WIDTH*FRAME_HEIGHT;
      while (i > 0)
      {
        *dst++ = sim_lt16[*src++];
        i--;
      }
    }
  }

//memcpy (real_Memory, Memory, FRAME_WIDTH*FRAME_HEIGHT*pfmt.PixelBytes);
#ifdef DO_SHM
  if (do_shm)
  {
    if (area)
      XShmPutImage (dpy, window, gc, &shm_image,
		    area->xmin, area->ymin, area->xmin, area->ymin,
		    area->Width (), area->Height (),
		    False);
    else
      XShmPutImage (dpy, window, gc, &shm_image, 0, 0, 0, 0, FRAME_WIDTH, FRAME_HEIGHT, False);
    XSync (dpy, False);
  }
  else
#endif
#ifdef DO_OPENGL
  {
      glXSwapBuffers(dpy,window);
      glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  } 
#else
  {
    if (area)
      XPutImage (dpy, window, gc, xim,
		 area->xmin, area->ymin, area->xmin, area->ymin,
		 area->Width (), area->Height ());
    else
      XPutImage (dpy, window, gc, xim, 0, 0, 0, 0, FRAME_WIDTH, FRAME_HEIGHT);
  }
  if (!LoopStarted)
    XFlush (dpy);
#endif
}

void SysGraphics2D::SetRGB(int i, int r, int g, int b)
{
#ifndef DO_OPENGL
  // If there is a colormap AND we are not simulating a display with
  // a colormap then we really set the color.
  if (cmap && !sim_depth)
  {
    XColor color;
    color.pixel = i;
    color.red = r*256;
    color.green = g*256;
    color.blue = b*256;
    color.flags = DoRed | DoGreen | DoBlue;

    XStoreColor (dpy, cmap, &color);
  }

  // If we are simulating an 8-bit display on truecolor displays then we
  // need to delete the lookup table. It has to be recomputed after changing
  // a palette entry.
  if (sim_depth == 8)
  {
    if (sim_lt16) { CHK (delete [] sim_lt16); sim_lt16 = NULL; }
  }
#endif
  csGraphics2D::SetRGB (i, r, g, b);
}

// Keyboard fonctions
SysKeyboardDriver::SysKeyboardDriver() : csKeyboardDriver ()
{
  // Create your keyboard interface
}

bool SysKeyboardDriver::Open(csEventQueue *EvQueue)
{
  csKeyboardDriver::Open (EvQueue);
  // Open your keyboard interface
  return true;
}

void SysKeyboardDriver::Close ()
{
  csKeyboardDriver::Close();
  // Close your keyboard interface
}

// Mouse fonctions
SysMouseDriver::SysMouseDriver() : csMouseDriver ()
{
// Initialize mouse system
}

bool SysMouseDriver::Open(csEventQueue *EvQueue)
{
  csMouseDriver::Open (EvQueue);
// Open mouse system
  return 1;
}

void SysMouseDriver::Close()
{
// Close mouse system
}

#endif //DO_X11
