/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifdef DISP_X11

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>

#define XK_MISCELLANY 1
#include <X11/Xlib.h>
#include <X11/keysymdef.h>

//changed Mar 9, 1998 by Jon Heiner for SGI compile
#ifndef OS_IRIX
#include <sys/timeb.h>
#else
#include <sys/time.h>
#endif

#include "util/def.h"
#include "system/system.h"
#include "util/inifile.h"
#include "system/xlib/xwin.h"
#include "system/xlib/driver2d/isysg2d.h"
//#include "render/graph3d.h"
#include "general/console.h"

static bool LoopStarted = false;// For Print() to know whenever to flush output

// The System driver ////////////////

BEGIN_INTERFACE_TABLE(SysSystemDriver)
  IMPLEMENTS_COMPOSITE_INTERFACE(XLibSystemDriver)
  IMPLEMENTS_COMPOSITE_INTERFACE(System)
END_INTERFACE_TABLE()

IMPLEMENT_UNKNOWN_NODELETE(SysSystemDriver)

void SysSystemDriver::SetSystemDefaults ()
{
  csSystemDriver::SetSystemDefaults ();

  //FULL_SCREEN = false;

  void init_sig ();
  init_sig ();
}

// Exit
void exit_clean (int rc)
{
  extern void cleanup ();
  cleanup ();
  exit (rc);
}

void SysSystemDriver::ParseArg (int argc, char* argv[], int &i)
{
  if (strcasecmp ("-shm", argv[i]) == 0)
  {
#if 0 // @@@ Temporarily disabled because we need a better place for this.
#   ifdef DO_SHM
      do_shm = true;
#   else
      CsPrintf (MSG_FATAL_ERROR, "SHM support not compiled into this version of Crystal Space!\n");
      exit_clean (0);
#   endif
#endif
  }
  else if (strcasecmp ("-noshm", argv[i]) == 0)
  {
#if 0 // @@@ Temporarily disabled because we need a better place for this.
#   ifdef DO_SHM
      do_shm = false;
#   else
      CsPrintf (MSG_FATAL_ERROR, "SHM support not compiled into this version of Crystal Space!\n");
      exit_clean (0);
#   endif
#endif
  }
  else if (strcasecmp ("-sdepth", argv[i]) == 0)
  {
    i++;
#if 0 // @@@ Temporarily disabled because we need a better place for this.
    sscanf (argv[i], "%d", &sim_depth);
    if (sim_depth != 8 && sim_depth != 15 && sim_depth != 16)
    {
      CsPrintf (MSG_FATAL_ERROR, "Crystal Space can't run in this simulated depth! (use 8, 15, or 16)!\n");
      exit (0);
    }
#endif
  }
  else csSystemDriver::ParseArg (argc, argv, i);
}

void SysSystemDriver::Help ()
{
  csSystemDriver::Help ();
  CsPrintf (MSG_STDOUT, "  -sdepth <depth>    set simulated depth (8, 15, or 16) (default=none)\n");
#if 0 // @@@ Temporarily disabled
#ifdef DO_SHM
  CsPrintf (MSG_STDOUT, "  -shm/noshm         SHM extension (default '%sshm')\n",
	  do_shm ? "" : "no");
#endif
#endif
}

// System loop !
void SysSystemDriver::Loop(void)
{
  XEvent event;
  int state, key;
  bool down;
  LoopStarted = true;
  Display* dpy;
  //int CompletionType = XShmGetEventBase (dpy) + ShmCompletion;
  IXLibGraphicsInfo* piG2D = NULL;
  HRESULT hRes = Gfx2D->QueryInterface (IID_IXLibGraphicsInfo, (void**)&piG2D);
  if (SUCCEEDED(hRes))
  {
    hRes = piG2D->GetDisplay (&dpy);
    piG2D->Release ();
    piG2D = NULL;
  }

  while (!Shutdown && !ExitLoop)
  {
    static long prev_time = -1;
    long elapsed = (prev_time == -1) ? 0 : Time () - prev_time;
    prev_time = Time ();
    NextFrame (elapsed, Time ());

    while (XCheckMaskEvent (dpy, ~0, &event))
      switch (event.type)
      {
        case ButtonPress:
          state = ((XButtonEvent*)&event)->state;
          Mouse->do_buttonpress (Time (), ((XButtonEvent*)&event)->button,
		((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y,
            state & ShiftMask, state & Mod1Mask, state & ControlMask);
          break;

        case ButtonRelease:
          Mouse->do_buttonrelease (Time (), ((XButtonEvent*)&event)->button,
		((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y);
          break;

        case MotionNotify:
          Mouse->do_mousemotion (Time (), ((XButtonEvent*)&event)->x, ((XButtonEvent*)&event)->y);
          break;

        case KeyPress:
          down = true;
          goto eatkey;

        case KeyRelease:
          down = false;
eatkey:
          key = XLookupKeysym ((XKeyEvent*)&event, 0);
          state = ((XKeyEvent*)&event)->state;
          switch (key)
          {
	    case XK_Meta_L:
	    case XK_Meta_R:
	    case XK_Alt_L:
	    case XK_Alt_R:      key = CSKEY_ALT; break;
	    case XK_Control_L:
	    case XK_Control_R:  key = CSKEY_CTRL; break;
	    case XK_Shift_L:
	    case XK_Shift_R:    key = CSKEY_SHIFT; break;
	    case XK_Up:         key = CSKEY_UP; break;
	    case XK_Down:       key = CSKEY_DOWN; break;
	    case XK_Left:       key = CSKEY_LEFT; break;
	    case XK_Right:      key = CSKEY_RIGHT; break;
	    case XK_BackSpace:  key = CSKEY_BACKSPACE; break;
	    case XK_Insert:     key = CSKEY_INS; break;
	    case XK_Delete:     key = CSKEY_DEL; break;
	    case XK_Page_Up:    key = CSKEY_PGUP; break;
	    case XK_Page_Down:  key = CSKEY_PGDN; break;
	    case XK_Home:       key = CSKEY_HOME; break;
	    case XK_End:        key = CSKEY_END; break;
	    case XK_Escape:     key = CSKEY_ESC; break;
	    case XK_Tab:        key = CSKEY_TAB; break;
	    case XK_Return:     key = CSKEY_ENTER; break;
	    case XK_F1:		key = CSKEY_F1; break;
	    case XK_F2:		key = CSKEY_F2; break;
	    case XK_F3:		key = CSKEY_F3; break;
	    case XK_F4:		key = CSKEY_F4; break;
	    case XK_F5:		key = CSKEY_F5; break;
	    case XK_F6:		key = CSKEY_F6; break;
	    case XK_F7:		key = CSKEY_F7; break;
	    case XK_F8:		key = CSKEY_F8; break;
	    case XK_F9:		key = CSKEY_F9; break;
	    case XK_F10:	key = CSKEY_F10; break;
	    case XK_F11:	key = CSKEY_F11; break;
	    case XK_F12:	key = CSKEY_F12; break;
            default:            break;
          }
          if (down)
            Keyboard->do_keypress (Time (), key);
          else
            Keyboard->do_keyrelease (Time (), key);
          break;
        default:
	  //if (event.type == CompletionType) shm_busy = 0;
	  break;
      } 
  }
}

// COM Implementation

IMPLEMENT_COMPOSITE_UNKNOWN_AS_EMBEDDED( SysSystemDriver, XLibSystemDriver )


/*
 * Signal handler to clean up and give some
 * final debugging information.
 */
extern void debug_dump ();
void handler (int sig)
{
  static bool in_exit = false;
  if (in_exit) exit (1);
  in_exit = true;

  if (sig == SIGINT)
    fprintf (stderr, "\n^C\n");
  else
    fprintf (stderr, "SIGNAL %d CAUGHT!!!\n", sig);

  int err;
  err = errno;
  printf ("error %d: %s\n", err, strerror (err));

  if (sig != SIGINT) debug_dump ();

  exit_clean (1);
} 

void init_sig ()
{
  void handler (int sig);

  signal (SIGHUP, handler);
  signal (SIGINT, handler);
  signal (SIGTRAP, handler);
  signal (SIGABRT, handler);
  signal (SIGALRM, handler);
  signal (SIGTERM, handler);
  signal (SIGPIPE, handler);
  signal (SIGSEGV, handler);
  signal (SIGBUS, handler);
  signal (SIGFPE, handler);
  signal (SIGILL, handler);
}


// Keyboard fonctions
SysKeyboardDriver::SysKeyboardDriver() : csKeyboardDriver ()
{
  // Create your keyboard interface
}

bool SysKeyboardDriver::Open(csEventQueue *EvQueue)
{
  csKeyboardDriver::Open (EvQueue);
  // Open your keyboard interface
  return true;
}

void SysKeyboardDriver::Close ()
{
  csKeyboardDriver::Close();
  // Close your keyboard interface
}

// Mouse fonctions
SysMouseDriver::SysMouseDriver() : csMouseDriver ()
{
// Initialize mouse system
}

bool SysMouseDriver::Open(csEventQueue *EvQueue)
{
  csMouseDriver::Open (EvQueue);
// Open mouse system
  return 1;
}

void SysMouseDriver::Close()
{
// Close mouse system
}


#endif //DISP_X11
