/*
  Crystal Space Windowing System: vector class
  Copyright (C) 1998 by Jorrit Tyberghein
  Written by Andrew Zabolotny <bit@eltech.ru>

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public
  License as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Library General Public License for more details.

  You should have received a copy of the GNU Library General Public
  License along with this library; if not, write to the Free
  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <string.h>
#include "util/csvector.h"

csVector::csVector (int ilimit, int ithreshold)
{
  root = (csSome *)malloc ((limit = ilimit) * sizeof (csSome));
  count = 0; threshold = ithreshold;
}

csVector::~csVector ()
{
//not much sense to call DeleteAll () since even for inherited classes
//anyway will be called csVector::FreeItem which is empty.
//DeleteAll ();
  if (root) free (root);
}

void csVector::DeleteAll ()
{
  for (int i = 0; i < count; i++)
    FreeItem (root[i]);
  SetLength (0);
}

void csVector::SetLength (int n)
{
  count = n;
  if ((n > limit) || ((limit > threshold) && (n < limit - threshold)))
  {
    n = ((n + threshold - 1) / threshold) * threshold;
    root = (csSome *)realloc (root, n * sizeof (csSome));
    limit = n;
  }
}

void csVector::FreeItem (csSome Item)
{
  (void)Item;
}

bool csVector::Delete(int n)
{
  int ncount = count - 1;

  if (n < count)
  {
    FreeItem (root[n]);
    memmove (&root[n], &root[n + 1], (ncount - n) * sizeof(csSome));
    SetLength (ncount);
    return true;
  } else
   return false;
}

bool csVector::Insert(int n, csSome Item)
{
  if (n <= count)
  {
    SetLength (count + 1);
    memmove (&root[n + 1], &root[n], (count - n) * sizeof(csSome));
    root[n] = Item;
    return true;
  } else
   return false;
}

int csVector::Find (csSome which)
{
  for (int i = 0; i < Length (); i++)
    if (root[i] == which)
      return i;
  return -1;
}
