#ifndef DEF_H
#define DEF_H

#include "util/types.h"
#include "util/memory.h"
#include "util/fixed.h"

//---------------------------------------------------------------
// Test if the makefile correctly defines all the operating
// system (one of the OS_ flags), the compiler (one of the
// COMP_ flags) and the processor (one of the PROC_ flags).
//---------------------------------------------------------------

#if (defined(OS_SOLARIS) || defined(OS_LINUX) || defined(OS_IRIX) || defined(OS_BSD) || defined(OS_BE)) && !defined(OS_UNIX)
#  define OS_UNIX
#endif

#if !defined(OS_SOLARIS) && !defined(OS_LINUX) && !defined(OS_DOS) && !defined(OS_UNIX) && !defined(OS_MACOS) && !defined(OS_AMIGAOS) && !defined(OS_WIN32) && !defined(OS_OS2) && !defined(OS_IRIX) && !defined(OS_BSD) && !defined(OS_BE)
#  error "Please specify the operating system in the makefile! (one of OS_...)"
#endif

#if !defined(COMP_GCC) && !defined(COMP_WCC) && !defined(COMP_UNKNOWN) && !defined(COMP_MWERKS) && !defined(COMP_VC)
#  error "Please specify the compiler in the makefile! (one of COMP_...)"
#endif

#if !defined(PROC_INTEL) && !defined(PROC_SPARC) && !defined(PROC_UNKNOWN) && !defined(PROC_POWERPC) && !defined(PROC_M68K)
#  error "Please specify the processor in the makefile! (one of PROC_...)"
#endif

#if defined(OS_SOLARIS) && !defined(DISP_X11) && !defined(DISP_GGI)
#  error "You need either X Windows or GGI support on a Solaris machine!"
#endif

#if defined(OS_LINUX) && !defined(DISP_X11) && !defined(DISP_SVGA) && !defined(DISP_GGI)
#  error "You need either X Windows, SVGALIB or GGI support on a Linux machine!"
#endif

#if (defined(OS_UNIX) && !defined(OS_LINUX)) && !defined(DISP_X11) && !defined(DISP_GGI) && !defined(DISP_OTHER)
#  error "Currently you need either X Windows or GGI support on a Unix machine!"
#endif

#if defined(DISP_SVGA) && !defined(OS_LINUX)
#  error "SVGALIB only works in combination with Linux!"
#endif

#if defined(DISP_SVGA) && defined(DISP_X11)
#  error "SVGALIB cannot be combined with X Windows support!"
#endif

#if defined(DISP_SVGA) && defined(DISP_GGI)
#  error "SVGALIB cannot be combined with GGI support!"
#endif

#if defined(DISP_GGI) && defined(DISP_X11)
#  error "GGI cannot be combined with X Windows support!"
#endif

#if defined(OS_AMIGAOS) && !defined(DO_AMIGAOS)
#  error "Currently you need AmigaOS support on an Amiga machine!"
#endif

//---------------------------------------------------------------

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <assert.h>

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

#ifndef MIN
#define MIN(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef MAX
#define MAX(a,b) ((a)>(b)?(a):(b))
#endif

#ifndef ABS
#define ABS(x) ((x)<0?-(x):(x))
#endif

#if !defined(SIGN) && !defined(DO_AMIGAOS)
#define SIGN(x) ((x)<0?-1:((x)>0?1:0))
#endif

#define EPSILON 0.001			/* Small value */
#define SMALL_EPSILON 0.000001		/* Very small value */
#ifndef OS_WIN32
#define INFINITE 999999000		/* Very large number */
#endif
#ifndef PI
#define PI	3.14159265358979323	/* You know this number, don't you? */
#endif
#ifndef M_PI
#define M_PI PI
#endif
#ifndef M_PI_2
#define M_PI_2	1.57079632679489661923	/* PI/2 */
#endif

#define DEBUG 1

#if defined(COMP_WCC)
#define strcasecmp stricmp
#define strncasecmp strnicmp
#endif

#if defined(COMP_VC)
#define strcasecmp _stricmp
#define strncasecmp _strnicmp
#endif

#ifdef PROC_INTEL
/*
//#pragma aux RoundToInt=\
//        "fistp DWORD [eax]"\
//        parm nomemory [eax] [8087]\
//        modify exact [8087];
*/

// This is 'stolen' from someone (I don't remember who anymore). It
// is a nice and fast way to convert a floating point number to int
// (only works on a i386 type processor).
// It is equivalent to 'i=(int)(f+.5)'. 
#define FIST_MAGIC ((((65536.0 * 65536.0 * 16)+(65536.0 * 0.5))* 65536.0))
inline long QuickRound (float inval)
{
  double dtemp = FIST_MAGIC + inval;
  return ((*(long *)&dtemp) - 0x80000000);
}

inline long QuickInt (float inval)
{
  double dtemp = FIST_MAGIC + (inval-.4999);
  return ((*(long *)&dtemp) - 0x80000000);
}

// This is my own invention derived from the previous one. This converts
// a floating point number to a 16.16 fixed point integer. It is
// equivalent to 'i=(int)(f*65536.)'.
#define FIST_MAGIC2 ((((65536.0 * 16)+(0.5))* 65536.0))
inline long QuickInt16 (float inval)
{
  double dtemp = FIST_MAGIC2 + inval;
  return ((*(long *)&dtemp) - 0x80000000);
}
#endif //PROC_INTEL

#ifdef PROC_M68K

#define FIST_MAGIC ((((65536.0 * 65536.0 * 16)+(65536.0 * 0.5))* 65536.0))
inline long QuickRound (float inval)
{
  double dtemp = FIST_MAGIC + inval;
  return (*(((long *)&dtemp) + 1)) - 0x80000000;
}
    
inline long QuickInt (float inval)
{
  double dtemp = FIST_MAGIC + (inval-.4999);
  return (*(((long *)&dtemp) + 1)) - 0x80000000;
}
	
#define FIST_MAGIC2 ((((65536.0 * 16)+(0.5))* 65536.0))
inline long QuickInt16 (float inval)
{
  double dtemp = FIST_MAGIC2 + inval;
  return (*(((long *)&dtemp) + 1)) - 0x80000000;
}
#endif

#if defined(PROC_INTEL) || defined(PROC_M68K)
#  define QRound(x) QuickRound(x)
#  define QInt(x) QuickInt(x)
#  define QInt16(x) QuickInt16(x)
#else
#  define QRound(x) ((int)((x)+.5))
#  define QInt(x) ((int)(x))
#  define QInt16(x) ((int)((x)*65536.))
#endif

// @@@ I don't know if there is a better way to convert
// a floating point to 8:24 fixed point (one with constants
// like the tricks above instead of the multiplication).
#define QInt24(x) (QInt16(((x)*256.)))

//#define SMALL_Z .01
#define SMALL_Z .1

#define USE_OCCLUSION 0 // Experimental feature, will not work in this version.

// Some useful macros: these should be true at least for 32-bit processors
#define LONGFROM2SHORT(s1,s2) (((short)s1) << 16 | (((short)s2) & 0xffff))
#define SHORT1FROMLONG(l)     (short)(((long)l) >> 16)
#define SHORT2FROMLONG(l)     (short)(((long)l) & 0xffff)

// Forward declaration for CrystalSpace classes
class csSystemDriver;
class csGraphics2D;
class csGraphics3D;
class TextureMM;
class csIniFile;

#endif /*DEF_H*/
