/*
    Copyright (C) 1998 by Jorrit Tyberghein    

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <math.h>
#include <time.h>
#include <stdio.h>

#include "system/system.h"
#include "util/csvector.h"

#include "sound/wave.h"
#include "sound/channel.h"
#include "sound/module.h"

#include "util/sound/modfile.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#if !defined(EXPLICIT_LOADER)
MODModuleLoader _MOD_module_type_supported_;
#endif

/** MOD file loader
 it's support old 4 channel mod format, protracker, fastracker, oktalizer, taketracker and startracker
 2, 4, 6, 8, 10, 12, 14, 16, 18, 20, 22, 24, 26, 28, 30 and 32 channels and 31/15 instruments.
*/

int readAmigaWord(int byte1, int byte2)
{
	return (byte1 * 256 + byte2) * 2;
}

#define canAddStream(x) {if((index+x)>size) goto exit_read;}
#define addStream(x) {if((index+x)>size) {goto exit_read;} else {index+=x;}}
#define Stream buf[index]

Module* MODModuleLoader::loadmodule(UByte* buf, ULong size)
{
  Module *ret = NULL;
  int i, p, r, n;
  unsigned char cdummy0, cdummy1, cdummy2, cdummy3;
  int index = 0;

  char name[21];
  int nchannel = 4;
  int nrow = 64;
  int ninstrument = 32;
  ModuleSample *instruments = NULL;
  int *orders = NULL;
  int norder = 0, maxorder;
  ModulePattern *patterns = NULL;
  int npattern = 0;

  char chunk[5];

  if(size<1084) goto exit_read;

  memcpy(&chunk, &buf[1080], 4);
  chunk[4]='\0';
  if (strcasecmp(chunk, "2CHN")==0)
    nchannel = 2;
  else if(strcasecmp(chunk, "M.K.")==0
    || strcasecmp(chunk, "M!K!")==0
    || strcasecmp(chunk, "4CHN")==0
    || strcasecmp(chunk, "FLT4")==0)
    nchannel = 4;
  else if (strcasecmp(chunk, "FLT6")==0
    || strcasecmp(chunk, "6CHN")==0)
    nchannel = 6;
  else if(strcasecmp(chunk, "FLT8")==0
    || strcasecmp(chunk, "8CHN")==0
    || strcasecmp(chunk, "CD81")==0
    || strcasecmp(chunk, "OKTA")==0)
    nchannel = 8;
  else if(strcasecmp(chunk, "10CH")==0)
    nchannel = 10;
  else if(strcasecmp(chunk, "12CH")==0)
    nchannel = 12;
  else if(strcasecmp(chunk, "14CH")==0)
    nchannel = 14;
  else if(strcasecmp(chunk, "16CN")==0
    || strcasecmp(chunk, "16CH")==0)
    nchannel = 16;
  else if(strcasecmp(chunk, "18CH")==0)
    nchannel = 18;
  else if(strcasecmp(chunk, "20CH")==0)
    nchannel = 20;
  else if(strcasecmp(chunk, "22CH")==0)
    nchannel = 22;
  else if(strcasecmp(chunk, "24CH")==0)
    nchannel = 24;
  else if(strcasecmp(chunk, "26CH")==0)
    nchannel = 26;
  else if(strcasecmp(chunk, "28CH")==0)
    nchannel = 28;
  else if(strcasecmp(chunk, "30CH")==0)
    nchannel = 30;
  else if(strcasecmp(chunk, "32CN")==0
   || strcasecmp(chunk, "32CH")==0)
    nchannel = 32;
  else ninstrument = 16;

  memcpy(name, &Stream, 20); addStream(20);
  name[20]='\0';

  if((instruments = new ModuleSample[ninstrument])==NULL) goto exit_read;
  for(i=1; i<ninstrument; i++)
  {
    char namei[23];

    memcpy(&namei, &Stream, 22); addStream(22);
    namei[22]='\0';
    instruments[i].Name=strdup(namei);
    
    cdummy0 = Stream; addStream(1);
    cdummy1 = Stream; addStream(1);
    instruments[i].nLength = readAmigaWord(cdummy0, cdummy1);

    cdummy0 = Stream; addStream(1);
    instruments[i].nFineTune = cdummy0;
    if(instruments[i].nFineTune>7) instruments[i].nFineTune-=16;

    cdummy0 = Stream; addStream(1);
    instruments[i].nVolume = cdummy0;

    cdummy0 = Stream; addStream(1);
    cdummy1 = Stream; addStream(1);
    instruments[i].nLoopStart = readAmigaWord(cdummy0, cdummy1);

    cdummy0 = Stream; addStream(1);
    cdummy1 = Stream; addStream(1);
    instruments[i].nLoopLength = readAmigaWord(cdummy0, cdummy1);

    instruments[i].nLoopEnd = instruments[i].nLoopStart + instruments[i].nLoopLength;
    
    if(instruments[i].nLoopEnd>instruments[i].nLength)
      instruments[i].nLoopEnd = instruments[i].nLength;

    instruments[i].Data = NULL;
  }
  cdummy0 = Stream; addStream(1);
  maxorder = cdummy0;

  cdummy0 = Stream; addStream(1);
  if(cdummy0!=127) goto exit_read;

  norder = 128;
  npattern = 0;
  if((orders = new int[norder])==NULL) goto exit_read;
  for(i=0; i<norder; i++)
  {
    cdummy0 = Stream; addStream(1);
    orders[i]=cdummy0;
    if(orders[i]>npattern) npattern = orders[i];
  }
  npattern++;

  // magic !
  cdummy0 = Stream; addStream(1);
  cdummy0 = Stream; addStream(1);
  cdummy0 = Stream; addStream(1);
  cdummy0 = Stream; addStream(1);

  if((patterns=new ModulePattern[npattern])==NULL) goto exit_read;
  for(p=0; p<npattern; p++)
  {
    if((patterns[p].rows=new ModuleRow[nrow])==NULL) goto exit_read;
    for(r=0; r<nrow; r++)
    {
      if((patterns[p].rows[r].notes=new ModuleNote[nchannel])==NULL) goto exit_read;
      for(n=0; n<nchannel; n++)
      {
        cdummy0 = Stream; addStream(1);
        cdummy1 = Stream; addStream(1);
        cdummy2 = Stream; addStream(1);
        cdummy3 = Stream; addStream(1);
    		
        
        int period= ((cdummy0 & 0x0F) << 8) | cdummy1;
        if(period)
          patterns[p].rows[r].notes[n].period_index = (int)((log(856) - log(period)) / log(1.007246412224) + 8);
        else
          patterns[p].rows[r].notes[n].period_index = -1;
        patterns[p].rows[r].notes[n].sample = (cdummy0 & 0xF0) | (cdummy2 >> 4);

        int eparmx = cdummy3 >> 4;
		    int eparmy = cdummy3 & 0xF;

        switch(cdummy2 & 0x0F)
        {
        case 0x00:
          patterns[p].rows[r].notes[n].effect = Arpeggio;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;

        case 0x01:
          patterns[p].rows[r].notes[n].effect = PortamentoUp;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x02:
          patterns[p].rows[r].notes[n].effect = PortamentoDown;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x03:
          patterns[p].rows[r].notes[n].effect = TonePortamento;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x04:
          patterns[p].rows[r].notes[n].effect = Vibrato;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x05:
          patterns[p].rows[r].notes[n].effect = TonePortamentoVolumeSlide;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x06:
          patterns[p].rows[r].notes[n].effect = VibratoVolumeSlide;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x07:
          patterns[p].rows[r].notes[n].effect = Tremolo;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x08:
          patterns[p].rows[r].notes[n].effect = SetPan;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x09:
          patterns[p].rows[r].notes[n].effect = SampleOffset;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0A:
          patterns[p].rows[r].notes[n].effect = VolumeSlide;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0B:
          patterns[p].rows[r].notes[n].effect = PositionJump;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0C:
          patterns[p].rows[r].notes[n].effect = SetVolume;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0D:
          patterns[p].rows[r].notes[n].effect = PatternBreak;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0E:
          switch (cdummy3)
          {
          case 0xE0:
            patterns[p].rows[r].notes[n].effect = SetFilter;
            break;
            
          case 0xE1:
            patterns[p].rows[r].notes[n].effect = FineTuneSlideUp;
            break;
            
          case 0xE2:
            patterns[p].rows[r].notes[n].effect = FineTuneSlideDown;
            break;
            
          case 0xE3:
            patterns[p].rows[r].notes[n].effect = GlissandoControl;
            break;
                        
          case 0xE4:
            patterns[p].rows[r].notes[n].effect = VibratoControl;
            break;
            
          case 0xE5:
            patterns[p].rows[r].notes[n].effect = SetFineTune;
            break;
            
          case 0xE6:
            patterns[p].rows[r].notes[n].effect = PatternLoop;
            break;
            
          case 0xE7:
            patterns[p].rows[r].notes[n].effect = TremoloControl;
            break;
            
          case 0xE8:
            patterns[p].rows[r].notes[n].effect = PanControl;
            break;
            
          case 0xE9:
            patterns[p].rows[r].notes[n].effect = NoteRetrig;
            break;
            
          case 0xEA:
            patterns[p].rows[r].notes[n].effect = FineTuneVolumeUp;
            break;
            
          case 0xEB:
            patterns[p].rows[r].notes[n].effect = FineTuneVolumeDown;
            break;
            
          case 0xEC:
            patterns[p].rows[r].notes[n].effect = NoteCut;
            break;
            
          case 0xED:
            patterns[p].rows[r].notes[n].effect = NoteDelay;
            break;
            
          case 0xEE:
            patterns[p].rows[r].notes[n].effect = PatternDelay;
            break;
            
          case 0xEF:
            patterns[p].rows[r].notes[n].effect = InvertLoop;
            break;
          }

          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        case 0x0F:
          patterns[p].rows[r].notes[n].effect = SetSpeed;
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
          
        default:
          patterns[p].rows[r].notes[n].effect = NoEffect; // or inknown effect
          patterns[p].rows[r].notes[n].effect_param = cdummy3;
          break;
        }
      }
    }
  }

  instruments[0].Data = NULL;
  for(i=1; i<ninstrument; i++)
  {
    unsigned char *data = NULL;
    if((data=new unsigned char[instruments[i].nLength+1])==NULL) goto exit_read;
    canAddStream(instruments[i].nLength);
    memcpy(data, &Stream, instruments[i].nLength); addStream(instruments[i].nLength);
 		int length = instruments[i].nLength;
    if(length>0)
    {
      data[length]=data[length-1];
      if(instruments[i].nLoopLength>2)
        data[instruments[i].nLoopEnd]=data[instruments[i].nLoopStart];
    }

    if((instruments[i].Data = new Wave(8000, false, false, false, instruments[i].nLength, data, ""))==NULL) goto exit_read;
  }

  ret = new Module();
  if(ret==NULL) goto exit_read;
  ret->setName(name);
  ret->setSpeed(6);
  ret->setBPM(125);
  ret->setSamples(ninstrument, instruments);
  ret->setOrders(norder, orders, maxorder);
  ret->setPatterns(nchannel, nrow, npattern, patterns);
  for(i=0; i<nchannel; i++)
    ret->setLeftRightChannelOnTrack(i, ((i&3)==0)||((i&3)==3) ? true : false);

  goto exit_ok;

exit_read:
  if(instruments) delete [] instruments;
  if(orders) delete [] orders;
  if(patterns) delete [] patterns;

exit_ok:
  return ret;
}
