/*
    Copyright (C) 1998 by Jorrit Tyberghein    

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __MODULELOADER_H
#define __MODULELOADER_H

#include <stdio.h>
#include "util/types.h"

///
class ModuleLoader  
{
public:
  /// Return the name of the wave type supported by this loader.
  virtual const char* get_name() const = 0;

  /// Return a descriptive line about this wave format.
  virtual const char* get_desc() const = 0;

  /**
   * Load an wave given the filename.
   * This routine will open the file named filename, try to recognize its
   * type, and return an Wave of the appropriate type.  Returns a
   * pointer to the Wave on success, or NULL on failure.
   */
  static Module* load (char* filename);

  /**
   * Load an module given a file pointer.
   * This routine will read from the given file, try to recognize its
   * type, and return an Module of the appropriate type.  Returns a
   * pointer to the Module on success, or NULL on failure.
   */
  static Module* load (FILE* fp);

  /**
   * Load an module from a buffer.
   * This routine will read from the buffer buf of length size, try to 
   * recognize the type of wave contained within, and return an Module
   * of the appropriate type.  Returns a pointer to the Module on 
   * success, or NULL on failure.
   */
  static Module* load (UByte* buf, ULong size);

protected:
  ///
  virtual ~ModuleLoader() {}

  /**
   * Register a loader for a given image type.
   * Adds 'loader' to the list of image formats to be checked during an
   * ModuleLoader::load(...) call.
   */
  static void reg_loader(ModuleLoader* loader);

  /**
   * Load an module from the given buffer.
   * Attempts to read an module from the buffer 'buf' of length 'size'.
   * If successful, returns a pointer to the resulting Module.  Otherwise
   * returns NULL.
   */
  virtual Module* loadmodule(UByte* buf, ULong size) = 0;

private:
  /// A static, NULL-terminated list of registered loaders.
  static csVector& loaderlist();
};

#endif // __MODULELOADER_H
