/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <time.h>

#include "system/system.h"
#include "system/sysprint.h"
#include "util/token.h"

//---------------------------------------------------------------------------

char* spaces = "                                                                                  ";

float get_token_float (char** buf)
{
  char* t = get_token (buf);
  int x=0;
  if (!((*t >= '0' && *t <= '9') || *t == '-' || *t == '.'))
  {
    CsPrintf (MSG_FATAL_ERROR, "Expected a floating point number! Got '%s' instead!\n", t);
    int a = 3/x; (void)a;
    fatal_exit (0, false);
  }
  float rc;
  sscanf (t, "%f", &rc);
  return rc;
}

int get_token_int (char** buf)
{
  char* t = get_token (buf);
  if (!((*t >= '0' && *t <= '9') || *t == '-'))
  {
    CsPrintf (MSG_FATAL_ERROR, "Expected an integer number! Got '%s' instead!\n", t);
    fatal_exit (0, false);
  }
  int rc;
  sscanf (t, "%d", &rc);
  return rc;
}

char* get_token (char** buf)
{
  static char token[100];
  char* b = *buf, * t = token;
  while (TRUE)
  {
    while (*b && (*b == ' ' || *b == '\n' || *b == 13 || *b == 10 || *b == '\t')) b++;
    if (*b == ';')
      while (*b && *b != '\n' && *b != 13 && *b != 10) b++;
    else break;
  }

  if (*b == '\'')
  {
    b++;
    while (*b && *b != '\'') { *t++ = *b++; }
    *t++ = 0;
    if (*b == '\'') b++;
  }
  else if (*b == '(' || *b == ')' || *b == '=' || *b == ',' || *b == '[' || *b == ']' ||
	*b == '%' || *b == ':' || *b == '{' || *b == '}')
  {
    *t++ = *b++;
    *t++ = 0;
  }
  else if (*b == '.' && (*(b+1) < '0' || *(b+1) > '9'))
  {
    *t++ = *b++;
    *t++ = 0;
  }
  else if ((*b >= '0' && *b <= '9') || *b == '.' || *b == '-')
  {
    while (*b && ((*b >= '0' && *b <= '9') || *b == '.' || *b == '-')) { *t++ = *b++; }
    *t++ = 0;
  }
  else if ((*b >= 'A' && *b <= 'Z') || (*b >= 'a' && *b <= 'z') || *b == '_')
  {
    while (*b && ((*b >= 'A' && *b <= 'Z') || (*b >= 'a' && *b <= 'z') || *b == '_')) { *t++ = *b++; }
    *t++ = 0;
  }
  else if (*b == 0)
  {
    *buf = b;
    return NULL;
  }
  else CsPrintf (MSG_FATAL_ERROR, "Que (%c%c%c%c%c...)?\n", *b, *(b+1), *(b+2), *(b+3), *(b+4));

  *buf = b;
  return token;
}

#ifdef COMP_WCC
void skip_token (char** buf, char* tok){skip_token (buf, tok, NULL);}
void skip_token (char** buf, char* tok, char* msg)
#else
void skip_token (char** buf, char* tok, char* msg)
#endif
{
  char* t = get_token (buf);
  if (strcmp (t, tok))
    if (msg) CsPrintf (MSG_FATAL_ERROR, msg, tok, t);
    else CsPrintf (MSG_FATAL_ERROR, "Expected '%s' instead of '%s'!\n", tok, t);
}

//---------------------------------------------------------------------------

