/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <ctype.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "util/def.h"

#ifdef COMP_BC
#include <dir.h>    // For getcwd()
#elif defined(COMP_VC)
#define getcwd _getcwd
#include <direct.h> // For _getcwd()
#endif
#ifdef OS_UNIX
#include <unistd.h>
#endif

#include "util/types.h"
#include "system/system.h"
#include "system/sysdriv.h"
#include "system/sysprint.h"
#include "util/util.h"

// return a random number from minRange to maxRange inclusive
long RndNum(long minRange, long maxRange)
{
  if (minRange > maxRange)
  {
    long sw = minRange;
    minRange = maxRange;
    maxRange = sw;
  }
  long range = maxRange - minRange + 1;
  return minRange + (long (rand ()) % range);
}

int ScanStr (char* in, char* format, ...)
{
  va_list arg;
  va_start (arg, format);

  int num = 0;
  in += strspn (in, " \t\n\f");

  while (*format)
  {
    if (*format == '%')
    {
      format++;
      switch (*format)
      {
        case 'd':
	{
	  int* a = va_arg (arg, int*);
	  in += strspn (in, " \t\n\f");
	  *a = atoi (in);
	  in += strspn (in, "0123456789+- \t\n\f");
	  num++;
	  break;
	}
	case 'D':
	{
	  int i;
	  int* list = va_arg (arg, int*);
	  int* nr = va_arg (arg, int*);
	  in += strspn (in, " \t\n\f");
	  i = 0;
	  while ((*in >= '0' && *in <= '9') || *in == '+' || *in == '-')
	  {
	    list[i++] = atoi (in);
	    in += strspn (in, "0123456789+-");
	    in += strspn (in, " \t\n\f");
	    if (*in != ',') break;
	    in++;
	    in += strspn (in, " \t\n\f");
	  }
	  *nr = i;
	  num++;
	  break;
	}
        case 'b':
	{
	  int* a = va_arg (arg, int*);
	  in += strspn (in, " \t\n\f");
	  char* in2 = in + strspn (in, "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ");
	  int l = (int)(in2-in);
	  *a = !strncasecmp (in, "yes", l) || !strncasecmp (in, "true", l) || !strncasecmp (in, "on", l) ||
	  	!strncasecmp (in, "1", l);
	  in = in2 + strspn (in2, " \t\n\f");
	  num++;
	  break;
	}
	case 'f':
	{
	  float* a = va_arg (arg, float*);
	  in += strspn (in, " \t\n\f");
	  *a = atof (in);
	  in += strspn (in, "0123456789.eE+- \t\n\f");
	  num++;
	  break;
	}
	case 'F':
	{
	  int i;
	  float* list = va_arg (arg, float*);
	  int* nr = va_arg (arg, int*);
	  in += strspn (in, " \t\n\f");
	  i = 0;
	  while ((*in >= '0' && *in <= '9') || *in == '.' || *in == '+' || *in == '-' || *in == 'e' || *in == 'E')
	  {
	    list[i++] = atof (in);
	    in += strspn (in, "0123456789.eE+-");
	    in += strspn (in, " \t\n\f");
	    if (*in != ',') break;
	    in++;
	    in += strspn (in, " \t\n\f");
	  }
	  *nr = i;
	  num++;
	  break;
	}
	case 's':
	{
	  char* a = va_arg (arg, char*);
	  in += strspn (in, " \t\n\f");
	  if (*in == '\'')
	  {
	    in++;
	    char* in2 = strchr (in, '\'');
	    strncpy (a, in, (int)(in2-in));
	    a[(int)(in2-in)] = 0;
	    in = in2+1;
	  }
	  else
	  {
	    char* in2 = in + strspn (in, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ_0123456789");
	    strncpy (a, in, (int)(in2-in));
	    a[(int)(in2-in)] = 0;
	    in = in2;
	  }
	  in += strspn (in, " \t\n\f");
	  num++;
	  break;
	}
      }
      if (*format) format++;
    }
    else if (*format == ' ') { format++; in += strspn (in, " \t\n\f"); }
    else if (*in == *format) { in++; format++; }
    else { num = -1; break; }
  }

  va_end (arg);

  return num;
}

static bool RecursiveCombinations (int *vector, int top, int mask, int m, int n,
  bool (*callback) (int *vector, int count, void *arg), void *arg)
{
  for (int i = 0; i < m; i++)
  {
    if (mask & (1 << i))
      continue;
    vector [top] = i;
    if (top + 1 >= n)
      if (callback (vector, n, arg))
        return true;
      else
        ;
    else if (RecursiveCombinations (vector, top + 1, mask | (1 << i),
                                    m, n, callback, arg))
      return true;
  } /* endfor */
  return false;
}

void Combinations (int m, int n, bool (*callback) (int *vector, int count,
  void *arg), void *arg)
{
  int *vector = new int [m];
  RecursiveCombinations (vector, 0, 0, m, n, callback, arg);
  delete [] vector;
}

char *expandname (char *iName)
{
  char outname [MAXPATHLEN + 1];
  int inp = 0, outp = 0, namelen = strlen (iName);
  while ((outp < MAXPATHLEN)
      && (inp < namelen))
  {
    char tmp [MAXPATHLEN + 1];
    int ptmp = 0;
    while ((inp < namelen)
        && (iName [inp] != '/')
        && (iName [inp] != PATH_SEPARATOR)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
        && (iName [inp] != ':')
#endif
          )
      tmp [ptmp++] = iName [inp++];
    tmp [ptmp] = 0;

    if ((ptmp > 0)
     && (outp == 0)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
     && ((inp >= namelen)
      || (iName [inp] != ':'))
#endif
        )
    {
      getcwd  (outname, sizeof (outname));
      outp = strlen (outname);
      if (strcmp (tmp, "."))
        outname [outp++] = PATH_SEPARATOR;
    } /* endif */

#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
    // If path starts with '/' (root), get current drive
    if ((ptmp == 0)
     && (outp == 0))
    {
      getcwd (outname, sizeof (outname));
      if (outname [1] == ':')
        outp = 2;
    } /* endif */
#endif

    if (strcmp (tmp, "..") == 0)
    {
      while ((outp > 0)
          && ((outname [outp - 1] == '/')
           || (outname [outp - 1] == PATH_SEPARATOR)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
           || (outname [outp - 1] == ':')
#endif
             )
            )
        outp--;
      while ((outp > 0)
          && (outname [outp - 1] != '/')
          && (outname [outp - 1] != PATH_SEPARATOR)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
          && (outname [outp - 1] != ':')
#endif
            )
        outp--;
    }
    else if (strcmp (tmp, ".") == 0)
    {
      // do nothing
    }
    else if (strcmp (tmp, "~") == 0)
    {
      // strip all output path; start from scratch
      strcpy (outname, "~/");
      outp = 2;
    }
    else
    {
      int sl = strlen (tmp);
      memcpy (&outname [outp], tmp, sl);
      outp += sl;
      if (inp < namelen)
      {
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
        if (iName [inp] == ':')
          outname [outp++] = ':';
#endif
        outname [outp++] = PATH_SEPARATOR;
      }
    } /* endif */
    while ((inp < namelen)
        && ((iName [inp] == '/')
         || (iName [inp] == PATH_SEPARATOR)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
         || (iName [inp] == ':')
#endif
           )
          )
      inp++;
  } /* endwhile */

  char *ret = new char [outp + 1];
  memcpy (ret, outname, outp);
  ret [outp] = 0;
  return ret;
}

void splitpath (char *iPathName, char *iPath, size_t iPathSize, char *iName,
  size_t iNameSize)
{
  size_t sl = strlen (iPathName);
  size_t maxl = sl;
  while (sl
      && (iPathName [sl - 1] != '/')
      && (iPathName [sl - 1] != PATH_SEPARATOR)
#if defined (OS_OS2) || defined (OS_DOS) || defined (OS_WIN32)
      && (iPathName [sl - 1] != ':')
#endif
        )
    sl--;

  if (sl >= iPathSize)
  {
    memcpy (iPath, iPathName, iPathSize - 1);
    iPath [iPathSize - 1] = 0;
  }
  else
  {
    memcpy (iPath, iPathName, sl);
    iPath [sl] = 0;
  }

  if (maxl - sl >= iNameSize)
  {
    memcpy (iName, &iPathName [sl], iNameSize - 1);
    iName [iNameSize - 1] = 0;
  }
  else
    memcpy (iName, &iPathName [sl], maxl - sl + 1);
}
