/*
    Copyright (C) 1998 by Jorrit Tyberghein
  
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.
  
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
  
    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef UTIL_H
#define UTIL_H

#include <stdio.h>

#if defined(COMP_WCC) && defined (OS_WIN32)
_WCRTLINK extern "C" char *           getcwd( char *__buf, size_t __size );
#endif

/// Return a random number.
long RndNum(long minRange, long maxRange);

/**
 * Own version of sscanf that is a bit more relaxed towards spaces
 * and also accepts quoted strings (the quotes will not be included into
 * the result string).<p>
 * It supports the following format commands:
 * <ul>
 * <li>	%d	integer number
 * <li>	%f	floating point
 * <li>	%b	boolean (0, 1, true, false, yes, no, on, off)
 * <li>	%s	string (with or without single quotes)
 * <li>	%D	list of integers, first argument should be a
 *		pointer to an array of integers, second argument
 *		a pointer to an integer which will contain the
 *		number of elements inserted in the list.
 * <li>	%F	similarly, a list of floats.
 * </ul>
 * Returns the number of successfully scanned arguments or -1 if there
 * was a mismatch.
 */
int ScanStr (char* in, char* format, ...);

/**
 * Compute all combinations of m by n. Calls the callback function with
 * a vector of numbers from 0 to m - 1; numbers never appears twice in vector.
 * Callback returns false for continuation and true to break the loop.
 */
extern void Combinations (int m, int n, bool (*callback) (int *vector, int count,
  void *arg), void *arg);

/**
 * Expand a filename if it contains shortcuts ('.', '~', '..' etc)<p>
 * Return a newly-allocated with new [] "char *".
 */
extern char *expandname (char *iName);
/**
 * Split a pathname into separate path and name. Path delimiters are either
 * '/', PATH_SEPARATOR and, for OS/2, MS-DOS and Win32 targets, ':'.
 */
extern void splitpath (char *iPathName, char *iPath, size_t iPathSize,
  char *iName, size_t iNameSize);

/// Swap two integer variables
inline void iSwap (int &a, int &b)
{
  int tmp = a;
  a = b; b = tmp;
}

/// Swap two floating-point variables
inline void fSwap (float &a, float &b)
{
  float tmp = a;
  a = b; b = tmp;
}

/// Return the argument squared
inline float fSquare (float x)
{
  return x*x;
}

#endif

