#include <iostream.h>
#include <string.h>

#include <qmsgbox.h>

#include <kapp.h>
#include <klocale.h>
#include <kwm.h>

#include "sessiondata.h"
#include "mywidget.h"
#include "panelwindow.h"
#include "kcmdlineargs.h"



// must be defined here, otherwise a get a core dump on program exit
KApplication* a;


int main (int paramAnz, char ** paramStr)
  {
    a = new KApplication (paramAnz, paramStr);

    // load the sessions
    SessionManager* theSessions = new SessionManager ();

    // check the commandline
    KCmdLineArgs* cmdline = new KCmdLineArgs (paramAnz, paramStr);

    if (cmdline->isArgumentPresent ("--help") == TRUE)
      {
	cout << endl;
	cout << "Available commandline options:" << endl << endl;
	cout << "--help     :  show this text" << endl;
	cout << "--panel    :  starts ktelnet on panel" << endl;
	cout << "--session  :  start the following session" << endl;
	cout << endl;
	
	return (0);
      }

    // start a selected session
    QString session = cmdline->getArgument ("--session");
    if (session.isNull() == FALSE)
      {
        theSessions->newSessionSelected ( (const char*)session );
	if (session != theSessions->session ())
	  {
	    cout << "Session not available. Exit." << endl;
	    return (0);
	  }
	
	cout << "Start session : " << (const char*)theSessions->session() 
	     << endl;
	theSessions->startSession ();
	return (0);
      }

    // start on panel?
    bool onPanel = cmdline->isArgumentPresent ("--panel");

    // ok, finished, delete the cmdline object
    delete cmdline;

    // start on panel?
    KConfig* k = a->getConfig ();
    k->setGroup ("common");
    if (k->readEntry ("startOnPanel", "FALSE") == QString("TRUE")) 
      onPanel = TRUE;

    // now choose the wanted mainwidget (on panel or real window)
    if (onPanel == FALSE)
      {
	MyWidget* w;
	w = new MyWidget (theSessions);

	a->setMainWidget (w);
	w->show ();
      }
    else
      {
	PanelWindow* p;
	p = new PanelWindow (theSessions);
	KWM::setDockWindow (p->winId ());
	
	a->setMainWidget (p);
	p->show ();
      }

    int result = a->exec ();

    // ckeck if options need saving
    k->setGroup ("common");
    if (k->readEntry ("autosaveOptions", "FALSE") == QString ("TRUE") ) 
      k->sync ();      
    
    // check if sessions need saving
    if (theSessions->needSessionSave() == TRUE) 
      {
	// autosaving, nice
	if (k->readEntry("autosaveSessions", "FALSE") == QString("TRUE"))
	  theSessions->saveSessions ();
	else
	  {
	    int res = QMessageBox::warning (0, i18n("KTelnet Warning"), 
	  i18n("Your sessions have changed but are not saved yet.\nSave now?"),
					    i18n("Yes"), i18n("No"), 
					    0, 0, 0); 	
	    // ok, save
	    if (res == 0) theSessions->saveSessions ();
	  }
      }

    return (result);
  }
