#include <iostream.h>

#include <qpushbt.h>
#include <qlined.h>
#include <qcombo.h>
#include <qtooltip.h>
#include <qmsgbox.h>

#include <kapp.h>
#include <kiconloader.h>
#include <klocale.h>

#include "mainwidget.h"
#include "mywidget.h"

#include "mainwidget.moc"




mainWidget::mainWidget (SessionManager* sess, QWidget* parent, const char* name)
  : QWidget (parent, name)
{
  int i;

  sessions = sess;

  // **************************  some labels  *******************************
  
  schrift = new QMLabel ("KTelnet", this);
  schrift->setFont (QFont ("Times" ,18, QFont::Bold));
  schrift->resize (120, 25);
  
  version = new QMLabel (VERSION, this);
  sessionName = new QMLabel (i18n("Session:"), this);
  accountname = new QMLabel (i18n("Account:"), this);
  hostname = new QMLabel (i18n("Host:"), this);

  //CT 8jan98 new items to eliminate newsession widget
  optionsLabel = new QMLabel (i18n("Further Options:"), this);
  
  // ***********************  the ComboBox  *********************************

  //CT 8jan98 read-write combo
  profileName = new QMComboBox (this, "profileName"); 

  // now fill the ComboBox
  for (i=0; i>-1; i++)
    {
      if (sessions->session(i) != 0)
	profileName->insertItem ((const char*)sessions->session(i));
      else break;
    }
  
  connect (profileName, SIGNAL (activated (int)), SLOT (changeLines ()));
  connect (profileName, SIGNAL (sigRightClick ()), 
				SLOT (changeProfileName()));
  
  QToolTip::add (profileName, 
	  i18n("Select session, right\nmousebutton to change name"));

  // ***********  if calling a new session a lineedit instead  **************

  newSessionLined = new QMLineEdit (this);
  QToolTip::add (newSessionLined, i18n("Enter new session name"));
  newSessionLined->setGeometry (90, 15, 150, 25);
  newSessionLined->hide ();
  
  // ***********************  the LineEdits  ********************************
  
  host = new QLineEdit (this, "host");
  host->setMaxLength (49);
  connect (host, SIGNAL (textChanged (const char*)), sessions, 
	   SLOT (hostHasChanged (const char*)) );
  QToolTip::add (host, i18n("Enter remote host"));
  
  account = new QLineEdit (this, "account");
  account->setMaxLength (29);
  connect (account, SIGNAL (textChanged (const char*)), 
	   sessions, SLOT (accountHasChanged (const char*)) );
  QToolTip::add (account, i18n("Enter account to log in"));

  // ********************** some more buttons *******************************

  lb1 = new QMLabel (i18n("Terminal type:"), this);
  lb2 = new QMLabel (i18n("Connect/Port:"), this);
  lb3 = new QMLabel (i18n("Textsize:"), this);
  lb4 = new QMLabel (i18n("Background Color:"), this);
  lb5 = new QMLabel (i18n("Foreground Color:"), this);

  //CT 8jan98 new button labels to avoid newsession widget
  lb6 = new QMLabel (i18n("Toolbar Icon:"), this);

  terminal = new QComboBox (this);
  terminal->insertStrList ( &(sessions->theTerminals()) );

  QToolTip::add (terminal, i18n("Change Terminal"));

  // *************************************************************************

  // the textsize widget
  textsize = new QComboBox (this);
  textsize->insertItem (i18n("Tiny - 6x13"));
  textsize->insertItem (i18n("Small - 7x14"));
  textsize->insertItem (i18n("Medium - 8x13"));
  textsize->insertItem (i18n("Large - 9x15"));
  textsize->insertItem (i18n("Huge - 10x20"));
  QToolTip::add (textsize, i18n("Change terminal textsize"));

  // the connection program
  connector = new QComboBox (this);
  connector->insertItem ("Telnet");
  connector->insertItem ("rLogin");
  connector->insertItem ("Secure Shell");
  QToolTip::add (connector, i18n("Change connection program"));

  // the portnumber for remote machine
  port = new KIntegerLine (this);
  QToolTip::add (port, i18n ("Set port number, Telnet default=23\nSecure Shell default=22\nSet to \"0\" if not shure"));

  bgcolorbutton = new KColorButton (this, "Change Background Color");
  QToolTip::add (bgcolorbutton, 
		 i18n("Click to change background color"));
  fgcolorbutton = new KColorButton (this, "Change Foreground Color");
  QToolTip::add (fgcolorbutton, 
		 i18n("Click to change foreground color"));

  //CT 8jan98 new lineEdits to avoid newsession widget
  options = new QLineEdit (this, "options");
  connect (options, SIGNAL (textChanged (const char*)), sessions,
	   SLOT (cmdlineHasChanged (const char *)) );
  QToolTip::add (options, 
		 i18n("Enter additional terminal options here"));

  //CT 8jan98 new buttons to avoid newsession widget
  toolbariconbutton = new KIconLoaderButton (this);
  QToolTip::add (toolbariconbutton, "Click to change icon");
  connect (toolbariconbutton, SIGNAL (iconChanged(const char*)), 
	   SLOT (changeIcon(const char*)));

  addtotoolbar = new QPushButton(i18n("Add to toolbar"), 
				 this, "Add to toolbar");
  QToolTip::add (addtotoolbar, 
		 i18n("Click to add/remove\nsession to toolbar"));
  connect (addtotoolbar, SIGNAL (clicked()), SLOT (slotToolbar ()));

  savenewbutton = new QPushButton (i18n("Accept"), this, "accept");
  QToolTip::add (savenewbutton, 
		 i18n("Accept the new session"));
  connect (savenewbutton, SIGNAL (clicked ()), parent, 
	   SLOT(appendNewSession()) );
  savenewbutton->hide ();

  restoreoldbutton = new QPushButton(i18n("Cancel"), this, 
				     "Cancel");
  QToolTip::add (restoreoldbutton, 
		 i18n("Cancel entering a new session"));
  connect (restoreoldbutton, SIGNAL(clicked()), parent, 
	   SLOT(restoreSession ()));
  restoreoldbutton->hide ();

  // **********************  the connect button  *****************************
  
  Connect = new QPushButton (i18n("Connect"), this, "Connect");
  connect (Connect, SIGNAL (clicked ()), sessions, SLOT (startSession ()) ); 
  Connect->setDefault (TRUE);
  QToolTip::add (Connect, i18n("Start the remote session"));

  // ******************** resize the widget over buttons ********************
  
  buttonUp = new QPushButton (this);
  buttonUp->setPixmap (KApplication::getKApplication()->getIconLoader()->
		       loadIcon("smallup.xpm"));

  buttonDown = new QPushButton (this);
  buttonDown->setPixmap (KApplication::getKApplication()->getIconLoader()->
			 loadIcon("smalldown.xpm"));

  QToolTip::add (buttonUp, i18n("Lower Window Size"));
  QToolTip::add (buttonDown, i18n("Raise Window Size"));

  // ***************************** some focusPolicy *************************

  connect (this, SIGNAL (showOptionsPopup(QPoint)), parent, 
	   SLOT (showOptionsPopup(QPoint)));
  connect (this, SIGNAL (showFilePopup(QPoint)), parent, 
	   SLOT (showFilePopup(QPoint)));

  connect (buttonUp, SIGNAL (clicked ()), SLOT (reduceSize ()));
  connect (buttonDown, SIGNAL (clicked ()), SLOT (increaseSize ()));

  connect (terminal, SIGNAL (activated (const char*)), sessions, 
	   SLOT (terminalHasChanged(const char*)));
  connect (textsize, SIGNAL (activated (int)),
	   SLOT (textSizeHasChanged(int)));
  connect (connector, SIGNAL (activated (int)),
	   SLOT (connectorHasChanged(int)));
  connect (connector, SIGNAL (activated (int)), 
	   SLOT (checkConnectorForPort (int)));

  connect (fgcolorbutton, SIGNAL (changed (const QColor&)), sessions, 
	   SLOT (fgcolorHasChanged (const QColor&)));
  connect (bgcolorbutton, SIGNAL (changed (const QColor&)), sessions, 
	   SLOT (bgcolorHasChanged (const QColor&)));

  connect (port, SIGNAL (textChanged (const char*)), sessions, 
	   SLOT (portHasChanged (const char*)));

  connect (this, SIGNAL (updateRects ()), parent, SLOT (updateRects ()));

  connect (this, SIGNAL (addToToolbar ()), parent, SLOT (addToToolbar ()));
  connect (this, SIGNAL (removeFromToolbar ()), parent, 
	   SLOT (removeFromToolbar ()));
  
  connect (this, SIGNAL (deactivateSLOTS ()),parent, SLOT(deactivateSLOTS ()));
  connect (this, SIGNAL (activateSLOTS ()), parent, SLOT (activateSLOTS ()));

  connect (this, SIGNAL (textsizeHasChanged (const char*)), sessions,
	   SLOT (textsizeHasChanged (const char*)));

  connect (this, SIGNAL (connectorHasChanged (const char*)), sessions, 
	   SLOT (connectorHasChanged (const char*)));

  connect (sessions, SIGNAL (appendedNewSession (int)), 
	   SLOT (gotNewSession(int)));

  connect (sessions, SIGNAL (deletedSession(const char*)), 
	   SLOT (SessionDeleted (const char*)));

  setFixedSize (375, 125);

  Connect->setGeometry (270, 15, 80, 25);
  schrift->move (268, 85);
  version->move (340, 84);
  sessionName->move (20, 12);
  accountname->move (20, 82);
  hostname->move (20, 47);
  host->setGeometry (90, 50, 150, 25);
  account->setGeometry (90, 85, 150, 25);
  profileName->setGeometry (90, 15, 150, 25);    // vorher alles y -= 5
  //CT 8jan98
  optionsLabel->setGeometry (20, 294, 157, 25);
  options->setGeometry (150, 294, 200, 25);

  lb1->setGeometry (20, 119, 130, 25);
  lb3->setGeometry (20, 154, 130, 25);
  lb2->setGeometry (20, 189, 130, 25);
  lb4->setGeometry (20, 224, 150, 25);
  lb5->setGeometry (20, 259, 150, 25);
  //CT 8jan98
  lb6->setGeometry (20, 329, 150, 25);

  terminal->setGeometry (120, 120, 120, 25);
  textsize->setGeometry (120, 155, 120, 25);
  connector->setGeometry (120, 190, 120, 25);
  port->setGeometry (270, 190, 80, 25);
  fgcolorbutton->setGeometry (160, 260, 80, 25);
  bgcolorbutton->setGeometry (160, 225, 80, 25);

  buttonUp->setGeometry (360, 14, 13, 13);
  buttonDown->setGeometry (360, 27, 13, 13);

  //CT 8jan98
  toolbariconbutton->setGeometry (150, 329, 35, 35);

  //CT 8jan98
  addtotoolbar->setGeometry (210, 334, 110, 25);
  savenewbutton->setGeometry (270, 120, 80, 25);
  restoreoldbutton->setGeometry (270, 155, 80, 25 );

  // some keyboard accelerators
  acc = new QAccel (this);
  acc_updown = new QAccel (this);
  acc->connectItem (acc->insertItem (Key_PageUp), this, SLOT (reduceSize()));
  acc->connectItem (acc->insertItem (Key_PageDown), this,  
		    SLOT (increaseSize()));

  // these accelerators can not be called when changing the session name
  acc_updown->connectItem (acc_updown->insertItem (Key_Return), sessions, 
			   SLOT (startSession()));
  acc_updown->connectItem (acc_updown->insertItem (Key_Up), this, 
			   SLOT (changeUp ()));
  acc_updown->connectItem (acc_updown->insertItem (Key_Down), this, 
			   SLOT (changeDown()));
  acc_updown->connectItem (acc_updown->insertItem (ALT+Key_S), this, 
			   SLOT (changeProfileName ()));

  // this is not really needed, but initialized variables are always nice.
  prev.status = 1;
  prev.session = sess->current();
}


mainWidget::~mainWidget ()
{
  delete Connect;
  delete schrift;
  delete version;
  delete sessionName;
  delete accountname;
  delete hostname;
  delete host;
  delete account;
  delete profileName;
  delete port;
  delete newSessionLined;
  delete lb1;
  delete lb2;
  delete lb3;
  delete lb4;
  delete lb5;
  delete buttonUp;
  delete buttonDown;
  //CT 9jan98
  delete lb6;
  delete addtotoolbar;
  delete toolbariconbutton;
  delete savenewbutton;
  delete restoreoldbutton;
}


void mainWidget::SessionDeleted (const char* the_session)
{
  int i;

  for (i=0; i<profileName->count(); i++)
    {
      if (strcmp (profileName->text (i), the_session) == 0)
	{
	  profileName->removeItem (i);
	  profileName->setCurrentItem (sessions->current());
	  changeLines ();
	  return;
	}
    }
}


void mainWidget::connectorHasChanged (int i)
{
  switch (i)
    {
    case 0 : emit (connectorHasChanged ("telnet"));
      break;
    case 1 : emit (connectorHasChanged ("rlogin"));
      break;
    case 2 : emit (connectorHasChanged ("ssh"));
    }
}


void mainWidget::textSizeHasChanged (int i)
{
  switch (i)
    {
    case 0 : emit (textsizeHasChanged ("6x12"));
      break;
    case 1 : emit (textsizeHasChanged ("7x14"));
      break;
    case 2 : emit (textsizeHasChanged ("8x13"));
      break;
    case 3 : emit (textsizeHasChanged ("9x15"));
      break;
    case 4 : emit (textsizeHasChanged ("10x20"));
    }
}



// after new session is edited, the widget has to be restored...

void mainWidget::restoreSession ()
{
  newSessionLined->hide ();
  profileName->show ();
  restoreoldbutton->hide ();
  savenewbutton->hide ();

  Connect->setEnabled (TRUE);
  
  connect (toolbariconbutton, SIGNAL (iconChanged(const char*)),
	   SLOT (changeIcon(const char*)));

  connect (this, SIGNAL (addToToolbar()), parent(),SLOT (addToToolbar ()));
  connect (this, SIGNAL (removeFromToolbar()), parent(),
	      SLOT (removeFromToolbar ()));

  // re enable the resize buttons
  buttonUp->setEnabled (TRUE);
  buttonDown->setEnabled (TRUE);

  if (prev.status == 1) showStandard();
  else if (prev.status == 2) showFull();
  else showMinimum();

  profileName->setCurrentItem (prev.session);
  
  changeLines();
} 


// this is called by parents function newSessionSelect to edit a new session

void mainWidget::editNewSessionSelected ()
{
  if(height()==125) prev.status = 1;
  else 
    if (height()>125) prev.status = 2;
    else prev.status = 0;
  prev.session = sessions->current();
  
  showFull();
  
  profileName->hide ();
  newSessionLined->setText("");
  newSessionLined->show ();
  restoreoldbutton->show ();
  savenewbutton->show ();

  Connect->setEnabled (FALSE);

  disconnect (toolbariconbutton, SIGNAL (iconChanged(const char*)), this,
	      SLOT (changeIcon(const char*)));

  disconnect (this, SIGNAL (addToToolbar()), parent(),SLOT (addToToolbar ()));
  disconnect (this, SIGNAL (removeFromToolbar()), parent(),
	      SLOT (removeFromToolbar()));

  // disable the resize buttons
  buttonUp->setEnabled (FALSE);
  buttonDown->setEnabled (FALSE);

  options->setText( "" );
  terminal->setCurrentItem(0);
  textsize->setCurrentItem (3);
  connector->setCurrentItem (0);      // telnet means ... (next line)
  port->setEnabled(TRUE);
  port->setText ("");
  addtotoolbar->setText( i18n("Add to toolbar"));
  
  host->setText( "" );
  account->setText( "" );
  fgcolorbutton->setColor (QColor(0, 0, 0));
  bgcolorbutton->setColor (QColor (255, 255, 255));
  toolbariconbutton->setIcon ("session1.xpm");
     
  newSessionLined->setFocus();
}


// **************** some slots for changing profile name *********************


void mainWidget::changeProfileName ()
  {
    emit (deactivateSLOTS ());

    profileName->hide ();
    QToolTip::add (newSessionLined, 
	  i18n("Enter new session name.\nPress RETURN to finish."));
    newSessionLined->show ();
    newSessionLined->setFocus ();
    newSessionLined->setText (sessions->session());
    newSessionLined->selectAll ();
    connect (newSessionLined, SIGNAL (returnPressed()), 
	     SLOT (profileNameChanged()));
    connect (newSessionLined, SIGNAL (gotFocusOutEvent()), 
	     SLOT (profileNameChanged ()));
    
    // disconnect the keyboard accelerators
    acc_updown->setEnabled (FALSE);
  }


void mainWidget::profileNameChanged ()
{
  emit (activateSLOTS ());

  newSessionLined->hide ();
  QToolTip::add (newSessionLined, i18n("Enter new session name"));
  profileName->show ();

  // change everything, but only if needed
  if (strcmp (sessions->session(), newSessionLined->text()) != 0)
    {
      sessions->sessionNameHasChanged ( newSessionLined->text() );

      profileName->changeItem (sessions->session(), 
			       profileName->currentItem ());
      
      if (sessions->addToToolbar() == TRUE)
	{
	  emit (addToToolbar ());
	}
    }
  
  // disconnect the signals/slots
  disconnect (newSessionLined, SIGNAL (returnPressed()), this, 
	      SLOT (profileNameChanged()));  
  disconnect (newSessionLined, SIGNAL (gotFocusOutEvent()), this,
	      SLOT (profileNameChanged ()));

  // reconnect the keyboard accelerators
  acc_updown->setEnabled (TRUE);
}


// ****************************************************************************


void mainWidget::checkConnectorForPort (int)
{
  if (strcmp(connector->currentText(), "rLogin") == 0) 
    port->setEnabled (FALSE);
  else port->setEnabled (TRUE);
}


void mainWidget::changeIcon (const char* newIcon)
{
  sessions->iconHasChanged ( newIcon );

  // check if also needs to be changed in toolbar
  if (sessions->addToToolbar() == TRUE)
    {
      emit(addToToolbar ());      
    }
}


void mainWidget::changeLines ()          // another session is desired
  {
    sessions->rejectChangingSignals (TRUE);
    
    sessions->newSessionSelected ( profileName->currentText () );

    account->setText (sessions->account());
    host->setText (sessions->host());

    // reset the terminal type
    terminal->setCurrentItem (sessions->theTerminals().find(sessions->terminal()));
    
    // reset the connection program
    if (strcmp("telnet", sessions->connector()) == 0) 
      connector->setCurrentItem (0);
    if (strcmp("rlogin", sessions->connector()) == 0) 
      {
	connector->setCurrentItem (1);
	port->setEnabled (FALSE);
      }
    else port->setEnabled (TRUE);
    if (strcmp("ssh", sessions->connector()) == 0) 
      connector->setCurrentItem (2);
    
    // reset the fontsize
    if (strcmp("6x12", sessions->textsize()) == 0) 
      textsize->setCurrentItem (0);
    if (strcmp("7x14", sessions->textsize()) == 0) 
      textsize->setCurrentItem (1);
    if (strcmp("8x13", sessions->textsize()) == 0) 
      textsize->setCurrentItem (2);
    if (strcmp("9x15", sessions->textsize()) == 0) 
      textsize->setCurrentItem (3);
    if (strcmp("10x20", sessions->textsize()) == 0) 
      textsize->setCurrentItem (4);

    // the port number
    port->setValue(sessions->port());
    
    // set the colors
    fgcolorbutton->setColor (sessions->fgColor());
    bgcolorbutton->setColor (sessions->bgColor());

    // the options line
    options->setText (sessions->cmdLine());

    // set the icon
    toolbariconbutton->setIcon(sessions->icon());

    if (sessions->addToToolbar() == TRUE)
      addtotoolbar->setText (i18n("Remove"));
    else addtotoolbar->setText (i18n("Add to toolbar"));
    
    sessions->rejectChangingSignals (FALSE);
  }


void mainWidget::gotNewSession (int index)
{
  profileName->insertItem (sessions->session(index));
}


//
// if the fontsize is changed this font should not be changed at all
//

void mainWidget::paintEvent (QPaintEvent*)
  {
    if (schrift->font() == QFont ("Times" ,18, QFont::Bold));
    else schrift->setFont (QFont ("Times" ,18, QFont::Bold));
  }  


void mainWidget::mousePressEvent (QMouseEvent* e)
{
  if (e->button () == RightButton) emit ( showOptionsPopup (e->pos()) );
  if (e->button () == LeftButton) emit ( showFilePopup (e->pos ()) );
}


void mainWidget::increaseSize ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");

  if (height() == 125) 
    {
      showFull ();
      k->writeEntry ("windowSize", "full");
    }
  else if (height() == 50) 
    {
      showStandard ();
      k->writeEntry ("windowSize", "standard");
    }
}


void mainWidget::reduceSize ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");

  if (height() == 125) 
    {
      showMinimum () ;
      k->writeEntry ("windowSize", "minimum");
    }
  else if (height() == 375) 
    {
      showStandard ();
      k->writeEntry ("windowSize", "standard");
    }
}


// slots for the keyboard accelerator

void mainWidget::changeUp ()
{
  if (profileName->currentItem() > 0)
    {
      profileName->setCurrentItem (profileName->currentItem()-1);
      changeLines ();
    }
}


void mainWidget::changeDown ()
{
  if (profileName->currentItem() < profileName->count()-1) 
    {
      profileName->setCurrentItem (profileName->currentItem()+1);
      changeLines ();
    }
}


// ****************************************************************************


void mainWidget::showMinimum ()
{
  setFixedSize (375, 50);

  Connect->setGeometry (270, 15, 80, 25);
  sessionName->move (20, 12);
  profileName->setGeometry (90, 15, 150, 25);

  hostname->hide ();
  host->hide ();

  host->setEnabled (FALSE);
  account->setEnabled (FALSE);
  terminal->setEnabled (FALSE);
  connector->setEnabled (FALSE);
  textsize->setEnabled (FALSE);
  port->setEnabled (FALSE);
  bgcolorbutton->setEnabled (FALSE);
  fgcolorbutton->setEnabled (FALSE);
  toolbariconbutton->setEnabled (FALSE);
  addtotoolbar->setEnabled (FALSE);
  options->setEnabled (FALSE);

  buttonUp->setEnabled (FALSE);
  buttonDown->setEnabled (TRUE);

  emit (updateRects());
}


void mainWidget::showStandard ()
{
  setFixedSize (375, 125);

  // push them back to a nice site
  Connect->setGeometry (270, 15, 80, 25);
  hostname->show ();
  host->show ();
  sessionName->move (20, 12);
  profileName->setGeometry (90, 15, 150, 25);    // vorher alles y -= 5

  host->setEnabled (TRUE);
  account->setEnabled (TRUE);

  terminal->setEnabled (FALSE);
  connector->setEnabled (FALSE);
  textsize->setEnabled (FALSE);
  port->setEnabled (FALSE);
  bgcolorbutton->setEnabled (FALSE);
  fgcolorbutton->setEnabled (FALSE);
  toolbariconbutton->setEnabled (FALSE);
  addtotoolbar->setEnabled (FALSE);
  options->setEnabled (FALSE);

  lb1->hide ();
  terminal->hide ();

  buttonUp->setEnabled (TRUE);
  buttonDown->setEnabled (TRUE);

  emit (updateRects());
}


void mainWidget::showFull ()
{
  setFixedSize (375, 375);

  lb1->show ();
  terminal->show ();

  hostname->show ();
  host->show ();

  host->setEnabled (TRUE);
  account->setEnabled (TRUE);
  terminal->setEnabled (TRUE);
  options->setEnabled (TRUE);
  connector->setEnabled (TRUE);
  textsize->setEnabled (TRUE);
  if (strcmp (connector->currentText(), "rLogin") != 0) port->setEnabled (TRUE);
  bgcolorbutton->setEnabled (TRUE);
  fgcolorbutton->setEnabled (TRUE);
  toolbariconbutton->setEnabled (TRUE);
  addtotoolbar->setEnabled (TRUE);

  buttonUp->setEnabled (TRUE);
  buttonDown->setEnabled (FALSE);

  emit (updateRects());
}


void mainWidget::slotToolbar ()
{
  if (strcmp(addtotoolbar->text(), i18n("Add to toolbar")) == 0)
    {
      addtotoolbar->setText(i18n("Remove"));
      emit(addToToolbar ());
    }
  else 
    {
      addtotoolbar->setText(i18n("Add to toolbar"));
      emit (removeFromToolbar ());
    }
}


SessionData* mainWidget::getNewSession ()
  {
    SessionData* newSess = new SessionData;

    newSess->commandLine = options->text();
    newSess->terminal = terminal->currentText();

    if (textsize->currentItem () == 0) newSess->font = "6x12";
    if (textsize->currentItem () == 1) newSess->font = "8x13";
    if (textsize->currentItem () == 2) newSess->font = "7x14";
    if (textsize->currentItem () == 3) newSess->font = "9x15";
    if (textsize->currentItem () == 4) newSess->font = "10x20";

    if (connector->currentItem () == 0)
      newSess->connectionProgram = "telnet";
    if (connector->currentItem () == 1)
      newSess->connectionProgram = "rlogin";
    if (connector->currentItem () == 2)
      newSess->connectionProgram = "ssh";

    newSess->port = port->value();

    if (strcmp(addtotoolbar->text(), i18n("Add to toolbar")) == 0)
      newSess->addToToolbar = FALSE;
    else newSess->addToToolbar = TRUE;

    newSess->toolbarIcon = toolbariconbutton->icon ();

    newSess->profileName = newSessionLined->text ();
    newSess->host = host->text ();
    newSess->account = account->text ();
    newSess->fgcolor = fgcolorbutton->color();
    newSess->bgcolor = bgcolorbutton->color();

    return (newSess);
  }




// ****************************************************************************
//
//  Inherited QLabel class implementation to receive mousePressEvents on
//  labels. The event is parsed to the parent widget.
//
// ****************************************************************************


QMLabel::QMLabel (const char* text, QWidget* parent)
  : QLabel (text, parent)
{
  parentW = parent;
}


void QMLabel::mousePressEvent (QMouseEvent* e)
{
  //
  // you must correct for the position of the Label
  //
  ((mainWidget*)parentW)->mousePressEvent (new QMouseEvent (Event_MouseButtonPress, 
	     	 e->pos()+pos(), e->button(), e->state()));
}


// *********************************************************************
//
//  inherited to receive mousePressEvents on right mousebutton
//
// *********************************************************************


QMComboBox::QMComboBox (QWidget* parent, const char* name)
  : QComboBox (parent, name)
{
}


void QMComboBox::mousePressEvent (QMouseEvent* e)
{
  if (e->button () == RightButton) emit (sigRightClick());
  else QComboBox::mousePressEvent (e);
}




// *********************************************************************
//
//  inherited to handle focusOutEvents
//
// *********************************************************************


QMLineEdit::QMLineEdit (QWidget* parent, const char* name)
  : QLineEdit (parent, name)
{
  // just call the parents constructor
}


void QMLineEdit::focusOutEvent (QFocusEvent* e)
{
  QLineEdit::focusOutEvent (e);
  emit (gotFocusOutEvent ());
}


