#ifndef __MAINWIDGET_H
#define __MAINWIDGET_H


#include <qdialog.h>
#include <qwidget.h>
#include <qlabel.h>
#include <qlined.h>
#include <qaccel.h>

#include <kcolorbtn.h>
#include <kiconloaderdialog.h>
#include <kintegerline.h>

#include "mywidget.h"
#include "sessiondata.h"


class QPushButton;
class QComboBox;
class QLineEdit;
class mainWidget;



/**
 * This class must be inherited to get a QLabel which does parse any
 * received mousePressEvents to its parent widget. The parent widget 
 * is mainWidget in this case.
 *
 * mousePressEvent must of course correct the mousePosition for the
 * parent widget
 *
 */

class QMLabel : public QLabel
{
  Q_OBJECT
  public:
    QMLabel (const char* text, QWidget* parent);

  protected:
    void mousePressEvent (QMouseEvent* e);

    QWidget* parentW;
};



// needed for accepting right mousebutton clicks to change session name

class QMComboBox : public QComboBox
{
Q_OBJECT
public:
  QMComboBox (QWidget* parent=0, const char* name=0);
protected:
  void mousePressEvent (QMouseEvent* e);
signals:
  void sigRightClick ();
};



// needed for receiving focusOutEvents

class QMLineEdit : public QLineEdit
{
Q_OBJECT
public:
  QMLineEdit (QWidget* parent=0, const char* name=0);
protected:
  void focusOutEvent (QFocusEvent* e);
signals:
  void gotFocusOutEvent ();
};


// **************************************************************************


class mainWidget : public QWidget
{
  Q_OBJECT
  friend class MyWidget;
  friend class QMLabel;

  public:
    mainWidget (SessionManager* sess, QWidget* parent=0, const char* name=0);
    ~mainWidget ();

  public slots:
    void showMinimum ();
    void showStandard ();
    void showFull ();
    void changeLines ();
    SessionData* getNewSession (); //CT 9jan98

    void editNewSessionSelected ();
    void restoreSession ();

    void changeUp ();
    void changeDown ();

    // the first is called before, the other after changing the profileName
    void profileNameChanged ();
    void changeProfileName ();

    void gotNewSession (int index);
    void SessionDeleted (const char* the_session);

  private slots:
    void increaseSize ();
    void reduceSize ();
    void changeIcon (const char*);
    void checkConnectorForPort (int);
    void textSizeHasChanged (int);
    void connectorHasChanged (int);

  protected slots:
    void slotToolbar ();

  protected:
    void paintEvent (QPaintEvent*);
    void mousePressEvent (QMouseEvent*);

  signals:
    void showOptionsPopup (QPoint);
    void showFilePopup (QPoint);
    void updateRects ();
    void addToToolbar ();
    void removeFromToolbar ();
    void deactivateSLOTS ();
    void activateSLOTS ();
    void connectorHasChanged (const char*);
    void textsizeHasChanged (const char*);

  private:
    SessionManager* sessions;

    QAccel* acc;
    QAccel* acc_updown;

    QMLabel* schrift;
    QMLabel* version;
    QMLabel* sessionName;
    QMLabel* accountname;
    QMLabel* hostname;
    //CT 8jan98
    QMLabel* optionsLabel;

    QPushButton* Connect;		// the connect - Button
    QMComboBox* profileName;
    QMLineEdit* newSessionLined;         // a lineedit to enter a new session

    QLineEdit* host; 
    QLineEdit* account;
    //CT 8jan98
    QLineEdit* options;

    QMLabel* lb1;
    QMLabel* lb2;
    QMLabel* lb3;
    QMLabel* lb4;
    QMLabel* lb5;
    //CT 8jan98
    QMLabel* lb6;

    QComboBox* terminal;
    QComboBox* connector;
    QComboBox* textsize;
    
    KIntegerLine* port;

    KColorButton* fgcolorbutton;
    KColorButton* bgcolorbutton;

    //CT 8jan98
    KIconLoaderButton* toolbariconbutton;

    QPushButton* buttonUp;
    QPushButton* buttonDown;
    //CT 8jan98
    QPushButton* addtotoolbar;
    QPushButton* savenewbutton;
    QPushButton* restoreoldbutton;

    struct previousStatus
    {
      int status;			   //CT 9jan98 needed to save the
      int session;		           // main widget's status
    } prev;				   // 0=min;1=standard;2=max

};



#endif









