#include <string.h>
#include <stdlib.h>

#include <qstring.h>
#include <qmenubar.h>
#include <qpopmenu.h>
#include <qmsgbox.h>
#include <qframe.h>
#include <qtooltip.h>
#include <qkeycode.h>

#include <kconfigbase.h>
#include <kapp.h>
#include <kmenubar.h>
#include <kcolordlg.h>
#include <kiconloader.h>
#include <kiconloaderdialog.h>
#include <kwm.h>

#include "mywidget.h"
#include "mainwidget.h"
#include "panelwindow.h"

#include "sessiondata.h"
#include "configtab.h"

#include "mywidget.moc"



// ************  implementation of MyWidget - the main widget  ************
//


void MyWidget::deactivateSLOTS ()
{
  filemenue->setItemEnabled (filemenue->idAt(0), FALSE);
  filemenue->setItemEnabled (filemenue->idAt(1), FALSE);
  filemenue->setItemEnabled (filemenue->idAt(2), FALSE);

  tools->setItemEnabled (1, FALSE);
  tools->setItemEnabled (2, FALSE);
  tools->setItemEnabled (3, FALSE);  

  // reconnect the keyboard accelerators
  m->acc_updown->setEnabled (FALSE);
  m->acc->setEnabled (FALSE);
}


void MyWidget::activateSLOTS ()
{
  filemenue->setItemEnabled (filemenue->idAt(0), TRUE);
  filemenue->setItemEnabled (filemenue->idAt(1), TRUE);
  if (sess->needSessionSave() == TRUE)
    filemenue->setItemEnabled (filemenue->idAt(2), TRUE);

  tools->setItemEnabled (1, TRUE);
  tools->setItemEnabled (2, TRUE);
  if (sess->needSessionSave() == TRUE) tools->setItemEnabled (3, TRUE);

  // reconnect the keyboard accelerators
  m->acc_updown->setEnabled (TRUE);
  m->acc->setEnabled (TRUE);
}


// ************************  append a new session  ***************************


bool MyWidget::appendNewSession ()
  {
    if (sess->appendSession (m->getNewSession()) == FALSE)
      {
	restoreSession ();
	return (FALSE);
      }

    // add it to toolbar?
    if (sess->addToToolbar() == TRUE)
      tools->insertButton (KApplication::getKApplication()->
			   getIconLoader()->loadIcon (sess->icon()), 
			   sess->ID(), TRUE, 
			   sess->session(), -1);
    
    restoreSession ();
    return (TRUE);
  }


// slot called to update the toolbar and menu entries for the "save" topic

void MyWidget::needSessionSaving (bool save)
{
  tools->setItemEnabled (3, save);
  filemenue->setItemEnabled (filemenue->idAt(2), save);
}


//
// CT redefined to spawn up the new way of new seesion.
//

void MyWidget::newSessionSelect ()
{
  sess->rejectChangingSignals (TRUE);

  // set new geometry and contents of widgets
  m->editNewSessionSelected ();

  // disallow some things within toolbar and popup menus
  deactivateSLOTS ();

  NewSessAccel = new QAccel (this);
  NewSessAccel->connectItem( NewSessAccel->insertItem(Key_Escape), 
			     this,     
			     SLOT(restoreSession()) );
  NewSessAccel->connectItem( NewSessAccel->insertItem(Key_Return), 
			     this,     
			     SLOT(appendNewSession()) );

}


void MyWidget::restoreSession ()
{
  m->restoreSession();
  activateSLOTS ();

  delete NewSessAccel;

  sess->rejectChangingSignals (FALSE);
}


void MyWidget::deleteSessionSelect ()
  {
    if (sess->count () == 1)
      {
        QMessageBox::warning (0, i18n("Error"), 
			 i18n("You need at least 1 session!"));
	return;
      }

    QString whatToSay = i18n("You attempt to delete session \"");
    whatToSay += sess->session();
    whatToSay += i18n("\"!\n\nContinue deleting?");

    int result = QMessageBox::warning (0, i18n("Delete session"), 
		    (const char*)whatToSay, i18n("Yes"), 
				       i18n("No"), 0, 1);

    if (result == 0) 
      {
	tools->removeItem (sess->ID());
	sess->deleteSession ();
      }
 
    return;
  }


void MyWidget::hideMenuBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showMenuBar", "FALSE");

  menue->hide ();
  optionMenue->removeItemAt (0);
  optionMenue->insertItem (i18n("Show &menubar"), -1, 0);
  optionMenue->connectItem (optionMenue->idAt (0), this, SLOT(showMenuBar ()));

  updateRects ();
}


void MyWidget::showMenuBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showMenuBar", "TRUE");

  menue->show ();
  optionMenue->removeItemAt (0);
  optionMenue->insertItem (i18n("Hide &menubar"), -1, 0);
  optionMenue->connectItem (optionMenue->idAt (0), this, SLOT(hideMenuBar ()));

  updateRects ();
}


void MyWidget::hideToolBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showToolBar", "FALSE");

  tools->hide ();
  optionMenue->removeItemAt (1);
  optionMenue->insertItem (i18n("Show &toolbar"), -1, 1);
  optionMenue->connectItem (optionMenue->idAt (1), this, SLOT(showToolBar ()));

  updateRects ();
}


void MyWidget::showToolBar ()
{
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("common");
  k->writeEntry ("showToolBar", "TRUE");

  tools->show ();
  optionMenue->removeItemAt (1);
  optionMenue->insertItem (i18n("Hide &toolbar"), -1, 1);
  optionMenue->connectItem (optionMenue->idAt (1), this, SLOT(hideToolBar ()));

  updateRects ();
}


void MyWidget::showOptionsPopup (QPoint p)
{
  optionMenue->popup ( p + geometry().topLeft () + m->pos());
}


void MyWidget::showFilePopup (QPoint p)
{
  filemenue->popup ( p + geometry().topLeft () + m->pos());
}


// *********************  adding icons to toolbar  ****************************


void MyWidget::addToToolbar ()
{
  int i;

  sess->shownOnToolbarHasChanged(TRUE);

  //
  // always put the icons in the same order in the toolbar, so remove
  // all and insert all afterwards
  //
  for (i=0; i>-1; i++)
    {
      if (sess->session(i) == 0) break;
      tools->removeItem (sess->ID(i));
    }

  for (i=0; i>-1; i++)
    {
      if (sess->session(i) == 0) break;
      if (sess->addToToolbar(i) == TRUE)
	tools->insertButton (KApplication::getKApplication()->getIconLoader()->
			     loadIcon (sess->icon(i)), 
			     sess->ID(i), TRUE, 
			     sess->session(i), -1);
    }
}


void MyWidget::removeFromToolbar ()
{
  sess->shownOnToolbarHasChanged (FALSE);

  tools->removeItem (sess->ID());     // remove a certain ID
}


// runs the modal setup tabdialog
void MyWidget::setup ()
{
  configTab* setup = new configTab (this, 0, TRUE);
  setup->show ();
}


void MyWidget::saveOptions ()
{
  KConfig* k = KApplication::getKApplication ()->getConfig();
  k->sync ();
}


// only parsing the commands to mainWidget m (coming from configTab)
void MyWidget::showMinimum ()
{
  m->showMinimum ();
}


void MyWidget::showFull ()
{
  m->showFull ();
}


void MyWidget::showStandard ()
{
  m->showStandard ();
}


void MyWidget::showHelper ()
{
  KApplication::getKApplication()->invokeHTMLHelp ("","");
}


void MyWidget::putOnPanel ()
{
  PanelWindow* panelWin = new PanelWindow (sess);

  KWM::setDockWindow (panelWin->winId());

  panelWin->show ();
  delete this;
}


MyWidget::MyWidget (SessionManager* _sessions, QWidget*, const char* name)
  : KTopLevelWidget (name)
  {
    int i;
    KConfig* k = KApplication::getKApplication()->getConfig();

    sess = _sessions;

    connect (sess, SIGNAL (needSessionSaving(bool)), 
	     SLOT (needSessionSaving (bool)));

    // ****************************  Menue  **********************************
    
    menue = new KMenuBar (this);
    m = new mainWidget (sess, this);
    tools = new KToolBar (this);

    // ************************  the toolbar  *******************************
    // load the icons
    KIconLoader* kiL = KApplication::getKApplication()->getIconLoader();

    tools->insertButton (kiL->loadIcon ("exit.xpm"), 0, SIGNAL (clicked()), 
			 KApplication::getKApplication(), SLOT (quit()), TRUE, 
			 i18n("Exit the remote login manager"));
    tools->insertSeparator ();
    tools->insertButton (kiL->loadIcon ("filenew.xpm"), 1, SIGNAL (clicked()), 
			 this, SLOT (newSessionSelect()), TRUE, 
			 i18n("Define new session"));
    tools->insertButton (kiL->loadIcon ("editcut.xpm"), 2, SIGNAL (clicked()), 
			 this, SLOT (deleteSessionSelect()), TRUE, 
			 i18n("Delete session"));
    tools->insertButton (kiL->loadIcon ("filefloppy.xpm"), 3, 
			 SIGNAL (clicked()), 
			 sess, SLOT (saveSessions()), TRUE,
			 i18n("Save sessions"));
    tools->insertSeparator ();
    tools->insertButton (kiL->loadIcon ("panelize.xpm"), 0, 
			 SIGNAL (clicked()), 
			 this, SLOT (putOnPanel()), TRUE, 
			 i18n("Panelize the program"));
    tools->insertButton (kiL->loadIcon ("help.xpm"), 0, SIGNAL (clicked()), 
			 this, SLOT (showHelper()), TRUE, 
			 i18n("Get Help"));

    //
    // now insert the session buttons
    //
    tools->insertSeparator ();

    for (i=0; i>-1; i++)
      {
	if (sess->session(i) == 0) break;

	if (sess->addToToolbar(i) == TRUE)
	  {
	    tools->insertButton (kiL->loadIcon (sess->icon(i)), 
				 sess->ID(i), TRUE, 
				 sess->session(i));
	  }
      }

    connect ( tools, SIGNAL (clicked (int)), sess, 
	      SLOT (startSession_ID(int)) );

    // ************************  new Popupmenus  ****************************  

    filemenue = new QPopupMenu;
    optionMenue = new QPopupMenu;

    // ************************  now the option menue  ************************

    optionMenue->insertItem (i18n("Hide &menubar"));
    optionMenue->insertItem (i18n("Hide &toolbar"));
    optionMenue->insertSeparator ();
    optionMenue->insertItem (i18n("On &panel"), this, SLOT (putOnPanel()), 
			     CTRL+Key_P);
    optionMenue->insertSeparator ();
    optionMenue->insertItem (i18n("&Options"), this, SLOT (setup ()) );
    optionMenue->insertItem (i18n("Sa&ve options"), this, SLOT(saveOptions()));

    // ************************ the file menue ********************************

    filemenue->insertItem (i18n("&New Session"), this, SLOT (newSessionSelect ()), CTRL+Key_N );
    filemenue->insertItem (i18n("&Delete Session"), this, 
			   SLOT (deleteSessionSelect ()) );
    filemenue->insertItem (i18n("&Save Sessions"), sess, 
			   SLOT (saveSessions ()), CTRL+Key_S );
    filemenue->insertSeparator ();
    filemenue->insertItem (i18n("&Create kdelnk"), sess, SLOT(createKDElnk()));
    filemenue->insertSeparator ();
    filemenue->insertItem (i18n("E&xit"), KApplication::getKApplication(), 
			   SLOT (quit()), CTRL+Key_Q );
 
     // *********************  build the help menu  ***************************

    char* help = new char [400];

    strcpy (help, "KTelnet  ");
    strcat (help, VERSION);
    strcat (help, i18n("     formerly Telly"));
    strcat (help, 
	    i18n("\n\n(c) 1997/98 by\nSpaghetti Code Computing"));
    strcat (help, 
	  i18n("\n\nWritten by Martin Hfner and Cristian Tibirna"));
    strcat (help, 
	   i18n("\n\nEmail: mh@ap-dec717c.physik.uni-karlsruhe.de"));
    strcat (help, i18n("\nHomepage: http://ap-dec717c.physik.uni-karlsruhe.de/~mh/ktelnet"));
    strcat (help, i18n("\n\nThanks to all translators!"));

    helpmenue = KApplication::getKApplication()->getHelpMenu (TRUE, 
						 i18n(help));
    delete help;

    // ***************** now insert the Popups into the menuebar **************

    menue->insertItem (i18n("&File"), filemenue);
    menue->insertItem (i18n("&Options"), optionMenue);
    menue->insertSeparator ();
    menue->insertItem (i18n("&Help"), helpmenue);

    // set up the menu entries
    m->changeLines ();

    // now insert all widgets to KTopLevelWidget
    setMenu (menue);
    addToolBar (tools);
    setView (m);

    // now after inserting the toolbar calling this as at this point of the
    // file nothing will be needed to save
    needSessionSaving (sess->needSessionSave());
    
    // watch on the common configuration file
    k->setGroup ("common");
    
    if (k->readEntry ("showMenuBar", "TRUE") == QString ("TRUE")) 
      showMenuBar ();
    else hideMenuBar ();
    if (k->readEntry ("showToolBar", "TRUE") == QString ("TRUE")) 
      showToolBar ();	
    else hideToolBar ();

    if (k->readEntry ("windowSize", "standard") == QString ("minimum")) 
      m->showMinimum ();
    else if (k->readEntry ("windowSize", "standard") == QString ("standard")) 
      m->showStandard ();
    else m->showFull ();
  }


MyWidget::~MyWidget ()
{
  delete tools;
  delete menue;

  delete m;

  delete filemenue;
  delete optionMenue;
}








