#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <qmsgbox.h>

#include <kapp.h>
#include <ksimpleconfig.h>
#include <kconfig.h>
#include <klocale.h>


#include "sessiondata.h"
#include "linkdirselect.h"

#include "sessiondata.moc"


SessionManager::SessionManager ()
{
  needSaving = FALSE;
  rejectChangeSignals = FALSE;

  loadSessions ();

  // get a config object for the list of terminals
  KConfig* k = KApplication::getKApplication()->getConfig();
  k->setGroup ("terminal");
  
  // different terminals need different commandline options
  k->readListEntry ("terminals", terminals);
  k->readListEntry ("termfontsize", termFontSize);
  k->readListEntry ("termfgcolor", termFgColor);
  k->readListEntry ("termbgcolor", termBgColor);

  if (terminals.isEmpty() == TRUE) 
    {
      QMessageBox::warning (0, i18n("Warning"), 
			    i18n("Something went wrong reading the global resource file! Could not find any terminals. Stop!")); 	
      exit (0);       
    }
  
  // set to the top of the list
  theList.setAutoDelete (TRUE);
  cur = 0;
  theList.at (cur);
}


SessionManager::~SessionManager ()
{
}


void SessionManager::createKDElnk ()
{
  LinkDirectorySelector theDir;
  QString s;

  int result = theDir.exec ();

  if (result == 1) 
    {
      // append the session name to the preferred directory
      QString fname = theDir.getDirectory () + QString ("/");
      fname = fname + theList.at(cur)->profileName;
      fname += ".kdelnk";

      QFile f ( (const char*)fname );
      
      // file already exists?
      if (f.exists () == TRUE)
	{
	  int overwrite = QMessageBox::warning (0, i18n("Warning"), i18n("File already exists!\nOverwrite it?"), i18n("Yes"), i18n("No"), 0, 1);
	  if (overwrite == 1) return;
	}

      // now really save it
      if (f.open(IO_WriteOnly) == TRUE)
	{
	  f.writeBlock ("# KDE Config File\n", strlen("# KDE Config File\n"));
	  f.writeBlock ("[KDE Desktop Entry]\n", strlen("[KDE Desktop Entry]\n"));
	  
	  //
	  // the exec entry
	  //
	  s = "Exec=";
	  s += commandLine (cur);
	  s += "\n";
	  f.writeBlock ((const char*)s, strlen ((const char*)s));
	  
	  // the name
	  s = "Name=";
	  s += (const char*)(theList.at(cur)->profileName);
	  s += "\n";
	  f.writeBlock ((const char*)s, strlen ((const char*)s));
	  
	  s = "Icon=ktelnet.xpm\n";
	  f.writeBlock ((const char*)s, strlen ((const char*)s));
	  
	  // the comment shown by QToolTip
	  s = "Comment=";
	  if (theList.at(cur)->host != QString ("-"))
	    {
	      s += i18n("Connects to ");
	      s += (const char*)(theList.at(cur)->account);
	      s += "@";
	      s += (const char*)(theList.at(cur)->host);
	      s += "\n";
	    }
	  else 
	    {
	      s += i18n("Opens Terminal");
	      s += "\n";
	    }
	  f.writeBlock ((const char*)s, strlen ((const char*)s));
	  
	  f.writeBlock ("Type=Application\n", strlen("Type=Application\n"));
	  f.writeBlock ("Terminal=0\n", strlen("Terminal=0\n"));

	  QMessageBox::information (0, i18n("Information"), 
				    (const char*)QString(i18n("Saved file  "))
				    + fname + QString (" !\n\n") + 
				    QString (i18n("Perhaps you need to resfresh your desktop\nor panel to display the KDElnk icon.")));

	  f.close ();
	}
      else QMessageBox::warning (0, i18n("Error"), 
       i18n("Could not open file!\nMaybe you don't have write permissions."));
    }
}


QList<QString>& SessionManager::commandLineList (int i)
{
  QList<QString>* s = new QList<QString>;

  QString s2;
  char s1[10];

  // the terminal
  s->append(new QString (theList.at(i)->terminal));

  // the fgcolor - terminal dependend argument
  s2 = "-";
  s2 = s2 + termFgColor.at(terminals.find(theList.at(i)->terminal));
  s->append (new QString (s2));
  s->append (new QString ( QColor2string (theList.at(i)->fgcolor)) );

  // the bgcolor - terminal dependend argument
  s2 = "-";
  s2 = s2 + termBgColor.at(terminals.find(theList.at(i)->terminal));
  s->append (new QString (s2));
  s->append ( new QString ( QColor2string ((theList.at(i)->bgcolor)) ));
  
  // the terminal fontsize - terminal dependent
  s2 = "-";
  s2 = s2 + termFontSize.at(terminals.find(theList.at(i)->terminal));
  s->append (new QString (s2));
  s->append (new QString (theList.at(i)->font));
  
  // much more commandline arguments
  s2 = theList.at(i)->commandLine;
  s2 = s2.simplifyWhiteSpace ();

  while (s2.length()>0) 
    {
      s->append (new QString (s2.left (s2.find (' '))));
      if (s2.contains (' ') > 0)
	s2 = (s2.right (s2.length()-s2.find(' '))).stripWhiteSpace();
      else s2 = "";
    }
  
  // the terminal title
  s->append (new QString ("-T"));
  s->append (new QString (theList.at(i)->profileName));
  
  // the connection program
  if (theList.at(i)->host != QString ("-"))
    {
      // what to execute
      s->append (new QString ("-e"));
      s->append (new QString (theList.at(i)->connectionProgram));
      
      // this order runs with all connection programs
      s->append (new QString ("-l"));
      s->append (new QString (theList.at(i)->account));
      
      // the connection programs want different commandline options for the
      // port.
      if (theList.at(i)->connectionProgram == "telnet")
	{
	  // beware of ports set to zero
	  s->append (new QString (theList.at(i)->host));
	  if (theList.at(i)->port != 0)
	    {
	      sprintf (s1, "%d", theList.at(i)->port);
	      s->append (new QString (s1));
	    }
	}
      else if (theList.at(i)->connectionProgram == "rlogin")
	{
	  // rlogin does not accept a port
	  s->append (new QString (theList.at(i)->host));	      
	}
      else if (theList.at(i)->connectionProgram == "ssh")
	{
	  // beware of ports set to zero
	  if (theList.at(i)->port != 0)
	    {
	      s->append (new QString ("-p"));
	      sprintf (s1, "%d ", theList.at(i)->port);
	      s->append (new QString (s1));
	    }
	  s->append (new QString (theList.at(i)->host));
	}
    }
  
  return (*s);
}


QString SessionManager::commandLine (int i)
{
  QString s;
  QString help;
  unsigned int j;

  QList<QString> sl;
  sl.setAutoDelete (TRUE);

  sl = commandLineList (i);

  for (j=0; j<sl.count(); j++)
    {
      help = *sl.at(j);
      if (help.contains (' ') > 0) 
	help = QString ("\"") + help + QString ("\"");
      s = s + help;
      if (j < sl.count()-1) s += " ";
    }

  // return the commandline needed to start the session
  return (s);
}


const char* SessionManager::QColor2string (QColor col)
{
  static char colour[10];
  int red, green, blue;
  char help[10];

  red = col.red ();
  green = col.green ();
  blue = col.blue ();
  
  strcpy (colour, "#");
  
  sprintf (help, "%x", red);
  if (red < 16) strcat (colour, "0");
  strcat (colour, help);
  
  sprintf (help, "%x", green);
  if (green < 16) strcat (colour, "0");
  strcat (colour, help);
  
  sprintf (help, "%x", blue);
  if (blue < 16) strcat (colour, "0");
  strcat (colour, help);
  
  return colour;
}


unsigned int SessionManager::getNewSessionID ()
{
  SessionData* iter;
  int newID = 99;
  bool foundNewID;

  do
    {
      foundNewID = TRUE;
      newID++;
      for (iter=theList.first(); iter!=0; iter=theList.next())
	{
	  if (iter->ID == newID) 
	    {
	      foundNewID = FALSE;
	      break;
	    }
	}
    }  
  while (foundNewID == FALSE);

  return (newID);
}


void SessionManager::rejectChangingSignals (bool reject)
{
  rejectChangeSignals = reject;
}


const QColor& SessionManager::fgColor ()
{
  return (theList.at(cur)->fgcolor);
}


const QColor& SessionManager::bgColor ()
{
  return (theList.at(cur)->bgcolor);
}


void SessionManager::loadSessions ()
{
  KConfig* kc = KApplication::getKApplication()->getConfig();
  kc->setGroup ("defaults");
  QString defaultTerminal = kc->readEntry ("defaultTerminal");
  
  KSimpleConfig* k = new KSimpleConfig ( (const char*)
					 (kapp->localconfigdir() + QString("/")
					  +  SESSIONSFILE) );
  
  QString s;
  char item[20];
  char s1 [4];
  
  do
    {
      //
      // append the list of sessions (the list starts with index 0)
      //
      theList.append( new SessionData );
      
      //
      // choose the item to load
      //
      strcpy (item, "session");
      sprintf ( s1, "%i", theList.at() );
      strcat (item, s1);
      k->setGroup (item);
      
      //
      // now read the sessions from file
      //
      theList.current()->host = k->readEntry ("host","-");
      
      theList.current()->account = k->readEntry ("account","-");
      
      theList.current()->font = k->readEntry ("font","9x15");
      
      theList.current()->terminal = 
	k->readEntry ("terminal",(const char*)defaultTerminal);
      
      theList.current()->connectionProgram = 
	k->readEntry ("connection","telnet");
      
      theList.current()->port = k->readNumEntry ("port", 0);
      
      theList.current()->bgcolor = k->readColorEntry ("bgcolor",
						      &QColor(255,255,255));
      
      theList.current()->fgcolor =k->readColorEntry ("fgcolor",&QColor(0,0,0));
      
      theList.current()->commandLine = k->readEntry ("options", "-ls");

      s = k->readEntry ("addToToolbar", "TRUE");
      if (s == QString("TRUE"))
	theList.current()->addToToolbar = TRUE;
      else theList.current()->addToToolbar = FALSE;
      
      theList.current()->toolbarIcon = k->readEntry ("toolbarIcon", 
						     "session1.xpm");
      
      theList.current()->ID = k->readNumEntry ("ID", 100+theList.at());  
      
      theList.current()->profileName = k->readEntry ("profile");
    }
  while ( theList.current()->profileName.isEmpty() == FALSE );
  
  //
  // if no session is available at least the ordinary terminal will be 
  // displayed
  //
  if (theList.count() == 1)
    theList.current()->profileName = i18n("ordinaryTerminal");
  else theList.removeLast();
  
  delete k;
}


void SessionManager::saveSessions ()
{
  //
  // now rewrite the configuration
  //
  KSimpleConfig* k = new KSimpleConfig ( (const char*)
					 (kapp->localconfigdir() + QString("/")
					  + SESSIONSFILE) );
  
  char s1[5];
  char item[20];
  int i=0;
  
  //
  // first delete the whole file
  //
  do
    {
      //
      // choose the item to remove
      //
      strcpy (item, "session");
      sprintf ( s1, "%i", i++ );
      strcat (item, s1);
      k->setGroup (item);
    }
  while (k->deleteGroup (item) == TRUE);
    
  int oldCurrent = theList.at ();
  //
  // now write it back to disk
  //
  SessionData* iter;
  for ( iter=theList.first(); iter!=0; iter=theList.next() )
    {
      strcpy (item, "session");
      sprintf ( s1, "%i", theList.find(iter) );
      strcat (item, s1);
      
      k->setGroup (item);
      
      k->writeEntry ("profile", (const char*)iter->profileName);
      k->writeEntry ("host", (const char*)iter->host);
      k->writeEntry ("account", (const char*)iter->account);
      k->writeEntry ("font", (const char*)iter->font);
      k->writeEntry ("terminal", (const char*)iter->terminal);
      k->writeEntry ("connection", (const char*)iter->connectionProgram);
      k->writeEntry ("port", iter->port);
      k->writeEntry ("bgcolor",  iter->bgcolor);
      k->writeEntry ("fgcolor", iter->fgcolor);
      k->writeEntry ("options", (const char*)iter->commandLine);
      k->writeEntry ("toolbarIcon", (const char*)iter->toolbarIcon);
      k->writeEntry ("ID", iter->ID);
      
      if (iter->addToToolbar == TRUE)
	k->writeEntry ("addToToolbar", "TRUE");
      else k->writeEntry ("addToToolbar", "FALSE");
    }
  
  k->sync ();
  
  delete k;
  
  theList.at (oldCurrent);
  
  needSaving = FALSE;
  emit (needSessionSaving (needSaving));
}


void SessionManager::deleteSession ()
  {
    needSaving = TRUE;

    QString deletedName = theList.at(cur)->profileName;

    theList.remove (cur);
    cur = theList.at ();

    emit (deletedSession( (const char*)deletedName ) );

    emit (needSessionSaving (needSaving));
  }


bool SessionManager::appendSession (SessionData* theNewSession)
{
  if (  ( theNewSession->profileName.isEmpty() )  
	||  ( theNewSession->host.isEmpty() )  )
    {
      QMessageBox::warning (0, i18n("Warning"), 
			    i18n("Nothing to append!"));
      
      delete theNewSession;
      return (FALSE);
    }
  
  // are there any white spaces in the strings (not allowed) ?
  
  if ( ( theNewSession->host.find (' ') >= 0 )  ||
	 ( theNewSession->account.find (' ') >= 0 ) )
    {
      QMessageBox::warning (0, i18n("Warning"), 
			    i18n("That doesn't seem to be a real account!"));
      
      delete theNewSession;
      return (FALSE);
    }
  
  SessionData* iter;
  for (iter=theList.first(); iter!=0; iter=theList.next())
    {
      if (iter->profileName == theNewSession->profileName)
	{
	  QMessageBox::warning (0, i18n("Warning"), 
				i18n("No 2 sessions with same name!"));
	  
	  delete theNewSession;
	  return (FALSE);
	}
    }
  
  theList.append (theNewSession);

  cur = theList.at();
  theList.current()->ID = getNewSessionID ();

  emit (appendedNewSession (cur));

  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  return (TRUE);
}


void SessionManager::startSession ()
{
  startSession (cur);
}


void SessionManager::startSession (int number)
{
  // change to the wanted sessionnumber
  unsigned int i;
  theList.at (number);
  
  // get config from main application
  KConfig* k = KApplication::getKApplication()->getConfig ();
  k->setGroup ("common");
  
  //
  // fork the process and store its process ID
  //
  int pid = fork ();
    
  if (k->readEntry ("callXhost", "FALSE") == QString ("TRUE"))
    {
      if (pid == 0)
	{
	  // calling xhost
	  execlp ("xhost", "xhost", (const char*)theList.at(number)->host, 0);
	}
      
      // fork another time
      pid = fork ();
    }
  
  // the next forked process
  // the process with PID 0 will be replaced by the terminal program	
  if (pid == 0)		
    {
      QList<QString> s = commandLineList (number);
      s.setAutoDelete (TRUE);

      // get memory for the arguments list
      char** cmdL = new char*[s.count()+1];
      for (i=0; i<s.count();i++) cmdL[i] = new char[50];

      // just make a "deep copy" of the stringlist
      for (i=0; i<s.count();i++) strcpy ( cmdL[i], (const char*)(*s.at(i)) );
      cmdL[s.count()] = 0;

      /*
       *  start the session now !!!!
       */
      execvp (cmdL[0], cmdL);
      
      // the program never reaches this point, I know, ...
      for (i=0; i<s.count(); i++) delete [] cmdL[i];
      delete [] cmdL;
    }
  
  // reset the current session pointer
  theList.at (cur);
    
  if (k->readEntry ("exitOnConnect", "FALSE") == QString ("TRUE")) exit (0);
}


void SessionManager::startSession_ID (int ID)
{
  //
  // we have to find out the correct session from ID
  //
  SessionData* iter;
  int i=0;

  for (iter=theList.first(); iter!=0; iter=theList.next(), i++)
    if (iter->ID == ID) break;

  startSession (i);

  theList.at(cur);
}


void SessionManager::portHasChanged (const char* newPort)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->port = atoi (newPort);
}


void SessionManager::hostHasChanged (const char* newText)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));
  
  theList.at(cur)->host = newText;
}


void SessionManager::accountHasChanged (const char* newText)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));
  
  theList.at(cur)->account = newText;
}


void SessionManager::terminalHasChanged (const char* newTerminal)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->terminal = newTerminal;
}


void SessionManager::textsizeHasChanged (const char* newTextsize)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->font = newTextsize;
}


void SessionManager::connectorHasChanged (const char* newConnector)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->connectionProgram = newConnector;
}


void SessionManager::fgcolorHasChanged (const QColor& newColor)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->fgcolor = newColor;
}


void SessionManager::bgcolorHasChanged (const QColor& newColor)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->bgcolor = newColor;
}


void SessionManager::cmdlineHasChanged (const char* newText)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));
 
  theList.at(cur)->commandLine = newText;
}


void SessionManager::shownOnToolbarHasChanged (bool show)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->addToToolbar = show;
}


int SessionManager::current ()
{
  return (cur);
}


int SessionManager::count ()
{
  return (theList.count());
}


const char* SessionManager::terminal ()
{
  return (theList.at(cur)->terminal);
}


const char* SessionManager::account ()
{
  return (theList.at(cur)->account);
}


const char* SessionManager::host ()
{
  return (theList.at(cur)->host);
}


const char* SessionManager::cmdLine ()
{
  return (theList.at(cur)->commandLine);
}


int SessionManager::port ()
{
  return (theList.at(cur)->port);
}


const char* SessionManager::textsize ()
{
  return (theList.at(cur)->font);
}


int SessionManager::ID ()
{
  return (theList.at(cur)->ID);
}


int SessionManager::ID (int i)
{
  if (i >= int(theList.count())) return (-1);
  if (i < 0) return (-1);

  return (theList.at(i)->ID);
}


const char* SessionManager::icon ()
{
  return (theList.at(cur)->toolbarIcon);
}


const char* SessionManager::icon (int i)
{
  if (i >= int(theList.count())) return (0);
  if (i < 0) return (0);

  return (theList.at(i)->toolbarIcon);
}


bool SessionManager::addToToolbar ()
{
  return (theList.at(cur)->addToToolbar);
}


bool SessionManager::addToToolbar (int i)
{
  if (i >= int(theList.count())) return (FALSE);
  if (i < 0) return (FALSE);

  return (theList.at(i)->addToToolbar);
}


const char* SessionManager::session ()
{
  return (theList.at(cur)->profileName);
}


const char* SessionManager::session (int i)
{
  if (i >= int(theList.count())) return (0);
  if (i < 0) return (0);

  return (theList.at(i)->profileName);
}


const char* SessionManager::connector ()
{
  return (theList.at(cur)->connectionProgram);
}


void SessionManager::newSessionSelected (const char* newSession)
{
  SessionData* iter;
  int i;

  for (iter=theList.first(), i=0; iter!=0; iter=theList.next(), i++)
    {
      //      if (iter->profileName == QString (newSession))
      // instead: a patch of Cristian Tibirna (11.06.98)
      if (QString (newSession)  == iter->profileName.left(strlen(newSession)) )
	{
	  cur = i;
	  return;
	}
    }
}


QStrList& SessionManager::theTerminals ()
{
  return terminals;
}


void SessionManager::sessionNameHasChanged (const char* newName)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->profileName = newName; 
}


void SessionManager::iconHasChanged (const char* newIcon)
{
  if (rejectChangeSignals == TRUE) return;
  needSaving = TRUE;
  emit (needSessionSaving (needSaving));

  theList.at(cur)->toolbarIcon = newIcon;
}


bool SessionManager::needSessionSave ()
{
  return (needSaving);
}
