/* *** l3
   *** Filename: l3.c
   *** Description: Add WAV files to encoding queue
   *** Version: 0.90 [19980217] | Sander Steffann & Michiel Bronswijk
   ***          1.00 [19980225] | Release version 1.00
   *** Remarks:
*/
#include <stdio.h>
#include <sys/param.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include <errno.h>
#include "l3spool.h"
#include <string.h>
#include "wav_interface.h"
                      
void main(int argc, char *argv[])
{
	int	id;
	int	curarg;
	int	bitrate;
	int	deleteflag;

	/* Show all the strange copyright stuff */
	printf("l3 - Version %s (%s %s)\n", L3VERSION, __DATE__, __TIME__);
	printf("Copyright (C) 1998 'Projectgroep MP3-studio voor Linux'\n");
	if (argc < 3) {
		printf("Usage:\n  %s <wav-file> <targetdir> [option]...\n", argv[0]);
		printf("  Options are: -br <bitrate> ; ");
		printf("select bitrate (default=112000)\n");
		printf("               -d            ; ");
		printf("delete <wav-file> after encoding\n");
		exit(1);
		};

	/* Set default parameters */
	bitrate = 112000;
	deleteflag = 0;
	
	/* Parse command line */	
	if (strncmp(argv[1], "-", 1) == 0) {
		printf("Bad usage\n");
		exit(1);
		};
	curarg = 3;
	while (curarg<argc) {
		if (strncmp(argv[curarg], "-", 1) != 0) {
			printf("Bad usage (argument: %s\n", argv[curarg]);
			exit(1);
			};
		if (strcmp(argv[curarg], "-br") == 0) {
			bitrate = atoi(argv[curarg+1]);
			if ((bitrate<8000) || (bitrate>256000))
				bitrate = 112000;
			curarg++; curarg++;
			}
		else {
			if (strcmp(argv[curarg], "-d") == 0) {
				deleteflag = 1;
				curarg++;
				}
			else {
				printf("Unknown switch %s\n", argv[curarg]);
				exit(1);
				}
			}
		}

	/* Do all the functional stuff */
	if (isWAVFile(argv[1], NULL) == -1) {
		printf("ERROR: %s is not a valid WAV-file.\n", argv[1]);
		exit(1);
		}	
	if (CreateLockFile() != 0) {
		printf("ERROR Could not create lockfile, ");
		printf("please try again later\n");
		exit(1);
		}
	id = GetHighestID();	
	WriteControlFile(++id, argv[1], argv[2], deleteflag, bitrate);
	SetHighestID(id);
	RemoveLockFile();
	
	/* Show what happened */
	printf("File queued.\n");
}
