/* *** l3d
   *** Filename: l3d.c
   *** Description: MPEG layer 3 spooling daemon
   *** Version: 19980210 0.80 | Sander Steffann & Michiel Bronswijk
   ***          19980225 1.00 | Release version 1.00
   *** Remarks:
*/

#include <signal.h>
#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <syslog.h>
#include <unistd.h>
#include "l3spool.h"
#include "wav_interface.h"

int	encodelock = 0;

void	RemoveLock(int sig)
{
	printf("l3d Closing down on signal number %d\n", sig);
	printf("Removing lockfile and tempfiles\n");

	RemoveLockFile();
	if (encodelock == 1) RemoveEncodeLockFile();
	unlink("/tmp/_l3dspool.mp3");
	unlink("/tmp/_l3dspool.wav");
	unlink("/tmp/l3/cfl3.cur");
	
	exit(0);
}

int	main(int argc, char *argv[])
{
	FILE		*ctrlfile;
	char		filename[MAXPATHLEN];
	char		filename2[MAXPATHLEN];
	char		commando[MAXPATHLEN];
	int		id;
	pid_t		encoder_pid;
	int		encoderin_fds[2];
	int		encoderout_fds[2];
	int		encodererr_fds[2];
	char		encoderinstr[256];
	int		returnvalue;
	struct stat	status;
		
	/* control file parameters */
	char		sourcename[MAXPATHLEN];
	char		targetname[MAXPATHLEN];
	int		do_delete;
	int		bitrate;
	
	char		bitratestr[7];
	
	/* Aren't we nice? */
	nice(20);

	/* Remove old locks */
	RemoveLockFile();
	signal(SIGHUP, RemoveLock);
	signal(SIGINT, RemoveLock);
	signal(SIGTERM, RemoveLock);
	signal(SIGQUIT, RemoveLock);

	printf("l3d - Version %s (%s %s)\n", L3VERSION, __DATE__, __TIME__);
	printf("Copyright (C) 1998 'Projectgroep MP3-studio voor Linux'\n");

	if (geteuid() == 0) {
		printf("Running as root\n");
	} else {
		while(CreateLockFile() != 0)
			sleep(1);

		/* Search for lowest ID control-file in the spool-dir */
		id = getLowestID();			
		
		RemoveLockFile();
		
		if (id < 1) {
			printf("Nothing to do, exiting...\n");
			exit(0);
		}

		printf("Not running as root! Exiting after processing queue\n");
	}
	
	if (CreateEncodeLockFile() != 0) {
		printf("Waiting to be the only l3d running...\n");
		fflush(NULL);
		while (CreateEncodeLockFile() != 0)
			sleep(1);
		printf("Ok! Now let's go to work\n");
	}

	encodelock = 1;

	/* Do we ever stop? No... */
	if (fork() == 0) while (1)
	{
		/* Try to establish a lock, sleep if currently locked */
		while(CreateLockFile() != 0)
			sleep(1);

		/* Search for lowest ID control-file in the spool-dir */
		id = getLowestID();			
		
		if (id != -1) {
			ReadControlFile(id, sourcename, targetname,
				        &do_delete, &bitrate);			
			
			sprintf(bitratestr, "%d", bitrate);
			
			/* now create the current spool control file */
			MakeControlFileCurrent(id);
			
			RemoveLockFile();

			symlink(sourcename, "/tmp/_l3dspool.wav");
			symlink(targetname, "/tmp/_l3dspool.mp3");

			/* create pipe for the encoder */
			if ((pipe(encoderin_fds) == -1) 
			|| (pipe(encoderout_fds) == -1) 
			|| (pipe(encodererr_fds) == -1)) {
				printf("ERR Couldn't create the pipe\n");
				exit(1);
				}
				
			/* do a fork for executing the encoder */
			if (encoder_pid = fork() == 0) {
				/* execute this as a child */
				int	org_stdin_fd;
				int	org_stdout_fd;
				
				org_stdin_fd = dup(STDIN_FILENO);
				org_stdout_fd = dup(STDOUT_FILENO);
				
				dup2(encoderin_fds[0],STDIN_FILENO);

				// Veroorzaakt soms fouten...
				// encoderout_fds[0] = open("/dev/null", 
				// 	O_WRONLY);
				// dup2(encoderout_fds[1],STDOUT_FILENO);

				// En deze ook... Oorzaak onbekend
				// encodererr_fds[0] = open("/dev/null", 
				// 	O_WRONLY);
				// dup2(encodererr_fds[1],STDERR_FILENO);
			
				// Dan maar op de botte manier
				close(STDERR_FILENO);
				
				/* for security now change gid/uid to
				   owner of controlfile */
				if (SetIdentityToOwner() != 0) {
					exit(-1);
				}
				execlp("l3enc", "l3enc", "/tmp/_l3dspool.wav",
				      "/tmp/_l3dspool.mp3", "-br", &bitratestr,
				      NULL);
				execl("./l3enc", "l3enc", "/tmp/_l3dspool.wav",
				      "/tmp/_l3dspool.mp3", "-br", &bitratestr,
				      NULL);
				/* if this gets executed someting went wrong */
				dup2(org_stdin_fd, STDIN_FILENO);
				dup2(org_stdout_fd, STDOUT_FILENO);
				printf("ERROR Failure executing the encoder\n");
				exit(1);
				};
				
			/* Remember the following lines have nothing to do
			   with bypassing the registration code!!! */
			strcpy(encoderinstr, "n\n");
			write(encoderin_fds[1], &encoderinstr, 
			      sizeof(encoderinstr));
			
			waitpid(encoder_pid, &returnvalue, 0);
			
			unlink("/tmp/_l3dspool.mp3");
			unlink("/tmp/_l3dspool.wav");
			
			RemoveControlFile(-1); /* remove current */
			
			if ((returnvalue == 0) && (do_delete != 0)) {
				unlink(sourcename);
			}
		} else {
			RemoveLockFile();
			if (geteuid() != 0) {
				RemoveEncodeLockFile();
				exit(0);
			}
			sleep(30);
		}
	}
}
