/* *** l3spool ***
   *** Filename: l3spool.c
   *** Version: 0.80 [19980210]
   ***          1.00 [19980225] Release version 1.00
*/

#include <dirent.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "l3spool.h"

int	SetIdentityToOwner(void)
{
	struct stat status;
	
	if (geteuid() != 0) {
		return(0);
	}
	
	if (stat(SPOOLDIR"/cfl3.cur", &status) != 0) {
		printf("ERROR: Could not read controlfile\n");
		return(-1);
	}
	
	setgid(status.st_gid);
	setuid(status.st_uid);
	
	return(0);
}

int	getLowestID(void)
{
	int		return_id, cur_id;
	DIR		*spooldir;
	char		tmpstr[1024];
	struct dirent	*direntry;
	struct stat	status;
	
	return_id = MAX_L3SPOOL_ID;
	spooldir = opendir(SPOOLDIR);
	while((direntry = readdir(spooldir)) != NULL) {
		if (sscanf(direntry->d_name, "cfl3.%d", &cur_id) == 1) {
			if (cur_id < return_id)
				if (geteuid() == 0) {
					return_id = cur_id;
				} else {
					strcpy(tmpstr, SPOOLDIR);
					strcat(tmpstr, "/");
					strcat(tmpstr, direntry->d_name);
					if (stat(tmpstr, &status) != 0) {
						printf("ERROR: Could not read controlfile\n");
						return(-1);
					}
					
					if (status.st_uid == geteuid()) {
						return_id = cur_id;
					}
				}
			}
		}
	closedir(spooldir);
	if (return_id != MAX_L3SPOOL_ID)
		return(return_id);
	else
		return(-1);
}

int	GetHighestFileID(void)
{
	int		return_id, cur_id;
	DIR		*spooldir;
	struct dirent	*direntry;
	
	return_id = 0;
	spooldir = opendir(SPOOLDIR);
	while((direntry = readdir(spooldir)) != NULL) {
		if (sscanf(direntry->d_name, "cfl3.%d", &cur_id) == 1) {
			if (cur_id > return_id)
				return_id = cur_id;
			}
		}
	closedir(spooldir);
	return(return_id);
}

int	GetHighestID(void)
{
	FILE	*countfile;
	int	value;

	if ((countfile = fopen(COUNTFILENAME, "r+")) == NULL) {
		printf("WARNING: Unable to open countfile\nChecking for existing files\n");
		return(GetHighestFileID());
	}

	if (fscanf(countfile, "%d", &value) < 1) {
		printf("WARNING: Unable to read countfile\nStarting at 0\n");
		fclose(countfile);
		return(GetHighestFileID());
	}

	fclose(countfile);
	return(value);
}

int	SetHighestID(int value)
{
	FILE	*countfile;
	int	oldumask;

	oldumask = umask(0);

	if ((countfile = fopen(COUNTFILENAME, "w")) == NULL) {
		printf("ERROR: Unable to create countfile\n");
		return(-1);
	}
	
	umask(oldumask);

	if (fprintf(countfile, "%d", value) < 1) {
		printf("ERROR: Unable to write countfile\n");
		fclose(countfile);
		return(-1);
	}

	fclose(countfile);
	return(0);
}

int	CreateLockFile(void)
{
	int	lockfile_fd;
	int	oldumask;
	struct stat status;
		
	if (stat(SPOOLDIR, &status) != 0) {
		oldumask = umask(0);
		printf("Creating spool directory\n");
		mkdir(SPOOLDIR, S_IRWXU | S_IRWXG | S_IRWXO );
		umask(oldumask);
	}
	
	if (lockfile_fd = open(LOCKFILENAME, O_CREAT|O_EXCL, 
	                    S_IRUSR|S_IWUSR) == -1) {
		if (errno == EEXIST)
			return(-1);
		else {
			printf("ERROR: Unable to create lockfile\n");
			exit(1);
			}
		}
	close(lockfile_fd);
	return(0);
}

void	RemoveLockFile(void)
{
	unlink(LOCKFILENAME);
}

int	CreateEncodeLockFile(void)
{
	int	lockfile_fd;
	int	oldumask;
	struct stat status;
		
	if (stat(SPOOLDIR, &status) != 0) {
		oldumask = umask(0);
		printf("Creating spool directory\n");
		mkdir(SPOOLDIR, S_IRWXU | S_IRWXG | S_IRWXO );
		umask(oldumask);
	}
	
	if (lockfile_fd = open(ENCODELOCKFILENAME, O_CREAT|O_EXCL, 
	                    S_IRUSR|S_IWUSR) == -1) {
		if (errno == EEXIST)
			return(-1);
		else {
			printf("ERROR: Unable to create lockfile\n");
			exit(1);
			}
		}
	close(lockfile_fd);
	return(0);
}

void	RemoveEncodeLockFile(void)
{
	unlink(ENCODELOCKFILENAME);
}

void	ReadControlFile(int id, char *sourcename, char *targetname,
			int *do_delete, int *bitrate)
{
	FILE		*ctrlStream;
	char		ctrlfilename[MAXPATHLEN];
	char		workspace[MAXPATHLEN];
	
	sprintf(ctrlfilename, "%s/cfl3.%d", SPOOLDIR, id);
	if ((ctrlStream = fopen(ctrlfilename, "r")) == NULL) {
		printf("ERROR: Unable to open control-file.\n");
		RemoveLockFile();
		exit(1);
		}
	/* format of a control-file
	   <source wav-filename>
	   <target mp3-filename>
	   <delete after encoding flag>
	   <bitrate>
	*/
	fgets(workspace, sizeof(workspace), ctrlStream);
	workspace[strlen(workspace)-1] = 0;
	strcpy(sourcename, workspace);
	fgets(workspace, sizeof(workspace), ctrlStream);
	workspace[strlen(workspace)-1] = 0;
	strcpy(targetname, workspace);
	fgets(workspace, sizeof(workspace), ctrlStream);
	workspace[strlen(workspace)-1] = 0;
	*do_delete = atoi(workspace);
	fgets(workspace, sizeof(workspace), ctrlStream);
	*bitrate = atoi(workspace);
	fclose(ctrlStream);
}

void	WriteControlFile(int id, char *WAVfilename, char *destdir, int do_delete,
                         int bitrate)
{
	FILE		*ctrlStream;
	char		ctrlfilename[MAXPATHLEN];
	char		fullfilename[MAXPATHLEN];
	char		targetfilename[MAXPATHLEN];
			
	sprintf(ctrlfilename, "%s/cfl3.%d", SPOOLDIR, id);
	if ((ctrlStream = fopen(ctrlfilename, "w")) == NULL)
	{
		printf("ERROR: Unable to create control-file.\n");
		RemoveLockFile();
		exit(1);
	}
	/* Lay-out of a control-file 
		<sourcefilename>
		<targetfilename>
		<deleteflag>
		<bitrate>
	*/
	realpath(WAVfilename, fullfilename);
	fprintf(ctrlStream, "%s\n", fullfilename);
	strcpy(&fullfilename[strlen(fullfilename)-3], "mp3");
	strcpy(targetfilename, destdir);
	strcat(&targetfilename, strrchr(&fullfilename, '/'));
	fprintf(ctrlStream, "%s\n", targetfilename);
	fprintf(ctrlStream, "%d\n", do_delete);
	fprintf(ctrlStream, "%d\n", bitrate);	
	fclose(ctrlStream);
}

void	MakeControlFileCurrent(int id)
{
	char	orgfilename[MAXPATHLEN];
	char	curfilename[MAXPATHLEN];
	
	sprintf(orgfilename, "%s/cfl3.%d", SPOOLDIR, id);
	sprintf(curfilename, "%s/cfl3.cur", SPOOLDIR);
	rename(orgfilename, curfilename);
}

void	RemoveControlFile(int id)
{
	char	filename[MAXPATHLEN];
	
	if (id == -1)
		sprintf(filename, "%s/cfl3.cur", SPOOLDIR);
	else
		sprintf(filename, "%s/cfl3.%d", SPOOLDIR, id);
	unlink(filename);
}
