#include "diawxxt.h"


PUBLIC PAGEPAD::PAGEPAD(const char *_title)
{
	title = strdup(_title);
	start = 0;
	stop = 0;
	below = false;
}
PUBLIC PAGEPAD::~PAGEPAD()
{
	free ((char*)title);
}
	
PUBLIC BOOK::BOOK (FORMBASE *_parent, const char *_id)
	: FORMBASE (_parent,_id)
{
	nbpage = 0;
	nopage = 0;
	offsetx = offsety = b_offsety = 0;
}
PUBLIC void BOOK::New_page (FORMBASE *sub, const char *s)
{
	pads[nbpage++] = new PAGEPAD(s);
	MFORM_C *m = alloc_mf();
	m->c = sub;
	m->type = T_FORM;
}
PUBLIC MFORM *BOOK::New_form (const char *s)
{
	pads[nbpage++] = new PAGEPAD(s);

	MFORM *ret = new MFORM (this,"");
	MFORM_C *m = alloc_mf();
	m->c = ret;
	m->type = T_FORM;
	return ret;
}
PRIVATE void BOOK::drawsel (wxDC *dc, int no, bool select)
{
	// The selected pad has a double edge because it is on
	// top while the non selected ones have a single edge
	// which shows that they are lower
	int x0 = pads[no]->start+1;
	int x1 = x0 + 3;
	int x3 = pads[no]->stop;
	int x2 = x3 - 3;
	if (pads[no]->below){
		int y1 = b_offsety;
		int y2 = b_offsety + 1;
		int by2 = b_offsety + 18;
		dc->SetPen (pen_black);
		dc->DrawText (pads[no]->title,x1+7,b_offsety + 15 -GetCharHeight());
		dc->DrawLine (x0 == 1 ? x0 +1 : x0 ,y1,x1,y1);
		dc->DrawLine (x0,y2,x1,y2);
		dc->DrawLine (x1,by2,x2,by2);

		dc->DrawLine (x2,y1,x3,y1);
		dc->DrawLine (x2,y2,x3,y2);


		dc->SetPen (pen_white);
		dc->DrawLine (x1,by2,x1,y1+1);
		if (!select) dc->SetPen (pen_back);
		{
			int x1_1 = x1 + 1;
			dc->DrawLine (x1_1,by2,x1_1,y1+1);
		}

		dc->SetPen (pen_black);
		dc->DrawLine (x2,by2,x2,y1);
		if (!select) dc->SetPen (pen_back);
		{
			int x2_1 = x2 + 1;
			dc->DrawLine (x2_1,by2,x2_1,y1+1);
			dc->DrawLine (x1,by2+1,x2,by2+1);
		}

		dc->SetPen (select ? pen_back : pen_black);
		dc->DrawLine (x1,y1,x2,y1);
		dc->DrawLine (x1,y2,x2,y2);
	}else{
		int y1 = offsety - 2;
		int y2 = offsety - 1;
		dc->SetPen (pen_black);
		dc->DrawText (pads[no]->title,x1+7,15-GetCharHeight());
		dc->SetPen (pen_white);
		dc->DrawLine (no == 0 ? x0 +1 : x0 ,y1,x1,y1);
		dc->DrawLine (x0,y2,x1,y2);
		dc->DrawLine (x1,2,x1,y1+1);
		dc->DrawLine (x1,2,x2,2);
		dc->DrawLine (x2,y1,x3,y1);
		dc->DrawLine (x2,y2,x3,y2);
		if (!select) dc->SetPen (pen_back);
		{
			int x1_1 = x1 - 1;
			dc->DrawLine (x1_1,2,x1_1,y1-1);
			dc->DrawLine (x1,1,x2,1);
		}

		dc->SetPen (pen_black);
		dc->DrawLine (x2,2,x2,y1);
		if (!select) dc->SetPen (pen_back);
		{
			int x2_1 = x2+1;
			dc->DrawLine (x2_1,2,x2_1,y1-1);
		}

		dc->SetPen (select ? pen_back : pen_white);
		dc->DrawLine (x1,y1,x2,y1);
		dc->DrawLine (x1,y2,x2,y2);
	}
}
PUBLIC void BOOK::selpage (int no)
{
	if (no != nopage){
		// wxDC *dc = GetDC();
		tbc[nopage]->c->Show(FALSE);
		drawsel (dc,nopage,false);
		nopage = no;
		tbc[nopage]->c->Show(TRUE);
		drawsel (dc,nopage,true);
	}
}
PRIVATE bool BOOK::locatepad (int x, bool below)
{
	for (int i=0; i<nbpage; i++){
		if (pads[i]->below == below
			&& pads[i]->start < x
			&& x < pads[i]->stop){
			selpage (i);
			tbc[i]->c->SetFocus();
			return true;
		}
	}
	return false;
}
PUBLIC void BOOK::OnEvent(wxMouseEvent & event)
{
	if (event.ButtonDown()){
		int y = (int)event.y;
		int x = (int)event.x;
		if (y < offsety){
			locatepad(x,false);
		}else if (y > b_offsety){
			locatepad(x,true);
		}
	}
}
PUBLIC void BOOK::OnPaint ()
{
	int w_width,w_height;
	GetSize(&w_width,&w_height);
	int y0 = offsety - 2;
	int y1 = y0 + 1;
	int lasty = b_offsety + 1;
	int lastx = w_width-4;
	dc->SetPen (pen_white);
	//dc->DrawLine (1,y0,3,y0);
	//dc->DrawLine (pads[nbpage-1]->stop-3,y0,lastx,y0);
	//dc->DrawLine (pads[nbpage-1]->stop-3,y1,lastx-1,y1);
	dc->DrawLine (1,y0,lastx,y0);
	dc->DrawLine (2,y1,lastx-1,y1);
	dc->DrawLine (1,y0,1,lasty);
	dc->DrawLine (2,y1,2,lasty);

	// Bottom shadow
	dc->SetPen (pen_black);
	dc->DrawLine (1,lasty,lastx,lasty);
	int lasty_1 = lasty - 1;
	dc->DrawLine (2,lasty_1,lastx,lasty_1);

	// Right shadow
	dc->DrawLine (lastx,lasty,lastx,y0+2);
	int lastx_1 = lastx - 1;
	dc->DrawLine (lastx_1,lasty,lastx_1,y0+2);
	
	for (int i=0; i<nbpage; i++){
		drawsel (dc,i,i==nopage);
	}
}


