#include "diawxxt.h"

PRIVATE void FORMBASE::init(FORMBASE *_parent, const char *_id)
{
	tbc = (MFORM_C**) malloc (400 * sizeof(MFORM_C*));
	maxtbc = 400;
	nbc = 0;
	f_parent = _parent;
	id = strdup(_id);
	modified = true;
 	stretch_mode = STRETCH_NONE;
	//setFont(new Font("Helvetica", Font.PLAIN, 14));
	pref_width = pref_height = -1;
}

PUBLIC FORMBASE::FORMBASE(FORMBASE *_parent, const char *_id)
	: wxPanel (_parent)
{
	init (_parent,_id);
}
PUBLIC FORMBASE::FORMBASE(const char *_id)
{
	init (NULL,_id);
}

PUBLIC FORMBASE::~FORMBASE()
{
	for (int i=0; i<nbc; i++) delete tbc[i];
	free (tbc);
	free (id);
}

// Compute the path of a form, by appending the IDs of the parents
void formbase_getabspath(FORMBASE *p, char *path)
{
	if (p->f_parent != NULL){
		char tmp[300];
		formbase_getabspath(p->f_parent,tmp);
		sprintf (path,"%s.%s",tmp,p->id);
	}else{
		strcpy (path,p->id);
	}
}
PUBLIC VIRTUAL bool FORMBASE::may_stretch()
{
	return stretch_mode != STRETCH_NONE;
}
PUBLIC VIRTUAL void FORMBASE::stretch (
	int,	// new_width
	int)	// new_height
{
}
PUBLIC bool FORMBASE::was_modified()
{
	return modified;
}
PUBLIC VIRTUAL void FORMBASE::set_modified()
{
	modified = true;
	if (f_parent != NULL) f_parent->set_modified();
}

// Display the current value of each field of the dialog
PUBLIC VIRTUAL void FORMBASE::dump()
{
	for (int i=0; i<nbc; i++){
		MFORM_C *c = tbc[i];
		if (c->type == T_FORM || c->type == T_SHEET){
			MFORM *m = (MFORM*)c->c;
			m->dump();
		}else if (c->type != T_UNKNOWN
			&& c->type != T_BUTTON
			&& c->type != T_SKIP
			&& c->type != T_ICON
			&& c->type != T_NEWLINE
			&& c->type != T_HLINE
			&& c->type != T_VLINE
			&& c->type != T_FILL
			&& c->type != T_LABEL
			&& c->type != T_RICHTEXT
			&& c->type != T_CLIST
			&& c->type != T_BUTTONFILL){
			const char *val = "";
			char tmp[100];
			if (c->type == T_STRING){
				val = ((wxText*)c->c)->GetValue();
			}else if (c->type == T_CHECKBOX){
				val = ((wxCheckBox*)c->c)->GetValue() ? "1" : "0";
			}else if (c->type == T_RADIO){
				#if 0
					CHECKBOX_RADIO *r = (CHECKBOX_RADIO*)c->c;
					if (r->GetValue()){
						sprintf (tmp,"%d",r->instance);
						val = tmp;
					}else{
						continue;
					}
				#else
					CHECKBOX_RADIO *radio = c->radio;
					if (radio->state){
						sprintf (tmp,"%d",radio->instance);
						val = tmp;
					}else{
						continue;
					}
				#endif
			}else if (c->type == T_COMBO){
				val = ((COMBO*)c->c)->getText();
			}else if (c->type == T_LIST){
				#if 0
				int sel = ((wxList*)c->c).getSelectedIndex();
				sprintf (tmp,"%d",sel);
				val = tmp;
				#endif
				val = "1";
			}else{
				fprintf (stderr,"code dump %d\n",c->type);
			}
			char path[300];
			formbase_getabspath(this,path);
			fprintf (mform_fout,"dump %s %s %s\n",path,c->id,val);
		}
	}
}

PUBLIC wxFrame *FORMBASE::getframe()
{
	MAINFORM *mf = gettop();
	return mf->fram;
}

/*
	Return the TOP MAINFORM owning this  FORM
*/
PUBLIC MAINFORM *FORMBASE::gettop()
{
	MAINFORM *ret = NULL;
	if (f_parent == NULL){
		ret = (MAINFORM*)this;
	}else{
		ret = f_parent->gettop();
	}
	return ret;
}

PUBLIC VIRTUAL void FORMBASE::dolayout()
{
}

PUBLIC void FORMBASE::doalllayout()
{
	int nb = nbc;
	for (int i=0; i<nb; i++){
		MFORM_C *c = tbc[i];
		if (c->type == T_FORM
			|| c->type == T_COMBO
			|| c->type == T_CLIST
			|| c->type == T_SHEET){
			FORMBASE *b = (FORMBASE*)c->c;
			b->doalllayout();
		}
	}
	dolayout();
	if (pref_width == -1) GetSize (&pref_width,&pref_height);
}

PUBLIC MFORM_C *FORMBASE::alloc_mf(const char *_id)
{
	set_modified();
	MFORM_C *m = new MFORM_C(_id);
	if (nbc == maxtbc){
		maxtbc += 400;
		tbc = (MFORM_C**)realloc(tbc,maxtbc*sizeof(MFORM_C*));
	}
	tbc[nbc++] = m;
	return m;
}
PUBLIC MFORM_C *FORMBASE::alloc_mf()
{
	return alloc_mf("");
}

/*
	Give focus to a specific field of this form.
	Return -1 if the form did not contain the proper field
*/
PUBLIC VIRTUAL int FORMBASE::setcurfield (
	const char *,
	wxWindow *[],
	int &)
{
	return -1;
}

/*
	Give focus to a specific field of this form.
	Return -1 if the form did not contain the proper field
*/
PUBLIC int MFORM::setcurfield (
	const char *id_suffix,
	wxWindow *tbfocus[],
	int &nbfocus)
{
	/* #Specification: setcurfield / strategy
		The setcurfield assume that the ID of field which may accept
		focus has this form

		#
		A single letter following by a suffix. Generally the
		suffix is the internal field number used by Linuxconf. For
		a text field (New_string()), this is S20 for example.

		When remadmin receive a Curfield command, it identify the proper
		MAINFORM and then lookup the proper ID in the various fields
		and sub-form.

		One the field is identify, proper action is done to make it visible
		if needed.
	*/
	int ret = -1;
	#if 0
		extern FILE *ftty;
		fprintf (ftty,"MFORM %s curfield %d %s\n",id,nbc,id_suffix);
		fflush (ftty);
	#endif
	for (int i=0; i<nbc; i++){
		MFORM_C *c = tbc[i];
		wxWindow *cc = c->c;
		const char *id = c->id;
		if ((c->type == T_BUTTONFILL || cc != NULL)
			&& c->type != T_BUTTON
			&& id != NULL
			&& id[0] != '\0'
			&& strcmp(id,id_suffix)==0){
			#if 0
				fprintf (ftty,"%s %s\n",id,id_suffix);
				fflush (ftty);
			#endif
			if (cc != NULL) tbfocus[nbfocus++] = cc;
			ret = 0;
		}else if (c->type == T_FORM){
			FORMBASE *b = (FORMBASE*)cc;
			ret = b->setcurfield (id_suffix,tbfocus,nbfocus);
		}
		if (ret == 0){
			int ww,wh;
			GetSize (&ww,&wh);
			int fx,fy,fw,fh;
			if (cc != NULL){
				cc->GetPosition (&fx,&fy);
				cc->GetSize (&fw,&fh);
			}else{
				fx = c->x;
				fy = c->y;
				fw = c->pref_width;
				fh = c->pref_height;
			}
			if (fy < 0
				|| fy + fh > wh){
				vmoveitems(voffset + fy);
			}
			break;
		}
	}
	if (ret == 0) tbfocus[nbfocus++] = this;
	return ret;
}
/*
	Give focus to a specific field of this form.
	Return -1 if the form did not contain the proper field
*/
PUBLIC int BOOK::setcurfield (
	const char *id_suffix,
	wxWindow *tbfocus[],
	int &nbfocus)
{
	int ret = -1;
	#if 0
		extern FILE *ftty;
		fprintf (ftty,"BOOK %s curfield %d %s\n",id,nbc,id_suffix);
		fflush (ftty);
	#endif
	for (int i=0; i<nbc; i++){
		MFORM_C *c = tbc[i];
		FORMBASE *m = (FORMBASE *)c->c;
		#if 0
			fprintf (ftty,"book %d m=%p\n",i,m);
			fflush (ftty);
		#endif
		ret = m->setcurfield (id_suffix,tbfocus,nbfocus);
		#if 0
			fprintf (ftty,"book %d m=%p ret=%d\n",i,m,ret);
			fflush (ftty);
		#endif
		if (ret == 0){
			selpage (i);
			break;
		}
	}
	if (ret == 0) tbfocus[nbfocus++] = this;
	return ret;
}


/*
	We walk recursivly to change the state of all CHECKBOX_RADIO
	sharing the same id. Only one will be left "on".
*/
PUBLIC void FORMBASE::processradio (MFORM_C *csel)
{
	for (int i=0; i<nbc; i++){
		MFORM_C *c = tbc[i];
		if (c->type == T_FORM){
			FORMBASE *b = (FORMBASE*)c->c;
			b->processradio (csel);
		}else if (c->type == T_RADIO){
			if (strcmp(csel->id,c->id)==0){
				bool newstate = csel == c;
				#if 0
					c->c->SetValue (newstate);
				#else
					c->radio->state = newstate;
					drawradio_but (c);
				#endif
			}
		}
	}
}

/*
	Draw only the button part of the radio button
*/
PUBLIC void FORMBASE::drawradio_but (MFORM_C *c)
{
	int x= c->x;
	int y= c->y;
	bool state = c->radio->state;
	const int nbpix=5;
	int x0 = x+1;
	int x1 = x+1+nbpix;
	int x2 = x+1+2*nbpix;
	int y0 = y+4;
	int y1 = y+4+nbpix;
	int y2 = y+4+2*nbpix;
	dc->SetPen (state ? pen_black : pen_white);
	dc->DrawLine (x0,y1,x1,y0);
	dc->DrawLine (x0+1,y1,x1,y0+1);
	dc->DrawLine (x1,y0,x2,y1);
	dc->DrawLine (x1,y0+1,x2-1,y1);
	dc->SetPen (state ? pen_white : pen_black);
	y1++;
	dc->DrawLine (x0,y1,x1,y2);
	dc->DrawLine (x0+1,y1,x1,y2-1);
	dc->DrawLine (x1,y2,x2,y1);
	dc->DrawLine (x1,y2-1,x2-1,y1);
}

