#pragma interface
#ifndef CONFDB_H
#define CONFDB_H

#ifndef MISC_H
	#include "misc.h"
#endif

class PRIVILEGE;

class CONFOBJ: public ARRAY_OBJ{
public:
	const char *sys;		// Pointer from tbsys[] below or NULL
	SSTRING key;
	SSTRING val;
	/*~PROTOBEG~ CONFOBJ */
public:
	CONFOBJ (const char *_sys,
		 const char *_key,
		 const char *_val);
	/*~PROTOEND~ CONFOBJ */
};

class CONFDB: public ARRAY{
	char *cursys;	// NULL or point into tbsys[]
	char **tbsys;
	int nbsys;
	int maxsys;
	CONFIG_FILE *fcfg;
	/*~PROTOBEG~ CONFDB */
public:
	CONFDB (CONFIG_FILE&_fcfg);
	CONFDB (void);
	void add (const char *prefix,
		 const char *key,
		 const SSTRING&val);
	virtual void add (const char *prefix,
		 const char *key,
		 const char *val);
	void add (const char *prefix,
		 const char *key,
		 int val);
	void addline (const char *buf);
	virtual int archive (SSTREAM&ss, const char *_sys);
private:
	void delsys (const char *_sys);
public:
	virtual int extract (SSTREAM&ss, const char *_sys);
	virtual int getall (const char *prefix,
		 const char *key,
		 SSTRINGS&lst,
		 bool copy);
	CONFOBJ *getitem (int no);
	const char *getval (const char *prefix,
		 const char *key);
	virtual const char *getval (const char *prefix,
		 const char *key,
		 const char *defval);
	double getvalf (const char *prefix,
		 const char *key,
		 double defval);
	int getvalnum (const char *prefix,
		 const char *key,
		 int defval);
private:
	void init (void);
	char *locatesys (const char *sys);
public:
	void patchsys (void);
	virtual void removeall (const char *prefix,
		 const char *key);
	void replace (const char *prefix,
		 const char *key,
		 char val);
	void replace (const char *prefix,
		 const char *key,
		 const SSTRING&val);
	void replace (const char *prefix,
		 const char *key,
		 const SSTRINGS&vals);
	void replace (const char *prefix,
		 const char *key,
		 const char *val);
	void replace (const char *prefix,
		 const char *key,
		 double val);
	void replace (const char *prefix,
		 const char *key,
		 int val);
	void replace (const char *prefix,
		 const char *key,
		 long val);
	virtual int save (PRIVILEGE *priv);
	int save (void);
private:
	void savesys (char *sys, SSTREAM&ss);
public:
	virtual void setcursys (const char *_subsys);
	~CONFDB (void);
	/*~PROTOEND~ CONFDB */
};


#endif

