#include <string.h>
#include <misc.h>
#include "../module_apis/dnsconf_apidef.h"
#include "dhcpd.h"
#include "dhcpd.m"
#include <ipstuff.h>
#include <netconf_def.h>

static HELP_FILE help_leases ("dhcpd","leases");
extern CONFIG_FILE f_lease;

class LEASE: public ARRAY_OBJ{
public:
	char ip[LEN_IP_ASC];
	SSTRING host;
	SSTRING starts;
	SSTRING ends;
	char mac[6*2+5+1];
	/*~PROTOBEG~ LEASE */
public:
	LEASE (const char *_host,
		 const char *_ip,
		 const char *_mac,
		 const char *_starts,
		 const char *_ends);
	/*~PROTOEND~ LEASE */
};

PUBLIC LEASE::LEASE(
	const char *_host,
	const char *_ip,
	const char *_mac,
	const char *_starts,
	const char *_ends)
{
	strcpy (ip,_ip);
	host.setfrom(_host);
	starts.setfrom (_starts);
	ends.setfrom (_ends);
	strcpy (mac,_mac);
}

class LEASES: public ARRAY{
	/*~PROTOBEG~ LEASES */
public:
	LEASES (void);
	LEASE *getitem (int no)const;
	void showalloc (void);
	/*~PROTOEND~ LEASES */
};

PUBLIC LEASE *LEASES::getitem (int no) const
{
	return (LEASE*)ARRAY::getitem(no);
}

PUBLIC LEASES::LEASES()
{
	FILE *fin = f_lease.fopen ("r");
	if (fin != NULL){
		char buf[1000];
		SSTRING starts,ends,ip,mac,host;
		while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
			char keyw[100];
			char *pt = strchr (buf,';');
			if (pt != NULL) *pt = '\0';
			pt = str_copyword (keyw,buf,sizeof(keyw));
			if (strcmp(keyw,"lease")==0){
				ip.copyword (pt);
			}else if (strcmp(keyw,"starts")==0){
				pt = str_skip(pt);
				pt = str_skipword(pt);
				starts.copyword (pt);
			}else if (strcmp(keyw,"ends")==0){
				pt = str_skip(pt);
				pt = str_skipword(pt);
				ends.copyword (pt);
			}else if (strcmp(keyw,"hardware")==0){
				pt = str_copyword(keyw,pt,sizeof(keyw));
				if (strcmp(keyw,"ethernet")==0){
					mac.copyword (pt);
				}
			}else if (strcmp(keyw,"client-hostname")==0){
				str_extract (pt,host);
			}else if (strcmp(keyw,"}")==0){
				add (new LEASE(host.get(),ip.get(),mac.get(),starts.get()
					,ends.get()));
				host.setfrom ("");
				ip.setfrom ("");
				mac.setfrom ("");
				starts.setfrom ("");
				ends.setfrom ("");
			}
		}
		fclose (fin);
	}
}

/*
	Update the DNS from the dhcp server
	Return 0 if the DNS was updated, != 0 if the DNS was left unchanged
*/
int leases_updatedns (
	DNSCONF_API *api,
	bool command_line,
	bool verbose)
{
	int ret = -1;
	const char *defdom = dhcp_getdefdom();
	if (defdom == NULL || defdom[0] == '\0'){
		if (command_line){
			fprintf (stderr,MSG_U(E_DNSDOM
				,"You must specify in which DNS domain\n"
				 "DHCP host must be inserted"));
		}else{
			xconf_error (MSG_R(E_DNSDOM));
		}
	}else{
		LEASES leases;
		bool must_write = false;
		for (int i=0; i<leases.getnb(); i++){
			LEASE *l = leases.getitem(i);
			const char *host = l->host.get();
			if (host[0] != '\0' && strlen(host)<999){
				// We remove the . if there (should not) and then add
				// the default domain
				char fqdn[1000];
				if (strchr(host,'.')!=NULL){
					strncpy (fqdn,host,sizeof(fqdn)-1);
					fqdn[sizeof(fqdn)-1] = '\0';
					char *pt = strchr(fqdn,'.');
					pt++;
					strcpy (pt,defdom);
				}else{
					snprintf (fqdn,sizeof(fqdn)-1,"%s.%s",host,defdom);
				}
				const char *ip = l->ip;
				IP_ADDRS ips;
				int nbip = api->geta (host,ips);
				if (nbip == 0){
					if (api->setfromip (fqdn,ip) != 0) must_write = true;
				}else if (nbip != 1
					|| ips.getitem(0)->cmp(ip)!=0){
					if (api->setfromip (fqdn,ip) != 0) must_write = true;
				}
			}
		}
		if (must_write){
			api->write();
			if (command_line){
				if (verbose) printf (MSG_U(N_UPDATED,"DNS was updated"));
			}else{
				xconf_notice (MSG_R(N_UPDATED));
			}
			ret = 0;
		}else{
			if (command_line){
				if (verbose) printf (MSG_U(N_UPTODATE,"DNS was already up to date"));
			}else{
				xconf_notice (MSG_R(N_UPTODATE));
			}
		}
	}
	return ret;
}

static int cmp_by_name (const ARRAY_OBJ *o1, const ARRAY_OBJ *o2)
{
	LEASE *l1 = (LEASE*)o1;
	LEASE *l2 = (LEASE*)o2;
	return l1->host.cmp(l2->host);
}
PUBLIC void LEASES::showalloc()
{
	DIALOG_RECORDS dia;
	dia.newf_head ("",MSG_U(H_SHOWALLOC,"Host\tIp\tMac address\tStarts\tStops"));
	sort (cmp_by_name);
	for (int i=0; i<getnb(); i++){
		LEASE *l = getitem(i);
		char buf[200];
		snprintf (buf,sizeof(buf)-1,"%s\t%s\t%s\t%s",l->ip,l->mac
			,l->starts.get(),l->ends.get());
		dia.new_menuitem (l->host.get(),buf);
	}
	int nof=0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_SHOWALLOC,"Lease allocations")
			,""
			,help_leases
			,nof,0);
		if (code == MENU_QUIT || code == MENU_ESCAPE){
			break;
		}
	}
}

/*
	Present the list of host currently served by the dhcpd server
*/
void leases_showalloc()
{
	LEASES leases;
	leases.showalloc();
}

