#pragma interface
#ifndef DNSCONF_H
#define DNSCONF_H

#include <limits.h>

class TBFILE;
class RECORD_IN_SOA;
class RECORD_IN_NS;
class FQHOST;
class DNS;
class POPEN;
class IP_ADDRS;

#ifndef MISC_H
	#include <misc.h>
#endif

class IN_NAME: public SSTRING{
	/*~PROTOBEG~ IN_NAME */
	/*~PROTOEND~ IN_NAME */
};

#ifndef IPSTUFF_H
	#include <ipstuff.h>
#endif

enum RECORD_TYPE{
	RTYPE_COMMENT,
	RTYPE_SOA,
	RTYPE_A,
	RTYPE_PTR,
	RTYPE_NS,
	RTYPE_MX,
	RTYPE_CNAME,
	RTYPE_INCLUDE,
	RTYPE_END_INCLUDE,
	RTYPE_ORIGIN
};

class RECORD: public ARRAY_OBJ{
public:
	RECORD_TYPE id;		// Used by the function is() to identify
				// the type of a record
	SSTRING comment;
	//virtual IP_ADDR *getip()const =0;
	//virtual IN_NAME *getname() const =0;
	virtual void print (FILE *fout) const =0;
	/*~PROTOBEG~ RECORD */
public:
	RECORD (RECORD_TYPE _id);
	int cmp (RECORD *other);
	virtual int cmp_left (const char *);
	virtual int cmp_right (const char *);
protected:
	virtual int cmpsame (const RECORD *);
public:
	virtual const char *get_left (void);
	bool is (RECORD_TYPE _id);
	virtual int set (const RECORD *);
	void setcomment (const char *str);
	virtual void sethostpart (const char *);
	/*~PROTOEND~ RECORD */
};

class RECORD_COMMENT: public RECORD{
	/*~PROTOBEG~ RECORD_COMMENT */
public:
	RECORD_COMMENT (const char *str);
	void print (FILE *fout)const;
	/*~PROTOEND~ RECORD_COMMENT */
};

class RECORD_PARSE;

class RECORD_IN: public RECORD{
protected:
	long ttl;
	char nottl;	// Is there a ttl field
	char ttlstr[20];
	/*~PROTOBEG~ RECORD_IN */
public:
	RECORD_IN (RECORD_TYPE _id);
	RECORD_IN (const RECORD_PARSE&p, RECORD_TYPE _id);
	/*~PROTOEND~ RECORD_IN */
};

class RECORD_IN_A: public RECORD_IN{
public:
	IN_NAME name;
	IP_ADDR addr;
	/*~PROTOBEG~ RECORD_IN_A */
public:
	RECORD_IN_A (const RECORD_PARSE&p);
	RECORD_IN_A (const char *_name, const char *_addr);
protected:
	int cmp_left (const char *str);
	int cmp_right (const char *str);
public:
	int cmphost (const char *host);
protected:
	int cmpsame (const RECORD *other);
public:
	virtual const char *get_left (void);
	void print (FILE *fout)const;
	int set (const RECORD *other);
	void sethostpart (const char *hostpart);
	void setip (const char *ip);
	/*~PROTOEND~ RECORD_IN_A */
};
class RECORD_IN_PTR: public RECORD_IN{
public:
	IN_NAME name;
	IN_NAME addr;
	/*~PROTOBEG~ RECORD_IN_PTR */
public:
	RECORD_IN_PTR (const RECORD_PARSE&p);
	RECORD_IN_PTR (const char *iprev,
		 const char *host);
protected:
	int cmp_left (const char *str);
	int cmp_right (const char *str);
public:
	int cmpip (const char *ip);
protected:
	int cmpsame (const RECORD *other);
public:
	void print (FILE *fout)const;
	int set (const RECORD *other);
	void sethostpart (const char *hostpart);
	/*~PROTOEND~ RECORD_IN_PTR */
};

class RECORDS: public ARRAY{
	/*~PROTOBEG~ RECORDS */
public:
	RECORD *getitem (int no)const;
	int save (TBFILE&tbf)const;
	/*~PROTOEND~ RECORDS */
};

class ORIGIN: public ARRAY_OBJ{
public:
	SSTRING origin;
	RECORDS tbrec;
	/*~PROTOBEG~ ORIGIN */
public:
	ORIGIN (const char *_origin);
	int getalladr (IP_ADDRS&adrs);
	void print (bool save_ori, TBFILE&tbf)const;
	int was_modified (void);
	/*~PROTOEND~ ORIGIN */
};

class ORIGINS: public ARRAY{
	/*~PROTOBEG~ ORIGINS */
public:
	ORIGIN *getitem (int no)const;
private:
	int parseend (char *pt,
		 RECORD_PARSE&p,
		 ORIGIN *ori);
	int parsespecial (const char *key,
		 const char *pt,
		 const char *first_origin,
		 TBFILE&tbf,
		 ORIGIN *&ori);
public:
	int read (const char *named_dir,
		 const char *fname,
		 const char *first_origin,
		 bool extract);
	int save (const char *named_dir,
		 const char *fname)const;
	/*~PROTOEND~ ORIGINS */
};

class PRIMARY_EDITINFO;

class PRIMARY: public ARRAY_OBJ{
public:
	SSTRING domain;
	SSTRING domainv;	// Use to present the domain
				// to the user, differ from domain
				// only for reverse mapping .. so far
	SSTRING file;
	ORIGINS origins;	// origins->getitem(0) is set by default
				// to the domain.
	char isrev;		// Tell if it is a domain or reverse
				// mapping primary.
	/*~PROTOBEG~ PRIMARY */
public:
	PRIMARY (const char *_domain,
		 const char *_file,
		 const char *named_dir,
		 bool extract);
	PRIMARY (void);
	void addrec (RECORD *rec);
	int edit (DNS&dns);
	void edithosts (DNS&dns);
	void format_revision (char *buf);
	int geta (SSTRING&dom, IP_ADDRS&adrs);
	int geta (SSTRING&dom, RECORDS&recs);
	int getalladr (IP_ADDRS&adrs);
	int getcname (SSTRING&dom, SSTRING&cname);
private:
	RECORD *getfirst (RECORD_TYPE rtype);
public:
	int getmx (SSTRING&dom, RECORDS&recs);
	int getmx (SSTRING&dom, SSTRINGS&strs);
	int getns (SSTRING&dom, RECORDS&recs);
	int getns (SSTRING&dom, SSTRINGS&strs);
	RECORD_IN_SOA *getsoa (void);
	virtual int is_reverse (void);
	int locate_left (FQHOST&fq,
		 RECORD_TYPE rtype,
		 RECORDS&recs);
	int locate_rev (const char *fullname, IP_ADDRS&tba);
	int locate_right (const char *value,
		 RECORD_TYPE rtype,
		 SSTRINGS&recs);
	int set (FQHOST&fq, RECORD *new_rec);
	virtual void setfromv (void);
	void setupdia (DIALOG&dia,
		 DNS&dns,
		 int level,
		 PRIMARY_EDITINFO&info);
	int unset (RECORD *new_rec);
	int unset_left (const char *left);
	void updatesoa (void);
	int was_modified (void);
	int write (bool bind8,
		 FILE *fout,
		 const char *named_dir)const;
	/*~PROTOEND~ PRIMARY */
};

class PRIMARY_REV: public PRIMARY{
public:
	/*~PROTOBEG~ PRIMARY_REV */
public:
	PRIMARY_REV (const char *_domain,
		 const char *_file,
		 const char *named_dir,
		 bool extract);
	PRIMARY_REV (void);
	int is_reverse (void);
	void setfromv (void);
	/*~PROTOEND~ PRIMARY_REV */
};


class PRIMARYS: public ARRAY{
	/*~PROTOBEG~ PRIMARYS */
public:
	int adddomain (const char *domain,
		 const char *example,
		 DNS&dns);
	void edit (DNS&dns);
	void editbydom (DNS&dns);
	int getalladr (IP_ADDRS&adrs);
	PRIMARY *getitem (FQHOST&fq,
		 char *hostpart,
		 int dontitself);
	PRIMARY *getitem (FQHOST&fq, char *hostpart);
	PRIMARY *getitem (const char *name)const;
	PRIMARY *getitem (int no)const;
	int locate_right (const char *value,
		 RECORD_TYPE rtype,
		 SSTRINGS&recs);
	virtual PRIMARY *new_PRIMARY (void);
private:
	void setselect (DIALOG&dia);
public:
	int write (bool bind8,
		 FILE *fout,
		 const char *named_dir)const;
	/*~PROTOEND~ PRIMARYS */
};
class PRIMARYS_REV: public PRIMARYS{
	/*~PROTOBEG~ PRIMARYS_REV */
public:
	PRIMARY *new_PRIMARY (void);
	/*~PROTOEND~ PRIMARYS_REV */
};

class SECONDARY: public ARRAY_OBJ{
public:
	SSTRING domain;
	IP_ADDR addr[4];
	SSTRING backup;
	/*~PROTOBEG~ SECONDARY */
public:
	SECONDARY (const char *_domain,
		 const char *_backup,
		 const char **_ip_addr,
		 int nb_ip);
	SECONDARY (void);
	int edit (DNS&dns);
	void write (bool bind8,
		 FILE *fout,
		 const char *named_dir)const;
	/*~PROTOEND~ SECONDARY */
};

class SECONDARYS: public ARRAY{
	/*~PROTOBEG~ SECONDARYS */
public:
	void edit (DNS&dns);
	SECONDARY *getitem (const char *name)const;
	SECONDARY *getitem (int no)const;
	void write (bool bind8,
		 FILE *fout,
		 const char *named_dir)const;
	/*~PROTOEND~ SECONDARYS */
};

class CACHEFILE: public ARRAY_OBJ{
public:
	SSTRING domain;
	SSTRING path;
	/*~PROTOBEG~ CACHEFILE */
public:
	CACHEFILE (const char *_domain, const char *_path);
	void abspath (const char *basepath, char *abspath);
	const char *getname (void);
	/*~PROTOEND~ CACHEFILE */
};
class CACHEFILES: public ARRAY{
public:
	/*~PROTOBEG~ CACHEFILES */
public:
	CACHEFILE *getitem (int no);
	/*~PROTOEND~ CACHEFILES */
};

/* #Specification: dnsconf / forwarders / number of
	dnsconf arbitrarily limit the number of forwarders to 5.
*/
#define MAX_FORWARDERS	5
class BIND8_LEXPARSE;

class DNS: public ARRAY_OBJ{
	SSTRING pathcfg;
	CACHEFILES cachefiles;
	PRIMARYS primarys;
	PRIMARYS_REV primarys_rev;
	SECONDARYS secondarys;
	IP_ADDR forwarders[MAX_FORWARDERS];
	int nbforward;
	SSTRING xfernets;
	SSTRING bogusns;
	SSTRING searchlist;
	struct {
		char forward_only;
		char no_recursion;
		char no_fetch_glue;
		char query_log;
		char fake_iquery;
		SSTRINGS unknown;
	} options;
public:
	bool bind8;		// Is it a bind8 or bind4 configuration
	/*~PROTOBEG~ DNS */
public:
	DNS (bool extract);
	DNS (void);
	void basiccheck (void);
	void check (void);
private:
	int check_dup (SSTRING&full,
		 IP_ADDRS&tba,
		 char doreverse[4]);
public:
	int edit (void);
	void editfeatures (void);
	void editforwarders (void);
	void editmxs (void);
	void editone (const char *name);
private:
	void editone_rev (SSTRING&full);
public:
	void editrecs (const char *preset);
	int empty (void);
	PRIMARY *finddomain (const char *name)const;
	PRIMARY *findprimary (const char *host);
	SECONDARY *findsecond (const char *name)const;
	void getalladr (IP_ADDRS&adrs);
	long getrevdate (void);
private:
	void init (bool extract);
public:
	void listconfigs (void);
	PRIMARY *locate_domain (const char *domain);
	int locate_ip (const char *ip, SSTRINGS&recs);
private:
	void mkzonedir (void);
public:
	int newdomain (const char *domain, const char *example);
private:
	void parse4 (char *buf,
		 const char *fpath,
		 int noline,
		 bool extract);
	void parse8 (BIND8_LEXPARSE&lex, bool extract);
	void parseforwarders (BIND8_LEXPARSE&lex);
	void parseoptions (BIND8_LEXPARSE&lex);
	void parsezone (BIND8_LEXPARSE&lex, bool extract);
public:
	int set (const char *fqhost,
		 const char *tbip[],
		 int nbip);
	int set (const char *fqhost,
		 const char *tbip[],
		 int nbip,
		 char doreverse[],
		 int &matchrev);
	int setcname (const SSTRING&fqhost, const SSTRING&cname);
private:
	void setconfig (SSTRING&p);
public:
	int setfromip (const char *host, const char *ip);
	void setip_in_dia (DIALOG&dia,
		 int level,
		 IP_ADDRS&tba,
		 const char *title);
	void setip_in_dia (DIALOG&dia, IP_ADDRS&tba);
	void setlocalhost (void);
	int setmx (const SSTRING&fqhost, const SSTRINGS&tbmx);
	int setns (const SSTRING&fqhost, const SSTRINGS&tbns);
	void unset (const char *fqhost);
	void unsetmx (const SSTRING&fqhost);
	void unsetns (const SSTRING&fqhost);
private:
	int update_one (SSTRING&full,
		 IP_ADDRS&tba,
		 SSTRINGS&tbns,
		 SSTRINGS&tbmx,
		 SSTRING&cname);
	int update_one (SSTRING&full,
		 IP_ADDRS&tba,
		 SSTRINGS&tbns,
		 SSTRINGS&tbmx,
		 SSTRING&cname,
		 char doreverse[4]);
public:
	int was_modified (void);
	int write (void);
	/*~PROTOEND~ DNS */
};
	
class DICTIONARY;
class USERACCT_COMNG;

#include "dnsconf.p"

#endif

