#pragma interface
#ifndef FIREWALL_H
#define FIREWALL_H

#ifndef MISC_H
    #include <misc.h>
#endif
#ifndef MODULE_H
    #include <module.h>
#endif

#if 1
	#include <dialog.h>
#else
	#include <modapi_def.h>
	#include <modapi.h>
#endif

class MODULE_firewall: public LINUXCONF_MODULE{
    /*~PROTOBEG~ MODULE_firewall */
public:
	MODULE_firewall (void);
	int dohtml (const char *key);
	int domenu (MENU_CONTEXT context,
		 const char *key);
	int execmain (int argc, char *argv[]);
	int message (const char *msg,
		 int ,
		 const char *[]);
	int probe (int level, int , bool);
	void setmenu (DIALOG&dia,
		 MENU_CONTEXT context);
    /*~PROTOEND~ MODULE_firewall */
};


struct IPFW_SRC{
	SSTRING host;
	SSTRING netmask;
	int nbbit_msk;		// Computed number of bit in the netmask
						// to speed up the sort.
	SSTRING portrange;
	SSTRING ports;
	SSTRING interface;	// eth0 ...
};

#define FW_ACCEPT	0
#define FW_REJECT	1
#define FW_DENY		2

class IPFW_RULE: public ARRAY_OBJ{
	friend class IPFW_RULES;
protected:
	char active;		// Is this rules has been activated
				// by the user.
	SSTRING protocol;	// icmp, udp, tcp, all
	IPFW_SRC from;
	IPFW_SRC to;
public:
	virtual void save(CONFDB *)=0;
	virtual int edit()=0;
	virtual int setup_left(int doit, SSTRING *collect, SSTRING &errmsg)=0;
	virtual int setup_right(int doit, SSTRING *collect, SSTRING &errmsg)=0;
	virtual int cmp_to (IPFW_RULE *r)=0;
	virtual int cmp_from (IPFW_RULE *r)=0;
	/*~PROTOBEG~ IPFW_RULE */
protected:
	IPFW_RULE (const char *&buf);
	IPFW_RULE (void);
public:
	int cmp_gen_from (IPFW_RULE *r,
		 int masq1,
		 int masq2,
		 int redir1,
		 int redir2);
	int cmp_gen_to (IPFW_RULE *r,
		 int masq1,
		 int masq2,
		 int redir1,
		 int redir2);
private:
	int diacheck (IPFW_SRC&s, int &nof);
public:
	int editk (DIALOG&dia, const char *title);
	int nbbitmask (IPFW_SRC&f);
	int nbbitmask_from (void);
	int nbbitmask_to (void);
	void present (char *buf);
protected:
	void savek (char *buf);
public:
	int setup (int policy,
		 IPFW_SRC&f,
		 IPFW_SRC&t,
		 const char *type,
		 int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	/*~PROTOEND~ IPFW_RULE */
};

class IPFW_RULEP: public IPFW_RULE{
protected:
	char policy; 	// FW_POLICY
	/*~PROTOBEG~ IPFW_RULEP */
protected:
	IPFW_RULEP (const char *&buf);
	IPFW_RULEP (void);
public:
	int cmp_policy_from (IPFW_RULEP *r,
		 int masq1,
		 int masq2,
		 int redir1,
		 int redir2);
	int cmp_policy_to (IPFW_RULEP *r,
		 int masq1,
		 int masq2,
		 int redir1,
		 int redir2);
protected:
	void setpolicy (DIALOG&dia);
	int setup (IPFW_SRC&f,
		 IPFW_SRC&t,
		 const char *type,
		 int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
public:
	/*~PROTOEND~ IPFW_RULEP */
};


class IPFW_RULE_FORWARD: public IPFW_RULEP{
	char masquerade;	// Masquerade while forwarding ?
	/*~PROTOBEG~ IPFW_RULE_FORWARD */
public:
	IPFW_RULE_FORWARD (const char *buf);
	IPFW_RULE_FORWARD (void);
	int cmp_from (IPFW_RULE *r);
	int cmp_to (IPFW_RULE *r);
	int edit (void);
	void save (CONFDB *db);
	int setup_left (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	int setup_right (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	/*~PROTOEND~ IPFW_RULE_FORWARD */
};

class IPFW_RULE_OUTPUT: public IPFW_RULEP{
	/*~PROTOBEG~ IPFW_RULE_OUTPUT */
public:
	IPFW_RULE_OUTPUT (const char *buf);
	IPFW_RULE_OUTPUT (void);
	int cmp_from (IPFW_RULE *r);
	int cmp_to (IPFW_RULE *r);
	int edit (void);
	void save (CONFDB *db);
	int setup_left (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	int setup_right (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	/*~PROTOEND~ IPFW_RULE_OUTPUT */
};

class IPFW_RULE_INPUT: public IPFW_RULEP{
	char doredir;
	SSTRING redirport;
	/*~PROTOBEG~ IPFW_RULE_INPUT */
public:
	IPFW_RULE_INPUT (const char *buf);
	IPFW_RULE_INPUT (void);
	int cmp_from (IPFW_RULE *r);
	int cmp_to (IPFW_RULE *r);
	int edit (void);
	void save (CONFDB *db);
	int setup_left (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	int setup_right (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	/*~PROTOEND~ IPFW_RULE_INPUT */
};

class IPFW_RULE_ACCT: public IPFW_RULE{
	/*~PROTOBEG~ IPFW_RULE_ACCT */
public:
	IPFW_RULE_ACCT (const char *buf);
	IPFW_RULE_ACCT (void);
	int cmp_from (IPFW_RULE *r);
	int cmp_to (IPFW_RULE *r);
	int edit (void);
	void save (CONFDB *db);
	int setup_left (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	int setup_right (int doit,
		 SSTRING *collect,
		 SSTRING&errmsg);
	/*~PROTOEND~ IPFW_RULE_ACCT */
};

extern char FIREWALL[];
extern char FORWARD[];
extern char BLOCK[];
extern char OUTPUT[];
extern char ACCT[];

#include "firewall.p"

#endif
