#include <string.h>
#include <stdlib.h>
	#include <sys/types.h>
	#include <netinet/in.h>
#include <features.h>
#ifdef __GLIBC__
#include <netinet/ip_fw.h>
#else
#include <sys/socket.h>
	#include <linux/ip.h>
	#include <linux/tcp.h>
	#include <linux/udp.h>
	#include <linux/icmp.h>
	#include <linux/if.h>
	#include <linux/timer.h>
	#include <linux/ip_fw.h>
#endif
#include <limits.h>
#include <netdb.h>
#include "firewall.h"
#include "netconf.h"
#include <dialog.h>
#include <confdb.h>
#include <misc.h>
#include <userconf.h>
#include "../../paths.h"
#include "firewall.m"

#ifndef IP_FW_POLICY_IN
	/* These are just here so it compiles */
	#define IP_FW_F_MASQ		0
	#define IP_FW_APPEND_OUT	1
	#define IP_FW_APPEND_IN		2
	#define IP_FW_APPEND_FWD	3
	#define IP_ACCT_APPEND		4
	#define FIREWAALL_NONE
#endif

HELP_FILE help_ipfw ("firewall","firewall");


PROTECTED IPFW_RULE::IPFW_RULE()
{
	from.interface.setfrom ("Any");
	to.interface.setfrom ("Any");
	protocol.setfrom ("all");
	/* #Specification: netconf / firewalling / active rule
		One rule may be defined and desactivated without erasing it.
		This give some flexibility to the administrator to define
		complex rules and "comment out" some.
	*/
	active = 1;
}
PROTECTED IPFW_RULEP::IPFW_RULEP()
{
	policy = FW_ACCEPT;
}
/*
	Extract a number from a buffer.
	The number may be enclosed in double quote.
*/
static const char *firewall_extract (const char *buf, char &val)
{
	SSTRING s;
	buf = str_extract (buf,s);
	val = s.getval();
	return buf;
}

static const char *firewall_extract (const char *buf, IPFW_SRC &s)
{
	buf = str_extract (buf,s.host);
	buf = str_extract (buf,s.netmask);
	buf = str_extract (buf,s.portrange);
	buf = str_extract (buf,s.ports);
	buf = str_extract (buf,s.interface);
	return buf;
}

PROTECTED IPFW_RULE::IPFW_RULE(const char * &buf)
{
	buf = firewall_extract (buf,active);
	buf = str_extract (buf,protocol);
	buf = firewall_extract (buf,from);
	buf = firewall_extract (buf,to);
}

PROTECTED IPFW_RULEP::IPFW_RULEP(const char * &buf)
	: IPFW_RULE (buf)
{
	// To keep compatibility with old installation, the policy is read
	// from the buffer by the IPFW_RULE_ child classes
}

PUBLIC IPFW_RULE_FORWARD::IPFW_RULE_FORWARD(const char *buf)
	: IPFW_RULEP (buf)
{
	buf = firewall_extract (buf,masquerade);
	buf = firewall_extract (buf,policy);
}
PUBLIC IPFW_RULE_FORWARD::IPFW_RULE_FORWARD()
{
	masquerade = 0;
}

PUBLIC IPFW_RULE_OUTPUT::IPFW_RULE_OUTPUT(const char *buf)
	: IPFW_RULEP (buf)
{
	buf = firewall_extract (buf,policy);
}
PUBLIC IPFW_RULE_OUTPUT::IPFW_RULE_OUTPUT()
{
}

PUBLIC IPFW_RULE_INPUT::IPFW_RULE_INPUT(const char *buf)
	: IPFW_RULEP (buf)
{
	buf = firewall_extract (buf,doredir);
	buf = str_extract (buf,redirport);
	buf = firewall_extract (buf,policy);
}
PUBLIC IPFW_RULE_INPUT::IPFW_RULE_INPUT()
{
	doredir = 0;
}

PUBLIC IPFW_RULE_ACCT::IPFW_RULE_ACCT(const char *buf)
	: IPFW_RULE (buf)
{
}
PUBLIC IPFW_RULE_ACCT::IPFW_RULE_ACCT()
{
}



static void firewall_append (char *buf, SSTRING &s)
{
	strcat (buf," \"");
	strcat (buf,s.get());
	strcat (buf,"\"");
}

static void firewall_append (char *buf, int val)
{
	char valstr[20];
	sprintf (valstr,"%d",val);
	strcat (buf," \"");
	strcat (buf,valstr);
	strcat (buf,"\"");
}

static void firewall_append (char *buf, IPFW_SRC &s)
{
	firewall_append (buf,s.host);
	firewall_append (buf,s.netmask);
	firewall_append (buf,s.portrange);
	firewall_append (buf,s.ports);
	firewall_append (buf,s.interface);
}

char FIREWALL[] = "firewall";
char FORWARD[]  = "forward";
char BLOCK[]  = "block";
char OUTPUT[]  = "output";
char ACCT[]  = "acct";

/*
	Save one forward line in /etc/conf.linuxconf
*/
PROTECTED void IPFW_RULE::savek(char *buf)
{
	buf[0] = '\0';
	firewall_append (buf,active);
	firewall_append (buf,protocol);
	firewall_append (buf,from);
	firewall_append (buf,to);
}

/*
	Save one forward line in /etc/conf.linuxconf
*/
PUBLIC void IPFW_RULE_FORWARD::save(CONFDB *db)
{
	char buf[2000];
	IPFW_RULE::savek(buf);
	firewall_append (buf,masquerade);
	firewall_append (buf,policy);
	db->add (FIREWALL,FORWARD,buf);
}
/*
	Save one forward line in /etc/conf.linuxconf
*/
PUBLIC void IPFW_RULE_INPUT::save(CONFDB *db)
{
	char buf[2000];
	IPFW_RULE::savek(buf);
	firewall_append (buf,doredir);
	firewall_append (buf,redirport);
	firewall_append (buf,policy);
	db->add (FIREWALL,BLOCK,buf);
}
/*
	Save one output line in /etc/conf.linuxconf
*/
PUBLIC void IPFW_RULE_OUTPUT::save(CONFDB *db)
{
	char buf[2000];
	IPFW_RULE::savek(buf);
	firewall_append (buf,policy);
	db->add (FIREWALL,OUTPUT,buf);
}
/*
	Save one output line in /etc/conf.linuxconf
*/
PUBLIC void IPFW_RULE_ACCT::save(CONFDB *db)
{
	char buf[2000];
	IPFW_RULE::savek(buf);
	db->add (FIREWALL,ACCT,buf);
}

/*
	Format the minimum information about the rule so it will
	be presentable in a menu.
*/
PUBLIC void IPFW_RULE::present (char *buf)
{
	sprintf (buf,"%s\t%s\t%s\t\t%s\t%s"
		,protocol.get()
		,from.interface.get()
		,from.host.get()
		,to.interface.get()
		,to.host.get());
}

/*
	try to convert something into an IP number
	Return -1 if it can't.
*/
static int ipfwrule_convert (
	const char *str,	// An IP number, a host name or an interface
					// name (eth0)
	char ip[],
	char msk[],		// Corresponding (computed) netmask
					// or extracted from the device
	SSTRING *errmsg)
{
	int ret = netconf_convert (str,ip,msk);
	if (ret == -1 && errmsg != NULL){
		char buf[300];
		sprintf (buf,MSG_U(E_IVLDIP,"Invalid IP number or host name or interface: %s\n")
			,str);
		errmsg->append (buf);
	}
	return ret;
}

static int ipfwrule_checkhn (const char *str)
{
	char msk[LEN_IP_ASC],ip[LEN_IP_ASC];
	SSTRING error;
	int ret = ipfwrule_convert (str,ip,msk,&error);
	if (ret == -1) xconf_error (error.get());
	return ret;
}

/*
	Return the number of significative bits in the netmask (1's)

	Return -1 if any error.
*/
PUBLIC int IPFW_RULE::nbbitmask(IPFW_SRC &f)
{
	int ret = f.nbbit_msk;
	if (ret == -1){
		const char *msk = f.netmask.get();
		char std_msk[LEN_IP_ASC];
		if (*msk == '\0'){
			/* #Specification: firewall / netmask
				The netmask is optionnal. It is automacly
				compute from the host/network number
			*/
			char ip[LEN_IP_ASC];
			if (ipfwrule_convert(f.host.get(),ip,std_msk,NULL)!=-1){
				msk = std_msk;
			}else{
				ret = -1;
			}
		}
		int num4[4];
		ipnum_aip24 (msk,num4);
		unsigned long nmsk = (num4[0] << 24) | (num4[1] << 16)
			| (num4[2] << 8) | num4[3];
		unsigned long cmp = 0x80000000l;
		int i;
		for (i=0; i<32 && (nmsk & cmp) != 0; i++, cmp >>= 1);
		ret = i;
		f.nbbit_msk = ret;
	}
	return ret;
}

/*
	Return the number of significative bits in the netmask (1's)

	This is taken in the from.
*/
PUBLIC int IPFW_RULE::nbbitmask_from()
{
	return nbbitmask (from);
}
/*
	Return the number of significative bits in the netmask (1's)

	This is taken in the "to".
*/
PUBLIC int IPFW_RULE::nbbitmask_to()
{
	return nbbitmask (to);
}

static int cmp_gen(
	int netmask1,	// Number of bits == 1in the netmask
	int netmask2,
	const char *iface1,	// Interface
	const char *iface2,
	int masq1,		// Is it a masquerading rule
	int masq2,
	int redir1,		// Is it a redirection rule
	int redir2)
{
	/* #Specification: firewall / sorting of rules
		Linuxconf knows automaticly how to sort rules so it makes sens
		Here are the rules it uses to sort

		redirection rules are always first
		masquerading rules are always last
		The most specific netmask is first (the one
			with the larger number of 1s).
		If all equals, the rules with the interface Any is last
			when compared to a rule with a specific interface
	*/

	int ret = redir2 - redir1;
	if (ret == 0){
		int ret = masq1 - masq2;
		if (ret == 0){	
			ret = netmask1 - netmask2;
			if (ret == 0){
				ret = strcmp(iface1,iface2);
				if (ret != 0){
					ret = stricmp(iface1,"any");
					if (ret == 0){
						ret = 1;
					}else{
						ret = stricmp (iface2,"any");
						if (ret == 0){
							ret = -1;
						}else{
							ret = 0;
						}
					}
				}
			}
		}
	}
	return ret;
}

PUBLIC int IPFW_RULE::cmp_gen_to(
	IPFW_RULE *r,
	int masq1,		// Is it a masquerading rule
	int masq2,
	int redir1,		// Is it a redirection rule
	int redir2)
{
	return cmp_gen (
		nbbitmask_to(),r->nbbitmask_to(),
		to.interface.get(),r->to.interface.get(),
		masq1,masq2,
		redir1,redir2);
}

PUBLIC int IPFW_RULE::cmp_gen_from(
	IPFW_RULE *r,
	int masq1,		// Is it a masquerading rule
	int masq2,
	int redir1,		// Is it a redirection rule
	int redir2)
{
	return cmp_gen (
		nbbitmask_from(),r->nbbitmask_from(),
		from.interface.get(),r->from.interface.get(),
		masq1,masq2,
		redir1,redir2);
}

PUBLIC int IPFW_RULEP::cmp_policy_to(
	IPFW_RULEP *r,
	int masq1,		// Is it a masquerading rule
	int masq2,
	int redir1,		// Is it a redirection rule
	int redir2)
{
	int ret = r->policy - policy;
	if (ret == 0) ret = IPFW_RULE::cmp_gen_to(r,masq1,masq2,redir1,redir2);
	return ret;
}

PUBLIC int IPFW_RULEP::cmp_policy_from(
	IPFW_RULEP *r,
	int masq1,		// Is it a masquerading rule
	int masq2,
	int redir1,		// Is it a redirection rule
	int redir2)
{
	int ret = r->policy - policy;
	if (ret == 0) ret = IPFW_RULE::cmp_gen_from(r,masq1,masq2,redir1,redir2);
	return ret;
}

PUBLIC int IPFW_RULE_INPUT::cmp_from(IPFW_RULE *r)
{
	IPFW_RULE_INPUT *f = (IPFW_RULE_INPUT*)r;
	return cmp_policy_from (f,0,0,doredir,f->doredir);
}


PUBLIC int IPFW_RULE_INPUT::cmp_to(IPFW_RULE *r)
{
	IPFW_RULE_INPUT *f = (IPFW_RULE_INPUT*)r;
	return cmp_policy_to (f,0,0,doredir,f->doredir);
}

PUBLIC int IPFW_RULE_OUTPUT::cmp_from(IPFW_RULE *r)
{
	return cmp_policy_from ((IPFW_RULE_OUTPUT*)r,0,0,0,0);
}


PUBLIC int IPFW_RULE_OUTPUT::cmp_to(IPFW_RULE *r)
{
	return cmp_policy_to ((IPFW_RULE_OUTPUT*)r,0,0,0,0);
}
PUBLIC int IPFW_RULE_FORWARD::cmp_from(IPFW_RULE *r)
{
	IPFW_RULE_FORWARD *f = (IPFW_RULE_FORWARD*)r;
	return cmp_policy_from (f,masquerade,f->masquerade,0,0);
}


PUBLIC int IPFW_RULE_FORWARD::cmp_to(IPFW_RULE *r)
{
	IPFW_RULE_FORWARD *f = (IPFW_RULE_FORWARD*)r;
	return cmp_policy_to (f,masquerade,f->masquerade,0,0);
}

PUBLIC int IPFW_RULE_ACCT::cmp_from(IPFW_RULE *r)
{
	return cmp_gen_from (r,0,0,0,0);
}


PUBLIC int IPFW_RULE_ACCT::cmp_to(IPFW_RULE *r)
{
	return cmp_gen_to (r,0,0,0,0);
}


/*
	Check one section of the dialog
	Return -1 if any error.
*/
PRIVATE int IPFW_RULE::diacheck (IPFW_SRC &s, int &nof)
{
	int ret = -1;
	bool allow_port = protocol.cmp("udp")==0 || protocol.cmp("tcp")==0;
	if (s.host.is_empty()){
		nof = 0;
		xconf_error (MSG_U(E_MISSINGHN,"Missing host or network"));
	}else if (ipfwrule_checkhn(s.host.get())==-1){
		nof = 0;
	}else if (!s.netmask.is_empty()
		&& !ipnum_validip(s.netmask.get(),false)){
		xconf_error (MSG_U(E_IVLDMASK,"Invalid netmask"),s.netmask.get());
		nof = 1;
	}else if (!allow_port && !s.portrange.is_empty()){
		xconf_error (MSG_U(E_ALLOWPORTRANGE
			,"Port range only allowed for TCP and UDP protocol"));
		nof = 2;
	}else if (ipfw_checkrange(s.portrange)==-1){
		xconf_error (MSG_U(E_IVLDPRANGE
			,"Invalid port range\n"
			 "Expecting xxxx:yyyy"));
		nof = 2;
	}else if (!allow_port && !s.ports.is_empty()){
		xconf_error (MSG_U(E_ALLOWPORTLIST
			,"Ports list only allowed for TCP and UDP protocol"));
		nof = 3;
	}else if (ipfw_checkports(s.ports,protocol)==-1){
		xconf_error (MSG_U(E_IVLDPORTS,"Invalid ports list"));
		nof = 3;
	}else{
		ret = 0;
	}
	return ret;
}

/*
	Return -1 if abort
	Return  0 if accept
	Return  1 if delete required
*/
PUBLIC int IPFW_RULE::editk(
	DIALOG &dia,
	const char *title)
{
	int i;
	int f_protocol = dia.getnb();
	FIELD_COMBO *comb = dia.newf_combo (MSG_U(F_PROTOCOL,"Protocol")
		,protocol);
	static const char *tbproto[]={
		"icmp","udp","tcp","all"
	};
	for (i=0; i<4; i++){
		comb->addopt (tbproto[i]);
	}

	dia.newf_title ("",MSG_U(F_FROM,"From"));
	int f_from_section = dia.getnb();
	dia.newf_str (MSG_U(F_HOSTORNET,"Host or Network"),from.host);
	comb = dia.newf_combo (MSG_U(F_HNETMASK,"Netmask"),from.netmask);
	host_setmasklist(comb);
	dia.newf_str (MSG_U(F_PORTRANGE,"Port range"),from.portrange);
	dia.newf_str (MSG_U(F_OTHERPORTS,"Other ports"),from.ports);
	comb = dia.newf_combo (MSG_U(F_INTERFACE,"Interface"),from.interface);
	comb->addopt ("Any",MSG_U(F_PACKFROMANY,"Packet come from anywhere"));
	devlist_setcombo (comb);

	dia.newf_title ("",MSG_U(F_TO,"To"));
	int f_to_section = dia.getnb();
	dia.newf_str (MSG_R(F_HOSTORNET),to.host);
	comb = dia.newf_combo (MSG_R(F_HNETMASK),to.netmask);
	host_setmasklist (comb);

	dia.newf_str (MSG_R(F_PORTRANGE),to.portrange);
	dia.newf_str (MSG_R(F_OTHERPORTS),to.ports);

	comb = dia.newf_combo (MSG_R(F_INTERFACE),to.interface);
	comb->addopt ("Any",MSG_R(F_PACKFROMANY));
	devlist_setcombo (comb);
	int ret = -1;
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit (title
			,""
			,help_ipfw
			,nof
			,MENUBUT_CANCEL|MENUBUT_ACCEPT|MENUBUT_DEL);
		if (code == MENU_ESCAPE || code == MENU_CANCEL){
			break;
		}else if (code == MENU_DEL){
			ret = 1;
			break;
		}else if (diacheck(from,nof)==-1){
			nof += f_from_section;
		}else if (diacheck(to,nof)==-1){
			nof += f_to_section;
		}else{
			for (i=0; i<4; i++){
				if (protocol.cmp(tbproto[i])==0) break;
			}
			if (i == 4){
				xconf_error (MSG_U(E_IVLDPROTO,"Invalid protocol"));
				nof = f_protocol;
			}else{
				ret = 0;
				break;
			}
		}
	}
	if (ret != 0) dia.restore();
	return ret;
}
PROTECTED void IPFW_RULEP::setpolicy(DIALOG &dia)
{
	static const char *tbpolicy[]={
		MSG_U(F_FW_ACCEPT,"Accept"),
		MSG_U(F_FW_REJECT,"Reject"),
		MSG_U(F_FW_DENY,"Deny"),
		NULL
	};
	dia.newf_chkm (MSG_U(F_RULEPOLICY,"Rule's policy"),policy,tbpolicy);
}

/*
	Return -1 if abort
*/
PUBLIC int IPFW_RULE_FORWARD::edit()
{
	DIALOG dia;
	dia.newf_chk ("",active,MSG_U(F_RULEACTIVE,"This rule is active"));
	setpolicy(dia);
	dia.newf_chk ("",masquerade,MSG_U(F_DOMASQUERADE,"Do masquerading"));
	return IPFW_RULE::editk (dia,MSG_U(T_FORWARDING,"Firewall forwarding rule"));
}
/*
	Return -1 if abort
*/
PUBLIC int IPFW_RULE_OUTPUT::edit()
{
	DIALOG dia;
	dia.newf_chk ("",active,MSG_R(F_RULEACTIVE));
	setpolicy(dia);
	return IPFW_RULE::editk (dia,MSG_U(T_OUTPUT,"Firewall Outputing rule"));
}
/*
	Return -1 if abort
*/
PUBLIC int IPFW_RULE_INPUT::edit()
{
	DIALOG dia;
	dia.newf_chk ("",active,MSG_R(F_RULEACTIVE));
	setpolicy(dia);
	dia.newf_chk ("",doredir,MSG_U(F_DOREDIR,"Redirect to local port"));
	dia.newf_str (MSG_U(F_REDIRPORT,"Redirection port"),redirport);
	return IPFW_RULE::editk (dia,MSG_U(T_INPUTING,"Firewall inputing rule"));
}

/*
	Return -1 if abort
*/
PUBLIC int IPFW_RULE_ACCT::edit()
{
	DIALOG dia;
	dia.newf_chk ("",active,MSG_R(F_RULEACTIVE));
	return IPFW_RULE::editk (dia,MSG_U(T_ACCT,"IP Accouting rule"));
}



PUBLIC int IPFW_RULE::setup(
	int policy,
	IPFW_SRC &f,
	IPFW_SRC &t,
	const char *type,
	int doit,			// Exec the command ?
	SSTRING *collect,	// Will contain a copy of the
						// command generated
	SSTRING &errmsg)	// Error message are appended to this SSTRING
{
	int ret = 0;
	if (active){
		char interbuf[100];
		strcpy (interbuf,"0.0.0.0");
		if (!f.interface.is_empty()
			&& f.interface.cmp("Any")!=0){
			/* #Specification: firewall / iface argument
				We accept a host name, an IP number or
				a network device name (eth0).
			*/
			char ip[20],msk[20];
			const char *ifacearg = f.interface.get();
			if (ipfwrule_convert (ifacearg,ip,msk,NULL)!=-1){
				sprintf (interbuf,"%s",ip);
			}else{
				xconf_error (MSG_U(E_IVLDINTER
					,"Invalid interface %s for firewalling rule")
					,ifacearg);
				ret = -1;
			}
		}
		/* #Specification: firewall / forwarding / accept
			The firewalling interface support only accepting rule
			so far. It is expect to be enough for most firewalls.

			Supporting denying and rejecting is not much of a problem
			although, Experience will shows us how to do this
			without confusing the user...

			Comments are welcome.
		*/
		char fhoststr[PATH_MAX],fhostmsk[LEN_IP_ASC];
		ret |= ipfwrule_convert (f.host.get(),fhoststr,fhostmsk,&errmsg);
		char thoststr[PATH_MAX],thostmsk[LEN_IP_ASC];
		ret |= ipfwrule_convert (t.host.get(),thoststr,thostmsk,&errmsg);
		struct ip_fw bf;
		if (ret == 0
			&& ipfw_baseinit(policy
			,interbuf,protocol.get()
			,fhoststr,f.netmask.is_empty() ? fhostmsk : f.netmask.get()
			,f.portrange.get(),f.ports.get()
			,thoststr,t.netmask.is_empty() ? thostmsk : t.netmask.get()
			,t.portrange.get(),t.ports.get()
			,type[0] == 'r' ? type + 1 : (const char *)NULL
			,bf)!=-1){
			int command = 0;
			if (type[0] == 'm'){
				command = IP_FW_APPEND_FWD;
				bf.fw_flg |= IP_FW_F_MASQ;
			}else if (type[0] == 'f'){
				command = IP_FW_APPEND_FWD;
			}else if (type[0] == 'o'){
				command = IP_FW_APPEND_OUT;
			}else if (type[0] == 'b'){
				command = IP_FW_APPEND_IN;
			}else if (type[0] == 'r'){
				command = IP_FW_APPEND_IN;
			}else if (type[0] == 'a'){
				command = IP_ACCT_APPEND;
			}
			ret = ipfw_append (doit,collect,command,bf);
		}else{
			ret = -1;
		}
	}
	return ret;
}

PROTECTED int IPFW_RULEP::setup(
	IPFW_SRC &f,
	IPFW_SRC &t,
	const char *type,
	int doit,			// Exec the command ?
	SSTRING *collect,	// Will contain a copy of the
						// command generated
	SSTRING &errmsg)	// Error message are appended to this SSTRING
{
	return IPFW_RULE::setup (policy,f,t,type,doit,collect,errmsg);
}

/*
	Setup one side of the forwarding rule (from -> to)
*/
PUBLIC int IPFW_RULE_FORWARD::setup_left(int doit, SSTRING *collect, SSTRING &errmsg)
{
	int ret = -1;
	if (masquerade){
		ret = setup (from,to,"m",doit,collect,errmsg);
	}else{
		ret = setup (from,to,"f",doit,collect,errmsg);
	}
	return ret;
}
/*
	Setup one side of the forwarding rule (to -> from)
*/
PUBLIC int IPFW_RULE_FORWARD::setup_right(int doit, SSTRING *collect, SSTRING &errmsg)
{
	int ret = 0;
	if (!masquerade){
		ret = setup (to,from,"f",doit,collect,errmsg);
	}
	return ret;
}
/*
	Setup one side of the forwarding rule (from -> to)
*/
PUBLIC int IPFW_RULE_OUTPUT::setup_left(int doit, SSTRING *collect, SSTRING &errmsg)
{
	return setup (from,to,"o",doit,collect,errmsg);
}
/*
	Setup one side of the forwarding rule (to -> from)
*/
PUBLIC int IPFW_RULE_OUTPUT::setup_right(int doit, SSTRING *collect, SSTRING &errmsg)
{
	return setup (to,from,"o",doit,collect,errmsg);
}


/*
	Setup one side of the blocking rule (from -> to)
*/
PUBLIC int IPFW_RULE_INPUT::setup_left(int doit, SSTRING *collect, SSTRING &errmsg)
{
	int ret = -1;
	if (doredir){
		char type[100];
		if (redirport.is_empty()){
			strcpy (type,"r0");
		}else{
			sprintf (type,"r%s",redirport.get());
		}
		ret = setup (from,to,type,doit,collect,errmsg);
	}else{
		ret = setup (from,to,"b",doit,collect,errmsg);
	}
	return ret;
}
/*
	Setup one side of the blocking rule (to -> from)
*/
PUBLIC int IPFW_RULE_INPUT::setup_right(int doit, SSTRING *collect, SSTRING &errmsg)
{
	int ret = 0;
	if (!doredir){
		ret = setup (to,from,"b",doit,collect,errmsg);
	}
	return ret;
}

/*
	Setup one side of the accounting rule (from -> to)
*/
PUBLIC int IPFW_RULE_ACCT::setup_left(int doit, SSTRING *collect, SSTRING &errmsg)
{
	return setup (FW_ACCEPT,from,to,"a",doit,collect,errmsg);
}
/*
	Setup one side of the accounting rule (to -> from)
*/
PUBLIC int IPFW_RULE_ACCT::setup_right(int doit, SSTRING *collect, SSTRING &errmsg)
{
	return setup (FW_ACCEPT,to,from,"a",doit,collect,errmsg);
}

