#include <stdlib.h>
#include "internal.h"
#include "samba.m"
#include "keyword.h"

extern SAMBA_HELP_FILE help_samba;
static SAMBA_HELP_FILE help_share ("share");
static SAMBA_HELP_FILE help_homes ("homes");
static SAMBA_HELP_FILE help_printers ("printers");
static SAMBA_HELP_FILE help_global ("global");

static SMB_CONF *samba;

PRIVATE void SMB_SHARE::setdia (
	DIALOG &dia, 
	SMB_FIELD tbf[],
	int nbf,
	SMB_FIELD_VAL tbv[])
{
	for (int i=0; i<nbf; i++){
		SMB_FIELD *pt = tbf + i;
		SMB_FIELD_TYPE type = pt->type;
		const char *title = pt->title;
		const char *val = getval (pt->key1,pt->key2,pt->key3);
		if (type == SMB_STR){
			tbv[i].str.setfrom (val);
			dia.newf_str (title,tbv[i].str);
		}else if (type == SMB_NUM){
			tbv[i].num = atoi(val);
			dia.newf_num (title,tbv[i].num);
		}else if (type == SMB_CHK){
			tbv[i].chk = stricmp(val,K_YES)==0;
			dia.newf_chk ("",tbv[i].chk,title);
		}
	}
}

PRIVATE void SMB_SHARE::savedia(
	SMB_FIELD tbf[],
	int nbf,
	SMB_FIELD_VAL tbv[])
{
	for (int i=0; i<nbf; i++){
		SMB_FIELD *pt = tbf + i;
		SMB_FIELD_TYPE type = pt->type;
		const char *key1 = pt->key1;
		const char *key2 = pt->key2;
		const char *key3 = pt->key3;
		if (type == SMB_STR){
			setval (key1,key2,key3,tbv[i].str);
		}else if (type == SMB_NUM){
			setval (key1,key2,key3,tbv[i].num);
		}else if (type == SMB_CHK){
			setval (key1,key2,key3,tbv[i].chk ? K_YES : K_NO);
		}
	}
}		

PUBLIC int SMB_SHARE::editgen(
	DIALOG &dia,
	SMB_FIELD tbf[],
	int nbf,
	bool may_del,		// May delete this share
	SAMBA_HELP_FILE &help_file)
{
	int ret = -1;
	SMB_FIELD_VAL tbv[nbf];
	setdia (dia,tbf,nbf,tbv);
	int nof = 0;
	while (1){
		int menubut = MENUBUT_ACCEPT|MENUBUT_CANCEL;
		if (may_del) menubut |= MENUBUT_DEL;
		MENU_STATUS code = dia.edit (MSG_U(T_SHARE,"Share setup")
			,MSG_U(I_SHARE,"You are allowed to enter configurations\n"
				"for one disk share\n")
			,help_file
			,nof,menubut);
		if (code == MENU_ESCAPE || code == MENU_CANCEL){
			break;
		}else if (code == MENU_DEL){
			if (xconf_delok()){
				ret = 1;
				break;
			}
		}else{
			savedia (tbf,nbf,tbv);
			ret = 0;
			break;
		}
	}
	if (ret != 0) dia.restore();
	return ret;
}

/*
	Edit a standard share (not homes, global or printers)
*/
PUBLIC int SMB_SHARE::edit()
{
	DIALOG dia;
	dia.newf_str (MSG_U(F_SHARENAME,"Share name"),name);

	static SMB_FIELD tbf[]={
		{K_COMMENT,NULL,NULL,SMB_STR,MSG_U(F_COMMENT,"Comment/description")},
		{K_AVAILABLE,NULL,NULL,SMB_CHK,MSG_U(F_ENABLE,"This share is enabled")},
		{K_COPY,NULL,NULL,SMB_STR,MSG_U(F_INHERIT,"Inherit settings from share")},
		{K_PATH,NULL,NULL,SMB_STR,MSG_U(F_PATH,"Directory to export")},
		{K_PUBLIC,NULL,NULL,SMB_CHK,MSG_U(F_PUBLIC,"Public access")},
		{K_GUEST,K_ONLY,NULL,SMB_CHK,MSG_U(F_GUESTONLY,"Guest access only")},
		{K_WRITABLE,NULL,NULL,SMB_CHK,MSG_U(F_WRITABLE,"Writable")},
		{K_BROWSEABLE,NULL,NULL,SMB_CHK,MSG_U(F_BROWSEABLE,"Browsable")},
		{K_WRITE,K_LIST,NULL,SMB_STR,MSG_U(F_WRITELIST,"Write list")},
		{K_PREEXEC,NULL,NULL,SMB_STR,MSG_U(F_PREEXEC,"Setup command")},
		{K_ROOT,K_PREEXEC,NULL,SMB_STR,MSG_U(F_ROOTPREEXEC,"Setup command (root)")},
		{K_POSTEXEC,NULL,NULL,SMB_STR,MSG_U(F_POSTEXEC,"Cleanup command")},
		{K_ROOT,K_POSTEXEC,NULL,SMB_STR,MSG_U(F_ROOTPOSTEXEC,"Cleanup command (root)")},
		{K_FORCE,K_USER,NULL,SMB_STR,MSG_U(F_FORCEUSER,"Force user")},
		{K_FORCE,K_GROUP,NULL,SMB_STR,MSG_U(F_FORCEGROUP,"Force group")},
		{K_ADMIN,K_USERS,NULL,SMB_STR,MSG_U(F_ADMINUSERS,"Admin users")},
		{K_ALLOW,K_HOSTS,NULL,SMB_STR,MSG_U(F_ALLOWHOSTS,"Allow hosts")},
		{K_DENY,K_HOSTS,NULL,SMB_STR,MSG_U(F_DENYHOSTS,"Deny hosts")},
		{K_DONT,K_DESCEND,NULL,SMB_STR,MSG_U(F_DONTDESCEND,"Don't descent")},
		{K_GUEST,K_ACCOUNT,NULL,SMB_STR,MSG_U(F_GUESTACCOUNTSH,"Guest account (this share)")},
		{K_VALID,K_USERS,NULL,SMB_STR,MSG_U(F_VALIDUSERS,"Valid users")},
		{K_INVALID,K_USERS,NULL,SMB_STR,MSG_U(F_IVLDUSERS,"Invalid users")},
		{K_MAGIC,K_SCRIPT,NULL,SMB_STR,MSG_U(F_MAGICSCRIPT,"Magic script")},
		{K_MAGIC,K_OUTPUT,NULL,SMB_STR,MSG_U(F_MAGICOUTPUT,"Magic output")},
		{K_MAX,K_CONNECTIONS,NULL,SMB_STR,MSG_U(F_MAXCONNECT,"Max. connections")},
		{K_USER,NULL,NULL,SMB_STR,MSG_U(F_USERLIST,"User list")},
		{K_READ,K_LIST,NULL,SMB_STR,MSG_U(F_READLIST,"Read only user list")},
		{K_ONLY,K_USER,NULL,SMB_CHK,MSG_U(F_ONLYUSER,"Only user may connect")},
	};
	return editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),true,help_share);
}

PUBLIC int SMB_SHARE::editglobal()
{
	DIALOG dia;

	dia.newf_chk ("",samba->syncpass,MSG_U(F_SYNCPASS,"Synchronise linux and SMB passwords"));

	static SMB_FIELD tbf[]={
		{K_COMMENT,NULL,NULL,SMB_STR,MSG_U(F_SERVERDESC,"Server description")},
		{K_WORKGROUP,NULL,NULL,SMB_STR,MSG_U(F_WORKGROUP,"Work group")},
		{K_GUEST,K_ACCOUNT,NULL,SMB_STR,MSG_U(F_GUESTACCOUNT,"Guest account")},
		{K_ENCRYPT,K_PASSWORDS,NULL,SMB_CHK,MSG_U(F_ENGRYPTPASS,"Encrypted password required")},
		{K_PASSWORD,K_SERVER,NULL,SMB_STR,MSG_U(F_PASSSERVER,"Password server")},
		{K_PASSWORD,K_LEVEL,NULL,SMB_NUM,MSG_U(F_PASSLEVEL,"Password level")},
		{K_PASSWD,K_PROGRAM,NULL,SMB_STR,MSG_U(F_PASSPROG,"Passwd program")},
		{K_PREFERRED,K_MASTER,NULL,SMB_CHK,MSG_U(F_PREFMASTER,"Preferred master")},
		{K_OS,K_LEVEL,NULL,SMB_NUM,MSG_U(F_OSLEVEL,"OS level")},
		{K_NULL,K_PASSWORDS,NULL,SMB_CHK,MSG_U(F_NULLPASSWORDS,"Allow null passwords account")},
		{K_ALLOW,K_HOSTS,NULL,SMB_STR,MSG_R(F_ALLOWHOSTS)},
		{K_DENY,K_HOSTS,NULL,SMB_STR,MSG_R(F_DENYHOSTS)},
		{K_DEAD,K_TIME,NULL,SMB_NUM,MSG_U(F_DEADTIME,"Dead time")},
		{K_DEBUG,K_LEVEL,NULL,SMB_NUM,MSG_U(F_DBGLEVEL,"Debug level")},
		{K_DEFAULT,K_SERVICE,NULL,SMB_STR,MSG_U(F_DEFSERVICE,"Default service")},
		{K_DOMAIN,K_MASTER,NULL,SMB_CHK,MSG_U(F_DOMMASTER,"Domain master")},
		{K_REMOTE,K_ANNOUNCE,NULL,SMB_STR,MSG_U(F_REMANNOUNCE,"Remote announce")},
		{K_LOAD,K_PRINTERS,NULL,SMB_CHK,MSG_U(F_LOADPRINTERS,"Show all available printers")},
		{K_MESSAGE,K_COMMAND,NULL,SMB_STR,MSG_U(F_MSGCOMMAND,"WinPopup command")},

	};
	int ret = editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),false,help_global);
	if (ret == 0) samba->write();
	return ret;
}

PUBLIC int SMB_SHARE::edithomes()
{
	DIALOG dia;
	static SMB_FIELD tbf[]={
		{K_COMMENT,NULL,NULL,SMB_STR,MSG_R(F_COMMENT)},
		{K_AVAILABLE,NULL,NULL,SMB_CHK,MSG_R(F_ENABLE)},
		{K_PUBLIC,NULL,NULL,SMB_CHK,MSG_R(F_PUBLIC)},
		{K_WRITABLE,NULL,NULL,SMB_CHK,MSG_R(F_WRITABLE)},
		{K_BROWSEABLE,NULL,NULL,SMB_CHK,MSG_R(F_BROWSEABLE)},
		{K_WRITE,K_LIST,NULL,SMB_STR,MSG_R(F_WRITELIST)},
		{K_PREEXEC,NULL,NULL,SMB_STR,MSG_R(F_PREEXEC)},
		{K_ROOT,K_PREEXEC,NULL,SMB_STR,MSG_R(F_ROOTPREEXEC)},
		{K_POSTEXEC,NULL,NULL,SMB_STR,MSG_R(F_POSTEXEC)},
		{K_ROOT,K_POSTEXEC,NULL,SMB_STR,MSG_R(F_ROOTPOSTEXEC)},
		{K_ALLOW,K_HOSTS,NULL,SMB_STR,MSG_R(F_ALLOWHOSTS)},
		{K_DENY,K_HOSTS,NULL,SMB_STR,MSG_R(F_DENYHOSTS)},
		{K_VALID,K_USERS,NULL,SMB_STR,MSG_R(F_VALIDUSERS)},
		{K_INVALID,K_USERS,NULL,SMB_STR,MSG_R(F_IVLDUSERS)},
		{K_MAX,K_CONNECTIONS,NULL,SMB_STR,MSG_R(F_MAXCONNECT)},
		{K_USER,NULL,NULL,SMB_STR,MSG_R(F_USERLIST)},
		{K_READ,K_LIST,NULL,SMB_STR,MSG_R(F_READLIST)},
		{K_ONLY,K_USER,NULL,SMB_CHK,MSG_R(F_ONLYUSER)},
	};
	int ret = editgen (dia,tbf,(int)(sizeof(tbf)/sizeof(tbf[0])),true,help_homes);
	if (ret == 0) samba->write();
	return ret;
}

PUBLIC int SMB_SHARE::editprinters()
{
	return 0;
}

PUBLIC int SMB_SHARES::write()
{
	return samba->write();
}

PUBLIC int SMB_CONF::editshares()
{
	samba = this;
	int ret = -1;
	int nof = 0;
	while (1){
		DIALOG_LISTE dia;
		dia.newf_head ("",MSG_U(H_SHARES,"Share name\tDescription"));
		int n = shares.getnb();
		int tblookup[n];
		int alloc = 0;
		for (int i=0; i<n; i++){
			SMB_SHARE *d = shares.getitem(i);
			const char *name = d->name.get();
			if (strcmp(name,"global")!=0
				&& strcmp(name,"printers")!=0
				&& strcmp(name,"homes")!=0){
				tblookup[alloc++] = i;
				dia.new_menuitem(name,d->getval(K_COMMENT));
			}
		}
		dia.addwhat (MSG_U(I_NEWSHARE,"Select [Add] to add a new disk share"));
		MENU_STATUS code = dia.editmenu (MSG_U(T_SHARES,"DIsk shares")
			,MSG_U(I_SHARES
				,"You can define multiple independant entry\n"
				 "points in your file server")
			,help_share
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else if (code == MENU_ADD){
			SMB_SHARE *d = new SMB_SHARE;
			ret = shares.editone (d);
		}else if (nof < n && nof >= 0){
			ret = shares.editone (tblookup[nof]);
		}
	}
	return ret;
}

PUBLIC void SMB_CONF::editdefaults()
{
	SMB_SHARE *d = shares.getitem(K_GLOBAL);
	if (d != NULL) d->editglobal();
}
PUBLIC void SMB_CONF::edithomes()
{
	SMB_SHARE *d = shares.getitem(K_HOMES);
	if (d != NULL) d->edithomes();
}
PUBLIC void SMB_CONF::editprinters()
{
	SMB_SHARE *d = shares.getitem(K_PRINTERS);
	if (d != NULL) d->editprinters();
}

PUBLIC void SMB_CONF::edit()
{
	samba = this;
	static const char *m_def = MSG_U(M_DEFAULT,"Defaults");
	static const char *m_homes = MSG_U(M_HOMES,"Default setup for users's home");
	static const char *m_printers = MSG_U(M_PRINTERS,"Default setup for printers");
	static const char *m_shares = MSG_U(M_SHARES,"Disk shares");
	static const char *menuopt[]={
		"",		m_def,
		"",		m_homes,
		"",		m_printers,
		"",		m_shares,
		NULL,
	};
	DIALOG_MENU dia;
	dia.new_menuitems (menuopt);
	int choice=0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_SAMBA,"Samba administration")
			,MSG_U(I_SAMBA,"This menu allows to configure the Samba SMB file server")
			,help_samba
			,choice,0);
		if (code == MENU_QUIT || code == MENU_ESCAPE){
			break;
		}else{
			const char *key = menuopt[choice*2+1];
			if (key == m_def){
				editdefaults();
			}else if (key == m_homes){
				edithomes();
			}else if (key == m_printers){
				editprinters();
			}else if (key == m_shares){
				editshares();
			}
		}
	}	
}

void samba_edit ()
{
	SMB_CONF smb;
	smb.edit();
}

