/* ***********

	Check comments starting with // ### and fill appropriate code there

   ***********
*/
#pragma implementation
#include <stdio.h>
#include <string.h>
#include <translat.h>
#include "status.h"
#include "status.m"

static HELP_FILE help_status("status","status");

MODULE_DEFINE_VERSION(status);

PUBLIC MODULE_status::MODULE_status()
	: LINUXCONF_MODULE("status")
{
	linuxconf_loadmsg ("status",PACKAGE_REV);
}


static const char *keymenu=NULL;

PUBLIC void MODULE_status::setmenu (
	DIALOG &dia,
	MENU_CONTEXT context)
{
	if (context == MENU_MAIN_CONTROL){
		keymenu = MSG_U(M_status,"Viewing system state");
		dia.new_menuitem ("status","",keymenu);
	}
}

static const char *report;
static const char *sysinfo;
static const char *memory;
static const char *disk;
static const char *inter;
static const char *iprouting;
static const char *ipxrouting;
static const char *processes;
static const char *ioports;
static const char *interrupts;

static void ft (void *p)
{
	const char *key = (const char *)p;
	if (key == report){
		status_report();
	}else if (key == sysinfo){
		sys_sysinfo_show();
	}else if (key == memory){
		sys_memory_show();
	}else if (key == disk){
		sys_disk_show();
	}else if (key == inter){
		sys_inter_show();
	}else if (key == iprouting){
		sys_iprouting_show();
	}else if (key == ipxrouting){
		sys_ipxrouting_show();
	}else if (key == processes){
		sys_processes_show();
	}else if (key == interrupts){
		sys_interrupts_show();
	}else if (key == ioports){
		sys_ioports_show();
	}else{
		const char *argv[2];
		argv[0] = key;
		argv[1] = NULL;
		module_sendmessage ("statusshow",1,argv);
	}
}

static void status_mainmenu()
{
	report = MSG_U(M_REPORT,"Prepare a report");
	sysinfo = MSG_U(M_SYSINFO,"System information");
	memory = MSG_U(M_MEMORY,"memory usage(RAM)");
	disk = MSG_U(M_DISK,"disk usage");
	inter = MSG_U(M_INTERFACE,"network devices");
	iprouting = MSG_U(M_IPROUTING,"IP routing table");
	ipxrouting = MSG_U(M_IPXROUTING,"IPX routing table");
	processes = MSG_U(M_PROCESSES,"processes");
	interrupts = MSG_U(M_INTERRUPTS,"interrupts mapping");
	ioports = MSG_U(M_IOPORTS,"I/O port mapping");

	static const char *tbopt[]={
		"", report,
		"",	sysinfo,
		"",	memory,
		"",	disk,
		"",	inter,
		"",	iprouting,
		"",	ipxrouting,
		"", processes,
		"", interrupts,
		"", ioports,
		NULL
	};
		
	DIALOG_MENU dia;
	dia.new_menuitems(tbopt);
	dia.setmenu (MENU_SYS_STATUS);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_STATUS,"System status")
			,MSG_U(I_STATUS
				,"You can view the current state of your computer.\n"
				 "You can't modidy anything from here.")
			,help_status
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			uithread (ft,(void*)key);
		}
	}
}

static void status_main()
{
	if (netconf_mainaccess()){
		status_mainmenu();
	}
}

PUBLIC int MODULE_status::domenu (
	MENU_CONTEXT context,
	const char *key)
{
	if (context == MENU_MAIN_CONTROL){
		if (key == keymenu){
			status_mainmenu();
		}
	}
	return 0;
}


PUBLIC int MODULE_status::dohtml (const char *key)
{
	int ret = LNCF_NOT_APPLICABLE;
	if (strcmp(key,"status")==0){
		// ### Insert any menu and dialog here
		ret = 0;
	}
	return ret;
}


static void usage()
{
	xconf_error (MSG_U(T_USAGE
		,"status usage\n"
		 "\n"
		 "status --option ...\n")
		);
}

PUBLIC int MODULE_status::execmain (int argc , char *argv[])
{
	int ret = LNCF_NOT_APPLICABLE;
	const char *pt = strrchr(argv[0],'/');
	if (pt != NULL){
		pt++;
	}else{
		pt = argv[0];
	}
	if (strcmp(pt,"status")==0){
		ret = -1;
		if (argc == 1){
			status_main();
		}else{
			// ### Add some option parsing for the module
			::usage();
		}
	}
	return ret;
}


static MODULE_status status;

