/* *** oss_interface ***
   *** Filename: oss_interface.c
   *** History: 19980111 Ver 1.00 Rev A | Michiel Bronswijk
   ***          19980225 Release version 1.00
   *** Description: Interface to the Open Sound System
   *** Changes:
*/

#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/soundcard.h>

#include "oss_interface.h"

#define DEVDSP	"/dev/dsp"
int	dsp_fd;

/* *** soundInitPlay ***
   *** History: 19980109 Ver 1.00 Rev A | Michiel Bronswijk
   *** Description: Initializes the soundcard for digital audio play
   *** Remarks: The sound format and number of channels will always
   		be set correctly when successful, sample-rate
   		can differ, check the returned sSpeed to see if
   		it's within your margins.
   *** Changes:
*/
int	soundInitPlay(int sFormat, int sChannel, int *sSpeed)
{
	int paramFormat, paramChannel;
	
	paramFormat = sFormat;
	paramChannel = sChannel;
	/* open /dev/dsp for output */
	if ((dsp_fd = open(DEVDSP, O_WRONLY | O_SYNC)) == -1)
		return(sound_DEVICENOTAVAIL);
	/* set sound format */
	ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &sFormat);
	if (sFormat != paramFormat) goto soundInitfailure;
	/* set number of channels */
	ioctl(dsp_fd, SNDCTL_DSP_STEREO, &sChannel);
	if (sChannel != paramChannel) goto soundInitfailure;
	/* set sample rate */
	ioctl(dsp_fd, SNDCTL_DSP_SPEED, sSpeed);
	return(sound_OK);
	
	soundInitfailure:
	close(dsp_fd);
	return(sound_NOTSUPPORTED);
}      	

/* *** soundInitRecord ***
   *** History: 19980121 Ver 1.00 | Michiel Bronswijk & Sander Steffann
   *** Description: Initializes the soundcard for digital audio play
   *** Remarks: The sound format and number of channels will always
   		be set correctly when successful, sample-rate
   		can differ, check the returned sSpeed to see if
   		it's within your margins.
   *** Changes:
*/
int	soundInitRecord (int sFormat, int sChannel, int *sSpeed)
{
	int paramFormat, paramChannel;
	int result;
		
	paramFormat = sFormat;
	paramChannel = sChannel;
	/* open /dev/dsp for output */
	if ((dsp_fd = open(DEVDSP, O_RDONLY | O_SYNC)) == -1)
		return(sound_DEVICENOTAVAIL);
	/* set sound format */
	ioctl(dsp_fd, SNDCTL_DSP_SETFMT, &sFormat);
	if (sFormat != paramFormat) goto soundInitfailure;
	/* set number of channels */
	ioctl(dsp_fd, SNDCTL_DSP_STEREO, &sChannel);
	if (sChannel != paramChannel) goto soundInitfailure;
	/* set sample rate */
	ioctl(dsp_fd, SNDCTL_DSP_SPEED, sSpeed);
	return(sound_OK);
	
	soundInitfailure:
	close(dsp_fd);
	return(sound_NOTSUPPORTED);
}      	

/* *** soundDone ***
   *** History: 19980109 Ver 1.00 Rev A | Michiel Bronswijk
   *** Description: Stops the soundcard playing digital audio
   *** Remarks: If you set the forced flag sound will stop immediately,
   		if not set current sound buffers will be emptied first.
   *** Changes:
*/
int	soundDone(int forced)
{
	if (forced)
		ioctl(dsp_fd, SNDCTL_DSP_RESET, 0);
	if (close(dsp_fd) == -1)
		return(sound_ERROR);
	else
		return(sound_OK);
}

/* *** soundPlay ***
   *** History: 19980109 Ver 1.00 Rev A | Michiel Bronswijk
   *** Description: Sends buffer to the soundcard
   *** Remarks: This call will block if the soundcard's own buffer is full
   *** Changes:
*/
int	soundPlay(const void *buffer, int buflength)
{
	if (write(dsp_fd, buffer, buflength) != buflength)
		return(sound_ERROR);
	else
		return(sound_OK);
}

/* *** soundRecord ***
   *** History: 19980121 Ver 1.00 | Michiel Bronswijk & Sander Steffann
   *** Description: Sends buffer to the soundcard
   *** Remarks: This call will block if the soundcard's own buffer is full
   *** Changes:
*/
int	soundRecord(void *buffer, int buflength)
{
	int dataread;

	if ((dataread=read(dsp_fd, buffer, buflength)) == -1)
		return(sound_ERROR);
	else
		return(dataread);
}

/* *** soundPause ***
   *** History: 19980111 Ver 1.00 Rev A | Michiel Bronswijk
   *** Description: Pauses playing/recording
   *** Remarks:
   *** Changes:
*/
int	soundPause(void)
{
	if (ioctl(dsp_fd, SNDCTL_DSP_POST, 0) == -1)
		return(sound_ERROR);
	else
		return(sound_OK);
}

