/* player.c */
/* Version 1.00
 * Copyright (C) 1998 Projectgroep MP3-studio voor Linux
 * 19980225
 * Coders: Sander Steffann & Michiel Bronswijk
 */

#define PLAYER_VERSION	"1.20"

#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <unistd.h>
#include <signal.h>

#include "mpeg_format.h"
#include "oss_interface.h"

// Waar gaan de standaard-meldingen naartoe?
#define TCL_Stream stdout

// De structuur van de buffer
typedef struct	sample_buffer {
	char		*data;		// Pointer naar de data
	int		data_size;	// Hoeveel ruimte is er voor data
	int		data_start;	// Positie waar eerste data staat
	int		data_length;	// Hoeveelheid data in buffer
	int		EOF_Reached;	// Vlag voor einde invoer
				
	pthread_mutex_t	mutex;		// Mutex voor buffer-lock
	pthread_cond_t	has_data;	// Condition voor "data in buffer"
	pthread_cond_t	has_room;	// Condition voor "plek in buffer"
	} buffer_t;
	
/* Global variables ********************************************** */
long			reqpos = 0;	// Welke sec beginnen met afspelen?

// Pipes voor omleiden in/uit/err van de decoder (l3dec)
int			decoderin_fds[2];
int			decoderout_fds[2];
int			decodererr_fds[2];

// Process ID van decoder
pid_t			decoder_pid;

// Threads voor buffer-lees en -schrijffuncties
pthread_t		tdec2buf;
pthread_t		tbuf2snd;

// Structuur MP3 header
struct mpegformat_t	cur_mp3;

// Declaratie buffer met initialisatie variabelen
buffer_t		buffer = {NULL, 0, 0, 0, 0, 
				PTHREAD_MUTEX_INITIALIZER,
				PTHREAD_COND_INITIALIZER,
				PTHREAD_COND_INITIALIZER};
		 
/*
 * RemoveTmp:
 *  Deze routine ruimt alle tijdelijke bestanden op die het programma aangemaakt
 *  heeft. Later in het programma zal deze routine aan verschillende afsluit-
 *  signals gekoppeld worden, zodat bij het afbreken van het programma geen
 *  rotzooi achterblijft.
 */
void	RemoveTmp(int sig)
{
	printf("Closing down on signal number %d\nCleaning up my room\n", sig);

	unlink("/tmp/mp3_studio_player_link.mp3");
	
	exit(0);
}

/* *** Decoder2Buffer ***
   *** History: 19980113 Ver 1.00 | Sander Steffann
   ***          19980114 Ver 1.01 | Sander Steffann & Michiel Bronswijk
   *** Description: Thread that buffers the output of the decoder
   *** Changes: 19980114 | optimized with memcpy
 *
 * Deze routine leest de data van de decoder uit de betreffende pipe, en zet
 * deze data in de buffer.
 */ 
void	*Decoder2Buffer(void *arg)
{
	char	input[4096];		// Buffer voor invoer
	int	todo, t;		// Tellers
	int	startpos, maxwrite;	// Variabelen voor berekening positie
					// van data in buffer
	
	// Oneindige lus voor lezen data
	while (1)
	{
	
	// Als data gelezen, dan...
	if ((todo = read(decoderout_fds[0], &input, sizeof(input))) > 0) {
		// Lock buffer voor verwerking
		pthread_mutex_lock(&buffer.mutex);
		
		// Teller op 0, begin met verwerken data zolang er nog
		// onverwerkte data is.
		t = 0;
		while (t < todo) {
			// Wacht zolang buffer vol is op condition
			while (buffer.data_length == buffer.data_size) {
				pthread_cond_wait(&buffer.has_room, 
					&buffer.mutex);
				}
				
			/* calculate start-position for writing */
			startpos = (buffer.data_start + buffer.data_length) %
				buffer.data_size;

			/* calculate free space in buffer */
			maxwrite = buffer.data_size - buffer.data_length;

			/* handle wraps */
			if ((startpos + maxwrite) > buffer.data_size)
				maxwrite = buffer.data_size-startpos;

			/* if there's not enough data, reduce maxwrite */
		        if (maxwrite>=(todo-t))
		        	maxwrite = (todo-t);

		        /* transfer data */
		        memcpy(&(buffer.data[startpos]), &(input[t]), 
		        	maxwrite);

			/* update counters */
		        buffer.data_length += maxwrite;
		        t += maxwrite;
			}
		// Unlock buffer en geef aan dat buffer data bevat
		pthread_mutex_unlock(&buffer.mutex);
		pthread_cond_signal(&buffer.has_data);
		}
	}
}

/* *** Buffer2Soundcard ***
   *** History: 19980113 Ver 1.00 | Sander Steffann
   ***          19980115 Ver 1.01 | Sander Steffann & Michiel Bronswijk
   ***          19980120 Ver 1.02 | Sander Steffann & Michiel Bronswijk
   *** Description: Thread that writes the buffer to the soundcard
   *** Changes: 19980115 | optimized with memcpy
   ***          19980120 | added seconds output
 *
 * Deze routine leest de data uit de buffer en stuurt deze direct naar de 
 * geluidskaart.
 */ 	
void	*Buffer2Soundcard(void *arg)
{
	char	output[4096];		// uitvoer-buffer
	int	outputsize;		// | Variabelen voor berekening
	int	startpos, maxread;	// | positie data
	int	totaloutput;		// Aantal bytes dat naar geluidskaart
					// is gestuurd
	long	currenttime = 0, 	// tijd die gespeeld is
		displaytime = -1;	// tijd voor op display
		
	totaloutput = 0;		// Nog niets verstuurd

	// Oneindige lus
	while (1)
	{
	
	// Lock buffer voor verwerken data
	pthread_mutex_lock(&buffer.mutex);
	
	// Als er geen data in de buffer zit...
	while (buffer.data_length == 0) {
		// Sluit af als geen nieuwe data meer zal komen
		if (buffer.EOF_Reached == 1) {
			pthread_mutex_unlock(&buffer.mutex);
			pthread_exit(0);
			}
		
		// Of wacht op nieuwe data
		pthread_cond_wait(&buffer.has_data, &buffer.mutex);
		}
		
	// Nog geen data verwerkt
	outputsize = 0;
	
	// Zolang er data in de buffer zit, en de uitvoerbuffer niet vol is
	while ((buffer.data_length>0) && (outputsize < sizeof(output))) {
		/* calculate startpos for reading */
		startpos = buffer.data_start;

		/* set size */
		maxread = buffer.data_length;

		/* handle wrap */
		if ((startpos+maxread)>buffer.data_size)
			maxread = buffer.data_size - startpos;

		/* reduce if buffer would overflow */
		if (maxread >= (sizeof(output)-outputsize)) 
			maxread = sizeof(output)-outputsize;

		/* transfer data */
		memcpy(&(output[outputsize]), &(buffer.data[startpos]),
			maxread);

		/* update counters */
		buffer.data_start = (buffer.data_start + maxread) % 
			buffer.data_size;
		buffer.data_length -= maxread;
		outputsize += maxread;
		}
	// Unlock buffer en geef aan dat er weer plek is
	pthread_mutex_unlock(&buffer.mutex);
	pthread_cond_signal(&buffer.has_room);
	
	// Pas aantal afgespeelde bytes aan
	totaloutput += outputsize;
	
	// Pas de display-tijd aan en laat tijd zien
	// Mono:   Tijd = aantal bytes / (samplerate * 2 (16-bit))
	// Stereo: Tijd = aantal bytes / (samplerate * 2 (stereo) * 2 (16-bit))
	currenttime = totaloutput / 
		( ((cur_mp3.channels == SINGLE_CHANNEL)? 1: 2) *
		 (cur_mp3.samplerate) * 2 );
		 
	// Als tijd veranderd, laat nieuwe tijd zien
	if (currenttime != displaytime) {
		displaytime = currenttime;
		fprintf(TCL_Stream, "STATUS Playpos %ld\r", displaytime + reqpos);
		fflush(NULL);
		}
	
	// Stuur de data naar de geluidskaart
	soundPlay(&output, outputsize);
	}
}

/* *** GetFileFormat ***
   *** History: 19980119 Ver 1.00 | Michiel Bronswijk
   ***          19980120 Ver 1.01 | Sander Steffann & Michiel Bronswijk
   *** Description: Reads the mp3-file and initializes audio-settings
 *
 * Leest informatie van MP3 bestand
 */ 	
int	GetFileFormat(const char filename[], struct mpegformat_t *mp3_fmt)
{
	struct stat		status;		// stat-struct voor MP3-file
	int			mp3_fd;		// file-descriptor voor file
	long			filesize;	// bestandsgrootte
	char			header_buffer[MPEG_HEADER_LENGTH]; // Buffer
				
	/* check if source file exists */
	if (stat(filename, &status) == -1) {
		fprintf(TCL_Stream, "ERROR %s File_not_found\n", filename);
		return(-1);
		};
	filesize = (long)status.st_size;

	/* determine format of source file */
	if ((mp3_fd = open(filename, O_RDONLY)) == -1) {
		fprintf(TCL_Stream, "ERROR %s Could_not_open\n", filename);
		return(-1);
		};

	/* Read header */
	if (read(mp3_fd, &header_buffer, MPEG_HEADER_LENGTH) == -1) {
		fprintf(TCL_Stream, "ERROR %s Could_not_read\n", filename);
		close(mp3_fd);
		return(-1);
		};
	
	close(mp3_fd);
	
	/* Process header */
	mpegGetFormat(&header_buffer, mp3_fmt);

	/* if it seems to be a valid file output its format */
	if (mp3_fmt->version != -1) {
		/* calculate duration of the MP3-song */
		mp3_fmt->totalframes = 
			(long)((double)filesize/mp3_fmt->framesize);
		mp3_fmt->duration = mp3_fmt->totalframes*
		           ((float)1152/(float)mp3_fmt->version)/
		           (float)mp3_fmt->samplerate;
#ifdef DEBUG
		printf("DEBUG filesize %ld\n", filesize);
		printf("DEBUG framecount %ld\n", mp3_fmt->totalframes);
#endif
		return(0);
		}
	else {
		fprintf(TCL_Stream, "ERROR %s No_MP3_file\n", filename);
		return(-1);	
		};
	}

/*
 * cmdINFO
 *  Laat informatie over een MP3 bestand zien
 */
void	cmdINFO(const char filename[])
{
	struct mpegformat_t	mp3_fmt;	// MP3 header

	// Lees header MP3
	if (GetFileFormat(filename, &mp3_fmt) != -1) {
		// Laat info van MP3 zien
		fprintf(TCL_Stream, "INFO %s Version %d Layer %d Bitrate %d Samplerate %d Mode ",
			filename, mp3_fmt.version, mp3_fmt.layer,
			mp3_fmt.kilobitrate, mp3_fmt.samplerate);
		switch(mp3_fmt.channels) {
			case SINGLE_CHANNEL: fprintf(TCL_Stream, "single_channel"); break;
			case DUAL_CHANNEL  : fprintf(TCL_Stream, "dual_channel"); break;
			case JOINT_STEREO  : fprintf(TCL_Stream, "joint_stereo"); break;
			case STEREO        : fprintf(TCL_Stream, "stereo"); break;
			default		   : fprintf(TCL_Stream, "unknown"); break;
			};		
		fprintf(TCL_Stream, " Duration %ld\n", mp3_fmt.duration);		
		}
}

/*
 * cmdPLAY
 *  Speel een MP3 bestand af. Deze routine verzorgt het aanmaken van de buffer
 *  e.d.
 */
void	cmdPLAY(const char filename[], int reqpos, int playtype)
{
	int			audio_format;	// Info
	int			audio_stereo;	// over
	int			audio_speed;	// audioformaat
	
	void*			resultfromthread;	// Exitvalue thread
	char			decoderinstr[8];	// invoer voor decoder

	// Lees formaat bestand
	if (GetFileFormat(filename, &cur_mp3) == -1)
		return;

	// Zet audio-formaat
	audio_format = AFMT_S16_LE;
	audio_stereo = (cur_mp3.channels == SINGLE_CHANNEL)? 0 : 1;
	audio_speed = cur_mp3.samplerate;
	
	// Open geluidskaart voor afspelen
	if (soundInitPlay(audio_format, 
			audio_stereo,
			&audio_speed) != sound_OK) {
	    	fprintf(TCL_Stream, "ERROR Failure initializing soundcard\n");
		return;
	    	};
	    	
	/* create pipes for the decoder. Wegens problemen niet allemaal
	   gebruikt */
	if ((pipe(decodererr_fds) == -1) 
	|| (pipe(decoderout_fds) == -1)
	|| (pipe(decoderin_fds) == -1)) {
		fprintf(TCL_Stream, "ERROR Couldn't create the pipes\n");
		return;
		}

	/* do a fork for executing the decoder */
	if (decoder_pid = fork() == 0) {
		/* execute this as a child */
		int		org_stdin_fd;
		int		org_stdout_fd;
		char		framestr[32];
		char		fullpath[MAXPATHLEN];
		
		/* Save original fd's */
		org_stdin_fd = dup(STDIN_FILENO);
		org_stdout_fd = dup(STDOUT_FILENO);
		
		/* Redirect in/output */
		dup2(decoderin_fds[0],STDIN_FILENO);
		dup2(decoderout_fds[1],STDOUT_FILENO);
		
		/* Leek een goed idee, maar werkt niet. Reden onbekend */
		// decodererr_fds[0] = open("/dev/null", O_WRONLY);
		// dup2(decodererr_fds[1],STDERR_FILENO);
		
		/* Dan maar op de botte manier */
		close(STDERR_FILENO);
		
		/* Links naar bestand maken. Dit is om problemen met spaties
		   in de bestandnaam af te vangen. Een symlink naar zo'n
		   bestand maken levert geen probleem op. Zo'n bestandsnaam
		   als parameter aan MP3 doorgeven wel. */
		unlink("/tmp/mp3_studio_player_link.mp3");
		realpath(filename, fullpath);
		symlink(fullpath, "/tmp/mp3_studio_player_link.mp3");

		// Als we een gewon bestand afspelen:
		if (playtype == 0) {
			// Bepaal de startpositie
			sprintf(framestr, "%ld", (reqpos*cur_mp3.totalframes) /
				cur_mp3.duration);

			/* try following path */
			execlp("l3dec", "l3dec", 
				"/tmp/mp3_studio_player_link.mp3",
				"-sto",
				"-fb", framestr, NULL);
			/* last try current dir */
			execl("./l3dec", "l3dec", 
				"/tmp/mp3_studio_player_link.mp3",
				"-sto",
				"-fb", framestr, NULL);
		} else {
			// EXTRA: Onofficiele optie
			// Speel gebluid af vanaf een DAT tape. Gebruikt iets
			// andere opties voor l3dec dan bij afspelen bestand
		
			/* try following path */
			execlp("l3dec", "l3dec",
				"/tmp/mp3_studio_player_link.mp3",
				"-sto", NULL);
			/* last try current dir */
			execl("./l3dec", "l3dec",
				"/tmp/mp3_studio_player_link.mp3",
				"-sto", NULL);
		}

		/* if this gets executed something went wrong */
		unlink("/tmp/mp3_studio_player_link.mp3");

		dup2(org_stdin_fd, STDIN_FILENO);
		dup2(org_stdout_fd, STDOUT_FILENO);
		fprintf(TCL_Stream, "ERROR Failure executing the decoder\n");
		exit(1);
		};
		
	/* Do the actual data-transfer */

	/* allocate buffer for 5 seconds of PCM data */
	buffer.data = (char*)malloc(audio_speed*2*(audio_stereo+1)*5);
	buffer.data_size = audio_speed*2*(audio_stereo+1)*5;
#ifdef DEBUG	
	printf("DEBUG Buffersize %d\n", buffer.data_size);
#endif		
	pthread_create(&tbuf2snd, NULL, Buffer2Soundcard, NULL);
	pthread_create(&tdec2buf, NULL, Decoder2Buffer, NULL);

	// Tell l3dec we don't want to register now. Remember you STILL
	// have to pay for l3dec!!!
	strcpy(decoderinstr, "n\n");
	write(decoderin_fds[1], &decoderinstr, sizeof(decoderinstr));

	// Wacht op einde decoder
	waitpid(decoder_pid, NULL, 0);
	
	printf("INFO Waiting for playback to finish\n");

	// Lock buffer and tell threads that no more data is coming
	pthread_mutex_lock(&buffer.mutex);
	buffer.EOF_Reached = 1;
	pthread_mutex_unlock(&buffer.mutex);
	pthread_cond_signal(&buffer.has_data);

	pthread_join(tbuf2snd, &resultfromthread);
	
	free(buffer.data);
	
	soundDone(0); /* don't truncate */
}

/*
 * The bootstrap
 *  Start het programma op
 */
int main(int argc, char *argv[])
{
	char		command[80];
	int		task;
		
	/* display notice */
	fprintf(stderr, "player - Version %s (%s %s)\n", 
		PLAYER_VERSION, __DATE__, __TIME__);	
	fprintf(stderr, 
		"Copyright (C) 1998 'Projectgroep MP3-studio voor Linux'\n");
#ifdef DEBUG
	fprintf(stderr, "DEBUG output enabled\n");
#endif
	/* check arguments */
	if (argc<3) {
		fprintf(stderr, "Usage:\n %s info <mp3-file>\n", argv[0]);
		fprintf(stderr, " %s play <mp3-file> [starttime]\n", argv[0]);
		fprintf(stderr, " %s tape <device>\n", argv[0]);
		exit(1);
		};
		
	/* What are we going to do? */
	if (strcmp(argv[1], "play") == 0) {
		task = 1;
		if (argc>3) reqpos = atoi(argv[3]);
		}
	else if (strcmp(argv[1], "info") == 0) {
		task = 2;
		}
	else if (strcmp(argv[1], "tape") == 0) {
		task = 3;
		}
	else {
		fprintf(stderr,"Wrong use of program!\n");
		exit(1);
		}
	
	/* Installing signal handlers */
	signal(SIGHUP, RemoveTmp);
	signal(SIGINT, RemoveTmp);
	signal(SIGTERM, RemoveTmp);
	signal(SIGQUIT, RemoveTmp);
	
	/* Do the work */
	switch(task) {
		case 1: cmdPLAY(argv[2], reqpos, 0); break;
		case 2: cmdINFO(argv[2]); break;
		case 3: cmdPLAY(argv[2], 0, 1); break;
		};
}
