/* recorder.c */
/* Version 1.00
 * Copyright (C) 1998 Projectgroep MP3-studio voor Linux
 * 19980225
 * Coders: Sander Steffann & Michiel Bronswijk
 */

#define RECORDER_VERSION 	"1.00"

#include <errno.h>
#include <fcntl.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>
#include <sys/stat.h>
#include <unistd.h>

#include "oss_interface.h"
#include "wav_interface.h"

// Waar gaan de standaard en error meldingen naartoe?
#define TCL_Stream stdout
#define ERR_Stream stderr

// De structuur van de buffer
typedef struct	sample_buffer {
	char		*data;		// Pointer naar de data
	int		data_size;	// Hoeveel ruimte is er voor data
	int		data_start;	// Positie waar eerste data staat
	int		data_length;	// Hoeveelheid data in buffer
	int		max_length;	// Voor timer: hoe lang opnemen?
	int		EOF_Reached;	// Vlag "stop met opnemen"
	int		Read_Finished;	// Vlag "laatste data in buffer"
				
	pthread_mutex_t	mutex;		// Mutex voor buffer-lock
	pthread_cond_t	has_data;	// Condition voor "data in buffer"
	pthread_cond_t	has_room;	// Condition voor "plek in buffer"
	} buffer_t;
	
/* Global variables ********************************************** */
int			outputfd;	// File descriptor: WAV file
int			samplerate;	// SampleRate
int			channels;	// Mono/Stereo

// Threads voor buffer-lees en -schrijffuncties en voor Abort-thread
pthread_t		tsnd2buf;	
pthread_t		tbuf2fil;
pthread_t		tabort;

// Declaratie buffer met initialosatie variabelen
buffer_t		buffer = {NULL, 0, 0, 0, 0, 0, 0,
				PTHREAD_MUTEX_INITIALIZER,
				PTHREAD_COND_INITIALIZER,
				PTHREAD_COND_INITIALIZER};
		 
/* *** Soundcard2Buffer ***
   *** History: 19980121 Ver 1.00 | Sander Steffann & Michiel Bronswijk
   ***          19980114 Ver 1.01 | Sander Steffann & Michiel Bronswijk
   *** Description: Thread that buffers the output of the soundcard
   *** Changes:
 *
 *  Leest data van geluidskaart en schrijft data naar buffer
 */ 
void	*Soundcard2Buffer(void *arg)
{
	char	input[4096];		// Invoer-buffer
	int	todo, t;		// Tellers
	int	startpos, maxwrite;	// Variabelen voor berekening positie
					// van data in buffer
	int	totaloutput;		// Aantal bytes dat van geluidskaart
					// ingelezen is
	long	currenttime = 0, 	// Tijd die opgenomen is
	        displaytime = -1;	// Tijd voor op display
		
	totaloutput = 0;		// Nog niets opgenomen
	
	// Onendige lus
	while (1)
	{
	
	// Als er data van de geluidskaart gelezen is...
	if ((todo = soundRecord(&input, sizeof(input))) > 0) {
		// Werk de getoonde tijd bij
		totaloutput += todo;
		currenttime = totaloutput /
			( (channels) *
			 (samplerate) * 2 );
			 
		// Alleen nieuwe tijd laten zien als de tijd echt veranderd is
		if (currenttime != displaytime) {
			displaytime = currenttime;
			fprintf(TCL_Stream, "STATUS Recpos %ld\r", 
				displaytime);
			fflush(NULL);
			}
			
		// Lock de buffer voor verwerking
		pthread_mutex_lock(&buffer.mutex);

		// Teller op 0, beginnen met verwerken data zolang er nog
		// onverwerkte data is.
		t = 0;
		while (t < todo) {
			// Wacht zolang buffer vol is op condition
			while (buffer.data_length == buffer.data_size) {
				pthread_cond_wait(&buffer.has_room, 
					&buffer.mutex);
				}
				
			/* calculate start-position for writing */
			startpos = (buffer.data_start + buffer.data_length) %
				buffer.data_size;
				
			/* calculate free space in buffer */
			maxwrite = buffer.data_size - buffer.data_length;

			/* handle wraps */
			if ((startpos + maxwrite) > buffer.data_size)
				maxwrite = buffer.data_size-startpos;

			/* if there's not enough data, reduce maxwrite */
		        if (maxwrite>=(todo-t))
		        	maxwrite = (todo-t);

		        /* transfer data */
		        memcpy(&(buffer.data[startpos]), &(input[t]), 
		        	maxwrite);

			/* update counters */
		        buffer.data_length += maxwrite;
		        t += maxwrite;
			}

		/* Exit on request */
		if (buffer.EOF_Reached == 1) {
			buffer.Read_Finished = 1;
			pthread_mutex_unlock(&buffer.mutex);
			pthread_cond_signal(&buffer.has_data);
			pthread_exit(0);
		}

		// Unlock buffer en geef aan dat er weer data is
		pthread_mutex_unlock(&buffer.mutex);
		pthread_cond_signal(&buffer.has_data);
		}
	}
}

/* *** Buffer2File ***
   *** History: 19980121 Ver 1.00 | Sander Steffann & Michiel Bronswijk
   *** Description: Thread that writes the buffer to a file
   *** Changes:
 *
 * Lees data uit buffer en zet in bestand
 */ 	
void	*Buffer2File(void *arg)
{
	char	output[4096];		// Buffer voor uitvoer
	int	outputsize;		// Aantal bytes in uitvoerbuffer
	int	startpos, maxread;	// Variabelen voor berekening positie
					// data in buffer

	// Oneindige lus
	while (1)
	{
	// Lock buffer voor verwerking
	pthread_mutex_lock(&buffer.mutex);
	
	// Als er geen data in de buffer zit...
	while (buffer.data_length == 0) {
		// Sluit af als geen nieuwe data meer zal komen
		if (buffer.Read_Finished == 1) {
			pthread_mutex_unlock(&buffer.mutex);
			pthread_exit(0);
			}
			
		// Of wacht op nieuwe data
		pthread_cond_wait(&buffer.has_data, &buffer.mutex);
		}

	// Nog geen data verwerkt
	outputsize = 0;
	
	// Zoalng er data in de buffer zit, en de uitvoerbuffer niet vol is
	while ((buffer.data_length>0) && (outputsize < sizeof(output))) {
		/* calculate startpos for reading */
		startpos = buffer.data_start;

		/* set size */
		maxread = buffer.data_length;

		/* handle wrap */
		if ((startpos+maxread)>buffer.data_size)
			maxread = buffer.data_size - startpos;

		/* reduce if buffer would overflow */
		if (maxread >= (sizeof(output)-outputsize)) 
			maxread = sizeof(output)-outputsize;

		/* transfer data */
		memcpy(&(output[outputsize]), &(buffer.data[startpos]),
			maxread);

		/* update counters */
		buffer.data_start = (buffer.data_start + maxread) % 
			buffer.data_size;
		buffer.data_length -= maxread;
		outputsize += maxread;
		}

	// Unlock buffer en geef aan dat er weer plek is
	pthread_mutex_unlock(&buffer.mutex);
	pthread_cond_signal(&buffer.has_room);
	
	// Schrijf data naar bestand, of stop bij error
	if (writeWAV(&output, outputsize) < outputsize) {
		fprintf(ERR_Stream, "ERROR Cannot write (disk full?)\n");
		pthread_exit(0);
		}
	}
}

/*
 * Abort
 *  In deze thread wordt bepaald wanneer we stoppen met opnemen
 */
void	*Abort(void *arg)
{
	int maxtime;

	// Lees maximale opnametijd uit buffer
	pthread_mutex_lock(&buffer.mutex);
	maxtime = buffer.max_length;
	pthread_mutex_unlock(&buffer.mutex);

	// Als tijdlimiet gesteld, dan daarop wachten
	if (maxtime > 0) {
		printf("INFO Recording for %d seconds\n", maxtime);
		printf("INFO Please wait...\n");
		
		while (maxtime > 0) {
			maxtime = sleep(maxtime);
			printf("INFO Someone disturbed my sleep... ");
			printf("%d seconds left\n", maxtime);
		}
	} else {
		// Anders wachten op toets
		char	ch;
	
		printf("INFO Press 'q' followed by <Enter> to quit <-\n\n");

		ch = '?';
		while ((ch != 'q') && (ch != 'Q'))
			ch = getchar();
	}
	
	printf("INFO Waiting for save to finish\n");

	// Geef aan threads door dat we gaan stoppen
	pthread_mutex_lock(&buffer.mutex);
	buffer.EOF_Reached = 1;
	pthread_mutex_unlock(&buffer.mutex);
	pthread_cond_signal(&buffer.has_data);
	pthread_exit(0);
}

/*
 * cmdRECORD
 *  Verzorgt het opnemen
 */
void	cmdRECORD(const char filename[], int chan, int sr)
{
	int			audio_format;
	int			result;
	void*			resultfromthread;
	
	// Maak WAV bestand aan
	creatWAV(filename, 16, chan, sr);

	// Zet audio-formaat
	audio_format = AFMT_S16_LE;
	channels = chan;
	samplerate = sr;
	
	// Open geluidskaart voor opnemen
	if ((result = soundInitRecord(audio_format, 
					channels-1, 
					&samplerate)) != sound_OK) {
	    	fprintf(ERR_Stream, "ERROR Failure initializing soundcard:");
		if (result == sound_DEVICENOTAVAIL) 
			fprintf(ERR_Stream, 
			"Device not available (busy or access problem)\n");
		if (result == sound_NOTSUPPORTED)
			fprintf(ERR_Stream, 
			"Sound parameters not supported\n"); 
		return;
	    	};

	/* Do the actual data-transfer */

	/* allocate buffer for 5 seconds of PCM data */
	buffer.data = (char*)malloc(samplerate*2*(channels)*5);
	buffer.data_size = samplerate*2*(channels)*5;
#ifdef DEBUG	
	printf("DEBUG Buffersize %d\n", buffer.data_size);
#endif
	
	// Maak threads voor lezen en schijven buffer, en voor afbreken
	pthread_create(&tbuf2fil, NULL, Buffer2File, NULL);
	pthread_create(&tsnd2buf, NULL, Soundcard2Buffer, NULL);
	pthread_create(&tabort, NULL, Abort, NULL);

	// Wacht tot wegschrijven data klaar
	pthread_join(tbuf2fil, &resultfromthread);
	
	// Sluit geluidskaart af
	soundDone(0);
#ifdef DEBUG	
	printf("DEBUG Soundcard shutdown!\n");
#endif
	free(buffer.data);
}

/*
 * Bootstrap
 */
int main(int argc, char *argv[])
{
	char		command[80];
	int		task;
	int		curarg;
	int		param_sr, param_ch;
		
	/* display notice */
	fprintf(stderr, "recorder - Version %s (%s %s)\n", 
		RECORDER_VERSION, __DATE__, __TIME__);	
	fprintf(stderr, 
		"Copyright (C) 1998 'Projectgroep MP3-studio voor Linux'\n");
#ifdef DEBUG
	fprintf(stderr, "DEBUG ouput enabled\n");
#endif

	/* check arguments */
	if (argc<2) {
		fprintf(ERR_Stream, "Usage:\n %s <wav-file> [option]...\n", 
			argv[0]);
		fprintf(ERR_Stream, "  Options are: -sr <samplerate>; ");
		fprintf(ERR_Stream, "selects samplerate (default=441000\n");
		fprintf(ERR_Stream, "               -ch <channels>  ; ");
		fprintf(ERR_Stream, "selects number of channels (default=2)\n");
		fprintf(ERR_Stream, "               -rt <seconds>   ; ");
		fprintf(ERR_Stream, "seconds to record (default=manual abort)\n");
		exit(1);
		};
		
	/* Set default parameters */
	param_sr = 44100;
	param_ch = 2;
	
	/* Parse command line */
	if (strncmp(argv[1], "-", 1) == 0) {
		fprintf(ERR_Stream, "Bad usage\n");
		exit(1);
		};
	curarg = 2;
	while (curarg<argc) {
		if (strncmp(argv[curarg], "-", 1) != 0) {
			fprintf(ERR_Stream, "Bad usage (argument: %s)\n", 
				argv[curarg]);
			exit(1);
			};
		if (strcmp(argv[curarg], "-sr") == 0) {
			param_sr = atoi(argv[curarg+1]);
			if ((param_sr<4000) || (param_sr>88200))
				param_sr = 44100;
			curarg++; curarg++;
		} else if (strcmp(argv[curarg], "-ch") == 0) {
			param_ch = atoi(argv[curarg+1]);
			if ((param_ch<1) || (param_ch>10))
				param_ch = 2;
			curarg++; curarg++;
		} else if (strcmp(argv[curarg], "-rt") == 0) {
			buffer.max_length = atoi(argv[curarg+1]);
			if (buffer.max_length<1) buffer.max_length = 0;
			curarg++; curarg++;
		} else {
			fprintf(ERR_Stream, "Unknown switch %s\n", 
				argv[curarg]);
			exit(1);
		}
	}
		
	// Start opname
	cmdRECORD(argv[1], param_ch, param_sr);

	// Sluit af, en maak WAV header
	closeWAV();
}
