#include "../config.h"
#ifdef GTKSTEP
#include <gtk/gtkfeatures.h>
#if !defined (GTK_MAJOR_VERSION) || (GTK_MAJOR_VERSION <= 1 && (GTK_MINOR_VERSION == 0 || (GTK_MINOR_VERSION == 1 && GTK_MICRO_VERSION <= 3)))
#if !defined (GTK_MINOR_VERSION) || GTK_MINOR_VERSION < 1
/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtklabel.h>


#define INDICATOR_SIZE     16
#define INDICATOR_SPACING  2

#define CHECK_BUTTON_CLASS(w)  GTK_CHECK_BUTTON_CLASS (GTK_OBJECT (w)->klass)


static void gtk_check_button_class_init          (GtkCheckButtonClass *klass);
static void gtk_check_button_init                (GtkCheckButton      *check_button);
static void gtk_check_button_draw                (GtkWidget           *widget,
						  GdkRectangle        *area);
static void gtk_check_button_draw_focus          (GtkWidget           *widget);
static void gtk_check_button_size_request        (GtkWidget           *widget,
						  GtkRequisition      *requisition);
static void gtk_check_button_size_allocate       (GtkWidget           *widget,
						  GtkAllocation       *allocation);
static gint gtk_check_button_expose              (GtkWidget           *widget,
						  GdkEventExpose      *event);
static void gtk_check_button_draw_indicator      (GtkCheckButton      *check_button,
						  GdkRectangle        *area);
static void gtk_real_check_button_draw_indicator (GtkCheckButton      *check_button,
						  GdkRectangle        *area);


static GtkToggleButtonClass *parent_class = NULL;


guint
gtk_check_button_get_type (void)
{
  static guint check_button_type = 0;

  if (!check_button_type)
    {
      GtkTypeInfo check_button_info =
      {
	"GtkCheckButton",
	sizeof (GtkCheckButton),
	sizeof (GtkCheckButtonClass),
	(GtkClassInitFunc) gtk_check_button_class_init,
	(GtkObjectInitFunc) gtk_check_button_init,
	(GtkArgSetFunc) NULL,
        (GtkArgGetFunc) NULL,
      };

      check_button_type = gtk_type_unique (gtk_toggle_button_get_type (), &check_button_info);
    }

  return check_button_type;
}

static void
gtk_check_button_class_init (GtkCheckButtonClass *class)
{
  GtkWidgetClass *widget_class;

  widget_class = (GtkWidgetClass*) class;
  parent_class = gtk_type_class (gtk_toggle_button_get_type ());

  widget_class->draw = gtk_check_button_draw;
  widget_class->draw_focus = gtk_check_button_draw_focus;
  widget_class->size_request = gtk_check_button_size_request;
  widget_class->size_allocate = gtk_check_button_size_allocate;
  widget_class->expose_event = gtk_check_button_expose;

  class->indicator_size = INDICATOR_SIZE;
  class->indicator_spacing = INDICATOR_SPACING;
  class->draw_indicator = gtk_real_check_button_draw_indicator;
}

static void
gtk_check_button_init (GtkCheckButton *check_button)
{
  check_button->toggle_button.draw_indicator = TRUE;
}

GtkWidget*
gtk_check_button_new (void)
{
  return GTK_WIDGET (gtk_type_new (gtk_check_button_get_type ()));
}


GtkWidget*
gtk_check_button_new_with_label (const gchar *label)
{
  GtkWidget *check_button;
  GtkWidget *label_widget;

  check_button = gtk_check_button_new ();
  label_widget = gtk_label_new (label);
  gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);

  gtk_container_add (GTK_CONTAINER (check_button), label_widget);
  gtk_widget_show (label_widget);

  return check_button;
}

static void
gtk_check_button_draw (GtkWidget    *widget,
		       GdkRectangle *area)
{
  GtkButton *button;
  GtkCheckButton *check_button;
  GdkRectangle child_area;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (area != NULL);

  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);

      if (check_button->toggle_button.draw_indicator)
	{
	  button = GTK_BUTTON (widget);

	  gtk_check_button_draw_indicator (check_button, area);

	  if (button->child && GTK_WIDGET_NO_WINDOW (button->child) &&
	      gtk_widget_intersect (button->child, area, &child_area))
	    gtk_widget_draw (button->child, &child_area);

	  gtk_widget_draw_focus (widget);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->draw)
	    (* GTK_WIDGET_CLASS (parent_class)->draw) (widget, area);
	}
    }
}

static void
gtk_check_button_draw_focus (GtkWidget *widget)
{
  GtkCheckButton *check_button;
  
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  
  if (GTK_WIDGET_DRAWABLE (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);
      if (check_button->toggle_button.draw_indicator)
	{
	  gint border_width;
	  
	  border_width = GTK_CONTAINER (widget)->border_width;
	  if (GTK_WIDGET_HAS_FOCUS (widget))
	    gdk_draw_rectangle (widget->window,
				widget->style->black_gc, FALSE,
				border_width, border_width,
				widget->allocation.width - 2 * border_width - 1,
				widget->allocation.height - 2 * border_width - 1);
	  else
	    gdk_draw_rectangle (widget->window,
				widget->style->bg_gc[GTK_STATE_NORMAL], FALSE,
				border_width, border_width,
				widget->allocation.width - 2 * border_width - 1,
				widget->allocation.height - 2 * border_width - 1);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->draw_focus)
	    (* GTK_WIDGET_CLASS (parent_class)->draw_focus) (widget);
	}
    }
}

static void
gtk_check_button_size_request (GtkWidget      *widget,
			       GtkRequisition *requisition)
{
  GtkCheckButton *check_button;
  GtkButton *button;
  gint temp;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (requisition != NULL);

  check_button = GTK_CHECK_BUTTON (widget);

  if (GTK_WIDGET_CLASS (parent_class)->size_request)
    (* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);

  if (check_button->toggle_button.draw_indicator)
    {
      button = GTK_BUTTON (widget);

      requisition->width += (CHECK_BUTTON_CLASS (widget)->indicator_size +
			     CHECK_BUTTON_CLASS (widget)->indicator_spacing * 3 + 2);

      temp = (CHECK_BUTTON_CLASS (widget)->indicator_size +
	      CHECK_BUTTON_CLASS (widget)->indicator_spacing * 2);
      requisition->height = MAX (requisition->height, temp) + 2;
    }
}

static void
gtk_check_button_size_allocate (GtkWidget     *widget,
				GtkAllocation *allocation)
{
  GtkCheckButton *check_button;
  GtkButton *button;
  GtkAllocation child_allocation;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (allocation != NULL);

  check_button = GTK_CHECK_BUTTON (widget);
  if (check_button->toggle_button.draw_indicator)
    {
      widget->allocation = *allocation;
      if (GTK_WIDGET_REALIZED (widget))
	gdk_window_move_resize (widget->window,
				allocation->x, allocation->y,
				allocation->width, allocation->height);

      button = GTK_BUTTON (widget);

      if (button->child && GTK_WIDGET_VISIBLE (button->child))
	{
	  child_allocation.x = (GTK_CONTAINER (widget)->border_width +
				CHECK_BUTTON_CLASS (widget)->indicator_size +
				CHECK_BUTTON_CLASS (widget)->indicator_spacing * 3 + 1);
	  child_allocation.y = GTK_CONTAINER (widget)->border_width + 1;
	  child_allocation.width = MAX (1, allocation->width - child_allocation.x  -
				    GTK_CONTAINER (widget)->border_width - 1);
	  child_allocation.height = MAX (1, allocation->height - child_allocation.y * 2);

	  gtk_widget_size_allocate (button->child, &child_allocation);
	}
    }
  else
    {
      if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
	(* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);
    }
}

static gint
gtk_check_button_expose (GtkWidget      *widget,
			 GdkEventExpose *event)
{
  GtkButton *button;
  GtkCheckButton *check_button;
  GdkEventExpose child_event;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_CHECK_BUTTON (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);

      if (check_button->toggle_button.draw_indicator)
	{
	  button = GTK_BUTTON (widget);

	  gtk_check_button_draw_indicator (check_button, &event->area);

	  child_event = *event;
	  if (button->child && GTK_WIDGET_NO_WINDOW (button->child) &&
	      gtk_widget_intersect (button->child, &event->area, &child_event.area))
	    gtk_widget_event (button->child, (GdkEvent*) &child_event);

	  gtk_widget_draw_focus (widget);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
	    (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
	}
    }

  return FALSE;
}


static void
gtk_check_button_draw_indicator (GtkCheckButton *check_button,
				 GdkRectangle   *area)
{
  GtkCheckButtonClass *class;

  g_return_if_fail (check_button != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (check_button));
  g_return_if_fail (CHECK_BUTTON_CLASS (check_button) != NULL);

  class = CHECK_BUTTON_CLASS (check_button);

  if (class->draw_indicator)
    (* class->draw_indicator) (check_button, area);
}

/* XPM */
static char *off2[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"................",
".##############b",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".bbbbbbbbbbbbbbb"
};
/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"bbbbbbbbbbbbbbb#",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"b##############.",
"#..............."
};

/* XPM */
static char *on2[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"................",
".##############b",
".#aaaaaaaaaabaab",
".#aaaaaaaaab.aab",
".#aaaaaaaab.#aab",
".#aaaaaaab.#aaab",
".#aab.aab.#aaaab",
".#aab.#b.#aaaaab",
".#aab#b.#aaaaaab",
".#aabb.#aaaaaaab",
".#aab.#aaaaaaaab",
".#aa.#aaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".bbbbbbbbbbbbbbb"
};
/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"bbbbbbbbbbbbbbb#",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaba#.",
"baaaaaaaaaab.a#.",
"baaaaaaaaab.#a#.",
"baaaaaaaab.#aa#.",
"baaab.aab.#aaa#.",
"baaab.#b.#aaaa#.",
"baaab#b.#aaaaa#.",
"baaabb.#aaaaaa#.",
"baaab.#aaaaaaa#.",
"baaa.#aaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"b##############.",
"#..............."
};

static void
gtk_real_check_button_draw_indicator (GtkCheckButton *check_button,
				      GdkRectangle    *area)
{
  GtkWidget *widget;
  GtkToggleButton *toggle_button;
  GdkRectangle restrict_area;
  GdkRectangle new_area;
  gint width, height;
  gint x, y;

  g_return_if_fail (check_button != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (check_button));

  if (GTK_WIDGET_DRAWABLE (check_button))
    {
      widget = GTK_WIDGET (check_button);
      toggle_button = GTK_TOGGLE_BUTTON (check_button);

      restrict_area.x = GTK_CONTAINER (widget)->border_width;
      restrict_area.y = restrict_area.x;
      restrict_area.width = widget->allocation.width - restrict_area.x * 2;
      restrict_area.height = widget->allocation.height - restrict_area.x * 2;

      if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
	{
	  gtk_style_set_background (widget->style, widget->window,
				    GTK_STATE_NORMAL);
	  gdk_window_clear_area (widget->window, new_area.x, new_area.y,
				 new_area.width, new_area.height);
	}
      
      x = CHECK_BUTTON_CLASS (widget)->indicator_spacing + GTK_CONTAINER (widget)->border_width;
      y = (widget->allocation.height - CHECK_BUTTON_CLASS (widget)->indicator_size) / 2;
      width = CHECK_BUTTON_CLASS (widget)->indicator_size;
      height = CHECK_BUTTON_CLASS (widget)->indicator_size;

      {
	 static GdkBitmap   *mask2_on     = NULL;
	 static GdkPixmap   *pixmap2_on   = NULL;
	 static GdkBitmap   *mask2_off    = NULL;
	 static GdkPixmap   *pixmap2_off  = NULL;
	 static GdkBitmap   *mask_on      = NULL;
	 static GdkPixmap   *pixmap_on    = NULL;
	 static GdkBitmap   *mask_off     = NULL;
	 static GdkPixmap   *pixmap_off   = NULL;

	 if (!pixmap_off || !pixmap_on)
	 {
	    pixmap2_off
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask2_off,
					       NULL, off2);
	    pixmap2_on
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask2_on,
					       NULL, on2);
	    pixmap_off
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask_off,
					       NULL, off);
	    pixmap_on
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask_on,
					       NULL, on);
	 }
	 gdk_gc_set_clip_mask( widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  GTK_TOGGLE_BUTTON (widget)->active
              ? ( GTK_BUTTON (widget)->button_down
              		? mask2_on : mask_on ) :
              ( GTK_BUTTON( widget)->button_down
                    ? mask2_off : mask_off ) );
	 gdk_gc_set_clip_origin( widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  x + 1, y + 1 );
	 gdk_draw_pixmap (widget->window,
			  widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  GTK_TOGGLE_BUTTON (widget)->active
			  ? ( GTK_BUTTON (widget)->button_down
					? pixmap2_on : pixmap_on ) :
			  ( GTK_BUTTON (widget)->button_down
				    ? pixmap2_off : pixmap_off ), 0, 0, x + 1, y + 1,
			  width, height);
	 gdk_gc_set_clip_mask( widget->style->bg_gc[GTK_WIDGET_STATE (widget)], NULL );
      }
    }
}
#else /* defined (GTK_MINOR_VERSION) && GTK_MINOR_VERSION >= 1 */
/* GTK - The GIMP Toolkit
 * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <gtk/gtkcheckbutton.h>
#include <gtk/gtklabel.h>


#define INDICATOR_SIZE     16
#define INDICATOR_SPACING  2

#define CHECK_BUTTON_CLASS(w)  GTK_CHECK_BUTTON_CLASS (GTK_OBJECT (w)->klass)


static void gtk_check_button_class_init          (GtkCheckButtonClass *klass);
static void gtk_check_button_init                (GtkCheckButton      *check_button);
static void gtk_check_button_draw                (GtkWidget           *widget,
						  GdkRectangle        *area);
static void gtk_check_button_draw_focus          (GtkWidget           *widget);
static void gtk_check_button_size_request        (GtkWidget           *widget,
						  GtkRequisition      *requisition);
static void gtk_check_button_size_allocate       (GtkWidget           *widget,
						  GtkAllocation       *allocation);
static gint gtk_check_button_expose              (GtkWidget           *widget,
						  GdkEventExpose      *event);
static void gtk_check_button_draw_indicator      (GtkCheckButton      *check_button,
						  GdkRectangle        *area);
static void gtk_real_check_button_draw_indicator (GtkCheckButton      *check_button,
						  GdkRectangle        *area);


static GtkToggleButtonClass *parent_class = NULL;


GtkType
gtk_check_button_get_type (void)
{
  static GtkType check_button_type = 0;

  if (!check_button_type)
    {
      GtkTypeInfo check_button_info =
      {
	"GtkCheckButton",
	sizeof (GtkCheckButton),
	sizeof (GtkCheckButtonClass),
	(GtkClassInitFunc) gtk_check_button_class_init,
	(GtkObjectInitFunc) gtk_check_button_init,
	/* reserved_1 */ NULL,
        /* reserved_2 */ NULL,
        (GtkClassInitFunc) NULL,
      };

      check_button_type = gtk_type_unique (gtk_toggle_button_get_type (), &check_button_info);
    }

  return check_button_type;
}

static void
gtk_check_button_class_init (GtkCheckButtonClass *class)
{
  GtkWidgetClass *widget_class;

  widget_class = (GtkWidgetClass*) class;
  parent_class = gtk_type_class (gtk_toggle_button_get_type ());

  widget_class->draw = gtk_check_button_draw;
  widget_class->draw_focus = gtk_check_button_draw_focus;
  widget_class->size_request = gtk_check_button_size_request;
  widget_class->size_allocate = gtk_check_button_size_allocate;
  widget_class->expose_event = gtk_check_button_expose;

  class->indicator_size = INDICATOR_SIZE;
  class->indicator_spacing = INDICATOR_SPACING;
  class->draw_indicator = gtk_real_check_button_draw_indicator;
}

static void
gtk_check_button_init (GtkCheckButton *check_button)
{
  check_button->toggle_button.draw_indicator = TRUE;
}

GtkWidget*
gtk_check_button_new (void)
{
  return GTK_WIDGET (gtk_type_new (gtk_check_button_get_type ()));
}


GtkWidget*
gtk_check_button_new_with_label (const gchar *label)
{
  GtkWidget *check_button;
  GtkWidget *label_widget;

  check_button = gtk_check_button_new ();
  label_widget = gtk_label_new (label);
  gtk_misc_set_alignment (GTK_MISC (label_widget), 0.0, 0.5);

  gtk_container_add (GTK_CONTAINER (check_button), label_widget);
  gtk_widget_show (label_widget);

  return check_button;
}

static void
gtk_check_button_draw (GtkWidget    *widget,
		       GdkRectangle *area)
{
  GtkButton *button;
  GtkCheckButton *check_button;
  GdkRectangle child_area;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (area != NULL);

  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);

      if (check_button->toggle_button.draw_indicator)
	{
	  button = GTK_BUTTON (widget);

	  gtk_check_button_draw_indicator (check_button, area);

	  if (GTK_BIN (button)->child && GTK_WIDGET_NO_WINDOW (GTK_BIN (button)->child) &&
	      gtk_widget_intersect (GTK_BIN (button)->child, area, &child_area))
	    gtk_widget_draw (GTK_BIN (button)->child, &child_area);

	  gtk_widget_draw_focus (widget);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->draw)
	    (* GTK_WIDGET_CLASS (parent_class)->draw) (widget, area);
	}
    }
}

static void
gtk_check_button_draw_focus (GtkWidget *widget)
{
  GtkCheckButton *check_button;
  
  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  
  if (GTK_WIDGET_DRAWABLE (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);
      if (check_button->toggle_button.draw_indicator)
	{
	  gint border_width;
	  
	  border_width = GTK_CONTAINER (widget)->border_width;
	  if (GTK_WIDGET_HAS_FOCUS (widget))
	    gdk_draw_rectangle (widget->window,
				widget->style->black_gc, FALSE,
				border_width, border_width,
				widget->allocation.width - 2 * border_width - 1,
				widget->allocation.height - 2 * border_width - 1);
	  else
	    gdk_draw_rectangle (widget->window,
				widget->style->bg_gc[GTK_STATE_NORMAL], FALSE,
				border_width, border_width,
				widget->allocation.width - 2 * border_width - 1,
				widget->allocation.height - 2 * border_width - 1);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->draw_focus)
	    (* GTK_WIDGET_CLASS (parent_class)->draw_focus) (widget);
	}
    }
}

static void
gtk_check_button_size_request (GtkWidget      *widget,
			       GtkRequisition *requisition)
{
  GtkCheckButton *check_button;
  GtkButton *button;
  gint temp;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (requisition != NULL);

  check_button = GTK_CHECK_BUTTON (widget);

  if (GTK_WIDGET_CLASS (parent_class)->size_request)
    (* GTK_WIDGET_CLASS (parent_class)->size_request) (widget, requisition);

  if (check_button->toggle_button.draw_indicator)
    {
      button = GTK_BUTTON (widget);

      requisition->width += (CHECK_BUTTON_CLASS (widget)->indicator_size +
			     CHECK_BUTTON_CLASS (widget)->indicator_spacing * 3 + 2);

      temp = (CHECK_BUTTON_CLASS (widget)->indicator_size +
	      CHECK_BUTTON_CLASS (widget)->indicator_spacing * 2);
      requisition->height = MAX (requisition->height, temp) + 2;
    }
}

static void
gtk_check_button_size_allocate (GtkWidget     *widget,
				GtkAllocation *allocation)
{
  GtkCheckButton *check_button;
  GtkButton *button;
  GtkAllocation child_allocation;

  g_return_if_fail (widget != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (widget));
  g_return_if_fail (allocation != NULL);

  check_button = GTK_CHECK_BUTTON (widget);
  if (check_button->toggle_button.draw_indicator)
    {
      widget->allocation = *allocation;
      if (GTK_WIDGET_REALIZED (widget))
	gdk_window_move_resize (widget->window,
				allocation->x, allocation->y,
				allocation->width, allocation->height);

      button = GTK_BUTTON (widget);

      if (GTK_BIN (button)->child && GTK_WIDGET_VISIBLE (GTK_BIN (button)->child))
	{
	  child_allocation.x = (GTK_CONTAINER (widget)->border_width +
				CHECK_BUTTON_CLASS (widget)->indicator_size +
				CHECK_BUTTON_CLASS (widget)->indicator_spacing * 3 + 1);
	  child_allocation.y = GTK_CONTAINER (widget)->border_width + 1;
	  child_allocation.width = MAX (1, allocation->width - child_allocation.x  -
				    GTK_CONTAINER (widget)->border_width - 1);
	  child_allocation.height = MAX (1, allocation->height - child_allocation.y * 2);

	  gtk_widget_size_allocate (GTK_BIN (button)->child, &child_allocation);
	}
    }
  else
    {
      if (GTK_WIDGET_CLASS (parent_class)->size_allocate)
	(* GTK_WIDGET_CLASS (parent_class)->size_allocate) (widget, allocation);
    }
}

static gint
gtk_check_button_expose (GtkWidget      *widget,
			 GdkEventExpose *event)
{
  GtkButton *button;
  GtkCheckButton *check_button;
  GdkEventExpose child_event;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_CHECK_BUTTON (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (GTK_WIDGET_VISIBLE (widget) && GTK_WIDGET_MAPPED (widget))
    {
      check_button = GTK_CHECK_BUTTON (widget);

      if (check_button->toggle_button.draw_indicator)
	{
	  button = GTK_BUTTON (widget);

	  gtk_check_button_draw_indicator (check_button, &event->area);

	  child_event = *event;
	  if (GTK_BIN (button)->child && GTK_WIDGET_NO_WINDOW (GTK_BIN (button)->child) &&
	      gtk_widget_intersect (GTK_BIN (button)->child, &event->area, &child_event.area))
	    gtk_widget_event (GTK_BIN (button)->child, (GdkEvent*) &child_event);

	  gtk_widget_draw_focus (widget);
	}
      else
	{
	  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
	    (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
	}
    }

  return FALSE;
}


static void
gtk_check_button_draw_indicator (GtkCheckButton *check_button,
				 GdkRectangle   *area)
{
  GtkCheckButtonClass *class;

  g_return_if_fail (check_button != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (check_button));
  g_return_if_fail (CHECK_BUTTON_CLASS (check_button) != NULL);

  class = CHECK_BUTTON_CLASS (check_button);

  if (class->draw_indicator)
    (* class->draw_indicator) (check_button, area);
}

/* XPM */
static char *off2[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"................",
".##############b",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".bbbbbbbbbbbbbbb"
};
/* XPM */
static char *off[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"bbbbbbbbbbbbbbb#",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"b##############.",
"#..............."
};

/* XPM */
static char *on2[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"................",
".##############b",
".#aaaaaaaaaabaab",
".#aaaaaaaaab.aab",
".#aaaaaaaab.#aab",
".#aaaaaaab.#aaab",
".#aab.aab.#aaaab",
".#aab.#b.#aaaaab",
".#aab#b.#aaaaaab",
".#aabb.#aaaaaaab",
".#aab.#aaaaaaaab",
".#aa.#aaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".#aaaaaaaaaaaaab",
".bbbbbbbbbbbbbbb"
};
/* XPM */
static char *on[] = {
/* width height num_colors chars_per_pixel */
"    16    16        4            1",
/* colors */
". c #000000",
"# c #505450",
"a c #a8a8a8",
"b c #f8fcf8",
/* pixels */
"bbbbbbbbbbbbbbb#",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaba#.",
"baaaaaaaaaab.a#.",
"baaaaaaaaab.#a#.",
"baaaaaaaab.#aa#.",
"baaab.aab.#aaa#.",
"baaab.#b.#aaaa#.",
"baaab#b.#aaaaa#.",
"baaabb.#aaaaaa#.",
"baaab.#aaaaaaa#.",
"baaa.#aaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"baaaaaaaaaaaaa#.",
"b##############.",
"#..............."
};

static void
gtk_real_check_button_draw_indicator (GtkCheckButton *check_button,
				      GdkRectangle    *area)
{
  GtkWidget *widget;
  GtkToggleButton *toggle_button;
  GdkRectangle restrict_area;
  GdkRectangle new_area;
  gint width, height;
  gint x, y;

  g_return_if_fail (check_button != NULL);
  g_return_if_fail (GTK_IS_CHECK_BUTTON (check_button));

  if (GTK_WIDGET_DRAWABLE (check_button))
    {
      widget = GTK_WIDGET (check_button);
      toggle_button = GTK_TOGGLE_BUTTON (check_button);

      restrict_area.x = GTK_CONTAINER (widget)->border_width;
      restrict_area.y = restrict_area.x;
      restrict_area.width = widget->allocation.width - restrict_area.x * 2;
      restrict_area.height = widget->allocation.height - restrict_area.x * 2;

      if (gdk_rectangle_intersect (area, &restrict_area, &new_area))
	{
	  gtk_style_set_background (widget->style, widget->window,
				    GTK_STATE_NORMAL);
	  gdk_window_clear_area (widget->window, new_area.x, new_area.y,
				 new_area.width, new_area.height);
	}
      
      x = CHECK_BUTTON_CLASS (widget)->indicator_spacing + GTK_CONTAINER (widget)->border_width;
      y = (widget->allocation.height - CHECK_BUTTON_CLASS (widget)->indicator_size) / 2;
      width = CHECK_BUTTON_CLASS (widget)->indicator_size;
      height = CHECK_BUTTON_CLASS (widget)->indicator_size;

      {
	 static GdkBitmap   *mask2_on     = NULL;
	 static GdkPixmap   *pixmap2_on   = NULL;
	 static GdkBitmap   *mask2_off    = NULL;
	 static GdkPixmap   *pixmap2_off  = NULL;
	 static GdkBitmap   *mask_on      = NULL;
	 static GdkPixmap   *pixmap_on    = NULL;
	 static GdkBitmap   *mask_off     = NULL;
	 static GdkPixmap   *pixmap_off   = NULL;

	 if (!pixmap_off || !pixmap_on)
	 {
	    pixmap2_off
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask2_off,
					       NULL, off2);
	    pixmap2_on
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask2_on,
					       NULL, on2);
	    pixmap_off
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask_off,
					       NULL, off);
	    pixmap_on
	       = gdk_pixmap_create_from_xpm_d (widget->window, &mask_on,
					       NULL, on);
	 }
	 gdk_gc_set_clip_mask( widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  GTK_TOGGLE_BUTTON (widget)->active
              ? ( GTK_BUTTON (widget)->button_down
              		? mask2_on : mask_on ) :
              ( GTK_BUTTON( widget)->button_down
                    ? mask2_off : mask_off ) );
	 gdk_gc_set_clip_origin( widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  x + 1, y + 1 );
	 gdk_draw_pixmap (widget->window,
			  widget->style->bg_gc[GTK_WIDGET_STATE (widget)],
			  GTK_TOGGLE_BUTTON (widget)->active
			  ? ( GTK_BUTTON (widget)->button_down
					? pixmap2_on : pixmap_on ) :
			  ( GTK_BUTTON (widget)->button_down
				    ? pixmap2_off : pixmap_off ), 0, 0, x + 1, y + 1,
			  width, height);
	 gdk_gc_set_clip_mask( widget->style->bg_gc[GTK_WIDGET_STATE (widget)], NULL );
      }
    }
}
#endif /* !defined (GTK_MINOR_VERSION) || GTK_MINOR_VERSION < 1 */
#endif /* GTK+ < 1.1.5 */
#endif /* GTKSTEP */
