#include "fileobj.h"
#include "mfinderapp.h"
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>

extern "C" {
	void XShapeCombineMask(Display*, Window, int, int, int, Pixmap, int);
 	#define ShapeBounding                   0 
	#define ShapeSet                        0 
}

extern ProgInfo prInfo[];
extern int prInfoNum;

extern char Editor[];
extern char Xterm[];

extern char IconPath[];
extern IconData *IData[];
extern int IDataNum;
extern FolderIconData *IFoldData, *IFoldLinkData;

DeskObj::DeskObj(QWidget *parent, const char *path, 
							const char *name, int id, int x, int y)
        :QWidget(0L, 0L, WStyle_Customize | WStyle_Tool | WStyle_NoBorder)
{
	char buff[256] = "", *token, *delims="/";
	int c = 0;

	sprintf(parentpath, "");
 	strncpy(this->path, path, 256);	
 	strncpy(this->name, name, 256);	
 	strncpy(buff, path, 256);	
	token = strtok(buff, delims);
	while(strcmp(token, name) != 0) {
		sprintf(parentpath, "%s/%s", parentpath, token);
		token = strtok(NULL, delims);
		c++;
	}	
	if(c == 0) sprintf(parentpath, "/");
	this->id = id;
	this->x = x;
	this->y = y;
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isDragging = FALSE;
	xOffset = 0;
	yOffset = 0;
	UpdateMyData();
	Icona = QPixmap();
	UpdateIcon();

	Icona.setMask(Icona.createHeuristicMask());
	font = QFont("Helvetica", 10);
	font.setItalic(TRUE);
	setFont(font);

	mask = QBitmap();	
	CalcDim();

	popupMenu = new Popup();
	popupMenu->insertItem("Open", this, SLOT(OpenFromPopUp()));
	popupMenu->insertItem("Info", this, SLOT(InfoFromPopUp()));
	popupMenu->insertItem("Remove from Desktop", this, SLOT(RemoveMe()));
	
	move(x, y);
	show();
	lower();
}

void DeskObj::UpdateMyData()
{
	struct stat listStat;

   lstat(path, &listStat);

	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		int ln = readlink(path, myLinkTo, 512);
		if(chdir(path) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(getenv("HOME"));
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		
}

void DeskObj::UpdateIcon()
{
	bool found = FALSE;
	char name2[256] = "";
	char *pref = "", *suff = "", *token = "", *delims = ".";
	
	sprintf(name2, "%s", name);
	token = strtok(name2, delims);
	pref = token;
	while(token != NULL) {
		suff = token;
		token = strtok(NULL, delims);
	}
	
	switch(type) {
		case FIL:
			if(name[0] == '.') {
				for(int i = 4; i < IDataNum; i++) {
					if(strcmp(IData[i]->pref, ".") == 0) {
						sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
						IDataRef = i;
						found = TRUE;
						break;
					}
				}
			}
			if(found) break;
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(pref, IData[i]->pref) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(suff, IData[i]->suff) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			sprintf(pixname, "%s/%s", IconPath, IData[0]->IcnName);
			IDataRef = 0;
			break;
		case FILEXE:
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(pref, IData[i]->pref) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			sprintf(pixname, "%s/%s", IconPath, IData[1]->IcnName);
			IDataRef = 1;
			break;
		case FILLNK:
			sprintf(pixname, "%s/%s", IconPath, IData[2]->IcnName);
			IDataRef = 2;
			break;
		case FILEXELNK:
			sprintf(pixname, "%s/%s", IconPath, IData[3]->IcnName);
			IDataRef = 3;
			break;
		case DR:
			sprintf(pixname, "%s/%s", IconPath, IFoldData->IcnName);
			break;
		case DRLNK:
			sprintf(pixname, "%s/%s", IconPath, IFoldLinkData->IcnName);
			break;
		default:
			sprintf(pixname, "%s/%s", IconPath, IData[0]->IcnName);
			IDataRef = 0;
			break;
	}

	if(type == DR) {
		Icona = QPixmap(IFoldData->Icona);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->Icona);
	} else {
		Icona = QPixmap(IData[IDataRef]->Icona);
	}
	
	update();
}

void DeskObj::CalcDim()
{
	QPainter p;
	p.begin(this);
	int ascent = p.fontMetrics().ascent();
	int descent = p.fontMetrics().descent();
	int width = p.fontMetrics().width(name);
	int w = width;
	if(Icona.width() > w) {
		w = Icona.width() + 8;
		textXOffset = (w - width) / 2;
		pixmapXOffset = 4;
	} else {
		w += 8;
		textXOffset = 4;
		pixmapXOffset = (w - Icona.width()) / 2;
	}
	pixmapYOffset = 2;
	textYOffset = Icona.height() + 3 + ascent + 2;
	p.end();
		
	mask.resize(w, Icona.height() + 3 + ascent + descent + 4);
	mask.fill(color0);
	
	QPainter p2;
	p2.begin(&mask);
	p2.setFont(font);
   p2.fillRect(textXOffset-2, 
   				textYOffset-10, 
   				p2.fontMetrics().width(name)+4, 
   				p2.fontMetrics().ascent()+p2.fontMetrics().descent()+4, white);
	p2.drawText(textXOffset, textYOffset, name);
	bitBlt(&mask, pixmapXOffset, pixmapYOffset, Icona.mask());
	p2.end();
	width = w;
	height = Icona.height() + 3 + ascent + descent + 4;
}

void DeskObj::Select()
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->IconaSel);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->IconaSel);
	} else {
		Icona = QPixmap(IData[IDataRef]->IconaSel);
	}
	CalcDim();
	update();
}

void DeskObj::Unselect()
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->Icona);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->Icona);
	} else {
		Icona = QPixmap(IData[IDataRef]->Icona);
	}
	CalcDim();
	update();
}

void DeskObj::SelOpen()
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	if(type == DR)
		Icona = QPixmap(IFoldData->IconaOpen);
	if(type == DRLNK)
		Icona = QPixmap(IFoldLinkData->IconaOpen);
	CalcDim();
	update();
}

void DeskObj::UnselOpen()
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	if(type == DR)
		Icona = QPixmap(IFoldData->IconaSelOpen);
	if(type == DRLNK)
		Icona = QPixmap(IFoldLinkData->IconaSelOpen);
	CalcDim();
	update();
}

void DeskObj::Close()
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->IconaSel);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->IconaSel);
	} else {
		Icona = QPixmap(IData[IDataRef]->IconaSel);
	}
	clickonme(id);
	CalcDim();
	update();
}

void DeskObj::OpenFromPopUp()
{
	emit doubleclick(name, path, type);	
}

void DeskObj::InfoFromPopUp()
{

}

void DeskObj::RemoveMe()
{
	emit removeme(id);
}

//-------------------------------------------------------------------------
//                               eventi
//-------------------------------------------------------------------------
void DeskObj::resizeEvent(QResizeEvent *)
{
	XShapeCombineMask(x11Display(), winId(), ShapeBounding, 0, 0, 
			mask.handle(), ShapeSet);
}

void DeskObj::mousePressEvent(QMouseEvent *e)
{
	if(e->button() == RightButton) 
		popupMenu->popup(mapToGlobal(e->pos()));

	if(isOpenUnsl)
		SelOpen();
	if(isSelect) {

	} else {
		Select();
	}
	emit clickonme(id);
}

void DeskObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	emit doubleclick(name, path, type);
}

void DeskObj::mouseMoveEvent(QMouseEvent *)
{
	if(!isDragging) {
		emit dragattempt(path, parentpath, id);
		isDragging = TRUE;
	} else {
		emit mousemove(id);
	}
}

void DeskObj::mouseReleaseEvent(QMouseEvent *e)
{
	if(isDragging) {
		isDragging = FALSE;
		if(e->state() & ControlButton) 
			emit dragdone(TRUE);
		else
			emit dragdone(FALSE);
	}
}

void DeskObj::paintEvent(QPaintEvent *)
{
	bitBlt(this, pixmapXOffset, pixmapYOffset, &Icona);
  
	QPainter p;
	p.begin(this);
	if(!isSelect) {
		p.setPen(black);   
		p.drawText(textXOffset, textYOffset, name);
   } else {
   	p.fillRect(textXOffset-2, textYOffset-10, 
   		p.fontMetrics().width(name)+4, 
   		p.fontMetrics().ascent()+p.fontMetrics().descent()+4, black);
   	p.setPen(white);   
		p.drawText(textXOffset, textYOffset, name);
	}    
	p.end();
}

void DeskObj::DragMove()
{
	if(type == DR || type == DRLNK) {
		if(! isSelect && ! isOpen) Select();
		if(! isSelect && isOpen) SelOpen();
	}
}

void DeskObj::DragLeave()
{
	if(type == DR || type == DRLNK) {
		if(isSelect && isOpen) UnselOpen();
		if(isSelect && !isOpen) Unselect();
	}
}

