#include "fileobj.h"
#include "mfinderapp.h"
#include <stdio.h>
#include <stdlib.h>

extern ProgInfo prInfo[];
extern int prInfoNum;

extern char Editor[];
extern char Xterm[];

extern char IconPath[];
extern IconData *IData[];
extern int IDataNum;
extern FolderIconData *IFoldData, *IFoldLinkData;

FileObj::FileObj(QWidget *parent, const char *name, 
							int n, int idParent, char *ParentName)
        : QWidget(parent, name)
{
	QColor x = QColor(230, 230, 230);
	QColorGroup CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));

	idLocal = n;
	this->idParent = idParent;
	strncpy(this->name, name, 256);
	if(this->name[0] == '.')
		isdotfile = TRUE;
	else
		isdotfile = FALSE;
	strncpy(this->ParentName, ParentName, 256);
	if(strcmp(ParentName, "/") == 0) 
		sprintf(myPath, "%s%s", ParentName, name);
	else
		sprintf(myPath, "%s/%s", ParentName, name);
		
	UpdateMyData();	
		
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isDragging = FALSE;
	isdothided = FALSE;
	
	Icona = QPixmap();
	UpdateIcon();
	
	Label = new FileLabel(parentWidget(), this->name);
	Label->installEventFilter(this); 
	Label->cutText();
	connect(Label, SIGNAL(selattempt(int)), this, SLOT(slotsel(int)));		

	popupMenu = new Popup();
	popupMenu->insertItem("Open", this, SLOT(OpenFromPopUp()));
	popupMenu->insertItem("Rename", this, SLOT(RenameFromPopUp()));
	popupMenu->insertItem("Info", this, SLOT(InfoFromPopUp()));
	if(type != DR && type != DRLNK) {
		popupMenu->insertItem("Edit", this, SLOT(EditFromPopUp()));
		popupMenu->insertItem("Open With...", this, SLOT(OpWithFromPopUp()));
	}
	popupMenu->insertItem("Delete", this, SLOT(DeleteFromPopUp()));

	resize(Icona.width(), Icona.height());
}

void FileObj::UpdateMyData()
{
	struct stat listStat;

   lstat(myPath, &listStat);

	if(S_ISREG(listStat.st_mode)) {
		if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH))
			type = FILEXE;
		else
			type = FIL;
	}
	if(S_ISDIR(listStat.st_mode)) 
		type = DR;
	if(S_ISLNK(listStat.st_mode)) {
		int ln = readlink(myPath, myLinkTo, 512);
		if(chdir(myPath) != 0) {
			if(listStat.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) {
				type = FILEXELNK;
			} else {
				type = FILLNK;
			}
		} else {
			chdir(getenv("HOME"));
			type = DRLNK;
		}
	}		
	if(S_ISCHR(listStat.st_mode)) 
		type = CHRDV;		
	if(S_ISBLK(listStat.st_mode)) 
		type = BLCKDV;	
	if(S_ISFIFO(listStat.st_mode)) 
		type = FIF;		
	if(S_ISSOCK(listStat.st_mode)) 
		type = SCKT;		
}

void FileObj::UpdateIcon()
{
	bool found = FALSE;
	char name2[256] = "";
	char *pref = "", *suff = "", *token = "", *delims = ".";
	
	sprintf(name2, "%s", name);
	token = strtok(name2, delims);
	pref = token;
	while(token != NULL) {
		suff = token;
		token = strtok(NULL, delims);
	}
	
	switch(type) {
		case FIL:
			if(name[0] == '.') {
				for(int i = 4; i < IDataNum; i++) {
					if(strcmp(IData[i]->pref, ".") == 0) {
						sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
						IDataRef = i;
						found = TRUE;
						break;
					}
				}
			}
			if(found) break;
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(pref, IData[i]->pref) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(suff, IData[i]->suff) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			sprintf(pixname, "%s/%s", IconPath, IData[0]->IcnName);
			IDataRef = 0;
			break;
		case FILEXE:
			for(int i = 4; i < IDataNum; i++) {
				if(strcmp(pref, IData[i]->pref) == 0) {
					sprintf(pixname, "%s/%s", IconPath, IData[i]->IcnName);
					IDataRef = i;
					found = TRUE;
					break;
				}
			}
			if(found) break;
			sprintf(pixname, "%s/%s", IconPath, IData[1]->IcnName);
			IDataRef = 1;
			break;
		case FILLNK:
			sprintf(pixname, "%s/%s", IconPath, IData[2]->IcnName);
			IDataRef = 2;
			break;
		case FILEXELNK:
			sprintf(pixname, "%s/%s", IconPath, IData[3]->IcnName);
			IDataRef = 3;
			break;
		case DR:
			sprintf(pixname, "%s/%s", IconPath,IFoldData->IcnName);
			break;
		case DRLNK:
			sprintf(pixname, "%s/%s", IconPath,IFoldLinkData->IcnName);
			break;
		default:
			sprintf(pixname, "%s/%s", IconPath, IData[0]->IcnName);
			IDataRef = 0;
			break;
	}

	if(type == DR) {
		Icona = QPixmap(IFoldData->Icona);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->Icona);
	} else {
		Icona = QPixmap(IData[IDataRef]->Icona);
	}
	
	update();
}

void FileObj::Select()
{
	isSelect = TRUE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->IconaSel);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->IconaSel);
	} else {
		Icona = QPixmap(IData[IDataRef]->IconaSel);
	}
	Label->select();
	update();
	emit selattempt(idLocal);
}

void FileObj::Unselect()
{
	isSelect = FALSE;
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->Icona);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->Icona);
	} else {
		Icona = QPixmap(IData[IDataRef]->Icona);
	}
	Label->unselect();
	update();
}

void FileObj::SelOpen()
{
	isOpenUnsl = FALSE;
	isOpen = TRUE;
	isSelect = TRUE;
	if(type == DR)
		Icona = QPixmap(IFoldData->IconaOpen);
	if(type == DRLNK)
		Icona = QPixmap(IFoldLinkData->IconaOpen);
	Label->select();
	update();
	emit selattempt(idLocal);
}

void FileObj::UnselOpen()
{
	isOpenUnsl = TRUE;
	isOpen = TRUE;
	isSelect = FALSE;
	if(type == DR)
		Icona = QPixmap(IFoldData->IconaSelOpen);
	if(type == DRLNK)
		Icona = QPixmap(IFoldLinkData->IconaSelOpen);
	Label->unselect();
	update();
}

void FileObj::Open(QMouseEvent *e)
{
	char cmd[1024] = "";
	char name2[256] = "";
	char *pref = "", *suff = "", *token = "", *delims = ".";
	bool found = FALSE;

	if(type == DR || type == DRLNK) {
		emit openattempt(idLocal, e);
	}
	else if(type == FIL) {
		sprintf(name2, "%s", name);
		token = strtok(name2, delims);
		pref = token;
		while(token != NULL) {
			suff = token;
			token = strtok(NULL, delims);
		}
		if(name[0] == '.') {
			for(int i = 0; i < prInfoNum; i++) {
				if(strcmp(prInfo[i].pref, ".") == 0) {
					sprintf(cmd, "%s", prInfo[i].ProgName);
					found = TRUE;
					break;
				}
			}
		}
		if(!found) {
			for(int i = 0; i < prInfoNum; i++) {
				if(strcmp(pref, prInfo[i].pref) == 0) {
					sprintf(cmd, "%s", prInfo[i].ProgName);
					found = TRUE;
					break;
				}
			}
		}
		if(!found) {
			for(int i = 0; i < prInfoNum; i++) {
				if(strcmp(suff, prInfo[i].suff) == 0) {
					sprintf(cmd, "%s", prInfo[i].ProgName);
					found = TRUE;
					break;
				}
			}
		}
		if(!found)
			sprintf(cmd, "%s", Editor);
			
		Unselect();
		chdir(ParentName);
		emit openfile(cmd, myPath);
		chdir(getenv("HOME"));
	}
}

void FileObj::Close()
{
	isOpen = FALSE;
	isOpenUnsl = FALSE;
	isSelect = TRUE;
	if(type == DR) {
		Icona = QPixmap(IFoldData->IconaSel);
	} else if(type == DRLNK) {
		Icona = QPixmap(IFoldLinkData->IconaSel);
	} else {
		Icona = QPixmap(IData[IDataRef]->IconaSel);
	}
	Label->select();
	emit selattempt(idLocal);
	update();
}

void FileObj::InfoObj()
{
	struct dirent **namelist;
	struct stat listStat;
	struct passwd *usr;
   struct group *grp;
	struct tm *t;
	mode_t oldperm = 0L, newperm = 0L;
	char oldusr[32] = "", oldgroup[32] = "";
	int m, ln;

   lstat(myPath, &listStat);
	
	usr = getpwuid(listStat.st_uid);
   grp = getgrgid(listStat.st_gid);
				
	IWin = new InfoWin(0L, name, TRUE);

	sprintf(IWin->fPath, "Path: %s", myPath);
	if(type == FILLNK || type == FILEXELNK || type == DRLNK) {
		sprintf(IWin->fSize, "Link to: %s", myLinkTo);
	} else if(type == FIL || type == FILEXE) {
		sprintf(IWin->fSize, "Size: %i", listStat.st_size);
	} else if(type == DR) {
		int m = scandir(myPath, &namelist, 0, alphasort);
		sprintf(IWin->fSize, "Elements: %i", m - 2);
	} else {
		sprintf(IWin->fSize, " ");
	}
	
	t = localtime(&listStat.st_mtime);
	sprintf(IWin->lastMod, 
		"Last modification: %02i:%02i %02i.%02i.%04i", 
		t->tm_hour, t->tm_min, t->tm_mday, 
		t->tm_mon + 1, t->tm_year + 1900);

   bool IamRoot = (geteuid() == 0);
   bool IsMyFile = (geteuid() == listStat.st_uid);
	IWin->userEdit->setText(usr->pw_name);
	sprintf(oldusr, "%s", IWin->userEdit->text());
	IWin->userEdit->setEnabled(IamRoot || IsMyFile);
	QString grname = grp->gr_name;
	if(grname.isEmpty()) grname.sprintf("%d", grp->gr_gid);
	IWin->groupEdit->setText(grname);
	sprintf(oldgroup, "%s", IWin->groupEdit->text());
	IWin->groupEdit->setEnabled(IamRoot || IsMyFile);

	IWin->ckUread->setChecked((listStat.st_mode & S_IRUSR) == S_IRUSR);
	IWin->ckUread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IRUSR);
	IWin->ckUwrite->setChecked((listStat.st_mode & S_IWUSR) == S_IWUSR);
	IWin->ckUwrite->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IWUSR);
	IWin->ckUexec->setChecked((listStat.st_mode & S_IXUSR) == S_IXUSR);
	IWin->ckUexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXUSR);
	IWin->ckUID->setChecked((listStat.st_mode & S_ISUID) == S_ISUID);
	IWin->ckUID->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISUID);

	IWin->ckGread->setChecked((listStat.st_mode & S_IRGRP) == S_IRGRP);
	IWin->ckGread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IRGRP);
	IWin->ckGwrite->setChecked((listStat.st_mode & S_IWGRP) == S_IWGRP);
	IWin->ckGwrite->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IWGRP);
	IWin->ckGexec->setChecked((listStat.st_mode & S_IXGRP) == S_IXGRP);
	IWin->ckGexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXGRP);
	IWin->ckGID->setChecked((listStat.st_mode & S_ISGID) == S_ISGID);
	IWin->ckGID->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISGID);

	IWin->ckOread->setChecked((listStat.st_mode & S_IROTH) == S_IROTH);
	IWin->ckOread->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IROTH);
	IWin->ckOwrite->setChecked((listStat.st_mode & S_IWOTH) == S_IWOTH);
	IWin->ckOwrite->setEnabled(IsMyFile || IamRoot);	
	oldperm |= (listStat.st_mode & S_IWOTH);
	IWin->ckOexec->setChecked((listStat.st_mode & S_IXOTH) == S_IXOTH);
	IWin->ckOexec->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_IXOTH);
	IWin->ckSticky->setChecked((listStat.st_mode & S_ISVTX) == S_ISVTX);
	IWin->ckSticky->setEnabled(IsMyFile || IamRoot);
	oldperm |= (listStat.st_mode & S_ISVTX);

	if(IWin->exec()) {

		if(IWin->ckUread->isChecked())
		   newperm |= S_IRUSR;
		if(IWin->ckUwrite->isChecked())
		   newperm |= S_IWUSR;
		if(IWin->ckUexec->isChecked())
		   newperm |= S_IXUSR;
		if(IWin->ckUID->isChecked())
		   newperm |= S_ISUID;

		if(IWin->ckGread->isChecked())
		   newperm |= S_IRGRP;
		if(IWin->ckGwrite->isChecked())
		   newperm |= S_IWGRP;
		if(IWin->ckGexec->isChecked())
		   newperm |= S_IXGRP;
		if(IWin->ckGID->isChecked())
		   newperm |= S_ISGID;

		if(IWin->ckOread->isChecked())
		   newperm |= S_IROTH;
		if(IWin->ckOwrite->isChecked())
		   newperm |= S_IWOTH;
		if(IWin->ckOexec->isChecked())
		   newperm |= S_IXOTH;
		if(IWin->ckSticky->isChecked())
		   newperm |= S_ISVTX;

		if(oldperm != newperm) {
			if(chmod(myPath, newperm)) {
				sprintf(ERR, "Could not change permissions\n");
				if(dialog != 0L)
					dialog->~Dialog();
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
			} else {
				UpdateMyData();
				UpdateIcon();
				update();
			}
		}
		if(strcmp(oldusr, IWin->userEdit->text()) != 0 ||
									strcmp(oldgroup, IWin->groupEdit->text()) != 0) {

			usr = getpwnam(IWin->userEdit->text());
			grp = getgrnam(IWin->groupEdit->text());
			if(usr == 0L) {
				sprintf(ERR, "No user %s!\n", IWin->userEdit->text());
				if(dialog != 0L)
					dialog->~Dialog();
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
				return;
			}
			if(grp == 0L) {
				sprintf(ERR, "No group %s!\n", IWin->groupEdit->text());
				if(dialog != 0L)
					dialog->~Dialog();
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
				return;
			}
			if(chown(myPath, usr->pw_uid, grp->gr_gid) != 0) {
				sprintf(ERR, "Could not change owner/group\n");
				if(dialog != 0L)
					dialog->~Dialog();
				dialog = new Dialog(0L, "Error", TRUE, 0, TRUE, TRUE, FALSE, FALSE, ERR);
				m = dialog->exec();						
				return;
			} 
			UpdateMyData();
		}
	}

}

//-------------------------------------------------------------------------
//                               slots
//-------------------------------------------------------------------------
void FileObj::slotsel(int)   // lo usa la label
{
	if(!isOpen) Select();
	else SelOpen();
}

void FileObj::OpenFromPopUp()
{
	QMouseEvent e = QMouseEvent(0L, this->pos(), 0L, 0L);
	Open(&e);
}

void FileObj::RenameFromPopUp()
{
	emit renameme();
}

void FileObj::InfoFromPopUp()
{
	InfoObj();
}

void FileObj::EditFromPopUp()
{
	emit editme();
}

void FileObj::OpWithFromPopUp()
{
	emit openmewith();
}

void FileObj::DeleteFromPopUp()
{
	emit deleteme();
}

//-------------------------------------------------------------------------
//                               eventi
//-------------------------------------------------------------------------
void FileObj::mousePressEvent(QMouseEvent *e)
{
	if(e->button() == RightButton) 
		popupMenu->popup(mapToGlobal(e->pos()));

	if(e->state() & ShiftButton) {
		emit shiftclick(1);
		if(isSelect) {
			if(isOpen) UnselOpen();
			else Unselect();
			return;
		} else {
			if(isOpen) SelOpen();
			else Select();
		}
	} else {
		emit shiftclick(0);
		if(isOpenUnsl)
			SelOpen();
		if(isSelect) {

		} else {
			Select();
		}
	}
}

void FileObj::mouseDoubleClickEvent(QMouseEvent *e)
{
	Open(e);
}

void FileObj::mouseMoveEvent(QMouseEvent *)
{
	if( ! isDragging) {
		emit dragattempt(name, ParentName, idLocal, idParent);
		isDragging = TRUE;
	} else {
		emit mousemove();
	}
}

void FileObj::mouseReleaseEvent(QMouseEvent *e)
{
	if(isDragging) {
		isDragging = FALSE;
		if(e->state() & ControlButton) 
			emit dragdone(TRUE);
		else
			emit dragdone(FALSE);
	}
}

void FileObj::DragMove()
{
	if(type == DR || type == DRLNK) {
		if(! isSelect && ! isOpen) Select();
		if(! isSelect && isOpen) SelOpen();
	}
}

void FileObj::DragLeave()
{
	if(type == DR || type == DRLNK) {
		if(isSelect && isOpen) UnselOpen();
		if(isSelect && ! isOpen) Unselect();
	}
}

bool FileObj::eventFilter(QObject *, QEvent *e)
{
  if(e->type() == Event_KeyPress) {
		QKeyEvent *k = (QKeyEvent*)e;
		if(k->key() == Key_Tab)
			return TRUE;
  }
  if(e->type() == Event_MouseButtonPress || e->type() == Event_MouseButtonDblClick) {
		this->event(e);
		return TRUE;
  }
  return FALSE;
}

void FileObj::closeEvent(QCloseEvent *e)
{
	Label->close();
	QWidget::closeEvent(e);
}

void FileObj::paintEvent(QPaintEvent *)
{
	QPainter p;
	
	p.begin(this);
	p.drawPixmap(0, 0, Icona);
	
	p.end();
}
