// ----------------------------------------------------------------------------
// $Header$
// ----------------------------------------------------------------------------
// $RCSfile$
// ----------------------------------------------------------------------------
// Written 1997 by Christoph Pesch
// ----------------------------------------------------------------------------
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

#include <qpainter.h>
#include "infopanel.h"

// ----------------------------------------------------------------------------
// METHOD: infopanel
// infopanel(QWidget* parent, const char* name, bool def)
//
// DESCRIPTION: 
// Default constructor.
// Set def to FALSE if you want to use your own design.
// Set def to TRUE if you want to use the default design (and maybe
// some additional own design).
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

infopanel::infopanel(QWidget* parent, const char* name, bool def)
   : QFrame(parent, name)
{
   version = title = comment = author = NULL;
   useDefault = def;
   setCaption("Info");
}



// ----------------------------------------------------------------------------
// METHOD: ~infopanel
// ~infopanel()
//
// DESCRIPTION: 
// Destructor for the infopanel class.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

infopanel::~infopanel() 
{
   hide();
   delete[] version;
   delete[] title;
   delete[] author;
   delete[] comment;
}


// ----------------------------------------------------------------------------
// METHOD: set_version
// void set_version(const char* v)
//
// DESCRIPTION: 
// Sets the version text of the application.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::set_version(const char* v)
{
   delete[] version;
   version = new char[strlen(v)+1];
   strcpy((char*)version, v);
   update();
}


// ----------------------------------------------------------------------------
// METHOD: set_title
// void set_title(const char* t)
//
// DESCRIPTION: 
// Sets the title text of the application.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::set_title(const char* t)
{
   delete[] title;
   title = new char[strlen(t)+1];
   strcpy((char*)title, t);
   update();
}


// ----------------------------------------------------------------------------
// METHOD: set_author
// void set_author(const char* a)
//
// DESCRIPTION: 
// Sets the text for the author of the application.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::set_author(const char* a)
{
   delete[] author;
   author = new char[strlen(a)+1];
   strcpy((char*)author, a);
   update();
}


// ----------------------------------------------------------------------------
// METHOD: set_comment
// void set_comment(const char* c)
//
// DESCRIPTION: 
// Sets the comment text of the application.
// Use "\n" to indicate a new line.
//
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::set_comment(const char* c)
{
   delete[] comment;
   comment = new char[strlen(c)+1];
   strcpy((char*)comment, c);
   update();
}


// ----------------------------------------------------------------------------
// METHOD: use_default
// void use_default(bool d)
//
// DESCRIPTION: 
// With d=TRUE the default design of the infopanel is used.
// With d=FALSE the infopanel is empty and you have to add you own widgets.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::use_default(bool d)
{
   useDefault = d;
   update();
}


// ----------------------------------------------------------------------------
// METHOD: get_version
// const char* get_version()
//
// DESCRIPTION: 
// Returns the version text.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

const char* infopanel::get_version() const
{
   return version;
}


// ----------------------------------------------------------------------------
// METHOD: get_title
// const char* get_title()
//
// DESCRIPTION: 
// Returns the title text.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

const char* infopanel::get_title() const
{
   return title;
}


// ----------------------------------------------------------------------------
// METHOD: get_author
// const char* get_author() const
//
// DESCRIPTION: 
// Returns the author text.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

const char* infopanel::get_author() const
{
   return author;
}


// ----------------------------------------------------------------------------
// METHOD: get_comment
// const char* get_comment()
//
// DESCRIPTION: 
// Returns the comment text.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

const char* infopanel::get_comment() const
{
   return comment;
}


// ----------------------------------------------------------------------------
// METHOD: use_default
// bool use_default()
//
// DESCRIPTION: 
// Returns whether the default design is used for drawing.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

bool infopanel::use_default() const
{
   return useDefault;
}


// ----------------------------------------------------------------------------
// METHOD: drawContents
// void drawContents(QPainter *p)
//
// DESCRIPTION: 
// Draws the contents of the infopanel.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

void infopanel::drawContents(QPainter *p)
{
   QFrame::drawContents(p);
   if(useDefault)
   {
      QRect area=rect();
      int upperHeight=area.height()/3, lowerHeight=area.height()-upperHeight;

      if(NULL != title)
      {
         p->setFont(QFont("Helvetica", 24, QFont::Bold ));
	 p->drawText(area.left(), area.top(), area.width(), upperHeight-12,
		     AlignCenter | WordBreak, title, -1);
      }
      p->drawLine(area.left(), area.top()+area.height()/3,
		  area.right(), area.top()+area.height()/3);
      if(NULL != comment)
      {
         p->setFont(QFont("Helvetica", 12, QFont::Normal ));
	 p->drawText(area.left()+area.width()/20,
		     area.top() + upperHeight + lowerHeight/20,
		     (9*area.width())/10, (9*lowerHeight)/10,
		     AlignLeft | AlignVCenter | WordBreak, comment, -1);
      }
      if(NULL != author)
      {
	 p->setFont(QFont("Helvetica", 12, QFont::Bold));
	 p->drawText(area.left(), area.top(), area.width()-1, upperHeight,
		     AlignHCenter | AlignBottom, author, -1);
      }
      if(NULL != version)
      {
         p->setFont(QFont("Helvetica", 10, QFont::Light));
	 p->drawText(area.left(), area.top(), area.width()-1, upperHeight,
		     AlignRight | AlignBottom, version, -1);
      }
   }
}

// ----------------------------------------------------------------------------
// $Log$
// ----------------------------------------------------------------------------

// EOF
