// ----------------------------------------------------------------------------
// $Header$
// ----------------------------------------------------------------------------
// $RCSfile$
// ----------------------------------------------------------------------------
// Written 1997 by Christoph Pesch
// Some minor modifications (NULL pointer checks) by Tim N. van der Leeuw
// ----------------------------------------------------------------------------
// LAST CHANGE DONE: 15. June 1998
// ----------------------------------------------------------------------------

#include <qapp.h>
#include <qevent.h>
#include <qsignal.h>
#include <qpixmap.h>
#include <qcolor.h>
#include <iostream.h>
#include "nextapp.h"
extern "C"
{
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <WMaker.h>
#include "app.h"
#include "menu.h"
   enum { wmSelectItem = 1 };
}
static Atom _XA_WINDOWMAKER_MENU=0;

// ----------------------------------------------------------------------------
// METHOD: nextapp
// nextapp::nextapp(int &argc, char** argv, const char *cln)
//
// DESCRIPTION: 
// The default construtor.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

nextapp::nextapp(int &argc, char** argv, const char *cln,
		 const unsigned char* icondata)
   : QApplication(argc, argv)
{
   WId win = info.winId();
   Display *dpy=info.x11Display();
   XWMHints *hints;
   XClassHint classhint;
   appClassName = (char*)(cln == NULL ? argv[0] : cln);

   if(icondata != NULL)
   {
      icon = new QPixmap();
      icon->loadFromData(icondata, strlen((const char*)icondata), "XPM");
      if(icon->isNull())
      {
	 delete icon;
	 icon = NULL;
      }
      else{ info.setIcon(*icon); }
   }
   else{ icon = NULL; }
   
   classhint.res_name = (char*)name();
   classhint.res_class = appClassName;
   XSetClassHint(dpy, win, &classhint);
   
   hints = XGetWMHints(dpy, win);
   hints->window_group = win;
   hints->flags |= WindowGroupHint; 
   XSetWMHints(dpy, win, hints);

   XSetCommand(dpy, win, argv, argc);
   
   app = WMAppCreateWithMain(dpy, DefaultScreen(dpy), win);
   menu = WMMenuCreate(app, (char*)name());
   WMAppSetMainMenu(app, menu);

}



// ----------------------------------------------------------------------------
// METHOD: addWidget
// void addWidget(QWidget *w)
//
// DESCRIPTION: 
// 
// SEE ALSO: 
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

void nextapp::addWidget(QWidget *w)
{
   if(w == NULL){ return; }

   if(icon != NULL){ w->setIcon(*icon); } 
   
   WId win = w->winId();
   Display *dpy=w->x11Display();
   XWMHints *hints;
   XClassHint classhint;
   
   WMAppAddWindow(app, win);
   
   classhint.res_name = (char*)name();
   classhint.res_class = appClassName;
   XSetClassHint(dpy, win, &classhint);
   
   hints = XGetWMHints(dpy, win);
   hints->window_group = info.winId();
   hints->flags |= WindowGroupHint; 
   XSetWMHints(dpy, win, hints);
}


// ----------------------------------------------------------------------------
// METHOD: hide
// int hide()
//
// DESCRIPTION: 
// Hides the application.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

int nextapp::hide()
{
   if(app != NULL)
   {
      WMHideApplication(app);
   }
   return 1;
}


// ----------------------------------------------------------------------------
// METHOD: hide_others
// int hide_others()
//
// DESCRIPTION: 
// Hides all other applications.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

int nextapp::hide_others()
{
   if(app != NULL)
   {
      WMHideOthers(app);
   }
   return 1;
}



// ----------------------------------------------------------------------------
// FUNCTION: findEntry
// static wmMenuEntry* findEntry(WMMenu *menu, int tag)
//
// DESCRIPTION: 
// findEntry taken from WindowMaker source:
// Copied from libwm. Copyright (c) Alfredo Kojima.
// I had to copy this because it is a static function in libwm ...
// 
// LAST CHANGE DONE: 15. June 1998, TNL. Check for possible NULL
// parameter menu.
// ----------------------------------------------------------------------------

static wmMenuEntry* findEntry(WMMenu *menu, int tag)
{
	wmMenuEntry *entry=NULL;

	if (menu == NULL)
		return NULL;

	entry = menu->first;

	while (entry) {
		if (entry->tag == tag) {
	    	return entry;
		}
		if (entry->cascade) {
	    	wmMenuEntry *tmp;
	    	tmp = findEntry(entry->cascade, tag);
	    	if (tmp)
	      	return tmp;
		}
		entry = entry->next;
	}
	return NULL;
}



// ----------------------------------------------------------------------------
// METHOD: x11EventFilter
// bool nextapp::x11EventFilter(XEvent *e)
//
// DESCRIPTION: 
// Handles the menu events and calls QApplication::x11EventFilter(e)
// to handle all other events.
// 
// LAST CHANGE DONE: 15. June 1998, TNL. Check for e == NULL or app == NULL
// ----------------------------------------------------------------------------

bool nextapp::x11EventFilter(XEvent *e)
{
   // I made the assumptions that for e == NULL
   // TRUE should be returned (no even, so it's
   // handled) and that for app == NULL the QApplication
   // should handle the event.
   if (e == NULL)
       return TRUE;
   if (app == NULL)
       return QApplication::x11EventFilter(e);
   
   if(!_XA_WINDOWMAKER_MENU)
   {
      _XA_WINDOWMAKER_MENU = XInternAtom(app->dpy, "_WINDOWMAKER_MENU", False);
   }
   if(e->type == ClientMessage && e->xclient.format == 32
      && e->xclient.message_type == _XA_WINDOWMAKER_MENU
      && e->xclient.window == app->main_window)
   {
      wmMenuEntry *entry;

      if(e->xclient.data.l[1] == wmSelectItem)
      {
	 entry = findEntry(app->main_menu, e->xclient.data.l[2]);
	 if(entry && entry->clientData)
	 {
	    ((QSignal*)(entry->clientData))->activate();
	 }
      }
      return TRUE;
   }
   
   return QApplication::x11EventFilter(e);
}


// ----------------------------------------------------------------------------
// METHOD: main_menu
// WMMenu* main_menu()
//
// DESCRIPTION: 
// Returns a pointer to the main menu.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

WMMenu* nextapp::main_menu()
{
   return menu;
}


// ----------------------------------------------------------------------------
// METHOD: add_submenu
// WMMenu* add_submenu(WMMenu* m, const char* name)
//
// DESCRIPTION: 
// Adds a submenu with title name to m and returns a pointer.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

WMMenu* nextapp::add_submenu(WMMenu* m, const char* name)
   return submenu;
{
   submenu = WMMenuCreate(app, (char*)name);
   WMMenuAddSubmenu(m, (char*)name, submenu);
}


// ----------------------------------------------------------------------------
// METHOD: add_item
// void add_item(WMMenu* m, const char* name, QSignal* s, char* rtext)
//
// DESCRIPTION: 
// Adds an item with text name and rtext(?) to menu m.
// Connects the menu via QSignal s to a slot.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

void nextapp::add_item(WMMenu* m, const char* name, QSignal* s, char* rtext)
{
   WMMenuAddItem(m, (char*)name, NULL, (WMMenuAction)s, NULL, rtext);
}


// ----------------------------------------------------------------------------
// METHOD: add_item
// void add_item(WMMenu* m, const char* name, const char* s, char* rtext)
//
// DESCRIPTION: 
// Adds an item with text name and rtext(?) to menu m.
// Connects the menu to the slot s.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

void nextapp::add_item(WMMenu* m, const char* name, QObject* r,
		       const char* slot, char* rtext)
{
   if(r != NULL && slot != NULL)
   {
      QSignal *s=new QSignal();
      s->connect(r, slot); 
      WMMenuAddItem(m, (char*)name, NULL, (WMMenuAction)s, NULL, rtext);
   }
   else
   {
      WMMenuAddItem(m, (char*)name, NULL, NULL, NULL, rtext);
   }
}


// ----------------------------------------------------------------------------
// METHOD: realize_menus
// void realize_menus()
//
// DESCRIPTION: 
// Calls the necessary WM function to realize the menus.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

void nextapp::realize_menus()
{
   if(app != NULL)
   {
      WMRealizeMenus(app);
   }
}


// ----------------------------------------------------------------------------
// METHOD: get_infopanel
// infopanel* get_infopanel()
//
// DESCRIPTION: 
// Returns a pointer to the infopanel widget.
// 
// LAST CHANGE DONE: 30. September 1997
// ----------------------------------------------------------------------------

infopanel* nextapp::get_infopanel()
{
   return &info;
}


// ----------------------------------------------------------------------------
// METHOD: show_info
// int show_info()
//
// DESCRIPTION: 
// Shows the information panel.
// 
// LAST CHANGE DONE: 29. September 1997
// ----------------------------------------------------------------------------

int nextapp::show_info()
{
   info.show();
}

// ----------------------------------------------------------------------------
// $Log$
// ----------------------------------------------------------------------------

// EOF
