#define INCLUDE_MENUITEM_DEF

#include "popup.h"
#include <qmenubar.h>
#include <qpainter.h>
#include <qdrawutl.h>
#include <qapp.h>
#include <qpmcache.h>
#include <ctype.h>

Popup::Popup(QWidget *parent, const char *name)
      : QPopupMenu(parent, name)
{
	x = QColor(160, 164, 160);
	CGnorm = QColorGroup(black, x, x.light(), x.dark(), x.dark(120), black, x);
	CGinv = QColorGroup(white, black, x.light(), x.dark(), x.dark(120), white, black);
	setPalette(QPalette(CGnorm, CGnorm, CGnorm));
	
	font = QFont("Helvetica", 12);
	setFont(font);
}

int Popup::cellWidth(int col)
{
	if(col == 0)
		return 2;
	else
		return width()-4;	
}

int Popup::cellHeight(int row)
{
	QFontMetrics fm = fontMetrics();

	int h = fm.height() + 4;
	
	return h;
}

void Popup::updateSize()
{
	int height = 0;
	int max_width = 10;
	QFontMetrics fm(font);
	QMenuItemListIt it(*mitems);
	register QMenuItem *mi;
    
	int cellh = fm.height() + 4;
    
	while((mi = it.current())) {
		int w = 0;
		int itemHeight = 0;

	   if(itemHeight < cellh)
			itemHeight = cellh;
	   const char *s = mi->text();

		w += fm.width(s);

		height += itemHeight;

		if(max_width < w)
	    	max_width = w;
	    	
		++it;
	}
	
	max_width += 6;
	max_width += 4;
	if(max_width < 100)
		max_width = 100;
	setNumRows(mitems->count());
	resize(max_width+2*frameWidth(), height+2*frameWidth());
	badSize = FALSE;
}

void Popup::show()
{
	updateSize();
	QWidget::show();
}

void Popup::paintCell(QPainter *p, int row, int col)
{
	QMenuItem *mi;
	int cellh = cellHeight(row);
	int cellw = cellWidth(col);
	bool act;

	act = (row == actItem);
	mi = mitems->at(row);
	if(!mi->isDirty())
		return;
		
	if(col == 0) {
		qDrawPlainRect(p, 0, 0, 0, 0, CGnorm.background(), 1, 0);
		if(act)
			p->fillRect(0, 0, totalWidth()-2, cellh, QColor(250, 250, 250));
		else
			p->fillRect(0, 0, totalWidth()-2, cellh, CGnorm.background());
			
		if(row > 0) {	
			p->setPen(QColor(20, 20, 20));
			p->drawLine(-1, 0, totalWidth(), 0);
			p->setPen(QColor(230, 230, 230));
			p->drawLine(0, 1, totalWidth()-2, 1);
		}	
		return;
	} 
	
	const int text_flags = AlignVCenter | ShowPrefix | DontClip | SingleLine;
	p->setPen(black);
	p->drawText(2, 3, cellw-2, cellh-2*2, text_flags, mi->text());
    
	mi->setDirty(FALSE);
}

